(* :Name: LinearProgramming *)
(* :Title: LinearProgramming *)
(* :Author: 
	Michael Carter
	Department of Economics
	University of Canterbury
	Christchurch
	New Zealand
*)

(* :Summary:
	This package contains functions designed to supplement the
	linear programming facility of Mathematica. *)
(* :Context:  *)
(* :Package Version: 1.0 *)
(* :Copyright: Copyright 1994,  Michael Carter  *)
(* :History: Originally written by Michael Carter 1994	*)
(* :Keywords: linear programming, optimization *)
(* :Source: Linear Programming and Mathematica: The Simplex Algorithm
	    Mathematica Journal, Volume 4, Issue 4 *)
(* :Warnings: None. *)
(* :Mathematica Version: 2.2 *)
(* :Limitations: *)
(* :Discussion: *)

BeginPackage["LinearProgramming`"]

StandardLP::usage =
"StandardLP[constraints] expresses a system of linear inequalities
in standard LP form with appropriate slack or surplus variables.
StandardLP[constraints,labels] uses the labels to specify the slack
variables."

Pivot::usage =
"Pivot[tableau] pivots the tableau.
Pivot[tableau, x[[j]] pivots variable x[[j]] into tableau, while
Pivot[tableau,x[[j]],s[[i]] substitutes x[[j]] for s[[i]]"

Simplex::usage =
"Simplex[tableau] applies the simplex algorithm to the tableau, assuming
a feasible solution. Simplex[objective,constraints] maximizes objective 
subject to constraints."

LP::usage =
"LP[objective,constraints] maximizes objective subject to the
constraints using the two phase simplex algorithm. 
LP[tableau] does the same to a problem in tableau form"

FinalTableau::usage=
"FinalTableau[objective,constraints] computes the final tableau for the
problem of maximizing objective subject to constraints."

BasicVariables::usage =
"BasicVariables[tableau] lists the basic variables in tableau"

NonbasicVariables::usage = "NonbasicVariables[tableau] "
"NonbasicVariables[tableau] lists the nonbasic variables in tableau"

ShadowPrice::usage =
"ShadowPrice[tableau,var] computes the shadow price of var in tableau."

ShadowPrices::usage =
"ShadowPrices[tableau,vars] computes the shadow prices of vars in tableau.
ShadowPrices[tableau] extracts the shadow prices of variables in the
first row of tableau."

SensitivityAnalysis::usage =
"SensitivityAnalysis[tableau,objective] extracts from the final tableau
the prices associated with each decision variable, the shadow price of
each resource constraint and their respective ranges of validity.
SensitivityAnalysis[tableau,s[[i]] performs a sensitivity analysis with
respect to the constraint s[[i]] while 
SensitivityAnalysis[tableau,objective,x[[j]] does the same for 
decision variable x[[j]]"

TableauQ::usage = 
"TableauQ[tableau] returns true if tableau is a list of equations"

Intercept::usage = "Intercept[form] returns the constant term in a linear form"

Zero::usage = "Zero[x] returns the transformation rule x -> 0"

Delta::usage = "Delta[i] represents the change in a parameter"

(*
s::usage = "s[j] represents a slack variable"
sl::usage = "sl[j] represents a slack variable"
sg::usage = "sg[j] represents a slack variable"
z::usage = "z is the value of the objective function"
*)

Begin["`Private`"]

(* Specifying the problem *)
StandardLP[constraints_, labels_List] :=
    Transpose @ {labels,constraints} /.
    {{i_, lhs_ <= rhs_}:> s[[i]] == rhs - lhs,
    {i_, lhs_ >= rhs_}:> s[[i]] == lhs - rhs,
    {i_, lhs_ == rhs_}:> {sl[[i]] == rhs - lhs,
                          sg[[i]] == lhs - rhs}} // Flatten

StandardLP[constraints_?TableauQ,labels_List] := constraints

StandardLP[constraints_] :=
	StandardLP[constraints, Range[Length[constraints]]]

StandardLP[constraints_?TableauQ] := constraints

(* The simplex algorithm *)
LimitingResource[tableau_?TableauQ,var_] := Module[{ExpansionRatios},
    ExpansionRatios = (Rest[tableau] /.
        {resource_ == (b_ ?  NumberQ) + a_.  var + x_:>
                                {resource,b/Abs[a]}/; a < 0,
         resource_ == a_.  var + x_ :> {resource,0} /; a < 0,
         resource_ == x_  :> {resource,Infinity}});
    If[#[[2]] < Infinity,#[[1]],None] & @
                MinPair[Sort @ ExpansionRatios]]

NextPivot[tableau_?TableauQ] := Module[{minshadow},
    minshadow = MinPair[ShadowPrices[First[tableau][[2]]]];
    If[minshadow[[2]] < 0,minshadow[[1]],None]
]	

Pivot[tableau_?TableauQ,x_[j_],s_[i_]] := Module[{pivotRow},
    pivotRow = First @ Cases[tableau, s[i] == _];
    tableau /. {pivotRow :> Roots[pivotRow,x[j]],
	  otherRow_Equal :> (otherRow /. 
		Flatten @ Solve[pivotRow,x[j]])} //
		ExpandAll
]

Pivot[tableau_?TableauQ,x_[j_]] := 
	Pivot[tableau,x[j],LimitingResource[tableau,x[j]]]

Pivot[tableau_?TableauQ] := Pivot[tableau,NextPivot[tableau]]

Pivot[tableau_?TableauQ,None] := tableau

Pivot[tableau_?TableauQ,x_[j_],None] := (
	Print["The problem is unbounded"];
	Return[tableau])

Simplex[tableau_?TableauQ] := FixedPoint[Pivot,tableau]

Simplex[objective_,constraints_] := 
	Simplex[Prepend[StandardLP[constraints],
			z == objective]]

LP[tableau_?TableauQ] := Module[{smallest,relaxed,phaseI,
		RevisedConstraints,RevisedObjective},
    smallest = MinPair[Rest[tableau] /. 
	rhs_ == lhs_ :> {rhs,Intercept[lhs]}];
    If[smallest[[2]] >= 0,	(* origin feasible ? *)
	Simplex[tableau],	(* single phase *)
				(* 2 phase required *)
        relaxed = Prepend[Relax[Rest[tableau],a[0]],
			  z == -a[0]];
	phaseI = Simplex @ 
	    Pivot[relaxed,a[0],smallest[[1]]];
        If[Intercept[First[phaseI][[2]]]==0, (* feasible? *)
				(* phaseII *)
	    RevisedConstraints = Rest[phaseI] /. a[0]->0;
	    RevisedObjective = First[tableau][[2]] /. 
	        Flatten @ (ToRules /@ RevisedConstraints) //
				ExpandAll;
	    Simplex @ Prepend[RevisedConstraints,
			z == RevisedObjective],
				(* infeasible *)
	Print["The problem is infeasible"]
    	]
    ]
]
	
LP[objective_,constraints_] := 
	LP[Prepend[StandardLP[constraints],
			z == objective]]

(* The final tableau *)
FindBasic[SFconstraints_,nonzerovars_,KSzerovars_]:= 
   Module[{rules},
    rules = Solve[SFconstraints,
		Join[nonzerovars,First[KSzerovars]]];
    If[rules != {},rules,
	FindBasic[SFconstraints,nonzerovars,Rest[KSzerovars]]]
]

FinalTableau[objective_,constraints_] :=
    Module[{SFconstraints,vars,solution,zerovars,nonzerovars,rules},
    SFconstraints = StandardLP[constraints];
    vars = Union @@ Variables /@ SFconstraints;
    solution =
        Check[ConstrainedMax[objective,SFconstraints,vars],
              nonexistent];
                        (* stop if no solution exists *)
    If[solution == nonexistent, Return[]];
                        (* else *)
      zerovars = Cases[solution[[2]], Rule[x_,0] -> x];
      nonzerovars = Complement[vars,zerovars];
      rules = Flatten @ FindBasic[SFconstraints,nonzerovars,
        KSubsets[zerovars,Length[SFconstraints]-Length[nonzerovars]]];
       Prepend[FromRules[rules],
        z == objective /.  rules] // ExpandAll
]

(* Sensitivity analysis *)
BasicVariables[tableau_?TableauQ] := First /@ Rest[tableau]

NonbasicVariables[tableau_] := Union @@
	(Variables[#[[2]]]& /@ tableau)

SetAttributes[Zero,Listable];
Zero[x_] := x -> 0
Off[General::spell1];
ShadowPrice[objective_,var_]:= -Coefficient[objective,var]
ShadowPrices[objective_] :=
    {#, ShadowPrice[objective,#]}& /@ Variables[objective]
On[General::spell1];

ShadowPrices[tableau_?TableauQ] :=
	ShadowPrices[First[tableau][[2]]]

ShadowPrice[tableau_?TableauQ,var_] :=
	ShadowPrice[First[tableau][[2]],var]

ShadowPrices[tableau_?TableauQ,vars_List] := 
	{#,ShadowPrice[First[tableau][[2]],#]}& /@ vars

ShadowPriceRange[tableau_?TableauQ,s_[i_]] := 
    SimplifyInequalities[
        Select[Rest[tableau] /. s[i] -> -Delta[i] /. 
			Zero[NonbasicVariables[tableau]],
	   !FreeQ[#,Delta[i]]&] /.
	{-Delta[i] == b_ :> Delta[i] >= -b,  (* basic *)
	         _ == b_  + a_. Delta[i]  :> (* nonbasic *)
		     Sign[a] Delta[i] >= -b/Abs[a]},
	Delta[i]]	

SensitivityAnalysis[tableau_?TableauQ,s_[i_]] :=
	{s[i],ShadowPrice[tableau,s[i]],
		ShadowPriceRange[tableau,s[i]]}

SensitivityAnalysis[tableau_?TableauQ] := 
	SensitivityAnalysis[tableau,#]& /@ 
	Cases[Join[BasicVariables[tableau],
		   NonbasicVariables[tableau]],	s[i_]]

PriceRange[tableau_?TableauQ,objective_,x_[i_]] := 
Module[{RevisedObjective},
        RevisedObjective =
                Collect[objective /.
        a_.  x[i] -> (a + Delta[i]) x[i] /.
                (ToRules /@ Rest[tableau] // Flatten),
                NonbasicVariables[tableau]];
        SimplifyInequalities[Cases[ShadowPrices[RevisedObjective],
            {_,b_  + a_.  Delta[i]}  :> Sign[a] Delta[i] >= -b/Abs[a] ],
	    Delta[i]]
]

SensitivityAnalysis[tableau_?TableauQ,objective_Plus,x_[i_]] :=
	{x[i], Coefficient[objective,x[i]],
		PriceRange[tableau,objective,x[i]]}

SensitivityAnalysis[tableau_?TableauQ,objective_Plus] := 
	Join[SensitivityAnalysis[tableau,objective,#]& /@
		Variables[objective],
	     SensitivityAnalysis[tableau,#]& /@ 
		Cases[Join[BasicVariables[tableau],
		   NonbasicVariables[tableau]],	s[i_]]]

(* Utility functions *)
Unprotect[Variables];
Variables[equation_Equal] := Variables[List @@ equation]
Protect[Variables];

FromRules[rules_List] := Apply[Equal,rules,1]

Intercept[form_] := If[NumberQ[form],form,0]
Intercept[form_Plus] := Intercept[First @ form]

KSubsets[l_List,0] := { {} }
KSubsets[l_List,1] := Partition[l,1]
KSubsets[l_List,k_Integer?Positive] := {l} /; (k == Length[l])
KSubsets[l_List,k_Integer?Positive] := {}  /; (k > Length[l])
KSubsets[l_List,k_Integer?Positive] :=
        Join[
                Map[(Prepend[#,First[l]])&, KSubsets[Rest[l],k-1]],
                KSubsets[Rest[l],k]
        ]

MinPair[pairs_List] := 
        	Fold[If[#1[[2]] <= #2[[2]],#1,#2]&,
			First[pairs],Rest[pairs]]

NextPivot[tableau_?TableauQ] := Module[{minshadow},
    minshadow = MinPair[ShadowPrices[First[tableau][[2]]]];
    If[minshadow[[2]] < 0,minshadow[[1]],None]
]	

Relax[ineq_,epsilon_] := ineq /.
	{lhs_ <= rhs_ :> lhs<= rhs + epsilon,
	 lhs_ == rhs_ :> lhs == rhs + epsilon}

SimplifyInequalities[ineq_List,var_] := 
	Max @ Join[	Cases[ineq, var >= a_ -> a],
			Cases[ineq, -var <= a_ -> -a]] <=
	var <= 
	Min @ Join[	Cases[ineq, var <= a_ -> a],
			Cases[ineq, -var >= a_ -> -a]]

TableauQ[_] := False
TableauQ[{_Equal..}] := True
$PrePrint = If[TableauQ[#], TableForm[#],#]&;

End[]
EndPackage[]

