(* :Title: MultiplierMethod *)
(* :Context: Optimize`MultiplierMethod` *)
(* :File:    Optimize`MMethod` *)
(* :Author: Jean-Christophe Culioli *)   
(* :Email: culioli@cas.ensmp.fr     *)
(* :Affiliation:  Centre Automatique et Systemes, 
   Ecole des Mines de Paris, FRANCE *)
(* :Summary: 
	This package provides the function MultiplierMethod
	for minimization of an objective function subject to
	equality and / or inequality constraints *)
(* :Package Version: 1.1.1 *)
(* :Mathematica Version: 2.2 *)
(* :Copyright: Copyright 1994, J.-C Culioli, 
         Ecole des Mines de Paris, France  
This package may be copied in its entirety for nonprofit purposes 
only. Sale, other than for the direct cost of the media, is 
prohibited.  This copyright notice must accompany all copies.  *)

(* :History: V. 1.0, 10 September 94, by JCC *)
(* :History: V. 1.1, 17 September 94, by JCC : changed the use of 
FindRoot for inner optimization to FindMinimum with specified
gradient, using the rule: Derivative[0, 1][Max][0., theta_]:> 1.
Suppressed any approximation to the Max[0,x] function. Slightly
slower on some problems but much more robust.*)
(* :History: V. 1.1.1, 2 October 95, by JCC : minor changes, 
including the file name for DOS compatibility and the MMethod` 
subcontext instead of MMethod`*)

(* :Keywords:
	optimization, objective function, minimize, constraints,
	linear and nonlinear constraints, equalities, inequalities,
	augmented Lagrangian, Uzawa algorithm, Primal-Dual method. *)

(* :Limitations: (Theoretical) The problem solved is Min f(x) 
    subject to g(x) = 0, h(x) <= 0 where f and  h  are convex and 
    g is affine -- no proof of convergence to a  solution in most 
    other cases *)

(* :Limitations: (Practical)  This program does not check for 
optimality of the returned "solution" which should thus be 
considered as a simple "candidate". Some information about possible
infeasibility or KKT conditions not met are given. Note that the
CheckConvergence option only controls convergence, neither optimality
nor feasibility. *)

(* :Discussion:  The method is a gradient ascent algorithm 
(Uzawa algorithm) applied to the dual functional with Augmented 
Lagrangian. It is often called the Multiplier Method and due to 
Hestenes, Powell. See for example D. G. Luenberger, "Linear and 
Nonlinear Programming", Addison-Wesley, second. ed., 1989. *)

(* :Implementation:  The use of FindMinimum as an internal
algorithm lead us to cancel all messages and to imitate its usual 
output even when internal convergence messages FindMinimum::fmlim
prevents the output. As a complement, one advantage of this is that
we can set the InnerIterations option to 1, thus mimicking 
Arrow-Hurwicz algorithm !*)


BeginPackage["Optimize`MultiplierMethod`"]

ison = Off[FindMinimum::fmcv,FindMinimum::fmlim,
           General::spell1,General::spell]

MultiplierMethod::usage = "MultiplierMethod[f,g,h,x,x0] finds 
a local solution to a minimization algorithm where:	
f is the criterion to be minimized, g is the list (possibly empty) 
of equality constraints [with assumed zero right hand side], 
h is the list (possibly empty) of inequality constraints 
[with assumed zero right hand side in the <= direction], x = {x1,x2,...}
is a generic list of the variables, x0 is an initial vector  for x. 
It returns a list {f*,{x1 -> x1*, etc}} like FindMinimum. 
With the option DualParameter -> True it can also provide information 
on  feasibility and Lagrange and/or  Karush Kuhn & Tucker multipliers.";

OuterIterations::usage = "OuterIterations is an option for
MultiplierMethod specifying the maximum number of outer iterations 
(gradient ascent for the dual parameters). The default setting is 
OuterIterations -> 30.";

InnerIterations::usage = "InnerIterations is an option for 
MultiplierMethod specifying  the maximum number of inner iterations. 
The default setting is InnerIterations -> 10 .";

DualParameter::usage = "DualParameter is an option for MultiplierMethod.
With the default setting DualParameter -> False, no information is 
provided on the dual parameters (Lagrange and/or Karush, Kuhn & Tucker
multipliers). With DualParameter -> True, feasibility information and the
dual parameter are appended to the primal solution with the 
following format {f*,{x -> x*,...},g*,{p -> p*,...},h*,{q -> q*,...}}
where p's and q's are automatically created variables.";

CheckConvergence::usage = "CheckConvergence is an option for 
MultiplierMethod. With the setting CheckConvergence -> False, 
convergence of the algorithm is not checked... and control is left 
to the user with the OuterIterations option. With the default 
CheckConvergence -> True, the stopping test is 
Function[{u,v},Max[Abs[u-v]]< 10^-k  where k is the value of 
the ConvergenceGoal option. The total number of iterations is
however limited to OuterIterations";

Augmentation::usage = "Augmentation is an option for MultiplierMethod. 
It gives the value of the augmentation parameter for the augmented
Lagrangian. The default setting is  Augmentation -> 10. It should be 
decreased in case of initial explosion and increased in case of slow 
convergence.";

ConvergenceGoal::usage = "ConvergenceGoal is an option for
MultiplierMethod. It gives the precision of the convergence test. Its 
default setting is 6 which is very low but quick. It should be increased 
to 10 or more for better precision."; 

MultiplierMethod::parchange =
"Inappropriate parameter: `1` -> `2`, changed to `3`.";

MultiplierMethod::nonKKT= 
"Karush, Kuhn & Tucker conditions not satisfied better than `1`."; 

MultiplierMethod::nonadmissible= 
"current point `1` from admissible."; 

MultiplierMethod::nonposq= 
"non admissible Karush Kuhn & Tucker multipliers."

MultiplierMethod::nonslackness= 
"complementary slackness not satisfied.";

Begin["`private`"]

Grad[function_,var_List]:= D[function,#]& /@ var ; 
pos[x_List]:=Map[Max[0.,#]&,x]; 
phi[h_,q_,c_]:= Plus @@ ( 1/(2. c) (pos[q + c h]^2 - q^2) );
pvar[i_]:= ToExpression[ToString["MMethod`p"]<>ToString[i]];
qvar[i_]:= ToExpression[ToString["MMethod`q"]<>ToString[i]];

Options[MultiplierMethod] = {Augmentation -> 10., 
CheckConvergence -> True, DualParameter -> False, 
InnerIterations -> 10, OuterIterations -> 30,
ConvergenceGoal -> 6};

MultiplierMethod[f_,g_List,h_List,x_List,x0_List,opts___]:= 
Module[
{xk = x0,                      (*  primal variables *)
p, pk = Table[0.,{Length[g]}], (* dual "<=" variables *)
q, qk = Table[0.,{Length[h]}], (*   dual "=" variables *)
AugmLag,                       (* augmented Lagrangian *)
c, maxiter, mode, cvgoal,      (* options values *)
repx, repp, repq},             (*   solutions  *)

{c, maxiter, mode, cvgoal} = {Augmentation, OuterIterations, 
CheckConvergence, ConvergenceGoal}  /. {opts} /.
Options[MultiplierMethod]; 

p = Map[pvar,Range[Length[g]]];    (* building auxiliary *)
q = Map[qvar,Range[Length[h]]];          (* variables *)
              
AugmLag = f + p.g + c/2. g.g +  phi[h,q,c];

If[ !NumberQ[cvgoal] || cvgoal < 0 || cvgoal > $MachinePrecision,
Message[MultiplierMethod::parchange,ConvergenceGoal,cvgoal, 
		 cvgoal = ConvergenceGoal /. Options[MultiplierMethod]]
];

Which[
	mode === False,  {xk, pk, qk} = 
		Nest[MultiplierMethodStep[AugmLag,g,h,c,x,p,q,#]&,
{xk,pk,qk}, maxiter],
	mode === True,   {xk, pk, qk} = 
		FixedPoint[MultiplierMethodStep[AugmLag,g,h,c,x,p,q,#]&,
{xk,pk,qk}, maxiter,
			SameTest -> Function[{u,v},Max[Abs[u-v]]<
10^-cvgoal] ],
	True, Message[MultiplierMethod::opttf,CheckConvergence,mode]
	];

repx =Thread[x -> xk]; repp =Thread[p -> pk]; repq =Thread[q -> qk];
OptimalityConditions[f,g,h,x,p,q,repx,repp,repq];

dualinfo = DualParameter /. {opts} /. Options[MultiplierMethod]; 
Which[
	dualinfo === False,  {(f /. repx), repx},
	dualinfo === True, {(f /. repx), repx, (g /. repx), repp, (h /.
repx), repq},
	True, Message[MultiplierMethod::opttf,DualParameter,dualinfo]
	]

]

MultiplierMethodStep[AugmLag_,g_,h_,c_,x_,p_,q_,{xk_,pk_,qk_}]:= 
Module[
{iter = InnerIterations /. Options[MultiplierMethod], 
lastx, lagk, gradk, repk},
lagk = (AugmLag /. Thread[p -> pk] ) /. Thread[q -> qk];
gradk = Grad[lagk,x] //. Derivative[0, 1][Max][0., theta_]:> 1;
FindMinimum[lastx = x; lagk, ##, MaxIterations -> iter,
Gradient -> gradk ]& @@ Transpose[{x,xk}]; 
repk = Thread[x -> lastx];   (* forcing output even if 
                              it is prevented by FindMinimum *)
{x, pk + c g , pos[qk + c h]  } /. repk  ];

OptimalityConditions[f_,g_,h_,x_,p_,q_,repx_,repp_,repq_]:= 
Module[ {toladm, tolKKT, tolslack, KKT,admissible,nadm,posq,slackness},

toladm = tolslack = tolKKT = 10^-5;
KKT = Grad[f,x] + Grad[g,x].p + Grad[h,x].q  /. repx /. repp /. repq;
If[Max[Abs[KKT]] > tolKKT, 
Message[MultiplierMethod::nonKKT,Sqrt[KKT.KKT]]]; 

admissible =  {g , posit[h]} /. repx ; 
If[(nadm = Max[Abs[admissible]]) > toladm, 
Message[MultiplierMethod::nonadmissible,nadm]];

posq = Apply[And,Positive[q]];  
If[!posq,Message[MultiplierMethod::nonposq]]; (* should never happen *)

slackness = q.h /. repq ; 
If[Max[Abs[slackness]] > tolslack,
Message[MultiplierMethod::nonslackness]]; 
]

End[]
On[ison]
EndPackage[]

