BeginPackage["DSPack`"]

DSPack::usage ="DSPack.m is a package that has two versions of a function
(demandsupply) that allows the user to supply values and costs and then
graphs the demand and supply functions for these values and costs.  The
package also has functions that compute the  equilibrium quantities(eq),
equilibrium prices(ep), and computes the available surplus(surplus). It
is assumed that trades take place on a unit basis." 

demandsupply1::usage = "demandsupply1[v,c] is the simplest graphical
function for graphing demand and supply curves.  It assumes that each
buyer cannot buy more than one unit , and each seller cannot sell more
than one unit.The input, v, is a vector of maximum prices that each buyer
will buy, e.g. {2,3,5,1,3} would denote a maximum buying price for five
buyers.  Similarly c is a vector of minimum prices at which each seller
would sell."

demandsupply::usage="demandsupply[v,c] is a more general version of its
two-element counterpart.  In this case buyers may be able to buy more
than one unit, and sellers sell more than one unit.  A 'v' for two buyers
might be {{2,1},{3,2}}, where {2,1} refers to the maximum the buyer 1 is
willing to pay for the first and second units respectively.  A similar
notation is used for sellers.   The user may supply additional
Mathematica  Graphical options following c."

eq::usage = "eq[v,c] determines the equilbrium quantity for a supplied
set of values or costs.  A v for two buyers might be {{2,1},{3,2}}, where
{2,1} refers to the maximum the buyer 1 is willing to pay for the first
and second units respectively,  A similar notation is used for sellers.
The output of this function may be an interval, i.e. a set of quantities
that would be consistent with the equilbrium price and the v, c.
vectors." 

ep::usage="ep[v,c] determines the equilbrium price for a supplied set of
values or costs.  A v for two buyers might be {{2,1},{3,2}}, where {2,1}
refers to the maximum the buyer 1 is willing to pay for the first and
second units respectively,  A similar notation is used for sellers.  The
output of this function may be an interval, i.e. a set of prices that
would be consistent with the equilbrium quantity and the v, c. vectors."

surplus::usage="surplus[v,c] determines the total surplus that can be
achieved from trade in a competitive equilbrium.   A v for two buyers
might be {{2,1},{3,2}}, where {2,1} refers to the maximum the buyer 1 is
willing to pay for the first and second units respectively,  A similar
notation is used for sellers."   

Begin["`Private`"]

(* To anyone who may wish to modify these functions for personalized use.
It is easiest to modify these functions by eliminating the context
statements, i.e. the Begin statements and End statements in this
packages, reading in the package,  and then putting the context
statements back in the program later. *)

(* functions used in preparation of points for plot routines *) 

finisherv[x_] := 
	Join[{{x[[1]][[1]], Max[Flatten[x]]}}, x, 
	   {{x[[-1]][[1]], 0}}]

finisherc[x_] := Join[{{x[[1]][[1]], 0}},x, {{x[[-1]][[1]],
	 Max[Flatten[x]]} } ]

graphpointsv[v_] := 
  finisherv[Flatten[Thread[{Thread[{
                 		Range[Length[Reverse[
                 		Sort[Flatten[v]]]]] - 1, 
                 		Reverse[Sort[Flatten[v]]]
                                    }], 
      				Thread[{Drop[Range[Length[
      					Reverse[Sort[Flatten[v]]]] + 
      						1] - 1, 1], 
        				Reverse[Sort[Flatten[v]]]
                            } ]
                             } ], 1 ] ]
 
graphpointsc[c_] := 
  	finisherc[Flatten[Thread[{Thread[{
  				Range[Length[Sort[Flatten[c]]]] - 1, 
        				Sort[Flatten[c]]
        				             }
        				            ], 		
        	            		Thread[{
        	            		Drop[Range[Length[
        	            		Sort[Flatten[c]]] + 
            					1] - 1, 1],
             					Sort[Flatten[c]]
                                        }
                                       ]
                               }
                              ], 1
                     ]
            ]
 
(* End of functions used in preparation for plot routines. *)

(* Actual demandsupply functions *)

demandsupply[v_, c_,opts___] := 
  Show[{Graphics[Line[graphpointsv[v]]], 
    		Graphics[{Dashing[{0.01, 0.01, 0.01, 0.01}], 
      				Line[graphpointsc[c]]
      		          }
      		         ]
      	},opts, 
   Axes -> True, 
   Ticks -> 
    {Range[Max[Length[Flatten[Drop[Transpose[c], 
           -1]]], Length[Flatten[Drop[Transpo\
             se[v], -1]]]]], Automatic}, 
   PlotRange -> 
    {{0, Max[Length[Flatten[Drop[Transpose[c], 
          -1]]], Length[Flatten[Drop[Transpose[v], 
          -1]]]]},
          {0,Max[Max[Flatten[v]],Max[Flatten[c]]]}}
         ] 
         
  demandsupply1[v_,c_]:=
		demandsupply[
			Thread[
					List[v,Table[0,{Length[v]}
															
								]
					     ]
				   ],
			Thread[List[c,Table[Max[c],{Length[c]}
							   ]
					   ]
				   ], 
  		AxesLabel -> {"Quantity", "Price"}
				]	

(* End of demandsupply functions. *)

(* Preliminary functions for equilbrium computations *)

counter[v_,c_]:=Reverse[Sort[Flatten[v]		 
					 		 ]
					 	]-				
					Sort[Flatten[c]
						]
greaterthan[x_]:=Function[z,z>=x]

buyerinterval[v_, c_] := 
  {Reverse[Sort[Flatten[v]]][[
  				Length[Select[counter[v, c],Positive
  				             ]
      				  ]
      						 ]], 
   Reverse[Sort[Flatten[v]]][[
   				Length[Select[counter[v, c],
greaterthan[0]
   							 ]
   						]+1   ]]
   	}

sellerinterval[v_, c_] := 
  			{Sort[Flatten[c]][[
  				Length[Select[counter[v, c],Positive
  				             ]
      				  ]
      						 ]], 
   			Sort[Flatten[c]][[
   				Length[Select[counter[v, c],
greaterthan[0]
   							 ]
   						]+1   ]]
   			}

(* Equilbrium Quantity, Price and Surplus Calculations *)

eq[v_,c_]:=If[Position[counter[v,c],0]=={},
{Select[counter[v,c],Positive][[-1]]},
Flatten[{Position[counter[v,c],0][[1]]-1,
Position[counter[v,c],0][[-1]]}]]

ep[v_, c_] := 
  If[Length[eq[v,c]]==1,
  {Max[Min[sellerinterval[v, c]], 
    Min[buyerinterval[v, c]]], 
   Min[Max[sellerinterval[v, c]], 
    Max[buyerinterval[v, c]]]},
    Reverse[Sort[Flatten[v]]][[
  				Length[Select[counter[v,
c],greaterthan[0]
  				             ]
      				  ]
      						 ]]
      ]

surplus[v_,c_]:=Apply[Plus,Select[counter[v,c],Positive
                                 ]
                     ]

End[]
EndPackage[]

