BeginPackage["Institutions`"]

Institutions::usage="This package implements rules for the First
Price,Second Price, and Double Auctions. The inputs to the program are
announced bids(asks) of participants, and the rules associate the auction
outcome with the inputs."

FirstPrice::usage ="FirstPrice[bids] takes a collection of bids and
returns the highest bid.  Ties are resolved by sampling from a uniform
distribution on the highest bids. FirstPrice identifies the winning
bidder and the price at which the unit is exchanged."

SecondPrice::usage ="SecondPrice[bids] takes a collection of bids and
returns   the bidder who bid the highest price, and indicates the second
highest price bid. Ties are resolved by sampling from a uniform
distribution on the highest bid."

message::usage="message[{a1,a2,a3},...{m1,m2,m3},{new1,new2,new3}] takes
a history generated by the double auction institution,
{a1,a2,a3},...{m1,m2,m3},
and a recent submission to the auction, namely {new1,new2,new3}, and
determines what the next history of the auction will be.  The first
element of a particular triple of a history indicates the identity of the
seller(an integer), the second the number of the buyer, and the final
number the price.  If the first element of triple is 0, then we have a
bid.  If the second element of a triple is 0 we have an ask.  If both are
non-zero we have a trade.  When simulating behavior in this auction it is
best to begin a history with one bid and one ask that are ploys, i.e. the
bid is at a price of 0 and the ask is at a price higher than the highest
costs." 

Begin["`Private`"]

(*  Code For First Price *)

FirstPrice[Bids_]:=
Module[{ii,Players,TiedValues,Winner,WinnerValue},
	Players=Table[ii,{ii,1,Length[Bids]}];
	g[i_]:=Bids[[i]]==Max[Bids];
 	TiedValues=Select[Players,g];
	Winner=TiedValues[[Random[Integer,{1,Length[TiedValues]}] ]];
		WinnerValue=Bids[[Winner]];
		Return[{Winner,WinnerValue}]
	  ]

(* Code for Second Price *)
SecondPrice[Bids_]:=
	Module[
		{ii,Players,TiedValues,Winner,WinnerValue},
		Players=Table[ii,{ii,1,Length[Bids]}];
		g[i_]:=Bids[[i]]==Max[Bids];
 		TiedValues=Select[Players,g];
		Winner=TiedValues[[Random[Integer,
					{1,Length[TiedValues]}]
						  ]
					     ];
		LowerSet=Complement[Players,{Winner}];
		WinnerValue=Max[Bids[[LowerSet]]];
		Return[{Winner,WinnerValue
		        }
			  ]
		]

(* Code to implement the improvement rule in double auction *)

valid[h_] :=
If[Length[h]==lasttrade[h],True,
	 z3 = f[Position[notzero /@ 
        			Drop[Column[Take[h, 
          		 			-(Length[h] -
lasttrade[h])
          		 			         ], 
         		 		3 - Position[h[[-1]],
0][[1]][[1]]
         		 			], -1
         		 	     	], True
         		 	     ]
       		   ]; 
                If[z3 == 0,
                	True, 
     		If[h[[lasttrade[h] + z3]][[3]] <= 
       				 h[[-1]][[3]] && h[[-1]][[2]] == 0,
False, 
     			If[h[[ lasttrade[h] + z3]][[3]] >= 
         						h[[-1]][[3]] &&
h[[-1]][[1]] == 0, False, 
         						True
         			    ]
         		    ]
         	        ]
         	]
 f[x_] := 0 /; x == {}
 f[x_] := x[[-1]][[1]] /; !x == {}

notzero = Function[x, !x == 0]

Function[x, !x[[1]] == 0 && !x[[2]] == 0]
Function[x, !x == 0]
lasttrade[h_] := 
  If [MemberQ[trade /@ h, True],
         Flatten[Position[trade /@ h, True
                         ]
                 ][[-1]],
         0
     ]

(* Code to Assign Messages in Double Auction *)
message[h_] := Drop[h, -1] /; !valid[h]
 
message[h_] := 
  h /; Length[h] == location[h][[-1]] || 
    f[Position[notzero /@ 
        Drop[Column[Take[h, 
           -(Length[h] - 
              (List /@ location[h])[[-1]][[1]])], 
          Position[h[[-1]], 0][[1]][[1]]], -1], 
       True]] == 0
 
message[h_] := 
  Join[Drop[h, -1], {{h[[lasttrade[h] + 
         f[Position[notzero /@ 
            Drop[Column[Take[h, 
               -(Length[h] - lasttrade[h])], 
              Position[h[[-1]], 0][[1]][[1]]], 
             -1], True]]]][[1]], h[[-1]][[2]], 
      h[[lasttrade[h] + 
         f[Position[notzero /@ 
            Drop[Column[Take[h, 
               -(Length[h] - lasttrade[h])], 
              Position[h[[-1]], 0][[1]][[1]]], -1]\
            , True]]]][[3]]}}] /; 
   h[[lasttrade[h] + 
        f[Position[notzero /@ 
           Drop[Column[Take[h, 
              -(Length[h] - lasttrade[h])], 
             Position[h[[-1]], 0][[1]][[1]]], -1]\
           , True]]]][[3]] <= h[[-1]][[3]] && 
    h[[-1]][[1]] == 0
 
message[h_] := 
  Join[Drop[h, -1], {{h[[-1]][[1]], 
      h[[lasttrade[h] + 
         f[Position[notzero /@ 
            Drop[Column[Take[h, 
               -(Length[h] - lasttrade[h])], 
              Position[h[[-1]], 0][[1]][[1]]], 
             -1], True]]]][[2]], 
      h[[lasttrade[h] + 
         f[Position[notzero /@ 
            Drop[Column[Take[h, 
               -(Length[h] - lasttrade[h])], 
              Position[h[[-1]], 0][[1]][[1]]], -1]\
            , True]]]][[3]]}}] /; 
   h[[lasttrade[h] + 
        f[Position[notzero /@ 
           Drop[Column[Take[h, 
              -(Length[h] - lasttrade[h])], 
             Position[h[[-1]], 0][[1]][[1]]], -1]\
           , True]]]][[3]] >= h[[-1]][[3]] && 
    h[[-1]][[2]] == 0
 
message[h_] := h

location[h_]:=If[MemberQ[trade /@ h, True            							]==False,
					{0},
					Flatten[Position[trade /@ h, True
					                ]
					       ]
                ]
 
notzero = Function[x, !x == 0]
lasttrade[h_] := 
  			If[MemberQ[trade /@ h, True
  					   ], 
  				 Flatten[Position[trade /@ h, True
  				 				  ]
  				 		][[-1]], 
  				 0
  				]

(* End of Double Auction Code *)

End[]
EndPackage[]

