(* :Title: B-Spline Basis *)
(* :Context: BSplineBasis` *)

(* :Author: Mark Fisher *)

(* :Mathematica Version: 2.2 *)

(* :Discussion: This package can be used to produce b-splines.  There
are three parts to this package: (1) functions that make, evaluate,
and plot B-splines and B-spline bases; (2) alternative functions that
make higher order B-splines (but less effiecient routines) and
functions to look inside InterpolatingFunctions; (3) IntegrateIF (and
its auxilliaries) that takes an InterpolatingFunction and returns its
integral as another InterpolatingFunction; and (4) functions that
produce a penalty matrix and a decomposition useful with generalized
cross validation (GCV).

At the heart of the b-spline routines is SplineList.
SplineList[order] can be created by the following code.

(* the b-spline recursion: i-th spline, r-th order, evaluated at x *)
phi[i_, r_, x_] := 
  ((x - k[i]) phi[i, r - 1, x])/(k[i + r] - k[i]) + 
    ((k[i + r + 1] - x) phi[i + 1, r - 1, x])/(k[i + r + 1] - k[i + 1])
(* terminal condition *)
phi[i_, 0, x_] := d[i]
(* i-th spline, j-th region, r-th order, evaluated at x *)
phi[i_, j_, r_, x_] :=
	phi[i, r, x] /. {d[j] -> 1, d[_] -> 0}
(* For example, SplineList[3] is created by: *)
Join[{{0, 0, 0}},
	Table[D[phi[1, i, 3, x], {x, j}] /. x -> k[i+1], {i, 3}, {j, 0, 2}] // 
		Simplify, {{0, 0, 0}}]

Reference: De Boor, C. (1978) A practical guide to splines (New York:
Springer-Verlag)

The levels and derivatives in SplineList are used to construct an
InterpolatingFunction that represents the heart of the BSplineFunction.
*)

(* :Limitation: B-splines of order 4 or greater use less efficient
routines.  The reason is that Interpolation does not respect the
higher-order derivatives and returns "incorrect"
InterpolatingFunctions. MakeFinalSpline only works for order 3 or less. *)

(* :Warning:
MakeBSpline and IntegrateIF rely on the internal structrure of
InterpolatingFunctions. If this structure changes with newer versions
of Mathematica, these functions may not work properly.
*) 

BeginPackage["BSplineBasis`", {"Statistics`DescriptiveStatistics`",
"Utilities`FilterOptions`"}]

PadKnots::usage = "PadKnots[knots, order] prepends \"order\" of the
first knot and appends \"order\" of the last knot to the list."

MakeKnotList::usage = "MakeKnotList[xvalues, knotnum, {f, l}]
takes a list of \"x\" values and returns a list of \"knots\" with knotnum
interior knots distributed according to quantiles. {f, l} are the
endpoints, which if omitted are minimum and maximum x values. Not
recommended for zero-order splines."

SplineMatrix::usage = "SplineMatrix[knots, xvalues, order, d:0] return
a \"vector\" for each xvalues evaluated according to the b-spline
basis determined by the knots. Thus if xvalues is a vector,
SplineMatrix returns a matrix. xvalues may be deeply nexted for
indirect splining. Valid orders are 0 (step function), 1 (linear), 2
(quadratic), and 3 (cubic), or higher. (Higher order splines use less
efficient routines.) If the optional third argument is given as 1,
then the derivative of the spline is returned.  If the optional fourth
argument is given as 1, then the values returned are calculated from
the derivatives of the B-spline basis, which is useful for calculating
standard errors of functions of the spline. If the optional fourth
argument is given as -1, then the values returned are calculated from
the integrals of the B-spline basis, which is useful for splining the
derivative of a function.  SplineMatrix calls MakeBSpline."

MakeBSpline::usage = "MakeBSpline[knots, d:0] takes a list of knot
points and returns an InterpolatingFunction that represents a B-spline
of order Length[knots] - 2. Valid orders are 0 (step function), 1
(linear), 2 (quadratic), and 3 (cubic), or higher. (Higher order
splines use less efficient routines.) If the optional second argument
is given as 1, then the derivative of the spline is returned. If the
optional third argument is given as -1, then the integral of the
spline is returned. Differentiation is supported to any order;
integration is supported to order -2."

BSplineFunction::usage = "BSplineFunction[range, internal] represents
a cubic b-spline or its integratal or derivative."

MakeBSplineBasis::usage = "MakeBSplineBasis[knots, order, d:0]
produces a list of InterpolatingFunctions that represent a b-spline
basis of order \"order\" and which can be plotted with
PlotBSplineBasis. Valid orders are 0 (step function), 1 (linear), 2
(quadratic), and 3 (cubic), or higher. (Higher order splines use less
efficient routines.) If the optional third argument is given as 1,
then the derivative of the spline is returned. If the optional fourth
argument is given as 1, then the values returned are calculated from
the derivatives of the B-spline basis. Differentiation is supported to
any order; integration is supported to order -2."

PlotBSplineBasis::usage = "PlotBSplineBasis[bsbasis] plots the list of
InterpolatingFunctions returned by MakeBSplineBasis.
PlotBSplineBasis[bsbasis, {x, a, b}] plots the basis over the range
specified."

IntegrateIF::usage = "IntegrateIF[ifcn] returns the integral of ifcn
where ifcn is an InterpolatingFunction."

DividedDifferences::usage = "DividedDifferences[knots, poly, var]
returns a list of divided differences given a list of knots, a
polynomial, and the variable for the polynomial. The length of the
list of divided differences is one greater that the length of the list
of knots. Example: DividedDifferences[{x0, x1, x2}, x^2, x]."

NewtonForm::usage = "NewtonForm[diffs, knots, var] takes a list of
divided differences, a list of knot points, and a variable and returns
a polynomial in the variable in Newton form. The length of the list of
divided differences must be one greater that the length of the list of
knots. Example: NewtonForm[{c0, c1, c2, c3}, {x0, x1, x2}, x]."

InfoIF::usage = "InfoIF[ifcn, var] returns a pair of lists. The first
is the list of knot points in the InterpolatingFunction ifcn, and the
second is the list of polynomials (in Newton form) in ifcn as a
function of var."

PlotIFPolynomials::usage = "PlotIFPolynomials[ifcn] plots the
polynomials contained in the InterpolatingFunction ifcn.
PlotIFPolynomials[ifcn] also works if ifcn is a BSplineFunction."

MakeFinalSpline::usage = "MakeFinalSpline[knots, betahat] returns an
InterpolatingFunction of order (Length[knots] - Length[betahat] + 1)
defined over the range of knots where the betahat coefficients can be
interpreted as the weights on b-spline basis functions."

PenaltyMatrix::usage = "PenaltyMatrix[knots] returns a matrix that
penalizes the integral of the second derivative of a cubic B-spline
basis. PenaltyMatrix[knots, 0] returns a matrix that penalizes the sum
of the squared deviations on an order 0 spline from \"linear\" step
function."

GCVDecomposition::usage = "GCVDecomposition[xmat, penaltymat]
effectively decomposes (xmat'xmat + lambda penaltymat)^-1 into
gmat.(I + lambda diag[dvec]).gmat', and returns {dvec, gmat}. This
function is useful for certain problems involving generalized cross
validation."

Begin["`Private`"]

(* Part I *)

(* pads a list of knots with its end points *)
PadKnots[knots_List, order_Integer?NonNegative] := 
	Join[Table[knots[[1]], {order}], knots, Table[knots[[-1]], {order}]]

MakeKnotList::toomanyknots = "`1` knots may be too many"

MakeKnotList[xvalues:{_?NumberQ ..}, knotnum_Integer?NonNegative, 
		endknots:{_, _}:({Automatic, Automatic})] :=
	With[{union = Union[xvalues]},
	If[knotnum > Length[union] - 2, 
		Message[MakeKnotList::toomanyknots, knotnum]];
	Module[{first, last,
	interiorknots = Quantile[ union, #]& /@ 
		Table[i/(knotnum + 1), {i, knotnum}]},
	{first, last} = If[endknots === {Automatic, Automatic}, 
		{First[union], Last[union]}, endknots];
	Append[Prepend[interiorknots, first], last]]]

Format[BSplineFunction[range_, internals_]] :=
	SequenceForm["BSplineFunction[", range, ", <>]"]

BSplineFunction[range_, internals_][x_] := internals[x]

MakeBSplineBasis[knots:{_?NumberQ ..} /; Length[knots] >= 2, 
		order_Integer?Positive, d_Integer:0] := 
	MakeBSpline[#, d]& /@ 
		Partition[PadKnots[knots, order], order + 2, 1]

PlotBSplineBasis[bs:{_BSplineFunction ..}, range:{x_, a_, b_}, 
	opts___Rule] := Plot[Evaluate[Through[bs[x]]], {x, a, b}, opts]

PlotBSplineBasis[bs:{_BSplineFunction ..}, opts___Rule] :=
	Module[{x, a, b},
	a = Min[#[[1,1]]& /@ bs];
	b = Max[#[[1,-1]]& /@ bs];
	PlotBSplineBasis[bs, {x, a, b}, opts]]

(* produce "matrix" of values from "vector" of points 
	for a given list of knots *)
SplineMatrix[knots:{_?NumberQ ..} /; Length[knots] >= 2, 
		points_List, order_Integer?NonNegative, d_Integer:0] :=
	With[{	union = Union[Flatten[points]],
		temp = MakeBSplineBasis[knots, order, d]},
	points /. Dispatch[Thread[union -> (Through[temp[#]]& /@ union)]]]

(* called by MakeBSplineBasis and SplineMatrix *)
MakeBSpline[knots:{_?NumberQ ..} /; 2 <= Length[knots] <= 5, 
		d_Integer:0] :=
	Block[{k, count, union, length, range, if,
		first = First[knots], 
		last = Last[knots],
		order = Length[knots] - 2},
	count = Count[knots, first];
	union = Union[knots];
	length = Length[union];
	range = Range[count, count + length - 1];
	if = Derivative[d][Interpolation[
			Transpose[{union, 
			SplineList[order][[range]] /.
			Thread[Array[k, order + 2] -> knots]}], 
				InterpolationOrder -> order]];
	BSplineFunction[{first, last}, Function @@ {
		Which @@ {# < first, 0,
		# <= last, if[#],
		True, Which[NonNegative[d], 0, d == -1, if[last],
				d == -2, if[last] + (#-last)(last-first)/4, True, 0]}}]
	]


(* MakeBSpline for order >= 4 (knots >= 6) *)
(* MakeBSplineAlt works for order >=1, but is less efficient *)
(* see Part II *)
MakeBSpline[knots:{_?NumberQ ..} /; Length[knots] >= 6, d_Integer:0] := 
	MakeBSplineAlt[knots, d]

(* takes a list of knots and list of coefficients and returns an
	interpolating function; only for 0 <= order <= 3 *)
MakeFinalSpline::badorder = "Number of knots, `1`, minus number of
coefficients, `2`, plus 1 equals `3`. It should be between 0 and 3
inclusive."

MakeFinalSpline[knots:{_?NumberQ ..}, betahat:{_?NumberQ ..}] :=
	Module[{paddedknots, temp, interpolationlist,
	order = Length[betahat] - Length[knots] + 1},
	Which[0 <= order <= 3, 
	paddedknots = Partition[PadKnots[knots, order], order + 2, 1];
	temp = MapThread[SplineListEvaluate, {paddedknots, betahat}];
	interpolationlist = Cases[Flatten[temp, 1], 
			{#, a:Table[_, {Max[1, order]}]} -> a]& /@ knots;
	interpolationlist = If[order == 0, First /@ interpolationlist, 
							Apply[Plus, interpolationlist, {1}]];
	Interpolation[Transpose[{knots, interpolationlist}],
		InterpolationOrder -> order],
	True, Message[MakeFinalSpline::badorder,
		Length[knots], Length[betahat], order]; Return[$Failed]]]

(* called by MakeFinalSpline; only for 0 <= order <= 3  *)
SplineListEvaluate[knots:{_?NumberQ ..}, bhat_?NumberQ] :=
	Block[{k, order, count, union, length, range},
	order = Length[knots] - 2;
	count = Count[knots, First[knots]];
	union = Union[knots];
	length = Length[union];
	range = Range[count, count + length - 1];
	Transpose[{union, 
		bhat SplineList[order][[range]] /.
		Thread[Array[k, order + 2] -> knots]}]]

(* used by MakeBSpline and EvaluateSplineList.
	SplineList[order] is the level, and derivatives up to (order - 1)
	of a b-spline with knots k[1] to k[order + 2]  *)
SplineList[0] = {{1}, {1}}
SplineList[1] = {{0}, {1}, {0}}
SplineList[2] =
{{0, 0}, {(-k[1] + k[2])/(-k[1] + k[3]), 2/(-k[1] + k[3])}, 
  {(-k[3] + k[4])/(-k[2] + k[4]), 2/(k[2] - k[4])}, {0, 0}}
SplineList[3] = 
{{0, 0, 0}, 
{(k[1] - k[2])^2/((k[1] - k[3])*(k[1] - k[4])), 
(3*(k[1] - k[2]))/((-k[1] + k[3])*(k[1] - k[4])), 
6/((k[1] - k[3])(k[1] - k[4]))}, 
{(2*k[1]*k[2]*k[3] - k[1]*k[3]^2 - k[2]*k[3]^2 - k[1]*k[2]*k[4] + 
	k[3]^2*k[4] - k[1]*k[2]*k[5] + k[3]^2*k[5] + k[1]*k[4]*k[5] + 
	k[2]*k[4]*k[5] - 2*k[3]*k[4]*k[5])/
	((k[1] - k[4])*(-k[2] + k[4])*(-k[2] + k[5])), 
(3*(k[1]*k[2] - k[1]*k[3] - k[2]*k[3] + k[3]*k[4] + k[3]*k[5] - 
	k[4]*k[5]))/((k[1] - k[4])*(-k[2] + k[4])*(-k[2] + k[5])), 
(6*(-k[1] - k[2] + k[4] + k[5]))/
	((k[2] - k[4])*(-k[1] + k[4])*(-k[2] + k[5]))}, 
{(-k[4] + k[5])^2/((k[2] - k[5])*(k[3] - k[5])), 
(3*(-k[4] + k[5]))/((k[3] - k[5])*(-k[2] + k[5])), 
6/((-k[2] + k[5])*(-k[3] + k[5]))}, 
{0, 0, 0}}

(* Part II *)
(* for order > 3; some code used by IntegrateIF (see Part III) *)
(* also code for looking inside InterpolatingFunctions *)

(* MakeBSplineAlt, for order >= 2 *)
MakeBSplineAlt[knots:{_?NumberQ ..} /; Length[knots] >= 3,
		d_Integer:0] :=
	With[{first = First[knots], last = Last[knots],
		if = Derivative[d][MakeBSIF[knots]]},
	BSplineFunction[{first, last}, Function @@ {
		Which @@ {# < first, 0,
		# <= last, if[#],
		True, Which[NonNegative[d], 0, d == -1, if[last],
				d == -2, if[last] + (#-last)(last-first)/4, True, 0]}}]
	]

(* MakeBSIF returns a B-spline as an InterpolatingFunction *)
MakeBSIF[knots_] :=
	BuildIF[Union[knots], BSDiffs[knots]]

(* BuildIF takes knots and divided diffs (defined on the upper knot 
		for each region) and returns an InterpolatingFunction *) 
BuildIF[knots_List, diffs:{__List}] :=
	InterpolatingFunction @@ {{First[knots], Last[knots]},
		Prepend[
			MapThread[
				{##, Table[0, {Length[First[diffs]]-1}]}&,
				{Rest[knots], Drop[knots, -1], diffs}],
		{First[knots], First[knots], {0,0}, {0}}]}

(* BSDiffs creates divided differences of B-spline polynomials 
		for use in InterpolatingFunction *)
(* this version uses all upper knots (IF uses upper and lower) *)
BSDiffs[knots_] :=
	Module[{(*order, first, count, union, length, range, knotrule*)},
	order = Length[knots] - 2;
	first = First[knots];
	count = Count[knots, first];
	union = Union[knots];
	length = Length[union];
	range = Range[count, count + length - 2];
	knotrule = Thread[Array[k, order + 2] -> knots];
	Table[
	DividedDifferences[
		If[order > 0,
			Table[k[i+1], {order}], {}] /. knotrule,
		phi[1,i,order,x] /. knotrule, 
		x],
	{i, First[range], Last[range]}]]

(* the b-spline recursion: i-th spline, r-th order, evaluated at x *)
phi[i_, r_, x_] := 
  ((x - k[i]) phi[i, r - 1, x])/(k[i + r] - k[i]) + 
    ((k[i + r + 1] - x) phi[i + 1, r - 1, x])/(k[i + r + 1] - k[i + 1])
(* terminal condition *)
phi[i_, 0, x_] := d[i]
(* i-th spline, j-th region, r-th order, evaluated at x *)
phi[i_, j_, r_, x_] :=
	phi[i, r, x] /. {d[j] -> 1, d[_] -> 0}

(* Note: both NewtonForm and DividedDifferences take a list of knots. The
		length of knots is one less than the length of diffs *)
(* NewtonForm returns a polynomial in var given divided differences
		and knots *)
NewtonForm[diffs_, knots_, var_] :=
	With[{rdiffs = Reverse[diffs],
		rknots = Reverse[knots]},
	Fold[#1 #2[[1]] + #2[[2]]&, First[rdiffs], 
		Transpose[{var - rknots, Rest[rdiffs]}]]]

(* find divided differences for Newton form using knots *)
DividedDifferences[knots_, poly_, var_] :=
	With[{diffs = var - knots},
		MapThread[PolynomialRemainder[##, var]&,
			{FoldList[PolynomialQuotient[##, var]&,
				poly, diffs], Append[diffs, var]}]]

(* this returns the list of knot points and the polynomials
		(in Newton form) in an InterpolatingFunction *)
InfoIF[InterpolatingFunction[range_, table_], var_] :=
	{First /@ table, Poly[#, var]& /@ Rest[table]}

PlotIFPolynomials[i:InterpolatingFunction[range_, table_], opts___Rule] :=
	Module[{x, info, xrange},
	With[{xrange = Prepend[range, x]},
		info = InfoIF[i, x];
		Plot[Evaluate[info[[-1]]], xrange, opts, 
				Ticks -> {info[[1]], Automatic}]]]

PlotIFPolynomials[b_BSplineFunction, opts___Rule] :=
	PlotIFPolynomials[First @ 
		Cases[b, a:InterpolatingFunction[__][_] :> Head[a], Infinity],
		opts]


(* Part III *)
(* for integrated b-splines *)

(* rule for applying IntegrateIF as negative order of derivative *)
Derivative[n_Integer /; Negative[n]][if:InterpolatingFunction[__]] :=
		Derivative[n+1][IntegrateIF[if]]

IntegrateIF[InterpolatingFunction[range_, table_]] :=
	Block[{order, x, ivar, knots, polys, intpolys, vals, 
		high, low, defintpolys, diffs},
	order = Length[table[[-1, -1]]]; (* order of spline *)
	ivar = Table[x^i/i, {i, order + 1}]; (* integrated variable *)
	knots = First /@ table; (* knot points *)
	polys = Poly[#, x]& /@ Rest[table]; (* get the polynomials *)
(* indefinite integration *)
	intpolys = (Table[Coefficient[#, x, i], {i, 0, order}]& /@ polys).ivar;
(* evaluate each intpoly at low and high knots *)
	vals = MapThread[#1 /. Partition[Thread[x -> #2], 1]&,
		{intpolys, Partition[knots, 2, 1]}];
(* accumulate values *)
	{low, high} = Transpose[FoldList[Plus, {0, 0}, vals]];
(* adjust intpolys for definite integration *)
	defintpolys = intpolys + Drop[high, -1] - Rest[low];
(* construct the divided differences *)
	diffs = MapThread[
		DividedDifferences[Table[#1, {order + 1}], #2, x]&, 
		{Rest[knots], defintpolys}];
(* construct the InterpolatingFunction *)
	BuildIF[knots, diffs]
]

(* reconstruct newton form of polynomial from (part of) 
		InterpolatingFunction *)
Poly[{t_, _, diffs_, coeffs_}, var_] := NewtonForm[diffs, t - coeffs, var]


(* Part IV *)
(* penalty matrix for cubic splines *)
PenaltyMatrix[knots_List, 3] := PenaltyMatrix[knots]

PenaltyMatrix[knots_List] := 
	With[{paddedknots = PadKnots[knots, 3]},
	Module[{z = Zmat[paddedknots], 
		r = Rmat[paddedknots]},
	Transpose[z].r.z / 3]]

Rmat[pk_List] := 
	With[{n = Length[pk] - 6},
	Module[{mat = Table[0, {n}, {n}]},
		mat[[1, 1]]    = pk[[5]] - pk[[4]];
		mat[[n, n]]    = pk[[n+3]] - pk[[n+2]];
		Do[mat[[i, i]] = pk[[i+4]] - pk[[i+2]], {i, 2, n-1}];
		Do[mat[[i, i+1]] = mat[[i+1, i]] = 
			(pk[[i+4]] - pk[[i+3]])/2, {i, n-1}];
	mat]]

Zmat[pk_List] := 
	With[{n = Length[pk] - 6},
	Module[{mat = Table[0, {n}, {n+2}]},
	Do[	mat[[i, i]]   = 6/
			((pk[[i+4]] - pk[[i+1]])(pk[[i+4]] - pk[[i+2]]));
		mat[[i, i+2]] = 6/
			((pk[[i+5]] - pk[[i+2]])(pk[[i+4]] - pk[[i+2]]));
		mat[[i, i+1]] = -mat[[i, i]] - mat[[i, i+2]],
		{i, n}];
	mat]]

(* penalty when spline order is 0: sum of squared deviations from
		linear regression of beta coefficients on midpoints *)
PenaltyMatrix[knots_, 0] :=
	Module[{n, midpts, x},
	n = Length[knots] - 1;
	midpts = Rest[knots] + 
		Apply[Subtract, Partition[knots, 2, 1], {1}]/2;
	x = Transpose[{Table[1, {n}], midpts}];
	mat = IdentityMatrix[n] - x.Inverse[Transpose[x].x].Transpose[x];
	N @ Transpose[mat].mat ]

(* there is no penalty matrix for orders 1 or 2 *)
PenaltyMatrix::nopen = "There is no penalty matrix for orders 1 or 2."
PenaltyMatrix[_, 1|2] := (Message[PenaltyMatrix::nopen]; $Failed)

GCVDecomposition[x_, penaltymatrix_] := 
	Module[{esv, vmatrix, u, dvector, gmatrix},
(* construct vmatrix *)
	esv = Eigensystem[Transpose[x].x];
	vmatrix = Transpose[ esv[[2]] ].
		DiagonalMatrix[1/Sqrt[ esv[[1]] ]].esv[[2]];
(* construct u and dvector *)
	{u, dvector} = SingularValues[vmatrix.penaltymatrix.vmatrix, 
			Tolerance -> 0][[{1, 2}]];
(* construct gmatrix *)
	gmatrix = vmatrix.Transpose[u]; 
	{dvector, gmatrix}]


End[]
(*Protect[PadKnots, MakeKnotList, MakeKnotList, SplineMatrix,
MakeBSpline, MakeBSplineBasis, PlotBSplineBasis, MakeFinalSpline,
IntegrateIF, PenaltyMatrix, GCVDecomposition]*)
EndPackage[]

(* :Examples:
(* example of cubic spline *)
x = Table[Log[i] + Random[], {i, 1, 20}]//N
y = # + Random[Real, .5{-1, 1}]& /@ x
(*knots = MakeKnotList[x, 3]*)
knots = Table[i, {i, Min[x], Max[x], (Max[x]-Min[x])/3}]
xmat = SplineMatrix[knots, x, 3]
b = Inverse[Transpose[xmat].xmat].Transpose[xmat].y
fs = MakeFinalSpline[knots, b]
Module[{one, two}, Block[{$DisplayFunction = Identity},
	one = Plot[fs[x], {x, Min[x], Max[x]}];
	two = ListPlot[Transpose[{x, y}]]];
	Show[one, two]]

(* this shows that DividedDifferences inverts NewtonForm *)
With[{knots = {x0, x1, x2}, var = x},
	DividedDifferences[knots, NewtonForm[{c0, c1, c2, c3}, knots, x], x]]
*)
