(* :Title: Comma Delimited *)

(* :Author: Mark Fisher *)

(* :Context: CommaDelimited` *)

(* :Requirements: Utilities`FilterOptions` *)

(* :Mathematica Version: 2.2 *)

(* :Source: Blachman, Nancy, "Importing and Exporting Data",
The Mathematica Journal, vol. 2, no. 3, Summer 1992, pp. 53-57. *)

(* :Summary: Routines to read and write comma-delimited ASCII files *)

(* :Discussion:
ReadDelimited is designed to read data from comma-delimited files containing
a mixture of numbers, dates, and strings. Many databases for PCs read and
write comma-delimited files rapidly. ReadDelimited is intended to obviate
the need for writing customized reports in the database to transfer data to
Mathematica can read. The default comma-delimited format seperates fields
by commas, surounds strings with double quotes, and formats dates as mm/dd/yy.
ReadDelimited works by first reading in the data as a matrix of strings using
ReadList and then reformating the data according to a user specified list of
data types: Number, String, Date, and QuotedString. Numbers are converted to
expressions, quoted strings have the extra quotes stripped off, dates are
converted by StringToDate, and String leaves the expression unchanged for
strings that are not surrounded by quotes.

WriteDelimited is a modified version of Nancy Blachman's function
writeMatrixToSpreadSheet that appeared in the Mathematica Journal. The
nested Scan loops are classically elegant Mathematica iteration. The
modifications, essentially, are the addition of the options to change the
delimiter, format the data, and OpenAppend in addition to OpenWrite. The
latter allows AppendDelimited simply to pass options to WriteDelimited.

StringToDate and DateToString convert dates from strings to Mathematica date
format and back. There are two verions of StringToDate. The default version
takes no options for efficiency reasons. The other version allows the user
to specify the order of parts of the date and the divider used. These
options can be passed through ReadDelimited via its option DateFormat.
*)

BeginPackage["CommaDelimited`", {"Utilities`FilterOptions`"}]

ReadDelimited::usage = "ReadDelimited[\"file\"] reads an array of data from
a comma-delimited ASCII file, returning an array of strings.
ReadDelimited[\"file\", {type1, type2, ...}] converts the data according to
the sequence of types. Valid types are String, Number, Date, and QuotedString.
The options of ReadList can be passed via ReadDelimited."

QuotedString::usage = "QuotedString is used in ReadDelimited to
indicate a string enclosed in quotes. ReadDelimited will strip the
quotes off."

WriteDelimited::usage = "WriteDelimited[\"file\", data] writes a
comma-delimited ASCII file for an array of data, for which
MatrixQ[data] is True. WriteDelimited takes options FieldDelimiter and
DataFormat."

AppendDelimited::usage = "AppendDelimited[\"file\", data] appends an
array of data to a comma-delimited ASCII file, for which MatrixQ[data]
is True. AppendDelimited takes options FieldDelimiter and DataFormat."

StringToDate::usage = "StringToDate[mm/dd/yy] converts a string date like
\"mm/dd/yy\" to a Mathematica date like {yyyy, mm, dd}. StringToDate takes
options DateOrder and DateDivider."

DateToString::usage = "DateToString[{year, month, day}] converts a
Mathematica date {yyyy, mm, dd} to a string date \"mm/dd/yyyy\"."

Open::usage = "Open is an option for WriteDelited that specifies whether to
OpenWrite or OpenAppend. The default is Open -> OpenWrite."

FieldDelimiter::usage = "FieldDelimiter is an option for ReadDelimited,
WriteDelimited, and AppendDelimited, which specifies the character to be
used to separate fields in a record. The default is FieldDelimiter ->
\",\". Use NullWords -> True to force null fields between single
delimiters." 

DataFormat::usage = "DataFormat is an option for WriteDelimited and
AppendDelimited, which specifies the format (e.g., CForm) to be applied to
the data prior to writing the file. The default is DataFormat -> InputForm."

DateFormat::usage = "DateFormat is an option for ReadDelimited, which
specifies the string representation a date. The default is DateFormat ->
Automatic, which is equivalent to DateFormat -> {DateOrder -> {Month, Day,
Year}, DateDivider -> \"/\"}."

DateDivider::usage = "DateDivider is an option for StringToDate, which
specifies the (single) character that separates the parts of a date in the
string representation. The default setting is DateDivider -> \"/\"."

DateOrder::usage = "DateOrder is an option for StringToDate, which specifies
the order of the parts of the date in the string representation. The default
setting is DateOrder -> {Month, Day, Year}."

StringMonth::usage = "StringMonth is an option for StringToDate, which
indicates whether the month is a \"string\" like Jan, Feb, etc. The
default setting is StringMonth -> False."

Month::usage = "Month represents the month component of a date in the
DateOrder option for DateToString."

Day::usage = "Day represents the day component of a date in the DateOrder
option for DateToString."

Year::usage = "Year represents the year component of a date in the DateOrder
option for DateToString."

StringMonthRule::usage = "StringMonthRule is the list of rules used to
change months represented as strings (Jan, Feb, ... or jan, feb,
...) to numbers (1, 2, ...). If the string representation is not one
or these two, you will need to set StringMonthRule yourself."

ReportForm::usage = "ReportForm[expr, {n, f}] prints expr with n
digits, f to right of the decimal. The following is an example of how
ReportForm can be used with WriteDelimited and AppendDelimited:
DataFormat -> (ReportForm[#, {6,6}]&)."

Begin["`Private`"]

ReportForm[n_, d_] :=
	PaddedForm[AccountingForm[n, NumberSigns -> {"-", ""},
		NumberPadding -> {" ", "0"}], d]

(* basic: each entry is a string if typelist == {}
   more advanced: user supplies data types
	data types are Number, Date, and String *)

ReadDelimited::badtype = "One or more of `1` is not
valid. Valid types are Number, Date, String, and QuotedString."
ReadDelimited::badlength = "`1` has `2` columns, but `3` has
length `4`."
ReadDelimited::noopen = "Could not open `1`"

Options[ReadDelimited] = {DateFormat -> Automatic, FieldDelimiter -> ","}

ReadDelimited[filename_String, typelist_List:{}, opts___Rule] :=
	With[{	rlopts = FilterOptions[ReadList, opts],
		dateform = DateFormat /. {opts} /. Options[ReadDelimited],
		delimiter = FieldDelimiter /. {opts} /. 
			Options[ReadDelimited]},
	With[{in = ReadList[filename, Word, rlopts, (* read as strings *)
		WordSeparators -> {delimiter},
		NullWords -> False,
		RecordLists -> True],
		df = If[dateform === Automatic, Automatic, 
			FilterOptions[DateToString, Sequence @@ dateform]]},
	Which[
	in === $Failed, (* couldn't open *)
		Message[ReadDelimited::noopen, filename];
		Return[$Failed],
	typelist === {}, in, (* basic, leave as strings *)
	Length[typelist] != Length[in[[1]]], (* bad typelist length *)
		Message[ReadDelimited::badlength, filename, 
			Length[in[[1]]], typelist, Length[typelist]];
		Return[$Failed],
	Not[And @@ (MemberQ[{Number, Date, String, QuotedString}, #]& /@ 
			typelist)], (* bad typelist types *)
		Message[ReadDelimited::badtype, typelist]; Return[$Failed],
	True, (* advanced, transform data *)
		MapThread[Map, 
			{(typelist /. {
					Number | QuotedString -> ToExpression, 
					If[df === Automatic, Date -> StringToDate,
						Date -> StringToDate[#, df]&],
					String -> Identity}),
			Transpose[in]}] // Transpose
		]]]


WriteDelimited::notmat = "`1` is not a two-dimensional matrix."

Options[WriteDelimited] = {Open -> OpenWrite, FieldDelimiter -> ",",
		DataFormat -> InputForm}

SetAttributes[WriteDelimited, HoldAll]

WriteDelimited[filename_String, data_, opts___Rule] :=
	Module[{format, file, delimiter},	
		{format, file, delimiter} = 
			{DataFormat, Open[filename], FieldDelimiter} /. {opts} /. 
				Options[WriteDelimited];
	Which[file === $Failed, Return[$Failed],
		!MatrixQ[data], Message[WriteDelimited::notmat, HoldForm[data]];
			Close[file]; Return[$Failed]];
	Scan[
	(* write 1st item in row *)	
		(WriteString[file, First[#]];
	(* preceed the rest with delimiters *)
		Scan[WriteString[file, delimiter, #]&, Rest[#]];
	(* end with linefeed *)
		WriteString[file, "\n"])&,
	(* do this for each row of formatted data *)
		Map[format, data, {2}]];
	Close[file]
	]

(*WriteDelimited[filename_String, data_, opts___Rule] :=
	With[{	format = DataFormat /. {opts} /. Options[WriteDelimited],
		file = Open[filename] /. {opts} /. Options[WriteDelimited],
		delimiter = FieldDelimiter /. {opts} /. 
			Options[WriteDelimited]},
	Which[file === $Failed, 
			Return[$Failed],
		!MatrixQ[data], 
			Message[WriteDelimited::notmat, HoldForm[data]];
			Close[file];
			Return[$Failed]];
	With[{temp = Map[format, data, {2}]},	(* format the data *)
	Scan[ (	WriteString[file, First[#]];	(* write 1st item in row *)
		Scan[	WriteString[file, delimiter, #]&,
			Rest[#]		(* preceed the rest with delimiters *)
		];
		WriteString[file, "\n"] )&,	(* end with linefeed *)
		temp				(* do this for each row *)
	];
	Close[file] 
	]]*)

Options[AppendDelimited] = {FieldDelimiter -> ",", DataFormat -> InputForm}

AppendDelimited[filename_String, data_, opts___Rule] := 
	WriteDelimited[filename, data, Open -> OpenAppend, 
		DataFormat ->  (DataFormat /. {opts} /. 
			Options[AppendDelimited]),
		FieldDelimiter -> (FieldDelimiter /. {opts} /. 
			Options[AppendDelimited])]

DateToString[date:{_, _, _}] := 
	With[{temp = ToString /@ date},
	temp[[2]] <> "/" <> temp[[3]] <> "/" <> temp[[1]] ]

Options[StringToDate] = {DateDivider -> "/", 
		DateOrder -> {Month, Day, Year}, StringMonth -> False}

StringMonthRule = Thread[
	{"Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep",
		"Oct", "Nov", "Dec"} -> ToString /@ Range[12]]

StringToDate[date_String] := (* no-options version for efficiency *)
	Block[{temp = 
		ToExpression["{" <> 
			StringReplace[date, "/" -> ","] <> "}"][[{3, 1, 2}]] },
		If[temp[[1]] < 100, temp + {1900, 0, 0}, temp]]

StringToDate[date_String, opts___Rule] := 
	Block[{temp = 
		ToExpression["{" <> 
			StringReplace[
				If[StringMonth /. {opts} /. Options[StringToDate],
					StringReplace[date, StringMonthRule, IgnoreCase -> True], 
					date], 
				(DateDivider /. {opts} /. Options[StringToDate]) -> ","]
			<> "}"][[ 
				Flatten[Position[DateOrder /. {opts} /. 
				Options[StringToDate], #]& /@ {Year, Month, Day}] ]] },
		If[temp[[1]] < 100, temp + {1900, 0, 0}, temp]]


End[]
EndPackage[]

(* :Example:

Write a matrix of numbers called "mat" to a file named 
	"test.txt" and have them separated by tabs and 
	printed in CForm, use:

WriteDelimited["test.txt", mat, FieldDelimiter -> "\t", DataFormat -> CForm]


To see what data types to use:
In[2]:=
	First @ ReadDelimited["test.dat"]
Out[2]=
	{"912810CL0", 10.5, 2/15/1995, 0, 8/19/1993, 109.24, 5515}
Then use data types:
In[3]:=
	First @ ReadDelimited["test.dat", {String, Number, Date, Number,
		Date, Number, Number}]
Out[3]=
	{912810CL0, 10.5, {1995, 2, 15}, 0, {1993, 8, 19}, 109.24, 5515}
*)
