(* read CRSP Daily U.S. Government Bond File *)
(* :Title: CRSPDailyBonds *)
(* :Context: CRSPDailyBonds` *)

(* :Author: Mark Fisher *)

(* :Summary: 
This package provides functions that read data from the CRSP Daily
U.S. Government Bond Files and also some functions that produce MD
objects for use with the package YieldCurve. *)

(* :Discussion: 
(1) CRSPDailyDirectory is the directory (a string) where the CRSP daily
bond files are located. You may need to modify this if the current
setting does not work. 
(2) The package looks for the file "header.m" in
CRSPDailyDirectory. This file contains the information from HeaderFile
in a form that can be read into Mathematica more rapidly that reading
and parsing HeaderFile directly. If "header.m" does not exist, the
package will try to create it unless the you set the variable
$HeaderListFile = False in the code.
(3) If the data set is updated, you will need to delete "header.m" so
it will be recreated with the new data set.

Many of the definitions in this package will shadow or be shadowed by
the same or similar definitions in the package
CRSPMonthlyBonds`. I have included some utilities to switch back
and forth between the two contexts. CRSPContext is defined in both
contexts and returns the name of which ever context is higher on the
context path. CRSPSwap[] will swap the positions of the two contexts
on the context path, making the other context's definitions available.
*)

CRSPBonds::both = "Warning: Both CRSPDailyBonds and CRSPMonthlyBonds
are loaded. Many definitions will be shadowed. Use CRSPSwap[] to swap
contexts as needed."

If[MemberQ[$ContextPath, "CRSPMonthlyBonds`"],
	Off[General::shdw];
	Message[CRSPBonds::both]]

BeginPackage["CRSPDailyBonds`", {"Miscellaneous`Calendar`", 
	"Graphics`Colors`"}]

FieldInformation::usage = "FieldInformation[] prints the usage
messages for the fields in the MergedFields. FieldInformation[list]
prints the information for the symbols in the list."

SwapContexts::usage = "SwapContext[context1, context2] will swap the
order of the two contexts in $ContextPath, thereby allowing direct
access to shadowed definitions. Note that the contexts must be
strings."

CRSPSwap::usage = "CRSPSwap[] swaps the context \"CRSPDailyBonds`\"
with the context \"CRSPMonthlyBonds`\" to allow direct access to
the definitions that are shadowed when both packages are loaded."

CRSPContext::usage = "CRSPContext is defined in both the Monthly and
Daily CRSP packages. It returns the value of the context that is
higher on the context path ($ContextPath)."

ReadDates::usage = "ReadDates[] returns a list of all valid quote
dates for the CRSP daily bond files in CRSP integer form."

DateToInteger::usage = "DateToInteger[{year, month, day}] returns an
integer representation of the date that is used by the CRSP daily bond
files."

IntegerToDate::usage = "IntegerToDate[integer] returns a date in the
form of {year, month, day} from an integer representation that is used
by the CRSP daily bond files."

CRSPDailyDirectory::usage = "CRSPDailyDirectory is the directory (a
string) where the CRSP daily bond files are located. You may need to
modify this if the current setting does not work."

CRSPFieldSeparators::usage = "CRSPFieldSeparators is a list of field
separators used in the CRSP daily bond files."

RecordLength::usage = "RecordLength[\"file\"] returns the number of
characters plus one of the first record in the file."

NumberOfFields::usage = "NumberOfFields [\"file\"] returns the number
of fields in a CRSP daily bond file." 

DatePrices::usage = "DatePrices[date] returns all price quote records
for a given date. The date may be given either in the CRSP integer
representation as as {year, month, day}."

DateYields::usage = "DateYields[date] returns all yield quote records
for a given date. The date may be given either in the CRSP integer
representation as as {year, month, day}."

SecurityPrices::usage = "SecurityPrices[CRSPID] returns all price
quote records for a given CRSPID."

SecurityYields::usage = "SecurityYields[CRSPID] returns all yield
quote records for a given CRSPID."

CRSPPayments::usage = "CRSPPayments[CRSPID] returns all payments
records for a given CRSPID."

DebtOutstanding::usage = "DebtOutstanding[CRSPID] returns all debt
outstanding records for a given CRSPID."

ReadCalendar::usage = "ReadCalendar[] returns all the records from the
CRSP calendar file."

MassageCRSPData::usage = "MassageCRSPData[date] returns an MD object
constructed from bond prices in the CRSP daily bond files for the date
specified. MassageCRSPData take the options Bills, NotesAndBonds,
Callable, MinimumMaturityCutoff, and MaximumMaturityCutoff."

Bills::usage = "Bills is an option for MassageCRSPData that works with
automatic SelectionCondition to specify whether to include bills in
the data. The default setting is Bills -> True."

NotesAndBonds::usage = "NotesAndBonds is an option for MassageCRSPData
 that works with automatic SelectionCondition to specify whether to
include notes and bonds in the data. The default setting is
NotesAndBonds -> True."

Callable::usage = "Callable is an option for MassageCRSPData
 that works with automatic SelectionCondition to specify whether to
include callable bonds that are not otherwise special (e.g., that are
not flower bonds)."

MinimumMaturityCutoff::usage = "MinimumMaturityCutoff is an option for
MassageCRSPData that works with automatic SelectionCondition to
specify the minimum maturity to include in the data. The default is
MinimumMaturityCutoff -> 0 Days."

MaximumMaturityCutoff::usage = "MaximumMaturityCutoff is an option for
MassageCRSPData that works with automatic SelectionCondition to
specify the maximum maturity to include in the data. The default is
MaximumMaturityCutoff -> 100 (years)."

PriceFunction::usage = "PriceFunction[bid, ask] is an option for
MassageCRSPData which specifies how to combine the bid and ask
prices. The default is PriceFunction -> (#1&), which uses only the bid
price."

SelectionCondition::usage = "SelectionCondition is an option for
MassageCRSPData which specifies which securities to select. With the
default setting (SelectionCondition -> Automatic, which eliminates
callable, flower, and other securities with special features), the
options Bills, NotesAndBonds, MinimumMaturityCutoff, and
MaximumMaturityCutoff are all operative."

HeaderList::usage = "HeaderList is the name of the list that the
HeaderFile is stored in."

HeaderListFile::usage = "HeaderListFile is the name of the file that
HeaderList is stored in as a Mathematica expression."

SearchEngine::usage = "SearchEngine is the main search engine."

CRSPSummary::usage = "CRSPSummary[date] returns summary information
about the data for the specified date. CRSPSummary takes the option
PlotYields."

PlotYields::usage = "PlotYields is an option for CRSPSummary which
specifies whether to plot the yields. The default is PlotYields ->
True." 

Observations::usage = "Observations is used in the output of
CRSPSummary to identify the number of observations."

Sources::usage = "Sources is used in the output of CRSPSummary to
identify the sources of the quotes."

Callable::usage = "Callable is used in the output of CRSPSummary to
identify the number of callable bonds."

FlowerBonds::usage = "FlowerBonds is used in the output of CRSPSummary
to identify the number of flower bonds."

TaxExemptInterest::usage = "TaxExemptInterest is used in the output of
CRSPSummary to identify the number of bonds with wholly or partially
tax-exempt interest."

OtherSpecial::usage = "OtherSpecial is used in the output of
CRSPSummary to identify the number of bonds with other special
features."

LongestMaturity::usage = "LongestMaturity is used in the output of
CRSPSummary to identify the longest maturity (in years) in the
sample."

MergeDateData::usage = "MergeDateData[date] merges the data from the
header, price, and yield files for the specified date."

SelectFields::usage = "SelectFields[data, fields] returns only the
fields specified."

SelectRows::usage = "SelectRows[data, condition] returns all rows for
which the condition evaluates to True."


ACCINT::usage = "ACCINT is the total accrued interest at the end of day."

ASK::usage = "ASK is the ask price."

BANKDT::usage = "BANKDT is the bank eligibility date at the time of
issue (0 = no restrictions apply)."

BID::usage = "BID is the bid price."

CD1M::usage = "CD1M is the one-month certificate of deposit rate."

CD3M::usage = "CD2M is the three-month certificate of deposit rate."

CD6M::usage = "CD3M is the six-month certificate of deposit rate."

COUPRT::usage = "COUPRT is the coupon rate."

CP30D::usage = "CP30D is the 30-day commercial paper rate."

CP60D::usage = "CP360D is the 60-day commercial paper rate."

CP90D::usage = "CP90D is the 90-day commercial paper rate."

CRSPID::usage = "CRSPID is the CRSP identification number."

CUSIP::usage = "CUSIP is the CUSIP number."

DATDT::usage = "DATDT is the dated date (issue date) (0 = not
available or not applicable)."

DELDAT::usage = "DELDAT is the delivery date."

DURATN::usage = "DURATN is (Macaulay's) duration."

FCALDT::usage = "FCALDT is the first eligible call date at time of
issue, in YYYYMMDD format."

FCPDT::usage = "FCPDT is the first coupon date, in YYYYMMDD format."

FCPDTF::usage = "FCPDTF is the first coupon date payment flag."

FFERFT::usage = "FFERFT is the Federal funds effective rate."

FFMAXR::usage = "FFMAXR is the Federal funds maximum trading range."

FFMINR::usage = "FFMINR is the Federal funds minimum trading range."

FLOWER::usage = "FLOWER indicates estate tax treatment (1 = no special
status; 2 or 3 = flower bond)."

FSTQUO::usage = "FSTQUO is the day number of an issue's first quote date."

FSTYLD::usage = "FSTYLD is the day number of an issue's first yield date."

LSTQUO::usage = "LSTQUO is the day number of an issue's last quote date."

LSTYLD::usage = "LSTYLD is the day number of an issue's last yield date."

MATDT::usage = "MATDT is the maturity date at the time of issuance."

NAME::usage = "NAME is the name of the government security."

NIPPY::usage = "NIPPY is the number of interest payments per year (0 =
bill or certificate paying interest only at maturity)."

NOTICE::usage = "NOTICE is the notice required on callable issues."

NUMACT::usage = "NUMACT is the number of active issues."

NUMDBT::usage = "NUMDBT is the number of amount outstanding observations."

NUMPAY::usage = "NUMPAY is the number of interest payments."

PDINT::usage = "PDINT is the interest paid."

PUBOUT::usage = "PUBOUT is the publicly held face value outstanding."

QDATE::usage = "QDATE is the quote date."

RETNUA::usage = "RETNUA is the unadjusted return."

SOURCE::usage = "SOURCE is the primary data source (R = FRBNY, S =
Salomon, W = WSJ, Q = Bank quotation, M = not avaiable)."

TAX::usage = "TAX indicates the taxability of interest (1 = fully
taxable; 2 = partially tax exempt; 3 = wholly tax exempt)."

TOTOUT::usage = "TOTOUT is the face value outstanding."

TYPE::usage = "TYPE is the type of issue (1 = noncallable bond; 2 =
noncallable note; 3 = certificate of indebtedness; 4 = Treasury bill;
5 = callable bond; 6 = callable note; 7 = tax anticipation certificate
of indebtedness; 8 = tax anticipation bill; 9 = unusual provisions)."

UNIQ::usage = "UNIQ is the uniqueness number."

VALFC::usage = "VALFC is the amount of the first coupon per $100 of
face value."

WHY::usage = "WHY indicates the reason for the end of data on the file."

YLD::usage = "YLD is the annual yield."

YMCNOT::usage = "YMCNOT is the year and month of first call notice, in
YYYYMM format (0 = not callable or not called)."

(************ field names *****************************)
MasterQuotesFields = {CRSPID, QDATE, BID, ASK, SOURCE}
CrossQuotesFields  = {CRSPID, QDATE, BID, ASK, SOURCE}
MasterYieldsFields = {CRSPID, QDATE, ACCINT, YLD, RETNUA, DURATN}
CrossYieldsFields  = {CRSPID, QDATE, ACCINT, YLD, RETNUA, DURATN}
MasterPaymentsFields        = {CRSPID, QDATE, PDINT}
MasterDebtOutstandingFields = {CRSPID, QDATE, TOTOUT, PUBOUT}
CalendarFields = {QDATE, DELDAT, CD1M, CD3M, CD6M, CP30D,
		CP60D, CP90D, FFERFT, FFMINR, FFMAXR, NUMACT}
HeaderFields = {CRSPID, CUSIP, NAME, MATDT, TYPE, COUPRT,
		UNIQ, WHY, DATDT, BANKDT, FCALDT, YMCNOT, NOTICE,
		TAX, FLOWER, NIPPY, FCPDT, FCPDTF, VALFC, FSTQUO, LSTQUO,
		FSTYLD, LSTYLD, NUMDBT, NUMPAY}

MergedFields = Join[HeaderFields, Rest @ CrossQuotesFields, 
		Drop[CrossYieldsFields, 2]]
(******************************************************)

Begin["`Private`"]

CRSPSwap[] := 
	SwapContexts["CRSPDailyBonds`", "CRSPMonthlyBonds`"] 

SwapContexts[a_String, b_String] :=
	Module[{order, neworder},
	order = Flatten[Position[$ContextPath, #]& /@ {a, b}];
	neworder = Range[Length[$ContextPath]] /.
		Thread[order -> Reverse[order]];
	$ContextPath = $ContextPath[[ neworder ]] ]

CRSPContext = "CRSPDailyBonds`"

(**************** define system stuff ******************)
CRSPDailyDirectory = "/tr/data4/m1grd00"
(* the " " allows Mma to squeeze out unwanted spaces *)
CRSPFieldSeparators = {"|", " "}
(**************** file names ***********************)
(*	"master" files are organized by id; 
	"cross" files are organized by date *)
HeaderListFile := CRSPDailyDirectory <> "/" <> "header.m"
HeaderFile := CRSPDailyDirectory <> "/" <> "bmheader.dat"
CalendarFile := CRSPDailyDirectory <> "/" <> "bxcalind.dat"
PaymentsFile := CRSPDailyDirectory <> "/" <> "bmpaymts.dat"
DebtOutstandingFile := CRSPDailyDirectory <> "/" <> "bmdebt.dat"
MasterAddressFile := CRSPDailyDirectory <> "/" <> "bmaddrs.dat"
MasterQuotesFile := CRSPDailyDirectory <> "/" <> "bmquotes.dat"
MasterYieldsFile := CRSPDailyDirectory <> "/" <> "bmyield.dat"
CrossAddressFile := CRSPDailyDirectory <> "/" <> "bxaddrs.dat"
CrossQuotesFile := CRSPDailyDirectory <> "/" <> "bxquotes.dat"
CrossYieldsFile := CRSPDailyDirectory <> "/" <> "bxyield.dat"

(*************** create header.m *************************)
(* if you wish to create header.m, use this *)
(*		Put[ReadHeaderFile[], HeaderListFile]	*)
(* then set $HeaderListFile = True *)

(**************** check for header.m *******************)
HeaderFile::oops = "Warning: $HeaderListFile is set to True, but
\"header.m\" does not exist in `1`. (You may need to reset
CRSPDailyDirectory.) Setting $HeaderListFile to False for this
session."

(* this indicates that "header.m" exists in CRSPDailyDirectory *)
$HeaderListFile = True

Off[FileNames::cdir]
(* check to see if it really does *)
If[	$HeaderListFile && 
	Not[FileNames["header.m", CRSPDailyDirectory] === {HeaderListFile}],
	(* then *)
	Message[HeaderFile::oops, CRSPDailyDirectory];
	$HeaderListFile = False]
On[FileNames::cdir]

(******************** utilities ***********************)
(* for plot label *)
DateToString[date:{_, _, _}] := 
	With[{temp = ToString /@ date},
	temp[[2]] <> "/" <> temp[[3]] <> "/" <> temp[[1]] ]

(* load HeaderList if not already loaded *)
LoadHeaderList[] :=	If[ValueQ[HeaderList]//Not,
	Print["Loading HeaderList... Please wait."]; 
	HeaderList = 
		If[$HeaderListFile, Get[HeaderListFile], ReadHeaderFile[]]];

(* date/string conversion utilities *)
DateToInteger[date:{_Integer, _Integer, _Integer}] :=
	Plus @@ (date * {10000, 100, 1})

SetAttributes[IntegerToDate, Listable]
IntegerToDate[int_Integer] :=
	MapThread[Quotient, 
	{FoldList[Mod, int, {10000, 100}], {10000, 100, 1}}]

(* get "record lengths" *)
RecordLength[file_String] := 
	RecordLength[file] = (* for quick recall *)
	(StringLength[First @ ReadList[file, Record, 1]] + 1)

(* get number of fields *)
NumberOfFields[file_String] := 
	NumberOfFields[file] = (* for quick recall *)
	Module[{stream, temp},
		stream = StringToStream[First @ ReadList[file, Record, 1]];
		temp = ReadList[stream, Word, 
				WordSeparators -> CRSPFieldSeparators];
		Close[stream];
		Length[temp]]

(* read all quote dates from calendar file *)
ReadDates[] := 
	ToExpression[StringTake[#, 8]]& /@ ReadList[CalendarFile, Record]

(* returns the "calendar" file *)
ReadCalendarFile[] := 
	ToExpression @ ReadList[CalendarFile, 
		Word, RecordLists -> True, WordSeparators -> CRSPFieldSeparators]


(* returns the "header" file *)
ReadHeaderFile[] := 
	Module[{temp = 
		Transpose[ReadList[HeaderFile, Word, 
			RecordLists -> True, WordSeparators -> CRSPFieldSeparators]]},
		Transpose[Join[Take[temp, 3], ToExpression[Drop[temp, 3]]]]]

FieldInformation[list:{__Symbol}] := (Information /@ list;)
FieldInformation[] := FieldInformation[MergedFields]

(******************* MergeDateData, etc ************************)
MergeDateData::notsame = "Warning: The number of price observations is
different from the number of yield observations."

MergeDateData[date:{_Integer, _Integer, _Integer}] := 
	MergeDateData[DateToInteger[date]]

MergeDateData[date_Integer] := 
	Module[{prices, ylds, ids, headers},
	prices = DatePrices[date];
	If[prices == $Failed, Return[prices]];
	ylds = DateYields[date];
	If[ylds == $Failed, Return[ylds]];
(* check for consistency *)
	If[Length[prices] != Length[ylds], (* not same number *)
	(* then *)
		Message[MergeDateData];
		ids = Intersection[First /@ prices, First /@ ylds],
	(* else *)
		ids = First /@ prices];
	LoadHeaderList[];
	headers = Cases[HeaderList, {Alternatives @@ ids, __}];
	MapThread[Join, {Sort[headers], Rest /@ Sort[prices],
		Drop[#, 2]& /@ Sort[ylds]}]
	]

SelectFields[data_?MatrixQ, field_Symbol] := SelectFields[data, {field}]

SelectFields[data_?MatrixQ, fields_List] := 
	Module[{pred = Function @@ {MergedFields, fields}},
		Apply[pred, data, {1}]]

SelectRows[data_?MatrixQ, cond_] := 
	Module[{pred = Function @@ {MergedFields, cond}},
		Select[data, Apply[pred, #]&]]
(******************************************************)


(******* functions that call the search engine ****************)
(* returns all price quotes on a given date *)
DatePrices[date_Integer] := 
	SearchEngine[	ToString[date],
					CrossAddressFile, 
					{2, 3},
					CrossQuotesFile, 
					{{ToExpression, {2, 3, 4}}}
				]

DatePrices[date:{_Integer, _Integer, _Integer}] :=
	DatePrices[DateToInteger[date]]

(* returns all yields on a given date *)
DateYields[date_Integer] :=
	SearchEngine[	ToString[date],
					CrossAddressFile, 
					{4, 5},
					CrossYieldsFile,
					{{ToExpression, {2, 6}},
					 {YieldConversion, {3, 4, 5}}}
				]

DateYields[date:{_Integer, _Integer, _Integer}] :=
	DateYields[DateToInteger[date]]

(* returns all price quotes for a given id *)
SecurityPrices[crspid_String] :=
	SearchEngine[	crspid,
					MasterAddressFile, 
					{6, 7},
					MasterQuotesFile,
					{{ToExpression, {2, 3, 4}}}
				]

(* returns all yields for a given id *)
SecurityYields[crspid_String] :=
	SearchEngine[	crspid,
					MasterAddressFile, 
					{8, 9},
					MasterYieldsFile,
					{{ToExpression, {2, 6}}, 
					 {YieldConversion, {3, 4, 5}}}
				]

(* returns payments for given id *)
CRSPPayments[crspid_String] :=
	SearchEngine[	crspid,
					MasterAddressFile, 
					{4, 5},
					PaymentsFile,
					{{ToExpression, {2, 3}}}
				]

(* returns debt outstanding for given id *)
DebtOutstanding[crspid_String] :=
	SearchEngine[	crspid,
					MasterAddressFile, 
					{2, 3},
					DebtOutstandingFile,
					{{ToExpression, {2, 3, 4}}}
				]

SetAttributes[YieldConversion, Listable]
YieldConversion[x_String] := 
	365.25 * ToExpression[StringReplace[x, "e" -> "*10^"]]

(********************* search engine *****************************)
SearchEngine::notavail = "The requested item is not available for
CRSP ID `1`."
SearchEngine::notvalid = "`1` is not a valid CRSP ID or a valid date."

SearchEngine[	key_String, 
				addressfile_String, 
				addressfieldpos:{_Integer, _Integer}, 
				datafile_String, 
				fcnpos:{{_, _List} ..}] := 
Module[{ws, out, stream, string, location, size, wordtab, 
		numrecords},
(* look up in the address file where to find the key in the data file *)
	Check[stream = OpenRead[addressfile], Return[$Failed]]; (* couldn't open *)
	string = Find[stream, key, AnchoredSearch -> True];
	Close[stream];
	If[string == EndOfFile, (* didn't find key *)
		Message[SearchEngine::notvalid, key];
		Return[$Failed]];
(* extract the location and size info (no. of chars) *)
	ws = WordSeparators -> CRSPFieldSeparators;
	{location, size} = ToExpression @ ReadList[stream = 
		StringToStream[string], Word, ws][[addressfieldpos]];
	Close[stream]; 
	If[size == 0, (* found key, but no data *)
		Message[SearchEngine::notavail, key];
		Return[{}]];
(* read the data from the quotes file *)
	wordtab = Table[Word, {NumberOfFields[datafile]}]; (* for efficiency *)
	numrecords = size/RecordLength[datafile];
	Check[stream = OpenRead[datafile], Return[$Failed]]; (* couldn't open *)
	SetStreamPosition[stream, location];
	out = Table[Read[stream, wordtab, ws], {numrecords}];
	Close[stream];
(* convert the strings as requested and return the data *)
	PostSearchConversion[out, fcnpos]
	]

PostSearchConversion[datalist_List, fcnpos:{{_, _List} ..}] := 
	Fold[MapAt[#2[[1]], #1, Partition[#2[[2]], 1]]&, 
		Transpose[datalist], fcnpos] //Transpose

(**********************************************************************)


(************************* MassageCRSPData ****************************)
MassageCRSPData::nobonds = "There are no securities that satisfy the
selection criteria `1` on `2`."

Options[MassageCRSPData] = Sort @ {Bills -> True, NotesAndBonds -> True,
MinimumMaturityCutoff -> 0, MaximumMaturityCutoff -> 36525,
PriceFunction -> (#1&), SelectionCondition -> Automatic,
Callable -> False}

MassageCRSPData[date:{_Integer, _Integer, _Integer}, opts___Rule] :=
	Module[{int, merged, cond, selected,
		dateplus, minmat, maxmat, bills, notesbonds, pfcn,
		id, mat, iss, coup, bid, ask},
	{bills, notesbonds, minmat, maxmat, pfcn, cond, call} = {Bills, 
		NotesAndBonds, MinimumMaturityCutoff, MaximumMaturityCutoff, 
		PriceFunction, SelectionCondition, Callable} /. {opts} /. 
				Options[MassageCRSPData];
	{minmat, maxmat} = DateToInteger[DaysPlus[date, Round[#]]]& /@ 
		{minmat, maxmat};
	int = DateToInteger[date]; (* integer representation *)
(* select securities according to condition *)
	LoadHeaderList[];
	merged = MergeDateData[date];
	If[merged == $Failed, Return[merged]];
	If[cond === Automatic,
	cond = And[
		Or[	If[notesbonds, Or @@ Thread[NAME == {"NOTE", "BOND"}], False],
			If[bills, NAME == "BILL", False]], (* name: notes and bonds *)
		If[!call, FCALDT == 0, True], (* callable? *)
		MATDT >= minmat,		(* maturity date => quote date plus min *)
		MATDT <= maxmat,		(* maturity date <= quote date plus max *)
		DATDT <= int,			(* issue date <= quote date *)
		DATDT >= 0,				(* issued after date 0 *)
		TAX == 1,				(* no special tax treatment *)
		FLOWER == 1,			(* no flower bonds *)
		SOURCE != "M"			(* no N/A prices *)
		]];
	selected = SelectRows[merged, cond];
	If[selected === {}, 
		Message[MassageCRSPData::nobonds, cond, date];
		Return[$Failed]];
(* calculate various things *)
	{id, mat, iss, coup, bid, ask} = 
			Transpose[selected][[{ 1, 4, 9, 6, 27, 28 }]];
	mat = IntegerToDate[mat];
	iss = IntegerToDate[iss];
	price = MapThread[pfcn, {bid, ask}];
	term = 	(Apply[Plus, {1., 1/12., 1/365.25} * # & /@
		(Subtract @@ {mat, iss}), {1}] //Round) /. 0 -> .5;
(* construct and return an MD object *)
	YieldCurve`MD[	date, mat, term, coup, price, id,
		{"CRSP Daily U.S. Government Bond File"}]
	]


(****************** CRSPSummary ********************)
Options[CRSPSummary] = {PlotYields -> True}

CRSPSummary[date:{_Integer, _Integer, _Integer}, opts___Rule] :=
	Module[{temp = DatePrices[date]},
		If[temp === $Failed, temp, CRSPSummary[temp, opts]]]

CRSPSummary[data_?MatrixQ, opts___Rule] :=
	Module[{date, noobs, sources, ids, bids, selected, name, nameunion, 
		namecount, type, other, bills, notes, bonds, flower, tax, call, 
		maxmat, plot, ylds, colors, deleted, mat, calldate},
	noobs = Length[data];
	{ids, bids, sources} = Transpose[data][[{ 1, 3, 5 }]];
	sources = Union[sources];
(* select records for these securities from HeaderList *)
	LoadHeaderList[];
	selected = Cases[HeaderList, {Alternatives @@ ids, __}];
	{name, mat, type, calldate, tax, flower} = 
		Transpose[selected][[{ 3, 4, 5, 11, 14, 15 }]];
	nameunion = Union[name];
	namecount = Count[name, #]& /@ nameunion;
	call   = Count[type, 5 | 6];
	other  = Count[type, 3 | 7 | 8 | 9];
	tax    = Count[tax, 2 | 3];
	flower = Count[flower, 2 | 3];
	maxmat = DaysBetween[IntegerToDate[Max[mat]], date];
(* shall we plot? *)
	intdate = data[[1, 2]];
	date = IntegerToDate[intdate];
	plot = If[PlotYields /. {opts} /. Options[CRSPSummary], 
		(* then plot the yields *)
		ylds = 100 * (#[[4]]& /@ DateYields[date]);
	(* use calldate if callable is priced at a premium *)
		mat = MapThread[If[#1 > 0 && #2 > 100, #1, #3]&,
			{calldate, bids, mat}];
		mat = If[# < intdate, intdate, #]& /@ mat;
		mat = Apply[Plus, {1., 1/12., 1/365.25} * # & /@ IntegerToDate[mat], 
				{1}] - Plus @@ ({1., 1/12., 1/365.25} * date);
	(* bills: green, coupons: blue, callable: brown, flower, etc: red *)
		colors = Which[
			(Or @@ Thread[#[[5]] == {3, 7, 8, 9}]) || (* other *)
				(Or @@ Thread[#[[14]] == {2, 3}]) || (* tax *)
				(Or @@ Thread[#[[15]] == {2, 3}]), Red, (* flower *)
			(Or @@ Thread[#[[5]] == {5, 6}]), Brown, (* callable *)
			#[[5]] == 4, Green, (* bills *)
			True, Blue (* notes, bonds w/ no special features *)
				]& /@ selected;
	(* delete missing values (big negative) *)
		deleted = Transpose @ DeleteCases[
			Transpose @ {colors, mat, ylds}, {_, _, _?Negative}];
		Show[Graphics[MapThread[{#1, Point[{#2, #3}]}&, deleted]],
			PlotRange -> All, Axes -> False, 
				Frame -> {True, True, False, False},
				FrameLabel -> {"years", "percent"},
				PlotLabel -> TableForm[{DateToString[date],
			"bills: green, coupons: blue, callable: brown, flower, etc: red"},
						TableAlignments -> Center,
						TableSpacing -> {0}]],
		{}, {}]; (* else *)
(* return summary info *)
	{	Date -> date,
		Observations -> noobs, Sources -> sources,
		Names -> Transpose[{nameunion, namecount}], 
		Callable -> call, FlowerBonds -> flower, 
		TaxExemptInterest -> tax, OtherSpecial -> other,
		LongestMaturity -> maxmat/365.25, Plot -> plot}
	]

End[]
EndPackage[]

If[MemberQ[$ContextPath, "CRSPMonthlyBonds`"],
	On[General::shdw]]


(* :Examples:

Needs["YieldCurve`"]

QuickBills[date:{_Integer, _Integer, _Integer}] :=
	(md = MassageCRSPData[date, NotesAndBonds -> False];
	cp = ConstructPayments[md];
	df = DiscountFunction[cp, LambdaValue -> 0, Knots -> 3, 
		MaximumMaturity -> 1, DropNumber -> 0];
	DFSummaryPage[df])

McCulloch[date:{_Integer, _Integer, _Integer}] := 
	(md = MassageCRSPData[date];
	cp = ConstructPayments[md];
	df = DiscountFunction[cp, FunctionalForm -> DLogDelta, 
		LambdaValue -> 0, DropNumber -> 0, 
		Knots -> Round[Length[Price[md]]/20],
		MaximumMaturity -> Round[Max[LastPayment[cp]]/Years]];
	DFSummaryPage[df])
*)

