(* read CRSP Monthly U.S. Government Bond File *)
(* :Title: CRSPMonthlyBonds *)
(* :Context: CRSPMonthlyBonds` *)

(* :Author: Mark Fisher *)

(* :Summary: 
This package provides functions that read data from the CRSP Monthly
U.S. Government Bond Files and also some functions that produce MD
objects for use with the package YieldCurve. *)

(* :Discussion: 
(1) CRSPMonthlyDirectory is the directory (a string) where the CRSP
monthly bond files are located. You may need to modify this if the
current setting does not work.
(2) The package looks for the file "mheader.m" in
CRSPMonthlyDirectory. This file contains the information from HeaderFile
in a form that can be read into Mathematica more rapidly that reading
and parsing HeaderFile directly. If "mheader.m" does not exist, the
package will try to create it unless the you set the variable
$MonthlyHeaderListFile = False in the code.
(3) If the data set is updated, you will need to delete "mheader.m" so
it will be recreated with the new data set.

Many of the definitions in this package will shadow or be shadowed by
the same or similar definitions in the package
CRSPDailyBonds`. I have included some utilities to switch back
and forth between the two contexts. CRSPContext is defined in both
contexts and returns the name of which ever context is higher on the
context path. CRSPSwap[] will swap the positions of the two contexts
on the context path, making the other context's definitions available.
*)

CRSPBonds::both = "Warning: Both CRSPDailyBonds and CRSPMonthlyBonds
are loaded. Many definitions will be shadowed. Use CRSPSwap[] to swap
contexts as needed."

If[MemberQ[$ContextPath, "CRSPDailyBonds`"],
	Off[General::shdw];
	Message[CRSPBonds::both]]

BeginPackage["CRSPMonthlyBonds`", {"Miscellaneous`Calendar`", 
	"Graphics`Colors`"}]

SwapContexts::usage = "SwapContext[context1, context2] will swap the
order of the two contexts in $ContextPath, thereby allowing direct
access to shadowed definitions. Note that the contexts must be
strings."

CRSPSwap::usage = "CRSPSwap[] swaps the context \"CRSPDailyBonds`\"
with the context \"CRSPMonthlyBonds`\" to allow direct access to
the definitions that are shadowed when both packages are loaded."

CRSPContext::usage = "CRSPContext is defined in both the Monthly and
Daily CRSP packages. It returns the value of the context that is
higher on the context path ($ContextPath)."

DateToInteger::usage = "DateToInteger[{year, month, day}] returns an
integer representation of the date that is used by the CRSP daily bond
files."

IntegerToDate::usage = "IntegerToDate[integer] returns a date in the
form of {year, month, day} from an integer representation that is used
by the CRSP daily bond files."

CRSPMonthlySummary::usage = "CRSPMonthlySummary[{year, month}] returns
summary information about the data for the specified
date. CRSPMonthlySummary takes the option PlotYields."

PlotYields::usage = "PlotYields is an option for CRSPSummary which
specifies whether to plot the yields. The default is PlotYields ->
True." 

Observations::usage = "Observations is used in the output of
CRSPSummary to identify the number of observations."

Sources::usage = "Sources is used in the output of CRSPSummary to
identify the sources of the quotes."

Callable::usage = "Callable is used in the output of CRSPSummary to
identify the number of callable bonds."

FlowerBonds::usage = "FlowerBonds is used in the output of CRSPSummary
to identify the number of flower bonds."

TaxExemptInterest::usage = "TaxExemptInterest is used in the output of
CRSPSummary to identify the number of bonds with wholly or partially
tax-exempt interest."

OtherSpecial::usage = "OtherSpecial is used in the output of
CRSPSummary to identify the number of bonds with other special
features."

LongestMaturity::usage = "LongestMaturity is used in the output of
CRSPSummary to identify the longest maturity (in years) in the
sample."

RecordLength::usage = "RecordLength[\"file\"] returns the number of
characters plus one of the first record in the file."

NumberOfFields::usage = "NumberOfFields [\"file\"] returns the number
of fields in a CRSP daily bond file." 

MergeDateData::usage = "MergeDateData[date] merges the data from the
header, price, and yield files for the specified date."

SelectFields::usage = "SelectFields[data, fields] returns only the
fields specified."

SelectRows::usage = "SelectRows[data, condition] returns all rows for
which the condition evaluates to True."

MassageCRSPData::usage = "MassageCRSPData[{year, month}] returns an MD
object constructed from bond prices in the CRSP daily bond files for
the year and month specified. MassageCRSPData take the options Bills,
NotesAndBonds, Callable, MinimumMaturityCutoff, and
MaximumMaturityCutoff."

Bills::usage = "Bills is an option for MassageCRSPData that works with
automatic SelectionCondition to specify whether to include bills in
the data. The default setting is Bills -> True."

NotesAndBonds::usage = "NotesAndBonds is an option for MassageCRSPData
 that works with automatic SelectionCondition to specify whether to
include notes and bonds in the data. The default setting is
NotesAndBonds -> True."

Callable::usage = "Callable is an option for MassageCRSPData
 that works with automatic SelectionCondition to specify whether to
include callable bonds that are not otherwise special (e.g., that are
not flower bonds)."

MinimumMaturityCutoff::usage = "MinimumMaturityCutoff is an option for
MassageCRSPData that works with automatic SelectionCondition to
specify the minimum maturity to include in the data. The default is
MinimumMaturityCutoff -> 0 Days."

MaximumMaturityCutoff::usage = "MaximumMaturityCutoff is an option for
MassageCRSPData that works with automatic SelectionCondition to
specify the maximum maturity to include in the data. The default is
MaximumMaturityCutoff -> 100 (years)."

PriceFunction::usage = "PriceFunction[bid, ask] is an option for
MassageCRSPData which specifies how to combine the bid and ask
prices. The default is PriceFunction -> (#1&), which uses only the bid
price."

SelectionCondition::usage = "SelectionCondition is an option for
MassageCRSPData which specifies which securities to select. With the
default setting (SelectionCondition -> Automatic, which eliminates
callable, flower, and other securities with special features), the
options Bills, NotesAndBonds, MinimumMaturityCutoff, and
MaximumMaturityCutoff are all operative."

FieldInformation::usage = "FieldInformation[] prints the usage
messages for the fields in the MergedFields. FieldInformation[list]
prints the information for the symbols in the list."

DateQuotes::usage = "DateQuotes[{year, month}] returns all quote
records for a given date. The field names for the data are in the list
MonthlyCrossFields. Information[MonthlyCrossFields] will print some
information about those fields."

MonthlyHeaderList::usage = "MonthlyHeaderList is the name of the list
that the MonthlyHeaderFile is stored in."

MonthlyHeaderListFile::usage = "MonthlyHeaderListFile is the name of
the file that MonthlyHeaderList is stored in as a Mathematica
expression."

MonthlyAddressList::usage = "MonthlyAddressList is the name of the
list that the MonthlyAddressFile is stored in."

MonthlyAddressListFile::usage = "MonthlyAddressListFile is the name of
the file that MonthlyAddressList is stored in as a Mathematica
expression."

(***************** field names ********************)
MonthlyCrossFields = {JDATE, CRSPID, PRIC1R, PRIC2R, ACCINT, PDINT,
		ITAX, IFLWR, YTM, RETADJ, DURATN, IOUT1R, IOUT2R}

MonthlyHeaderFields = {CRSPID, CUSIP, NAME, IDTMAT, ITYPE, COUPRT,
		IUNIQ, MSTART, MFINIS, IWHY, IDTDTD, IDBNK, IDTCP, IYMCN, NOTICE,
		ITAX, IFLWR, NIPPY, IDTFC, VALFC, IDTEX1, VALEX1, IDTEX2,
		VALEX2}
MergedFields = Join[Drop[MonthlyHeaderFields, -4],
	Delete[MonthlyCrossFields, {{2}, {7}, {8}}]]

JDATE::usage = "JDATE is the quotation Date (YYMMDD)"
CRSPID::usage = "CRSPID is the CRSP identification number."
PRIC1R::usage = "PRIC1R is the month-end bid price where available"
PRIC2R::usage = "PRIC2R is the month-end ask price where available"
ACCINT::usage = "ACCINT is the accrued interest as of month end"
PDINT::usage = "PDINT is the interest payable during month"
YTM::usage = "YTM is the annualized yield to maturity"
RETADJ::usage = "RETADJ is the one-month holding period return"
DURATN::usage = "DURATN is the Duration"
IOUT1R::usage = "IOUT1R is the face value outstanding"
IOUT2R::usage = "IOUT2R is the face value publicly held"

CUSIP::usage = "CUSIP is the CUSIP number."

NAME::usage = "NAME is the name of the government security."

IDTMAT::usage = "IDTMAT is the maturity date at the time of issuance."

ITYPE::usage = "ITYPE is the type of issue (1 = noncallable bond; 2 =
noncallable note; 3 = certificate of indebtedness; 4 = Treasury bill;
5 = callable bond; 6 = callable note; 7 = tax anticipation certificate
of indebtedness; 8 = tax anticipation bill; 9 = unusual provisions)."

COUPRT::usage = "COUPRT is the coupon rate."

IUNIQ::usage = "IUNIQ is the uniqueness number."

MSTART::usage = "MSTART is the month of first price"

MFINIS::usage = "MFINIS is the month of last price"

IWHY::usage = "IWHY indicates the reason for the end of data on the file."

IDTDTD::usage = "IDTDTD is the dated date (issue date) (0 = not
available or not applicable)."

IDTCP::usage = "FCALDT is the first eligible call date at time of
issue, in YYYYMMDD format."

IYMCN::usage = "IYMCN is the year and month of the first call notice,
in YYYYMM format."

NOTICE::usage = "NOTICE is the notice required on callable issues."

ITAX::usage = "ITAX indicates the taxability of interest (1 = fully
taxable; 2 = partially tax exempt; 3 = wholly tax exempt)."

IFLWR::usage = "IFLWR indicates estate tax treatment (1 = no special
status; 2 or 3 = flower bond)."

NIPPY::usage = "NIPPY is the number of interest payments per year (0 =
bill or certificate paying interest only at maturity)."

IDTFC::usage = "IDTFC is the first coupon date, in YYYYMMDD format."

VALFC::usage = "VALFC is the amount of the first coupon per $100 of
face value."

IDTEX1::usage = "Date of date of first exchange offer. (Not in use.)"
VALEX1::usage = "Value at date of first exchange offer. (Not in use.)"
IDTEX2::usage = "Date of date of second exchange offer. (Not in use.)"
VALEX2::usage = "Value at date of second exchange offer. (Not in use.)"

Begin["`Private`"]

(****************************************************************)
CRSPContext = "CRSPMonthlyBonds`"

CRSPMonthlyDirectory = "/tr/datasets/crsp/bondfile"
MonthlyHeaderListFile := CRSPMonthlyDirectory <> "/" <> "mheader.m"
MonthlyAddressListFile := CRSPMonthlyDirectory <> "/" <> "maddress.m"

MonthlyMasterFile := CRSPMonthlyDirectory <> "/" <> "mbm.dat"
MonthlyCrossFile := CRSPMonthlyDirectory <> "/" <> "mbx.dat"
MonthlyCalendarFile := CRSPMonthlyDirectory <> "/" <> "mbi.dat"

(************ HeaderList and AddressList stuff ****************)
$MonthlyHeaderListFile = True
$MonthlyAddressListFile = True

ReadMonthlyHeaderFile[] := 
	Module[{found, string, stream, temp},
	found = StringDrop[#, 1]& /@ 
		FindList[MonthlyMasterFile, "1", AnchoredSearch -> True];
	string = StringJoin[Transpose[{found, Table["\n", {Length[found]}]}]];
	stream = StringToStream[string];
	temp = ReadList[stream, Join[{Word, Word, Word}, Table[Number, {21}]]];
	Close[stream];
	temp]

ReadMonthlyAddressFile[] :=
	Module[{index, subindex, datekey, numobs, startpos},
	index = ReadList[MonthlyCalendarFile, Word, RecordLists -> True];
	subindex = DeleteCases[index, {__, "0"}];
(* {JAHRMO, NUMDAT} *)
	{datekey, numobs} = Transpose[#[[{4, -1}]]& /@ subindex];
	numobs = ToExpression[numobs];
	startpos = Drop[FoldList[Plus, 0, numobs], -1] * 
		RecordLength[MonthlyCrossFile];
	Transpose[{datekey, startpos, numobs}]]

(* load MonthlyHeaderList if not already loaded *)
LoadMonthlyHeaderList[] :=	If[ValueQ[MonthlyHeaderList]//Not,
	Print["Loading MonthlyHeaderList... Please wait."]; 
	MonthlyHeaderList = 
		If[$MonthlyHeaderListFile, Get[MonthlyHeaderListFile], 
		ReadMonthlyHeaderFile[]]]

(* load MonthlyAddressList if not already loaded *)
LoadMonthlyAddressList[] :=	If[ValueQ[MonthlyAddressList]//Not,
	Print["Loading MonthlyAddressList... Please wait."]; 
	MonthlyAddressList = 
		If[$MonthlyAddressListFile, Get[MonthlyAddressListFile], 
		ReadMonthlyAddressFile[]]]

(* to create the files:
Begin["CRSPMonthlyBonds`Private`"]
Put[ReadMonthlyAddressFile[], MonthlyAddressListFile]
Put[ReadMonthlyHeaderFile[], MonthlyHeaderListFile]
End[]
*)

(*************** create maddress.m and mheader.m ************)
(* if you wish to create mheader.m and mheader.m , use this:

Begin["CRSPMonthlyBonds`Private`"]
Put[ReadMonthlyAddressFile[], MonthlyAddressListFile]
Put[ReadMonthlyHeaderFile[], MonthlyHeaderListFile]
End[]

then set 
$MonthlyHeaderListFile = True 
$MonthlyAddressListFile = True 
*)

(**************** check for header.m and mheader.m ************)
MonthlyHeaderFile::oops = "Warning: $MonthlyHeaderListFile is set to
True, but \"mheader.m\" does not exist in `1`. You may need to set
CRSPMonthlyDirectory. Temporarily setting $MonthlyHeaderListFile =
False."

MonthlyAddressFile::oops = "Warning: $MonthlyAddressListFile is set to
True, but \"maddress.m\" does not exist in `1`. You may need to set
CRSPMonthlyDirectory. Temporarily setting $MonthlyAddressListFile =
False."

(* this indicates that "mheader.m" exists in CRSPMonthlyDirectory *)
$MonthlyHeaderListFile = True

Off[FileNames::cdir]
(* check to see if it really does *)
If[	$MonthlyHeaderListFile && 
	Not[FileNames["mheader.m", CRSPMonthlyDirectory] === 
		{MonthlyHeaderListFile}],
	(* then *)
	Message[MonthlyHeaderFile::oops, CRSPMonthlyDirectory];
	$MonthlyHeaderListFile = False]

(* this indicates that "maddress.m" exists in CRSPMonthlyDirectory *)
$MonthlyAddressListFile = True

(* check to see if it really does *)
If[	$MonthlyAddressListFile && 
	Not[FileNames["maddress.m", CRSPMonthlyDirectory] === 
		{MonthlyAddressListFile}],
	(* then *)
	Message[MonthlyAddressFile::oops, CRSPMonthlyDirectory];
	$MonthlyAddressListFile = False]

On[FileNames::cdir]
(*********************************************************)

(* load MonthlyAddressList automatically *)
If[$MonthlyAddressListFile, LoadMonthlyAddressList[];]

(* load MonthlyHeaderList on demand *)
(*LoadMonthlyHeaderList[];*)


(******************* utilities ******************************)
CRSPSwap[] := 
	SwapContexts["CRSPDailyBonds`", "CRSPMonthlyBonds`"] 

SwapContexts[a_String, b_String] :=
	Module[{order, neworder},
	order = Flatten[Position[$ContextPath, #]& /@ {a, b}];
	neworder = Range[Length[$ContextPath]] /.
		Thread[order -> Reverse[order]];
	$ContextPath = $ContextPath[[ neworder ]] ]

(* date/string conversion utilities *)
DateToInteger[date:{_Integer, _Integer, _Integer}] :=
	Plus @@ (date * {10000, 100, 1})

SetAttributes[IntegerToDate, Listable]
IntegerToDate[int_Integer] :=
	MapThread[Quotient, 
	{FoldList[Mod, int, {10000, 100}], {10000, 100, 1}}]

(* for plot label *)
DateToString[date:{_, _, _}] := 
	With[{temp = ToString /@ date},
	temp[[2]] <> "/" <> temp[[3]] <> "/" <> temp[[1]] ]

(* get "record lengths" *)
RecordLength[file_String] := 
	RecordLength[file] = (* for quick recall *)
	(StringLength[First @ ReadList[file, Record, 1]] + 1)

(* get number of fields *)
NumberOfFields[file_String] := 
	NumberOfFields[file] = (* for quick recall *)
	Module[{stream, temp},
		stream = StringToStream[First @ ReadList[file, Record, 1]];
		temp = ReadList[stream, Word, 
				WordSeparators -> CRSPFieldSeparators];
		Close[stream];
		Length[temp]]

FieldInformation[list:{__Symbol}] := (Information /@ list;)
FieldInformation[] := FieldInformation[MergedFields]


(******************* MergeDateData, etc ************************)
MergeDateData::notsame = "Warning: The number of price observations is
different from the number of yield observations."

MergeDateData[date:{_Integer, _Integer}] := 
	Module[{quotes, ids, headers},
	quotes = DateQuotes[date];
	If[quotes === $Failed, Return[$Failed]];
	ids = #[[2]]& /@ quotes;
	LoadMonthlyHeaderList[];
	headers = Cases[MonthlyHeaderList, {Alternatives @@ ids, __}];
	MapThread[Join, {Drop[#, -4]& /@ Sort[headers], 
		Delete[#, {{6}, {7}}]& /@
			RotateRight /@ (Rest /@ Sort[RotateLeft /@ quotes])}]
	]

SelectFields[data_?MatrixQ, field_Symbol] := SelectFields[data, {field}]

SelectFields[data_?MatrixQ, fields_List] := 
	Module[{pred = Function @@ {MergedFields, fields}},
		Apply[pred, data, {1}]]

SelectRows[data_?MatrixQ, cond_] := 
	Module[{pred = Function @@ {MergedFields, cond}},
		Select[data, Apply[pred, #]&]]
(******************************************************)

(******************** DateQuotes ************************)
(* DateQuotes searchs MonthlyAddressList for info to give to
		FindDateQuotes *)

DateQuotes[date:{_Integer, _Integer}] :=
	FindDateQuotes[
		Cases[MonthlyAddressList,
		{ToString[{100, 1} . date], _, _}][[1]]
		]

FindDateQuotes[{date_String, startpos_Integer, numobs_Integer}] :=
	Module[{stream, temp, form},
	stream = OpenRead[MonthlyCrossFile];
	SetStreamPosition[stream, startpos];
	form = Join[Table[Word, {2}], Table[Number, {11}]];
	temp = Table[Read[stream, form], {numobs}];
	Close[stream];
	temp]


(************************* MassageCRSPData ****************************)
MassageCRSPData::nobonds = "There are no securities that satisfy the
selection criteria `1` on `2`."

Options[MassageCRSPData] = Sort @ {Bills -> True, NotesAndBonds -> True,
MinimumMaturityCutoff -> 0, MaximumMaturityCutoff -> 36525,
PriceFunction -> (#1&), SelectionCondition -> Automatic,
Callable -> False}

MassageCRSPData[date:{yr_Integer, mo_Integer}, opts___Rule] :=
	Module[{int, merged, cond, selected,
		dateplus, minmat, maxmat, bills, notesbonds, pfcn,
		id, mat, iss, coup, bid, ask, day},
	{bills, notesbonds, minmat, maxmat, pfcn, cond, call} = {Bills, 
		NotesAndBonds, MinimumMaturityCutoff, MaximumMaturityCutoff, 
		PriceFunction, SelectionCondition, Callable} /. {opts} /. 
				Options[MassageCRSPData];
(* select securities according to condition *)
	merged = MergeDateData[date];
	If[merged == $Failed, Return[merged]];
	day = ToExpression[StringTake[merged[[1, 21]], -2]];
	{minmat, maxmat} = DateToInteger[DaysPlus[{yr, mo, day}, Round[#]]]& /@ 
		{minmat, maxmat};
	int = DateToInteger[{yr, mo, day}]; (* integer representation *)
	If[cond === Automatic,
	cond = And[
		Or[	If[notesbonds, Or @@ Thread[NAME == {"NOTE", "BOND"}], False],
			If[bills, NAME == "BILL", False]], (* name: notes and bonds *)
		If[!call, IDTCP == 0, True], (* callable? *)
		IDTMAT >= minmat,		(* maturity date => quote date plus min *)
		IDTMAT <= maxmat,		(* maturity date <= quote date plus max *)
		IDTDTD <= int,			(* issue date <= quote date *)
		IDTDTD >= 0,			(* issued after date 0 *)
		ITAX == 1,				(* no special tax treatment *)
		IFLWR == 1,				(* no flower bonds *)
		PRIC1R > 0,				(* positive bid price *)
		PRIC2R > 0				(* positive ask price *)
		]];
	selected = SelectRows[merged, cond];
	If[selected === {}, 
		Message[MassageCRSPData::nobonds, cond, date];
		Return[$Failed]];
(* calculate various things *)
	{id, mat, iss, coup, bid, ask} = 
			Transpose[selected][[{ 1, 4, 11, 6, 22, 23 }]];
	mat = IntegerToDate[mat];
	iss = IntegerToDate[iss];
	price = MapThread[pfcn, {bid, ask}];
	term = 	(Apply[Plus, {1., 1/12., 1/365.25} * # & /@
		(Subtract @@ {mat, iss}), {1}] //Round) /. 0 -> .5;
(* construct and return an MD object *)
	YieldCurve`MD[	{yr, mo, day}, mat, term, coup, price, id,
		{"CRSP Monthly U.S. Government Bond File"}]
	]


(****************** CRSPMonthlySummary ********************)
Options[CRSPMonthlySummary] = {PlotYields -> True}

CRSPMonthlySummary[date:{_Integer, _Integer}, opts___Rule] :=
	Module[{temp = DateQuotes[date]},
		If[temp === $Failed, temp, CRSPMonthlySummary[temp, opts]]]

CRSPMonthlySummary[data_?MatrixQ, opts___Rule] :=
	Module[{date, numobs, sources, ids, bids, selected, name, nameunion, 
		namecount, type, other, bills, notes, bonds, flower, tax, call, 
		maxmat, plot, ylds, colors, deleted, mat, calldate},
	numobs = Length[data];
	{ids, bids, tax, flower, ylds} = Transpose[data][[{ 2, 3, 7, 8, 9 }]];
	LoadMonthlyHeaderList[];
	selected = Cases[MonthlyHeaderList, {Alternatives @@ ids, __}];
	{name, mat, type, iss, calldate} = 
		Transpose[selected][[{ 3, 4, 5, 11, 13 }]];
	nameunion = Union[name];
	namecount = Count[name, #]& /@ nameunion;
	call   = numobs - Count[calldate, 0];
	other  = Count[type, 3 | 7 | 8 | 9];
	tax    = Count[tax, 2 | 3];
	flower = Count[flower, 2 | 3];
	maxmat = DaysBetween[IntegerToDate[Max[mat]], date];
(* shall we plot? *)
	intdate = ToExpression[data[[1,1]]] + 19000000;
	date = IntegerToDate[intdate];
	plot = If[PlotYields /. {opts} /. Options[CRSPMonthlySummary], 
		(* then plot the yields *)
	(* use calldate if callable is priced at a premium *)
		mat = MapThread[If[#1 > 0 && #2 > 100, #1, #3]&,
			{calldate, bids, mat}];
		mat = If[# < intdate, intdate, #]& /@ mat;
		lastdate = DateToInteger[{2035, 12, 31}]; (* last date to use *)
		mat = If[# > lastdate, lastdate, #]& /@ mat;
		mat = Apply[Plus, {1., 1/12., 1/365.25} * # & /@ IntegerToDate[mat], 
				{1}] - Plus @@ ({1., 1/12., 1/365.25} * date);
		(* ((DayCount /@ IntegerToDate[mat]) - DayCount[date])/365.25;*)
	(* bills: green, coupons: blue, callable: brown, flower, etc: red *)
		colors = Which[
			(Or @@ Thread[#[[5]] == {3, 7, 8, 9}]) || (* other *)
				(Or @@ Thread[#[[14]] == {2, 3}]) || (* tax *)
				(Or @@ Thread[#[[15]] == {2, 3}]), Red, (* flower *)
			(Or @@ Thread[#[[5]] == {5, 6}]), Brown, (* callable *)
			#[[5]] == 4, Green, (* bills *)
			True, Blue (* notes, bonds w/ no special features *)
				]& /@ selected;
	(* delete missing values (big negative yld) or consol (big mat) *)
		deleted = Transpose @ DeleteCases[
			Transpose @ {colors, mat, ylds}, 
				{_, _, _?Negative}|{_, _?(#>80&), _}];
		Show[Graphics[MapThread[{#1, Point[{#2, #3}]}&, deleted]],
			PlotRange -> All, Axes -> False, 
				Frame -> {True, True, False, False},
				FrameLabel -> {"years", "percent"},
				PlotLabel -> TableForm[{DateToString[date],
			"bills: green, coupons: blue, callable: brown, flower, etc: red"},
						TableAlignments -> Center,
						TableSpacing -> {0}]],
		{}, {}]; (* else *)
(* return summary info *)
	{	Date -> date,
		Observations -> numobs, (*Sources -> sources,*)
		Names -> Transpose[{nameunion, namecount}], 
		Callable -> call, FlowerBonds -> flower, 
		TaxExemptInterest -> tax, OtherSpecial -> other,
		LongestMaturity -> N[maxmat/365.25], Plot -> plot}
]

End[]
EndPackage[]

If[MemberQ[$ContextPath, "CRSPDailyBonds`"],
	On[General::shdw]]


