(* :Title: Page Layout *)

(* :Author: Mark Fisher *)

(* :Summary:
This package allows the placement of text and graphics on a page using the
coordinates of the page itself.  For example, the coordinates for a
letter-sized page in portrait orientation run from {0, 0} in the lower
left-hand corner to {8.5, 11} in the upper right-hand corner.
*)

(* :Context: PageLayout` *)

(* :Mathematica Version: 2.2 *)

(* :History: Version 1.0 by Mark Fisher, June 1993. *)

(* :Keywords: graphics, printing, page layout *)

(* :Warning: This package turns off the warning message Show::shx. *)

(* :Limitation:
PSPrintPage is based on PSPrint, which uses psfix to set the page margins to
zero and to change the orientation to landscape on request. Modifications to
the Display call would be necessary to make PSPrintPage run on platforms
where psfix is not available.
*)

(* :Discussion:
PageLayout defines a Rectangle that has the dimensions of the page, which
allows the natural use of coordinates to place text and graphics objects
using PlaceText and PlacePlot. The options AspectRatio -> Automatic and
PlotRange -> {{0, pgW}, {0, pgH}} enforce the use of the entire page.  Any
part of a graphic that is outside the page Rectangle will not be printed.
Note, pgW and pgH refer to the width and height of the page. These are
set in PageLayout.
*)

BeginPackage["PageLayout`", "Utilities`FilterOptions`"]

PageLayout::usage = "PageLayout[primitives] displays Text, Rectangle, and
other primitives on a page. Typically, one would use PlaceText and PlacePlot
to create and position the primitives. A second argument specifies the page
size, the default value of which is {8.5, 11}.  Landscape orientation can be
specified with {11, 8.5}. PageLayout takes the option ShowPageBorders."

ShowPageBorders::usage = "ShowPageBorders is an option for PageLayout that
specifies whether the page borders should be shown. The default is
ShowPageBorders -> True. The page borders will show on screen but will not
be printed because they fall in the nonprinting zone. Note: On the Colorolcs
CP 4007 color printer (gcpr2), however, the registration is slightly off and
ShowPageBorders -> False should be used."

PSPrintPage::usage = "PSPrintPage[page] prints the output from PageLayout. A
second optional argument may be given to explicitly choose a printer, for
example -Pgcpr2. A third optional argument specifies the number of copies
to print."

PageToFile::usage = "PageToFile[page, \"file\"] sends the output from
PageLayout to a file."

PlacePlot::usage = "PlacePlot[plot, {bm, lm, tm, rm}] places a plot on the
page according to the margins specified, starting at the bottom and continuing
clockwise. PlacePlot[plot, {x, y}, {w, h}] places a plot on the page,
centering it at {x, y} and giving it dimensions {w, h}. PlacePlot takes the
option ShowPlotBorders."

ShowPlotBorders::usage = "ShowPlotBorders is an option for PlacePlot that
specifies whether to show the borders around the plot. The default is
ShowPlotBorders -> False."

PlaceText::usage = "PlaceText[text, {x, y}] places text at the position
specified. PlaceText[text, tmarg] centers the text tmarg inches from the top
of the page. PlaceText takes the option TextFont."

TextFont::usage = "TextFont is an option for PlaceText that specifies the
font to use. The default is TextFont -> {\"Courier\", 10}."

Begin["`Private`"]

Off[Show::shx]

PSPrintPage[
	page:Graphics[{Rectangle[{0, 0}, {w_?NumberQ, h_?NumberQ}, {}], ___}, _],
 	ptr_:"", copies_Integer:1] := 
	With[{land = If[w > h, "-land", ""]},
	Display[ToString[StringForm[
		"!psfix -rmarg 0 -lmarg 0 -tmarg 0 -bmarg 0
		-width `1` -height `2` `3` | lpr `4` -#`5`", 
		w, h, land, ptr, copies]],
	page ]
	]

PageToFile[
	page:Graphics[{Rectangle[{0, 0}, {w_?NumberQ, h_?NumberQ}, {}], ___}, _],
 	filename_String] := 
	With[{land = If[w > h, "-land", ""]},
	Display[ToString[StringForm[
		"!psfix -rmarg 0 -lmarg 0 -tmarg 0 -bmarg 0
		-width `1` -height `2` `3` > `4`", 
		w, h, land, filename]],
	page ]
	]

Options[PageLayout] = {ShowPageBorders -> True}

PageLayout[grObj__, pgsize:{w_?NumberQ, h_?NumberQ}:({8.5, 11}), 
		opts___Rule] := 
	With[{graphopts = FilterOptions[Graphics, opts]},
	Block[{pgFrame, pgBorders, pgW = w, pgH = h},
	pgFrame = Rectangle[{0, 0}, {pgW, pgH}, {}];
	pgBorders = If[ShowPageBorders /. {opts} /. Options[PageLayout], 
		{Thickness[0.001], Line[{{0, 0}, {0, pgH}, 
			{pgW, pgH}, {pgW, 0}, {0, 0}}]}, {}];
	Show[Graphics[{pgFrame, pgBorders, grObj}], 
		AspectRatio -> Automatic, PlotRange -> {{0, pgW}, {0, pgH}},
		graphopts ]
		]]

Options[PlacePlot]  = {ShowPlotBorders -> False}

PlacePlot[grObj_, {bm_, lm_, tm_, rm_}, opts___Rule] :=
	With[{pgW = pgW, pgH = pgH},
	{Rectangle[{lm, bm}, {pgW - rm, pgH - tm}, grObj],
	If[ShowPlotBorders /. {opts} /. Options[PlacePlot],
		{Thickness[0.001],
		Line[{{lm, bm}, {lm, pgH - tm}, {pgW - rm, pgH - tm}, 
		{pgW - rm, bm}, {lm, bm}}]}, 
		{}] }]

PlacePlot[grObj_, pos:{x_, y_}, size:{w_, h_}, opts___Rule] :=
	{Rectangle[pos - size/2, pos + size/2, grObj],
	If[ShowPlotBorders /. {opts} /. Options[PlacePlot],
		{Thickness[0.001],
		Line[{pos - size/2, {x - w/2, y + h/2},
		pos + size/2, {x + w/2, y - h/2}, pos - size/2}]},
		{}] }

Options[PlaceText]  = {TextFont -> {"Courier", 10}}

PlaceText[txt_String, pos:{x_, y_}, opts___Rule] := 
	With[{font = TextFont /. {opts} /. Options[PlaceText]},
	Text[FontForm[txt, font], pos]]

PlaceText[txt_String, tmarg_, opts___Rule] :=
	With[{font = TextFont /. {opts} /. Options[PlaceText],
	pgW = pgW, pgH = pgH},
	Text[FontForm[txt, font], {pgW/2, pgH - tmarg}]]
	

End[]

Protect[PageLayout, PSPrintPage, PageToFile, PlacePlot, PlaceText]

EndPackage[]

(* :Examples:
lp3d = ListPlot3D[ Table[Sin[i + Exp[j]], {i, 5}, {j, 5}] // N,
	DisplayFunction -> Identity ]
page = PageLayout[PlacePlot[lp3d, {2, 1, 2, 1}, ShowPlotBorders -> True],
	PlaceText["This is the page title", 1, TextFont -> {"Helvetica", 15}] ]
PSPrintPage[page]


plt = Plot[Sin[x], {x, -3Pi, 3Pi}, AspectRatio -> Automatic,
	DefaultFont -> {"Courier",6}]
tab = Table[PlacePlot[plt, {(i(11/Pi) + 11)/2, 3 Sin[i] + 4.25}, {2.5, 1}], 
	{i, -Pi, Pi, Pi/4}]
sin = Plot[Sin[x], {x, -Pi, Pi}, AspectRatio -> 11/(6Pi), Axes->False, 
	PlotStyle -> {Thickness[0.001]}]
pg = PageLayout[ PlacePlot[sin,{0,0,0,0}], tab, {11, 8.5}]
PSPrintPage[pg]
*)
