(* :Title: Yield Curve *)

(* :Authors: Mark Fisher and David Zervos *)

(* :Summary:
This package fits the term structure of interest rates using cubic
splines where the effective number of parameters is determined endogenously
by generalized cross validation.
*)

(* :Context: YieldCurve` *)

(* :Mathematica Version: 2.2 *)

(* :History: Version 1.1 by Mark Fisher and David Zervos, September 1994 
				Various fixes by Mark Fisher 1995 *)

(* :Keywords: term structure, interest rates, yield curve, cubic spline,
generalized cross validation *)

(* :Warning: Setting PrintReport -> True for either DurationReport or
OpenMarketReport relies on $OperatingSystem to choose how to print. If
$OperationSystem is "Unix" the print comand is "... >> !lpr". If
$OperatingSystem is "MSDOS" then the print command is "... >> prn". These
commands may not work for all installations of those operating systems. No
other operating system is supported, though any can be added easily. *)

(* :Limitation: The functions that take objects as input and produce objects
as output (ConstructPayments, DiscountFunction, YieldCalc, and
StandardErrors) have the attribute HoldFirst, allowing the output objects to
keep track of the input objects by reference to Hold[inputname]. In order
for various functions to work properly, the input name must be a symbol
rather than "%" for example. Thus, ConstructPayments[%] will return an error
message, whereas ConstructPayments[md] will work properly assuming md is a
valid MD object.
*)

(* :References: 
Fisher, Nychka, and Zervos, "Fitting the term structure of interest
rates with smoothing splines." Working paper 95-1, Federal Reserve
Board, January 1995.

Mayle, Standard Securities Calculation Methods: Fixed Income
Securities Formulas for Price, Yield, and Accrued Interest. Vol. I,
3rd ed., Securities Industry Association, New York, 1993.
*)

(* :Requirements:
The file "daycount.m" is a "calendar" used by a number of functions in
this package.  It must be located in the Packages/FRB subdirectory along
with YieldCurve.m and the other FRB packages.

The following packages are called by this package.
Miscellaneous`Calendar` is used by MassageData and ConstructPayments.
BSplineBasis is used by DiscountFunction.
Statistics`DescriptiveStatistics` is used by DiscountFunction.
Utilities`FilterOptions` is used by a variety of functions.
PageLayout` is used by DFSummaryPage and SESummaryPage.
CommaDelimited` is used by DataOut and DateToString is used by a
variety of functions.  TriangularPlot` is used by PlotSwapSurface.
ShowTime is a modification of Utilities`ShowTime`. It need not be
loaded for this package to run properly.
*)


BeginPackage["YieldCurve`", {"ShowTime`", "PageLayout`",
"BSplineBasis`", "CommaDelimited`", "Miscellaneous`Calendar`",
"Utilities`FilterOptions`", "Statistics`DescriptiveStatistics`",
"TriangularPlot3D`", "Graphics`Colors`"}]

(********** MassageData **********)
MassageData::usage = "MassageData[OpenMarket] takes the ASCII file
\"openmkt.dat\" returns an MD object that can be used as the argument
in ConstructPayments.  MassageData[Treasury] does the same thing for
the ASCII file named \"treasury.dat\". In each case, a different file
name can be specified by MassageData[OpenMarket|Treasury,
\"file\"]. MassageData takes the option Description."

ReadData::usage = "ReadData[file] reads the data from the file and
returns an MD object. ReadData takes the option Description."

(* options *) Description::usage = "Description is an option for
MassageData and ReadData, which specifies a list of strings and/or
symbols to be associated with the MD object that is produced. The
default is Description -> {}."

QuoteToPrice::usage = "QuoteToPrice is an option for MassageData which
specifies whether to adjust the quotes in the flat file. Bills are quoted
as banker's discounts; notes and bonds are quoted in 32nds. The default is
QuoteToPrice -> True. QuoteToPrice[quote, maturitydate, couponrate,
settlementdate] returns the price given the quote and other information."

(* argument tokens *) 
OpenMarket::usage = "OpenMarket is an argument for MassageData that
indicates how the input file is structured."

Treasury::usage = "Treasury is an argument for MassageData that indicates
how the input file is structured."

Strips::usage = "Strips is an an argument for MassageData that
indicates how the input file is structured."

SAS::usage = "SAS is an argument for MassageData that indicates how the
input file is structured."

Working::usage = "Working is an argument for MassageData that
indicates how the input file is structured."

(* MD objects and selectors *) 
MD::usage = "MD is the data object produced by MassageData. MDSelectors is a
list of selectors for MD objects."

MDSelectors::usage = "MDSelectors is a list of selectors that can be
applied to an MD object."

QuoteDate::usage = "QuoteDate[md|cp|df|yc|se] returns the quote date."

SettlementDate::usage = "SettlementDate[md|cp|df|yc|se] extracts the
settlement date. The settlement date is determined by the variable
BusinessDaysToSettlement."

BusinessDaysToSettlement::usage = "BusinessDaysToSettlement is the
number of business days until settlement. The default setting is
BusinessDaysToSettlement = 1; i.e., \"regular\" delivery. For \"cash\"
delivery set BusinessDaysToSettlement = 0; for \"skip-day\" delivery
set BusinessDaysToSettlement = 2."

NumberOfSecurities::usage = "NumberOfSecurities[md|cp|df|yc|se] returns
the total number of securities in the MD object."

MaturityDate::usage = "MaturityDate[md|cp|df|yc|se] extracts the maturity
date from an object."

Term::usage = "Term[md|cp|df|yc|se] extracts the term-to-maturity from an
object."

CouponRate::usage = "CouponRate[md|cp|df|yc|se] extracts the coupon
rate from an object."

Price::usage = "Price[md|cp|df|yc|se] extracts the price from an object.
Price[accdays, next, daysinper, c/yr, yld, c, npmts] calculates the price
given the number of accrued days, the number of days until the next payment,
the number of days in the coupon period, the number of coupon payments per
year, the yield, the coupon rate (as a decimal), and the number of coupon
payments remaining. Price[couprate, matdate, quotedate, yld] calculates the
price given the coupon rate, maturity date, quote date, yield of a bond."

IDNumber::usage = "IDNumber[md|cp|df|yc|se] extracts the IDNumber from an object."
(*----------------------------*)

MDSubset::usage = "MDSubset[md, poslist] creates an MD object that contains
a subset of bonds from another MD object according to the list of positions
given."

(********** ConstructPayments **********)
ConstructPayments::usage = "ConstructPayments[md] takes an MD object and
returns a CP object.  Run it prior to DiscountFunction. ConstructPayments
takes options PaymentsPerYear and FullPrice."

(* options *) 
PaymentsPerYear::usage = "PaymentsPerYear is an option for
ConstructPayments, which specifies the number of payments per year for
coupon bonds. The default is PaymentsPerYear -> 2. PaymentsPerYear is
also a CPSelector that returns the number of payments per year."

FullPrice::usage = "FullPrice is an option for ConstructPayments, which
specifies whether accrued interest is already included in the price. The
default is FullPrice -> False."

(* CP objects and selectors *)
CP::usage = "CP is the data object returned by ConstructPayments."

CPSelectors::usage = "CPSelectors is a list of selectors that can be
	applied to an CP object."

DaysInPeriod::usage = "DaysInPeriod[cp|df|yc|se] extacts the number of days
in the current coupon period."

FullPriceQ::usage = "FullPriceQ[cp|df|yc|se] returns True if
Price[cp|df|yc|se] includes accrued interest and False if it does
not."

Payments::usage = "Payments[cp|df|yc|se] extracts the payments from an
object."

DaysToPayments::usage = "DaysToPayments[cp|df|yc|se] extracts the days to
payments from an object."

AccruedInterest::usage = "AccruedInterest[cp|df|yc|se] extracts the accrued
interest from an object."

AccruedDays::usage = "AccruedDays[cp|df|yc|se] extracts the number of days
from the beginning of the coupon period to settlement date."

NextPayment::usage = "NextPayment[cp|df|yc|se] extracts the number of days
until the next payment from an object."

LastPayment::usage = "LastPayment[cp|df|yc|se] extracts the number of days
until the final payment from an object."

RemainingPayments::usage = "RemainingPayments[cp|df|yc|se] returns the
number of payments remaining."
(*----------------------------*)

(********** DiscountFunction **********)
DiscountFunction::usage = "DiscountFunction[cp] estimates a discount
function for a given day.  Run MassageData and ConstructPayments first to
create the CP object. The default options are Knots -> (number of
securities)/3, DropByTerm -> 0 (drop the 0 most recently issued securities
of each term), MaximumMaturity -> 30. The default FunctionalForm is
DLogDelta, which uses a nonlinear routine to estimate the forward rate
function; other functional forms are LogDelta and Delta. If DLogDelta or
LogDelta is used, DiscountFunction takes options to set the starting values
(Lambda0, Bounds, and options to tune FindMinimum) and looks for a file
named \"LStart.m\" or \"DLStart.m\" to find starting values for beta0 and
lambda0.  If the appropriate file cannot be opened, internal default values
are used."

DiscFunDelta::usage = "DiscFunDelta is called by DiscountFunction when
FunctionalForm -> Delta."

DiscFunNonLin::usage = "DiscFunNonLin is called by DiscountFunction when
FunctionalForm -> LogDelta or FunctionalForm -> DLogDelta."


(* options *)
Restriction::usage = "Restriction is an option of DiscountFunction."

Knots::usage = "Knots is an option of DiscountFunction, which is used
to specify the number of interior knots to be used. The default is
Knots -> Automatic, which is translated into the length of the
KeepList divided by three. Specific (interior) knot points can be
given in a list. For example, Knots -> {1, 2, 5, 7, 10, 20}
Years. Knot points at zero and the maximum maturity are automatically
included.  Knots[df|yc] extracts the list of knots (including the end
knots) from a DF object."

DropByTerm::usage = "DropByTerm is an option of DiscountFunction, which
specifies the how many of the most recently issued securities of each
original term-to-maturity to drop from the estimation. The default is
DropByTerm -> 0. Alternatively, one may specify the number of each term
to be dropped individually, as in DropByTerm -> {2, 0, 1, 2, 3, 1}, 
assuming there were 6 different maturities."

CountByTerm::usage = "CountByTerm[md] returns a table of the number of
securities of each original term."

IDNumberDropList::usage = "IDNumberDropList is an option for DiscountFunction,
which specifies (by IDNumber) which securities are to be omitted from the
estimation. The default is IDNumberDropList -> {}."

TuningParameter::usage = "TuningParameter is an option of
DiscountFunction that specifies the marginal tradeoff between the sum
of squared deviations and the penalty when the GCV function is
minimized. This is sometimes refered to as the cost. The default is
TuningParameter -> 2. TuningParameter is also an option for
GCVFunction. The default setting is TuningParameter -> Automatic,
which uses the value of TuningParameter from the previous use of
DiscountFunction. TuningParameter[df|yc|se] returns the value of the
tuning parameter used in the estimation."

FunctionalForm::usage = "FunctionalForm is an option for DiscountFunction,
which specifies the functional form to spline. Valid functional forms are
Delta (spline the discount function), LogDelta (spline (the negative of) the
log of the discount function, and DLogDelta (spline (the negative of) the
derivative of the log of the discount function; i.e., spline the forward
curve). The default is Method -> DLogDelta. FunctionalForm[df|yc|se] is a
DFSelector that extracts the FunctionalForm that was used."

LogDelta::usage = "LogDelta is a FunctionalForm for DiscountFunction."

DLogDelta::usage = "DLogDelta is a FunctionalForm for DiscountFunction."

Lambda0::usage = "Lambda0 is an option for DiscountFunction, which
specifies the starting value for lambda, the penalty. It may be either
Automatic or a list of length two, for example, {10^16, 10^16}.
The default is Lambda0 -> Automatic. The starting values determined by
Lambda0 -> Automatic depend on the FunctionalForm. If FunctionalForm
-> Delta, the starting values are {10.^10, 10.^11}. If FunctionalForm
-> LogDelta or DLogDelta, the starting values are determined by the
value of lambdastar in the file \"LStart.m\" or \"DLStart.m\", or, if
that value is unavailable, the starting values are {1.5 10^10, 1.8
10^10}."

LambdaValue::usage = "LambdaValue is an option for DiscountFunction, that
specifies how lambda, the weight given to the penalty, is to be chosen. The
default is LambdaValue -> Automatic, which specifies that generalized cross
validation should be used to determine the value of lambda. Alternatively,
the user may supply a fixed value for lambda. LambdaValue -> 0 would result
in no penalty at all."

Bounds::usage = "Bounds is an option for DiscountFunction. It puts bounds on
the range that FindMinimum uses. The default is Bounds -> {0, 10^30}.  The
user may be able to speed up convergence with a narrower choice of bounds.
However, choosing bounds too narrow may cause DiscountFunction to bomb."

MaximumMaturity::usage = "MaximumMaturity is an option for DiscountFunction
and DiscountFunction that sets the upper bound for the range to be splined
and for the interpolation function that is stores the final spline. The
default is MaximumMaturity -> 30, signifying 30 years."

ObservationWeights::usage = "ObservationWeights is an option for
DiscountFunction that will determine the weight each observation gets
in the least squares component. The default is ObservationWeights ->
Automatic which weights each observation equally.
ObservationWeights[df|yc|se] is a DFSelector that extracts the weights
used in the estimation."

ShowLambdaProgress::usage = "ShowLambdaProgress is an option for
DiscountFunction that indicates whether to print values for lambda at
each step of the FindMinimum routine. The default is
ShowLambdaProgress -> True."

SplineOrder::usage = "SplineOrder is an option for DiscountFunction
that indicates the order of the spline to use. The default is
SplineOrder -> 3, indicating a cubic spline. No penalty function is
available for SplineOrder -> 1 or 2, so generalized cross-validation
will be turned off and the penalty will be set to zero (as if
LambdaValue -> 0 were used). For SplineOrder -> 0, see PenaltyMatrix
for a description of the penalty. SplineOrder is also a DFSelector; it
returns the order of the spline that was used in the estimation."

OvernightForwardRate::usage = "OvernightForwardRate is an option for
DiscountFunction that restricts the zero-maturity forward rate when
the FunctionalForm is DLogDelta. For example, to set the zero-maturity
forward rate to 3 percent per annum (continuously compounded), use
OvernightForwardRate -> 3. The default setting is OvernightForwardRate
-> Automatic, which imposes no restricton."

BetaConvergenceTest::usage = "BetaConvergenceTest is an option for
DiscountFunction that determines when the beta coefficients have
converged in the nonlinear-least-squares fixed point problem that is
solved when the FunctionalForm is either LogDelta or DLogDelta. The
test function must be a function of two lists that returns True or
False. The default is BetaConvergenceTest -> (Max[Abs[(#2 - #1)/#2]] <
0.0001 &)."

InitialCurve::usage = "InitialCurve is an option for DiscountFunction
that sets the level of the initial (flat) yield curve to use for the
starting values for the beta coefficients when FunctionalForm is
LogDelta or DLogDelta. For example, to set the initial curve to 10
percent, use InitialCurve -> 10. The default setting is InitialCurve
-> Automatic, which uses a 5 percent curve."

(* DF objects and selectors *)
DF::usage = "DF is the data object returned by DiscountFunction."

DFSelectors::usage = "DFSelectors is a list of selectors that can be
applied to an DF object."

RestrictionQ::usage = "RestrictionQ[df|yc|se] returns True if a
restriction at maturity zero was imposed and False otherwise."

FinalSpline::usage = "FinalSpline[df|yc|se] extracts the interpolation object
from a DF object."

Delta::usage = "Delta[df|yc|se] extracts the estimated discount function
from a DF object. Delta is also a possible value for the option
FunctionalForm for DiscountFunction."

ForwardCurve::usage = "ForwardCurve[df|yc|se] extracts the estimated
continuously-compounded forward rate curve from a DF object."

ZeroCurve::usage = "ZeroCurve[df|yc|se] extracts the estimated
continuously-compounded zero-coupon curve from a DF object."

SemiAnnualForwardCurve::usage = "SemiAnnualForwardCurve[df|yc|se]
extracts the estimated semi-annually-compounded forward rate curve
from a DF object/"

SemiAnnualZeroCurve::usage = "SemiAnnualZeroCurve[df|yc|se] extracts
the estimated semi-annually-compounded zero-coupon curve from a DF
object."

ContinuousToSemiAnnual::usage = "ContinuousToSemiAnnual[function]
takes a function like ZeroCurve[df] or ForwardCurve[df] and returns a
modified function that uses semi-annual compounding. For example,
ContinuousToSemiAnnual[ZeroCurve[df]] returns a function that can be
sampled or plotted."

Lambda::usage = "Lambda[df|yc|se] extracts the value of lambda from the
current object."

FixedLambdaQ::usage = "FixedLambdaQ[df|yc|se] is a DFSelector that
returns True is lambda was fixed or False is lambda was determined by
minimizing the GCV function."

GCVMinimizedQ::usage = "GCVMinimizedQ[df|yc|se] is a DFSelector that
returns True if minimizing the generalized cross validation functon
was used to determine lambda and False otherwise."

GCVValue::usage = "GCVValue[df|yc|se] is a DFSelector that returns the
value of the generalized cross valuation function associated with the
penalty Lambda." 

EffectiveParameters::usage = "EffectiveParameters[df|yc|se] extracts the
number of effective parameters. EffectiveParameters[lambda] returns the
number of effective parameters as a function of lambda. This is defined
when FunctionalForm -> Delta has been used. Otherwise,
EffectiveParameters[] should be run without argumnets, after
GCVFunction[lambda] has been run."

MakeNonlinearSpline::usage = "MakeNonlinearSpline[] is intended to be
used after (i) DiscountFunction has been run with the FunctionalForm
set to either LogDelta or DLogDelta and (ii) GCVFunction has been run.
MakeNonlinearSpline[] returns the spline of the functional form using 
the same value of lambda that was used in GCVFunction."

PredictedPrice::usage = "PredictedPrice[df|yc|se] extracts the predicted
prices."

PriceError::usage = "PriceError[df|yc|se] extracts the difference
between fitted and actual prices."

KeepList::usage = "KeepList[df|yc|se] returns the list of positions of the
bonds that were used in the estimation."

NumberOfObservations::usage = "NumberOfObservations[df|yc|se] returns
the number of observations that were used the estimation."

ResidualVariance::usage = "ResidualVariance[df|yc|se] returns the sum of
the squared residuals divided by the number of observations minus one."

AverageAbsoluteError::usage = "AverageAbsoluteError[df|yc|se] returns
the average absolute error of the residuals."

Significance::usage = "Significance[df|yc|se] returns PriceError
divided by its standard errors."

BetaHat::usage = "BetaHat[df|yc|se] is a DFSelector that returns the vector
of estimated coefficients."

BetaHatCovarianceMatrix::usage = "BetaHatCovarianceMatrix[df|yc|se] is
a DFSelector that returns the covariance matrix of the estimated
coefficients."
(*----------------------------*)

(********** YieldCalc **********)
YieldCalc::usage = "YieldCalc[df|cp] calculates the actual and predicted
yields to maturity, the difference between the two (YieldError = Yield -
PredictedYield), duration, and convexity. YieldCalc takes a DF object as
input and returns a YC object. If YieldCalc is given a CP object, it does
not calculate PredictedYield or YieldError."

(* YC options *) 
Compounding::usage = "Compounding is an option for YieldCalc that
specifies how to handle the compounding of yields. The Default is
Compounding -> SIACompounding, which specifies the use of the
Securities Industry Association formulas. Currently this handles zeros
incorrectly. The other valid setting is Compounding ->
ContinuousCompounding, which specifies the use of continuous
compounding for bills and zeros. Compounding is also a YCSelector. It
returns the setting used for the constructions of the YC object."

SIACompounding::usage = "SIACompounding is a setting for the option
Compounding. It specifies the use of the Securities Industry
Association formulas for calculating yields."

ContinuousCompounding::usage = "ContinuousCompounding is a setting for
the option Compounding. It specifies the use of continuous compounding
for all securities for for calculating yields."

(* YC objects and selectors *)
YC::usage = "YC is the data object created by YieldCalc. YCSelectors is a
list of selectors for YC objects."

YCSelectors::usage = "YCSelectors is a list of selectors that can be
applied to an YC object."

Yield::usage = "Yield[yc] is a YCSelector that returns the yield to
maturity. Yield[accdays, next, daysinper, c/yr, p, c, npmts] calculates the
yield given the number of accrued days, the number of days until the next
payment, the number of days in the coupon period, the number of coupon
payments per year, the price, the coupon rate (as a decimal), and the number
of coupon payments remaining.  It is called by YieldCalc.  Yield[couprate,
matdate, quotedate, price] calculates the yield given the coupon rate,
maturity date, quote date, and price of a bond."

PredictedYield::usage = "PredictedYield[yc] is the predicted yield to
maturity."

YieldError::usage = "YieldError[yc] is the difference between
actual and predicted yield to maturity."

Duration::usage = "Duration[yc] is (minus) the modified duration,
-(dP/dy)(1/P), as calculated from the yield formulas. Duration[x][yc]
returns duration as a function of x, where x can be AccruedDays,
NextPayment, DaysInPeriod, PaymentsPerYear, Yield, CouponRate, or
RemainingPayments. Duration[accdays, next, daysinper, c/yr, yld, c, npmts]
calculates the duration given the number of accrued days, the number of days
until the next payment, the number of days in the coupon period, the number
of coupon payments per year, the yield, the coupon rate (as a decimal), and
the number of coupon payments remaining. Duration[couprate, matdate,
quotedate, yld] calculates the price given the coupon rate (in percent),
maturity date, quote date, and yield of a bond (in percent)."

Convexity::usage = "Convexity[yc] is (1/2)(d^2P/dy^2)(1/P).
Convexity[accdays, next, daysinper, c/yr, yld, c, npmts] calculates the
convexity given the number of accrued days, the number of days until the
next payment, the number of days in the coupon period, the number of coupon
payments per year, the yield, the coupon rate (as a decimal), and the number
of coupon payments remaining. Convexity[couprate, matdate, quotedate, yld]
calculates the convexity given the coupon rate (in percent), maturity date,
quote date, and yield of a bond (in percent)."

MacaulayDuration::usage = "MacaulayDuration[yc] is Macaulay's measure of
duration, the weighted maturity of the payments."
(*----------------------------*)

DFSummaryStatistics::usage = "DFSummary[df] produces a table of summary
statistics regarding the DF object."

DFSummaryPage::usage = "DFSummaryPage[df, title] formats a ShowGraph and a
DFSummaryStatistics on a page with an optional title. Plot options can
be passed to the graph."

Title::usage = "Title is an option for DFSummaryPage that overrides the
default title given by the Description."

SummaryTextFont::usage = "SummaryTextFont is an option for DFSummaryPage and
SESummaryPage that specifies the font to be used for the DFSummaryStatistics
or SESummaryStatistics included. The default is SummaryTextFont ->
{\"Courier\", 10}."

DataOut::usage = "DataOut[OpenMarket, yc] takes a YC object and produces an
ACSII file that can be imported by Paradox, containing the output from
YieldCalc, etc."

(********** ShowGraph **********)
ShowGraph::usage = "ShowGraph[df] displays the instantaneous rate
curves when given a DF object created by DiscountFunction. ShowGraph
take options Color and PeriodLength. In addition, Plot options can be
used."

(* options (also used by PlotByTerm) *) 
Color::usage = "Color is an option of ShowGraph and PlotByTerm, which
specifies whether color output should be used. The default is Color
-> True."

PeriodLength::usage = "PeriodLength is an option for PlotByTerm and
ShowGraph, which specifies the period to be used in plots. The default
is PeriodLength -> Years."
(*----------------------------*)

PlotByTerm::usage = "PlotByTerm[f[cp|df|yc]] plots f[cp|df|yc] vs. maturity
for an interactively chosen initial term-to-maturity. PlotByTerm[f[x], term]
plots f[x] for a given term. For example, PlotByTerm[PriceError[df], 10].
PlotByTerm takes options Color, PeriodLength, and ShowPoints."

ShowPoints::usage = "ShowPoints is an option for PlotByTerm which specifies
whether to show the points or not."

PlotAllByTerm::usage = "PlotAllByTerm[f[x]] runs PlotByTerm[f[x]] for all
terms."

PlotCombined::usage = "PlotCombined[df] combines ShowGraph and
PlotAllByTerm."

MaturityPlot::usage = "MaturityPlot[f[cp|df|yc]] plots f[cp|df|yc] versus
LastPayment. For example, MaturityPlot[Yield[yc]] plots the yield to
maturity versus LastPayment[yc]. ListPlot options can be used."

SignificanceByTerm::usage = "SignificanceByTerm[df, sigma] counts the number 
of bonds that have Significance[df] >= sigma by term-to-maturity."

(********** StandardErrors **********)
StandardErrors::usage = "StandardErrors[df] produces an SE object that
contains forward and zero-coupon rates and standard errors evaluated on
a grid of 60 points.  StandardErrors[df, gridlist] evaluates the rates
and standard errors on the gridlist given. The SE object can be used as
input for StandardErrorsPlot, SESummaryStatistics, and SESummaryPage."

(* SE objects and selectors *)
SE::usage = "SE is the data object created by StandardErrors. SESelectors
is a list of selectors for SE objects."

SESelectors::usage = "SESelectors is a list of selectors that can be
applied to an SE object."

StandardErrorsGrid::usage = "StandardErrorsGrid[se] is an SESelector that
returns the grid used to produce the standard errors."

ForwardRates::usage = "ForwardRates[se] is an SESelector that returns
the forward rates calculated on the grid."

ForwardRateErrors::usage = "ForwardRateErrors[se] is an SESelector that
returns the forward rate standard errors calculated on the grid."

ZeroCouponRates::usage = "ZeroCouponRates[se] is an SESelector that returns
the zero-coupon rates calculated on the grid."

ZeroCouponRateErrors::usage = "ZeroCouponRateErrors[se] is an SESelector
that returns the zero-coupon rate standard errors calculated on the grid."

StandardErrorsPlot::usage = "StandardErrorsPlot[se] plots the forward
and zero standard errors. StandardErrorsPlot takes the option PlotRates."

SESummaryStatistics::usage = "SESummaryStatistics[se] produces a table of
summary statistics regarding forward and zero standard errors."

SESummaryPage::usage = "SESummaryPage[se] combines StandardErrorsPlot
and SESummaryStatistics"

(*----------------------------*)

WhenceMD::usage = "WhenceMD[md|cp|df|yc|se] returns the MD object that
was used in the creation of the current object."

WhenceCP::usage = "WhenceCP[cp|df|yc|se] returns the CP object that
was used in the creation of the current object."

WhenceDF::usage = "WhenceDF[df|yc|se] returns the DF object that was
used in the creation of the current object."

WhenceYC::usage = "WhenceYC[yc|se] returns the YC object that was used
in the creation of the current object."

PlotRates3D::usage = "PlotRates3D[dfseries, Zero|Forward] produces
a 3-D plot of either zero-coupon or forward rates from a time series
of DF objects. The latter need not be in any specific order, however,
there may only be only DF object per date. PlotRates3D take options
PointsPerDay and InteriorDateTicks."

PointsPerDay::usage = "PointsPerDay is an option for PlotRates3D, which
specifies the number of points to sample per day. The default is
PointsPerDay -> 20."

InteriorDateTicks::usage = "InteriorDateTicks is an option for PlotRates3D,
which specifies the number of tick points to use for the date axis. The
default is InteriorDateTicks -> Automatic."

PlotRates::usage = "PlotRates is an option for ShowGraph, StandardErrorsPlot,
and PlotRates3D that specifies whether to show Zero, Forward, or Both.
The default is PlotRates -> Both."

Zero::usage = "Zero represents an option value for ShowGraph,
StandardErrorsPlot, and PlotRates3D that signifies which rate to plot."

Forward::usage = "Forward represents is an option value for ShowGraph,
StandardErrorsPlot and PlotRates3D that signifies which rate to plot."

Both::usage = "Both represents an option value for ShowGraph and
StandardErrorsPlot that signifies both forward and zero-coupon rate curves
should be drawn."

AssociateName::usage = "AssociateName[name, obj, pos] associates a name
with an object and a position.  For example, AssociateName[new30yr, yc, -1]
allows one to refer to the last bond as new30yr.  Thus, Price[new30yr]
would correctly return the price."

DurationReport::usage = "DurationReport[yc] produces a report of Macaulay's
duration, modified duration, and convexity for each bond. DurationReport
takes the options ShowReport and PrintReport."

OpenMarketReport::usage = "OpenMarketReport[yc] produces a report of the
price error and its significance and the yield, predicted yield, and yield
error for each bond.  OpenMarketReport takes options ShowReport and
PrintReport."

ShowReport::usage = "ShowReport is an option for DurationReport and
OpenMarketReport that specifies whether to show the report on screen. The
default is ShowReport -> True."

PrintReport::usage = "PrintReport is an option for DurationReport and
OpenMarketReport that specifies whether to show the report on screen. The
default is ShowReport -> False."

BreushPagan::usage = "BreushPagan[df, criteria] takes a DF object and a
sequence of one or more criteria. BreushPagan calculates the chi-square
statistic associated with the null hypothesis that the error variance is not
linearly related to the criteria. The function returns a chi-square
statistic and the associated degrees of freedom. Usage example:
BreushPagan[df, Duration[yc]]."

(* useful values *)
Days::usage = "Days equals 1, the number of days in a day." 

Weeks::usage = "Weeks equals 7, the number of days in a week."

Months::usage = "Months equals (365 + 1/4)/12, the average number of
days in a month."

Years::usage = "Years equals 365 + 1/4, the average number of days in
a year."

Object::usage = "Object[name] returns the object that name was
associated with." 

DayCount::usage = "DayCount[date] returns the number of days from
{1990, 1, 1} to the first business day on or following date."
FirstBusinessDay::usage = "FirstBusinessDay[date] returns the date of
the first business date on or following date."

FirstDate::usage = "FirstDate returns the first valid date for the
calendar."

LastDate::usage = "LastDate returns the last valid date for the calendar."

DaysInMonth::usage = "DaysInMonth[{year, month}] returns the number of
days in the given month."

BusinessDayQ::usage = "BusinessDayQ[date] returns True if date is a
busniess day and False if it is not."

LastBusinessDayInMonth::usage = "LastBusinessDayInMonth[{year, month}]
returns the date of the last business day in the given month."

(* Par coupon and swap stuff *)
ParCouponYield::usage = "ParCouponYield[n, d] returns the coupon rate
on a bond that matures in n years and pays semi-annual coupons that is
priced at par according to discount function d. For efficiency,
extract the discount function before calling ParCouponYield; i.e., d =
Delta[df]; Table[ParCouponYield[i, d], {i, .5, 30, .5}]."

ParCouponYieldCurve::usage = "ParCouponYieldCurve[d] takes a discount
function and returns the par coupon yield for a series of 60 bonds
that pay semi-annual coupons and that have maturities ranging from 1/2
year to 30 years. The par coupon yield is the yield that makes the
bond be priced at par. For example, ParCouponYieldCurve[Delta[df]]."

ForwardParCouponYieldCurve::usage = "ForwardParCouponYieldCurve[d, m,
step, maxmat] takes discount function d and returns the forward par
coupon yield for a series of bonds that pay coupons at frequency step
and that have maturities ranging from m years to maxmat years. The par
coupon yield is the yield that makes the bond be priced at par. The
last two arguments have defaults of .5 and 30. For example,
ForwardParCouponYieldCurve[Delta[df], 1]."

SwapRate::usage = "SwapRate[d, m, mp] uses discount function d to
calculate the the continuous swap rate from m to mp, where mp >= m. If
m > 0, the result is the continuous forward swap rate. SwapRate[d, m,
mp, n] returns the swap rate assuming n payments per year. For
example, SwapRate[d, 0, 2 Years, 2] would return the spot swap rate
(par coupon yield) with a maturity of two years, assuming semi-annual
coupons."

PlotSwapSurface::usage = "PlotSwapSurface[d] returns a 3-dimensional
plot of the spot/forward swap surface calculated from discount
function d. PlotSwapSurface[d, n] returns the spot/forward swap
surface assuming n payments per period."

PlotParCouponYieldSurface::usage = "PlotParCouponYieldSurface[d, step,
maxmat] returns a triangular surface of spot/forward par coupon yields
compounded every step size out to maxmat. The last two arguments have
defaults of .5 and 30. For example, PlotParCouponYieldSurface[d]
returns a plot of semi-annually compounded yields out to 30 years."

GCVFunction::usage = "GCVFunction[lambda] returns the generalized
cross validation value (GCVValue) for the given value of the penalty
lambda when the functional form is DLogDelta or LogDelta. As a
side-effect (which is of central importance to DiscountFunction), it
solves a non-linear least squares problem. It can be used to plot the
GCV function, which can be valuable if you are having trouble with
convergence. GCVFunction should only be run after you have run
DiscountFunction with the settings you wish to use. GCVFunction
relies on \"global\" variables in the Private context to
work. GCVFunction takes the option TuningParameter."

PlotGCVFunction::usage = "PlotGCVFunction[list] maps GCVFunction onto
list and plots the result vesus Log[list, 10]. PlotGCVFunction[] uses
10^Range[22] as the list."
(***************************************************************************)

Begin["`Private`"]
Unprotect[MD, CP, DF, YC, SE]

Days 	= 1
Weeks 	= 7
Months 	= 487/16 (* 365.25/12 *) (* to avoid rounding problems with *)
Years 	= 1461/4 (* 365.25 *)    (* InterpolatingFunction           *)

(* swap rate stuff *)
SwapRate[d_, m_, mp_] :=
	Which[mp == m, -100 Years d'[m]/d[m],
		mp > m, 100 Years
			(d[m]-d[mp])/NIntegrate[d[u], {u, m, mp}],
		True, {}]

SwapRate[d_, m_, mp_, n_] := 
	With[{h = Years/n},
	Which[mp == m, -100 Years d'[m]/d[m],
		mp > m, 100 n
			(d[m]-d[mp])/Sum[d[u], {u, m+h, mp, h}],
		True, {}]]

PlotSwapSurface[d_] := TriangularPlot3D[SwapRate[d, m Years, mp Years],
		{mp, 0, 30}, {m, 0, 30}, PlotPoints -> 31]

PlotSwapSurface[d_, n_] := ListTriangularPlot3D[
		Reverse /@ Table[{j, i, SwapRate[d, i Years, j Years, 2]},
				{i, 29}, {j, i+1, 30}]]

(*ParCouponYield[n_, d_] := 
	Block[{days, i, c, price}, 
		days = Table[i Years, {i, n, 10^-6, -.5}];
		price = d[n Years] + c/2 Plus @@ (d /@ days);
		c /. FindRoot[
			price == 1 + c If[Mod[n, .5] == 0, .5, .5 - Mod[n, .5]], 
		{c, .05}]]*)

ParCouponYield[n_, d_] := 
	Block[{days, i, c, price}, 
		days = Table[i Years, {i, n, 10^-6, -.5}];
		price = d[n Years] + c/2 Plus @@ (d /@ days);
		c /. FindRoot[price == 1, {c, .05}]]

ParCouponYieldCurve[d_] := 
	With[{list = d /@ Table[i, {i, Years/2, 30*Years, Years/2}]}, 
			2(1 - list)/Rest[FoldList[Plus, 0, list]]]

ForwardParCouponYieldCurve[d_, m_, step_:.5, maxmat_:30] := 
	With[{list = d /@ Table[i, {i, m Years, maxmat Years, step Years}]}, 
		1/step (First[list] - Rest[list])/
				Rest[FoldList[Plus, 0, Rest[list]]]]

PlotParCouponYieldSurface[d_, step_:.5, maxmat_:30] :=
	ListTriangularPlot3D[Reverse /@ 
		Table[
			Thread[{Range[i + step, maxmat, step], 
					i, 
					ForwardParCouponYieldCurve[d, i, step, maxmat]
			}], 
		{i, 0, maxmat - step, step}]
	]

AssociateName::outarg = "`1` is not a valid argument"
AssociateName::nameval = "`1` has a value and cannot be used"
AssociateName::notobj = "`1` is not a valid object"

SetAttributes[AssociateName, HoldAll]

AssociateName[name_Symbol, obj_, pos_] :=
	Which[
	ValueQ[name], 
		Message[AssociateName::nameval, HoldForm[name]];
		Return[$Failed],
	MatchQ[Hold[obj], Hold[Out[___]]],
		Message[AssociateName::outarg, HoldForm[obj]];
		Return[$Failed],
	Not[MatchQ[Head[obj], MD | CP | YC | DF | SE]],
		Message[AssociateName::notobj, HoldForm[obj]];
		Return[$Failed],
	MatchQ[Hold[pos], Hold[Out[___]]], 
		Message[AssociateName::outarg, HoldForm[pos]];
		Return[$Failed],
	True, 	Evaluate[name] /: f_[name] := f[obj][[pos]] /;
			MemberQ[Complement[Union[YCSelectors, DFSelectors,
			CPSelectors, MDSelectors], {QuoteDate}], f];
		Evaluate[name] /: QuoteDate[name] := QuoteDate[obj];
		Evaluate[name] /: Object[name] := obj;
		name]

DiscountFunction::maxmatshort = "Notice: increasing MaximumMaturity to `1`"
DiscountFunction::maxmatlong = "Warning: MaximumMaturity is set to `1`
	but their are no payments with maturity longer than `2`"
DiscountFunction::nokeep = "There are no remaining securities: all
securities were dropped."
DiscountFunction::badopts = "Either Lambda0 or Bounds is not a list of
	length two."
DiscountFunction::noopenortooshort = "Cannot open `1` or dsave is
	too short: using internal defaults"
DiscountFunction::startval = "The information in `1` is corrupted.
	Change the file or delete it."
DiscountFunction::noopen = "Cannot open `1`. Using internal defaults."
DiscountFunction::tooshort = "The InterpolatingFunction in `1` is too
short. Using internal defaults."
DiscountFunction::badvalues = "The information in `1` is
corrupted. Using internal defaults."
DiscountFunction::badform = "FunctionalForm -> `1` is bad. FunctionalForm
	should be either Delta, LogDelta, or DLogDelta"
DiscountFunction::outarg = "`1` is not a valid argument"
DiscountFunction::notobj = "`1` is not a valid object"
DiscountFunction::baddropnum = "The length of `1` does not match the
	length of the union of terms, `2`"
DiscountFunction::badweights = "The weights specified are bad in some
	way. Equal weights will be used."
DiscountFunction::badknots = "The knot specification is must be a
number or a vector of numbers."
DiscountFunction::badorder = "The SplineOrder is bad. It must be
either 0, 1, 2, or 3."
DiscountFunction::badcurve = "Notice: The initial curve found in `1` is too
short or contaminated. Using `2` percent flat curve."
DiscountFunction::badlambda = "Notice: The value for Lambda0 found in
`1` is not valid. Using `2`."

Options[DiscountFunction] = Sort @ {TuningParameter -> 2, 
	Lambda0 -> Automatic, Bounds -> {0., 10.^30}, 
	Restriction -> True, Knots -> Automatic, 
	DropByTerm -> 0, MaximumMaturity -> 30, IDNumberDropList -> {}, 
	LambdaValue -> Automatic, FunctionalForm -> DLogDelta, 
	ObservationWeights -> Automatic, ShowLambdaProgress -> True, 
	SplineOrder -> 3, OvernightForwardRate -> Automatic, 
	BetaConvergenceTest -> (Max[Abs[(#2 - #1)/#2]] < 0.0001 &),
	InitialCurve -> Automatic}

SetAttributes[DiscountFunction, HoldFirst]

DiscountFunction[cp_, opts___Rule] :=
	Which[
	MatchQ[Hold[cp], Hold[Out[___]]],
		Message[DiscountFunction::outarg, HoldForm[cp]];
		Return[$Failed],
	Not[MatchQ[Head[cp], CP]],
		Message[DiscountFunction::notobj, HoldForm[cp]];
		Return[$Failed],
	True,
	With[{term = Term[cp], lastpayment = LastPayment[cp],
		daystopayments = DaysToPayments[cp]},
	Module[{fform, method, knotnum, dropnum, maxmatyears, termlist, 
		droplist, poslist, sorted, keeplist, maxmat, maxlastpmt,
		iddrop, weights, sqw, phimat},
	{fform, knotnum, dropnum, maxmatyears, iddrop, order} =
	{FunctionalForm, Knots, DropByTerm, MaximumMaturity, 
		IDNumberDropList, SplineOrder} /. {opts} /. Options[DiscountFunction];
(* Part I: error checking *)
	Switch[order, 0|1|2|3, Null, 
		_, Message[DiscountFunction::badorder]; Return[$Failed]];
	Switch[fform, 
		Delta | LogDelta | DLogDelta, Null,
		_, Message[DiscountFunction::badform, fform]; Return[$Failed]];
(* Part II: create keeplist *)
(* create a list of how many to drop of each term *)
	termlist = Union[term];
	droplist = If[Head[dropnum] === List, 
		If[Length[dropnum] != Length[Union[term]], 
			Message[DiscountFunction::baddropnum, 
				dropnum, Union[term]]; Return[$Failed],
			dropnum], 
			Table[ dropnum, {Length[termlist]} ]];
(* locate those to be dropped *)
	poslist = Flatten[Position[term, #]]& /@ termlist;
(* sort postlist according to lastpayment *)
	lastpaymentlist = lastpayment[[#]]& /@ poslist;
	sorted = Map[Last, Sort /@ 
		MapThread[Transpose[{##}]&, {lastpaymentlist, poslist}], {2}];
(* don't drop more than there are *)
	keeplist = Sort @ Flatten @
		MapThread[If[Length[#1] <= #2, {}, Drop[#1, -#2]]&, 
				{sorted, droplist}];
(* substract out iddrop *)
	keeplist = Complement[keeplist, 
		Flatten[Position[IDNumber[cp], Alternatives @@ iddrop]]];
	If[Length[keeplist] == 0, 
		Message[DiscountFunction::nokeep]; Return[$Failed]];
(* Part III: create phimat *)
(* knot points, with maximum maturity appended *)
	maxmat = Ceiling[Years maxmatyears]; (* 30 years == 10958 days *)
(* compare maxmat with the maturity of the longest payment *)
	maxlastpmt = Max[ lastpayment[[keeplist]] ];
	If[maxmat < maxlastpmt, maxmat = maxlastpmt;
		Message[DiscountFunction::maxmatshort, N[maxlastpmt/Years]]];
	If[maxmat > 1.1 maxlastpmt, 
		Message[DiscountFunction::maxmatlong, 
				maxmatyears, N[maxlastpmt/Years]]];
	If[knotnum == Automatic, knotnum = Floor[ Length[keeplist]/3 ] ];
	Which[NumberQ[knotnum], 
		knots = MakeKnotList[lastpayment[[keeplist]], knotnum, {0,maxmat}],
		VectorQ[knotnum, NumberQ],
		knots = Join[{0}, Union[knotnum], {maxmat}],
		True, Message[DiscountFunction::badknots]; Return[$Failed]];
(* Part IV: spline and choose method *)
(* do splining *)
	phimat = Switch[fform,
		DLogDelta, SplineMatrix[N[knots], N[daystopayments], order, -1],
		_,		   SplineMatrix[N[knots], N[daystopayments], order]];
(* choose method *)
	Switch[fform,
		Delta,	DiscFunDelta[ cp, {knots, keeplist, phimat}, opts],
		_,		DiscFunNonLin[cp, {knots, keeplist, phimat}, opts]]
	]] ]

SetAttributes[DiscFunDelta, HoldFirst]

DiscFunDelta[ cp_ /; Head[cp] == CP, 
		{knots_, keeplist_, phimat_}, 
		opts___Rule ] :=
	With[{	nobs = Length[keeplist],
		fullprice = Price[cp] + AccruedInterest[cp],
		payments = Payments[cp],
		fmopts = FilterOptions[FindMinimum, opts]},
	Module[{lambda0, bounds, fixedlambda, restrict, dropnum, weights,
			order, x, y, xfull, yfull, sqw, penaltymatrix, pm,
			restrictpm, lambdavalue, dvector, gmatrix, yvec, hvec,
			dlam, num, den, dlambdastar, xpxi, betahat, pdiff,
			e, effnoparms, ssq, avgabserror, bbt, signif, d},
	$NonlinearFlag = False; (* set flag for GCVFunction *)
	{restrict, $TuningParameter, dropnum, weights, order} = 
	{Restriction, TuningParameter, DropByTerm, ObservationWeights,
		SplineOrder} /. {opts} /. Options[DiscountFunction];
(* error checking for starting value options *)
	fixedlambda = If[(lambdavalue = LambdaValue /. {opts} /. 
		Options[DiscountFunction]) =!= Automatic, True, False];
	{lambda0, bounds} = {Lambda0, Bounds} /. {opts} /. 
		Options[DiscountFunction];
	If[Length[bounds] != 2 || 
		lambda0 =!= Automatic && Length[lambda0] != 2,
		Message[DiscountFunction::badopts]; Return[$Failed], 
		{lower, upper} = bounds];
	If[lambda0 === Automatic, lambda0 = {10.^10, 10.^11}];
(* weight matrix *)
	Which[	weights === Automatic, 
			sqw = N @ IdentityMatrix[Length[fullprice]],
		Length[weights] != Length[fullprice],
			sqw = N @ IdentityMatrix[Length[fullprice]];
			Message[DiscountFunction::badweights],
		True,
			sqw = DiagonalMatrix[Sqrt[N @ weights]]];
(* construct the data matrix and x and y, using only those in keeplist *)
	pvpmts = MapThread[Dot, {payments, phimat}];
	xfull = sqw . pvpmts;
	yfull = sqw . fullprice;
	If[restrict, yfull = yfull - First /@ xfull; xfull = Rest /@ xfull];
	x = xfull[[keeplist]];
	y = yfull[[keeplist]];
(* penalty matrix *)
	Switch[order, 
		0 | 3,		If[restrict,
					(*then*)
					With[{pm = Rest @ PenaltyMatrix[knots, order]//N},
						penaltymatrix = Rest /@ pm; 
						restrictpm = First /@ pm],
					(*else*)
					penaltymatrix = PenaltyMatrix[knots, order]//N],
		1 | 2,	penaltymatrix = IdentityMatrix[ Dimensions[x][[2]] ]; 
				restrictpm = First @ penaltymatrix;
				fixedlambda = True; (* flag for no GCV *)
				lambdavalue = 0  (* flag for no penalty *)
		];
(* calculate betahat *)
	{dvector, gmatrix} = GCVDecomposition[x, penaltymatrix];
	yvec = Transpose[x.gmatrix].y;
	hvec = If[restrict, Transpose[gmatrix].restrictpm, 0];
	dlam = 1/(1 + lambda dvector);
	num = (y.y + 2(lambda hvec yvec - yvec^2).dlam +
			(lambda^2 hvec^2 - 2 lambda hvec yvec + yvec^2).(dlam^2));
	den = (nobs (1 - tuningparm (Plus @@ dlam)/nobs)^2);
	(* global, for GCVFunction *)
	$GCVFunction = Function @@ {{lambda, tuningparm}, num/den};
	$EffectiveParameters = Function @@ {{lambda}, Plus @@ dlam};
	If[fixedlambda, 
		(*then*)
		lambdastar = N[lambdavalue];
		gcvvalue = $GCVFunction[lambdastar, $TuningParameter],
		(*else*)
		foundmin = FindMinimum[
			$GCVFunction[lambda, $TuningParameter], 
			{lambda, lambda0, upper, lower}, fmopts];
		lambdastar = lambda /. foundmin[[2]];
		gcvvalue = foundmin[[1]]];
	dlambdastar = 1/(1 + lambdastar dvector);
	xpxi = gmatrix.DiagonalMatrix[dlambdastar].Transpose[gmatrix];
	If[restrict, betahat = Prepend[xpxi.(Transpose[x].y - 
				lambdastar restrictpm), 1],
				betahat = xpxi.Transpose[x].y];
(* calculate fitted values, residuals, etc. *)
	pdiff = fullprice - pvpmts.betahat;
	e = pdiff[[keeplist]];
	effnoparms = Plus @@ dlambdastar; 
	ssq = e.e/(nobs - effnoparms);
	avgabserror = Apply[Plus, Abs[e]]/nobs;
	bbt = gmatrix.DiagonalMatrix[dlambdastar^2].Transpose[gmatrix];
	signif = pdiff/(ssq(1 + #.bbt.# & /@ xfull) // Sqrt);
(* fill out bbt if necessary *)
	If[restrict, bbt = Prepend[Prepend[#, 0]& /@ bbt, 
			Table[0, {Length[bbt] + 1}]] ];
(* construct final spline as interpolation function *)
	d = MakeFinalSpline[knots, betahat];
(* produce DF object *)
	DF[ QuoteDate[cp], d, lambdastar, effnoparms, Delta, pdiff, 
		keeplist, ssq, avgabserror, signif, knots, bbt, betahat, 
		restrict, dropnum, fixedlambda, $TuningParameter, weights, order,
		gcvvalue, Hold[cp] ] ] ]

(****************Nonlinear routine***********************)
SetAttributes[DiscFunNonLin, HoldFirst]

DiscFunNonLin[ cp_ /; Head[cp] == CP, 
		{knots_, kplst_, phimat_}, opts___Rule ] :=
	Block[{lambda, gcvvalue, order, onrate, dropnum, weights, 
		fform, prog, curve0, startfile, lambda0, e, xpxi, x, y, bbt,
		pdiff, effnoparms, signif, finalbeta, avgabserror, bounds,
		beta0autoQ, lambda0autoQ, dsaveOKQ, lambdastarOKQ, initval,
		initfcn, upper, lower, beta0fcn, divisor},
(* global variables have "$" prefix *)
		$NonlinearFlag = True; (* set flag for GCVFunction *)
		$keeplist = kplst;
		$nobs = Length[$keeplist];
		$phimat = phimat;
		$fullprice = Price[cp] + AccruedInterest[cp];
		$payments = Payments[cp];
		$knots = knots;
	{$restrict, $TuningParameter, dropnum, weights, fform, prog, 
		order, onrate, $test, curve0} = 
	{Restriction, TuningParameter, DropByTerm, ObservationWeights, 
		FunctionalForm, ShowLambdaProgress, SplineOrder, 
			OvernightForwardRate, BetaConvergenceTest, InitialCurve} /. 
				{opts} /. Options[DiscountFunction];
	If[fform === DLogDelta, 
		If[NumberQ[onrate], $restrict = True, $restrict = False];
		startfile = "DLStart.m", 
		(*else LogDelta *)
		startfile = "LStart.m"];
(* restricted value *)
	If[$restrict, 
		$restrictval = If[fform === DLogDelta, 
			onrate/(100. Years), 0],
		$restrictval = 0];
(* error checking for starting value options *)
fixedlambda = If[(lambdavalue = LambdaValue /. {opts} /. 
		Options[DiscountFunction]) =!= Automatic, True, False];
	{lambda0, bounds} = {Lambda0, Bounds} /. {opts} /. 
		Options[DiscountFunction];
	If[Length[bounds] != 2 || 
		lambda0 =!= Automatic && Length[lambda0] != 2,
		Message[DiscountFunction::badopts]; Return[$Failed], 
		{lower, upper} = bounds];
(* weight matrix *)
	Which[	weights === Automatic, 
			$sqw = N @ IdentityMatrix[Length[$fullprice]],
		Length[weights] != Length[$fullprice],
			$sqw = N @ IdentityMatrix[Length[$fullprice]];
			Message[DiscountFunction::badweights],
		True,
			$sqw = DiagonalMatrix[Sqrt[N @ weights]]];
(* starting values for lambda and beta *)
	(* setup *)
	beta0autoQ = If[curve0 === Automatic, True, False, False];
	lambda0autoQ = If[lambda0 === Automatic && !fixedlambda, 
		True, False, False];
	(* make a list of maturities to average the starting function over *)
	knottab = If[order == 0, 
				(*then*)
				({Plus @@ #}& /@ Partition[knots, 2, 1]),
				(*else*)
				Partition[PadKnots[knots, order-1], order, 1]];
	divisor = If[order == 0, 2, order];
	initval = If[beta0autoQ, 5, curve0]; (* 5% flat curve *)
	initfcn = Switch[fform,
		LogDelta,	Function @@ {# initval/(100.0 Years)},
		_,			Function @@ {  initval/(100.0 Years)}];
	beta0fcn = Function[x, Apply[Plus, Map[x, knottab, {2}], {1}]/divisor];
	(* execute *)
	Clear[$dsave, $lambdastar]; (* global *)
	Off[Get::noopen]; 
	If[beta0autoQ || lambda0autoQ,
		(* then: open file *)
		If[Get[startfile] === $Failed, 
			(* then: can't open file *)
			Message[DiscountFunction::noopen, startfile]; 
			dsaveOKQ = lambdastarOKQ = False,
		(* else: file was opened; are the values good? *)
			dsaveOKQ = If[NumberQ[$dsave[Last[knots]]], 
				True, False, False];
			If[!dsaveOKQ && beta0autoQ,
				Message[DiscountFunction::badcurve, startfile, initval]];
			lambdastarOKQ = If[NumberQ[$lambdastar] && Positive[$lambdastar],
				True, False, False];
			If[!lambdastarOKQ && lambda0autoQ,
				Message[DiscountFunction::badlambda, startfile, lambda0]]];
		If[beta0autoQ,
			(* then: see if $dsave is OK *)
			If[dsaveOKQ, 
				(* then: use $dsave *)
				$beta0 = beta0fcn[$dsave],
				(* else: use default *)
				$beta0 = beta0fcn[initfcn] ]];
		If[lambda0autoQ,
			(* then: see if $lambdastar is OK *)
			If[lambdastarOKQ, 
				(* then: use lambdastar *)
				lambda0 = $lambdastar {1, 10},
				(* else: use default *)
				lambda0 = {1.5 10^10, 1.8 10^10}]],
		(* else: use initfcn for $beta0; lambda0 is already set *)
		$beta0 = beta0fcn[initfcn]];
	On[Get::noopen];	
	If[$restrict, $beta0 = Rest @ $beta0];
	(* done *)			
(* penaltymatrix *)
	Switch[order,
		0 | 3,	If[$restrict,
					(*then*)
					With[{pm = Rest @ PenaltyMatrix[knots, order]//N},
						$penaltymatrix = Rest /@ pm; 
						$restrictpm = First /@ pm],
					(*else*)
					$penaltymatrix = N @ PenaltyMatrix[knots, order]],
		1 | 2, 	$penaltymatrix = 1; (* no penalty *)
				$restrictpm = 0; (* no penalty *)
				fixedlambda = True; (* flag for no GCV *)
				lambdavalue = 0]; (* flag for no penalty *)
(* find the minimum gcv *)
	If[!fixedlambda, 
	findmin = CheckAbort[(* from within GivenLambdaFindBeta *)
			Check[ (* for error messages from FindMinimum *)
			With[{fmopts = FilterOptions[FindMinimum, opts]},
			FindMinimum[
				If[prog, Print[StringForm["lambda = `1`", 
						InputForm[lambda]]]];
				GivenLambdaFindBeta[lambda],
				{lambda, lambda0, lower, upper}, fmopts]],
			Return[$Failed]], 
			Return[$Failed]];
	$lambdastar = lambda /. findmin[[2]],
	$lambdastar = N[lambdavalue]];
	If[prog, Print["lambdastar = ", $lambdastar]];
(* final estimate: optimal lambda may not be the last one *)
	gcvvalue = CheckAbort[GivenLambdaFindBeta[$lambdastar], 
		Return[$Failed]];
	finalbeta = If[$restrict, Prepend[$beta0, $restrictval], $beta0];
(* construct x matrix *)
	pvpmts = $payments Exp[-(# . finalbeta&) /@ phimat];
	xfull = $sqw . MapThread[Dot, {-pvpmts, phimat}];
	If[$restrict, xfull = Rest /@ xfull];
	x = xfull[[$keeplist]];
(* calculate various statistics *)
	xpxi = Inverse[Transpose[x].x + ($lambdastar $penaltymatrix)];
	pdiff = $fullprice - Apply[Plus, pvpmts, {1}];
	e = pdiff[[$keeplist]];
	effnoparms = Plus @@ Transpose[$amat, {1,1}];
	ssq = e.e/($nobs - effnoparms);
	avgabserror = Apply[Plus, Abs[e]]/$nobs;
	bbt = xpxi.Transpose[x].x.Transpose[xpxi];
	signif = pdiff/(ssq(1 + #.bbt.# & /@ xfull) // Sqrt);
(* construct final spline as interpolation function *)
	$dsave = MakeFinalSpline[knots, finalbeta];
(* save these for next time *)
	Put[Definition[$lambdastar, $dsave], startfile];
	If[fform === LogDelta, 
		If[$restrict, 
			bbt = Prepend[Prepend[#, 0]& /@ bbt, 
				Table[0, {Length[bbt] + 1}]] ]];
(* produce DF object *)
	DF[ QuoteDate[cp], $dsave, $lambdastar, 
		effnoparms, fform, pdiff, $keeplist, ssq,
		avgabserror, signif, knots, bbt, finalbeta, $restrict, 
		dropnum, fixedlambda, $TuningParameter, weights, order, 
		gcvvalue, Hold[cp] ] ]


(* The purpose of the FixedPoint in GivenLambdaFindBeta is to find the
correct values of x and y that can then be used to calculate the GCV
value (that in turn is returned to the FindMinimum). What the
FixedPoint returns, however, is a the beta at the fixed point that can
then be used in the next iteraton. GivenLambdaFindBeta needs
$restrict, $payments, $sqw, $keeplist, phimat, $fullprice,
$penaltymatrix, $restrictval, $restrictpm, $test, and
$TuningParameter, all of which it gets via "global" variables. It
passes back $amat. *)

GivenLambdaFindBeta[lambda_] := 
	( $beta0 = 	(* save this iteration's result for *)
	FixedPoint[	(* next iteration's starting value *)
	(* nonlinear least squares *)
		Function[betavec, 
		Module[{bvalues, pvpmts},
		bvalues = If[$restrict, Prepend[betavec, $restrictval], betavec];
		pvpmts = $payments[[$keeplist]] 
			Exp[-(#. bvalues&) /@ $phimat[[$keeplist]]];
		$x = $sqw[[$keeplist, $keeplist]] . 
			MapThread[Dot, {-pvpmts, $phimat[[$keeplist]]}];
		If[$restrict, $x = Rest /@ $x; bvalues = Rest @ bvalues];
		$y = $sqw[[$keeplist, $keeplist]] . ($fullprice[[$keeplist]] - 
			Apply[Plus, pvpmts, {1}]) + $x.bvalues;
		Check[ (* abort if singular or badly conditioned *)
		Inverse[Transpose[$x].$x + lambda $penaltymatrix] .
				(Transpose[$x].$y - 
				If[$restrict, lambda $restrictval $restrictpm, 0]),
			Abort[], Inverse::sing, Inverse::luc]]],
	(* this iteration's starting value *)
		$beta0, 
	(* convergence test *)
		SameTest -> $test];
	(* the generalized cross validation value to be minimized *)
		$amat = $x.Inverse[Transpose[$x].$x + 
			lambda $penaltymatrix].Transpose[$x];
		With[{e = $y - $x.$beta0}, e.e/($nobs (1 - $TuningParameter (
			Plus@@Transpose[$amat, {1,1}]/$nobs ))^2 )] )

(* DF objects *)
DF /: QuoteDate[df_DF] 		:= df[[1]]
DF /: FinalSpline[df_DF]	:= df[[2]]
DF /: Lambda[df_DF] 		:= df[[3]]
DF /: EffectiveParameters[df_DF] := df[[4]]
DF /: FunctionalForm[df_DF]	:= df[[5]]
DF /: PriceError[df_DF] 	:= df[[6]]
DF /: KeepList[df_DF] 		:= df[[7]]
DF /: ResidualVariance[df_DF] := df[[8]]
DF /: AverageAbsoluteError[df_DF] := df[[9]]
DF /: Significance[df_DF] 	:= df[[10]]
DF /: Knots[df_DF] 		:= df[[11]]
DF /: BetaHatCovarianceMatrix[df_DF] := df[[12]]
DF /: BetaHat[df_DF]		:= df[[13]]
DF /: RestrictionQ[df_DF]	:= df[[14]]
DF /: DropByTerm[df_DF]		:= df[[15]]
DF /: FixedLambdaQ[df_DF]	:= df[[16]]
DF /: TuningParameter[df_DF]	:= df[[17]]
DF /: ObservationWeights[df_DF]	:= df[[18]]
DF /: SplineOrder[df_DF]	:= df[[19]]
DF /: GCVValue[df_DF]		:= df[[20]]
DF /: GCVMinimizedQ[df_DF]	:= Not[FixedLambdaQ[df]]
DF /: NumberOfObservations[df_DF] := Length[KeepList[df]]
DF /: Delta[df_DF] 		:= Switch[FunctionalForm[df], 
	Delta, FinalSpline[df],
	LogDelta, Function @@ {m, Exp[-FinalSpline[df][m]]},
	DLogDelta, Function @@ {m, Exp[-IntegrateIF[FinalSpline[df]][m]]}]
DF /: ForwardCurve[df_DF]	:= Switch[FunctionalForm[df], 
	Delta, With[{d = FinalSpline[df]}, Function @@ {m, -100 Years d'[m]/d[m]}],
	LogDelta, Function @@ {m, 100 Years FinalSpline[df]'[m]}, 
	DLogDelta, Function @@ {m, 100 Years FinalSpline[df][m]}]
DF /: ZeroCurve[df_DF]		:= Switch[FunctionalForm[df],
	Delta, Function @@ {m, -100 Years Log[FinalSpline[df][m]]/m},
	LogDelta, Function @@ {m, 100 Years FinalSpline[df][m]/m}, 
	DLogDelta, Function @@ {m, 100 Years IntegrateIF[FinalSpline[df]][m]/m}]
DF /: SemiAnnualForwardCurve[df_DF] := ContinuousToSemiAnnual[ForwardCurve[df]]
DF /: SemiAnnualZeroCurve[df_DF] := ContinuousToSemiAnnual[ZeroCurve[df]]
DF /: MaximumMaturity[df_DF]	:= Last[Knots[df]]
DF /: PredictedPrice[df_DF] 	:= Price[df] - PriceError[df] (* net of accrued *)
DF /: WhenceCP[df_DF] 		:= df[[-1]] // ReleaseHold
DF /: WhenceMD[df_DF] 		:= WhenceMD[WhenceCP[df]]
DF /: WhenceDF[df_DF] 		:= df
Format[df_DF] := StringForm["-DF[`1`]-", QuoteDate[df]]

DFSelectors = Sort @ {QuoteDate, FinalSpline, Lambda,
EffectiveParameters, PredictedPrice, PriceError, KeepList,
ResidualVariance, AverageAbsoluteError, Significance, Knots,
BetaHatCovarianceMatrix, BetaHat, RestrictionQ, DropByTerm, Delta,
MaximumMaturity, FixedLambdaQ, ForwardCurve, ZeroCurve, GCVMinimizedQ,
TuningParameter, ObservationWeights, SplineOrder, GCVValue,
NumberOfObservations, SemiAnnualForwardCurve, SemiAnnualZeroCurve}

(#[x_ /; MatchQ[Head[x], YC | SE]] := #[WhenceDF[x]])& /@ 
	DeleteCases[DFSelectors, QuoteDate]

(* converts ForwardCurve and ZeroCurve to semi-annual compounding *)
ContinuousToSemiAnnual[f_Function] := 
		Function @@ {m, 100 (2 Exp[f[m]/200] - 2)}

(* auxillary DiscountFunction functions *)
Options[GCVFunction] = {TuningParameter -> Automatic}

GCVFunction[lambdavalue_ /; !$NonlinearFlag, opts___Rule] := 
	Module[{tuningparm},
	tuningparm = TuningParameter /. {opts} /. Options[GCVFunction];
	If[tuningparm === Automatic, tuningparm = $TuningParameter];
	$GCVFunction[lambdavalue, tuningparm]]

GCVFunction[lambdavalue_ /; $NonlinearFlag, opts___Rule] := 
	Module[{tuningparm},
	tuningparm = TuningParameter /. {opts} /. Options[GCVFunction];
	If[tuningparm === Automatic, 
		GivenLambdaFindBeta[lambdavalue],
		(* else: use value given *)
		Block[{$TuningParameter = tuningparm}, 
		GivenLambdaFindBeta[lambdavalue]]]
	]

PlotGCVFunction[list_List:(10^Range[22]), opts___Rule] :=
	Module[{x, y, pos, data},
	x = N[Log[10, list]];
	y = CheckAbort[GCVFunction[#], $Failed]& /@ list;
	pos = Position[y, $Failed];
	data = Delete[Transpose[{x, y}], pos];
	ListPlot[data, opts, PlotJoined -> True, Axes -> False,
		Frame -> {True, True, False, False},
		Epilog -> {Red, PointSize[.015], Point /@ data}]
	]

EffectiveParameters::lin = "$NonlinearFlag is False.
EffectiveParameters requires an argument (lambda)."

EffectiveParameters::nonlin = "$NonlinearFlag is True.
EffectiveParameters takes no arguments."

EffectiveParameters[] := 
	If[$NonlinearFlag,
		Plus @@ Transpose[$amat, {1,1}],
		Message[EffectiveParameters::lin];
		Return[$Failed]]

EffectiveParameters[lambda_] :=
	If[$NonlinearFlag, 
		Message[EffectiveParameters::nonlin];
		Return[$Failed],
		$EffectiveParameters[lambda]]

MakeNonlinearSpline::lin = "$NonlinearFlag is False.
MakeNonlinearSpline is not defined."

MakeNonlinearSpline[] :=
	If[$NonlinearFlag,
		MakeFinalSpline[$knots,If[$restrict, 
			Prepend[$beta0, $restrictval], $beta0]],
		Message[MakeNonlinearSpliner::lin];
		Return[$Failed]]
(***************************************************************************)
YieldCalc::outarg = "`1` is not a valid argument"
YieldCalc::notobj = "`1` is not a valid object"
YieldCalc::badcomp = "`1` is not a valid setting for the option
Compounding."

Options[YieldCalc] = {Compounding -> SIACompounding}

SetAttributes[YieldCalc, HoldFirst]

YieldCalc[obj_, opts___Rule] := 
	Which[
	MatchQ[Hold[obj], Hold[Out[___]]],
		Message[YieldCalc::outarg, HoldForm[obj]];
		Return[$Failed],
	Not[MatchQ[Head[obj], CP | DF]],
		Message[YieldCalc::notobj, HoldForm[obj]];
		Return[$Failed],
	True,
	Module[{accdays, next, daysinper, m, price, pred, couprate, npmts,
		comp, ytm, pytm, ydiff, yfcn, dur, con, term, termlist, pairs,
		allargs, ytmrule},
	comp = Compounding /. {opts} /. Options[YieldCalc];
	pred = If[Head[obj] === DF, PredictedPrice[obj], "NA"];
	accdays = AccruedDays[obj];
	next = NextPayment[obj];
	daysinper = DaysInPeriod[obj];
	m = PaymentsPerYear[obj];
	price = Price[obj];
	couprate = CouponRate[obj]/100;
	npmts = RemainingPayments[obj];
	term = Term[obj]; (* standing in for security Type *)
	termlist = Union[term];
	allargs = Transpose[
		{accdays, next, daysinper, m, price, couprate, npmts}];
	Switch[comp, SIACompounding | ContinuousCompounding, Null,
		_, Message[YieldCalc::badcomp, comp]; Return[$Failed]];
	Switch[comp, 
		SIACompounding,
		(* this shows how to use Type (term here) to apply fcns *)
			pairs = {{-1, ZeroYield},
				{Alternatives @@ Complement[termlist, {-1}], Yield}};
			ytmrule = MapThread[( {#1, {args__}} :> #2[args] )&, 
				Transpose[pairs]];
			ytm = Transpose[{term, allargs}] /. ytmrule;
			(*ytm = MapThread[Yield,
				{accdays, next, daysinper, m, price, couprate, npmts}];*)
			ytm = 100 ytm;
			dur = MapThread[Duration, 
				{accdays, next, daysinper, m, ytm/100, couprate, npmts}];
			dur = -dur;
			con = MapThread[Convexity, 
				{accdays, next, daysinper, m, ytm/100, couprate, npmts}],
		ContinuousCompounding, 
			ytm = 100 MapThread[CCYield,
				{accdays, next, daysinper, m, price, couprate, npmts}];
			dur = "N/A";
			con = "N/A"];
	yfcn = Switch[comp, SIACompounding, Yield, _, CCYield];
	If[Head[obj] === DF,
		pytm = 100 MapThread[yfcn, 
		{accdays, next, daysinper, m, pred, couprate, npmts}];
		ydiff = 100 (ytm - pytm), 
		(* else *)
		pytm = ydiff = "N/A"];
	YC[QuoteDate[obj], ytm, pytm, ydiff, dur, con, comp, Hold[obj]] ]]

(* calculate yields: *)
YC /: QuoteDate[yc_YC] 		:= yc[[1]]
YC /: Yield[yc_YC] 		:= yc[[2]]
YC /: PredictedYield[yc_YC] 	:= yc[[3]]
YC /: YieldError[yc_YC] 	:= yc[[4]]
YC /: Duration[yc_YC]		:= yc[[5]]
YC /: Convexity[yc_YC]		:= yc[[6]]
YC /: Compounding[yc_YC]	:= yc[[7]]
YC /: MacaulayDuration[yc_YC]	:= Duration[yc](1 + 
					Yield[yc]/100/PaymentsPerYear[yc])
YC /: WhenceDF[yc_YC] 		:= yc[[-1]] // ReleaseHold
YC /: WhenceCP[yc_YC] 		:= WhenceCP[WhenceDF[yc]]
YC /: WhenceMD[yc_YC] 		:= WhenceMD[WhenceDF[yc]]
YC /: WhenceYC[yc_YC] 		:= yc
Format[yc_YC] := StringForm["-YC[`1`]-", QuoteDate[yc]]

YCSelectors = Sort @ {QuoteDate, Yield, PredictedYield, YieldError, Duration,
	Convexity, MacaulayDuration, Compounding}

(* yield functions *)
(* from Standard Securities Calculation Methods *)
(* a - AccruedDays, dsr - NextPayment, e - DaysInPeriod, m - PaymentsPerYear,
	p - Price, yld - Yield, r - CouponRate, n - RemainingPayments,
	rv - redemption value *)
(* coupons with one coupon remainng *)
Yield[a_, dsr_, e_, m_, p_, r_/;r>0, n_/;n==1, rv_:100] := 
	CouponYield1[a, dsr, e, m, p, r, n, rv]

CouponYield1 =
Compile[{a, dsr, e, m, p, r, n, rv},
	(((rv/100 + r/m) - (p/100 + a/e r/m))/
	(p/100 + a/e r/m)) (m e/dsr)]

(* coupons with more than one coupon remaining *)
Yield[a_, dsr_, e_, m_, p_, r_ /; r > 0, n_ /; n > 1, rv_:100] := 
	yld /. FindRoot[p + (100 r a)/(m e) == 
	rv/(1 + yld/m)^(n - 1 + dsr/e) + 
	(100 r (m + yld)(1 + yld/m)^(-(dsr/e) - n) *
		(-1 + (1 + yld/m)^n))/(m yld),
	{yld, r/p}]

(* bills *)
Yield[a_, dsr_, e_, m_, p_, r_/;r==0, n_/;n==1, rv_:100] := 
	BillYield[a, dsr, e, m, p, r, n, rv]

BillYield = 
	Compile[{a, dsr, e, m, p, r, n, rv}, (rv - p)/p (360/dsr)]

(* zeros with one coupon remainng *)
ZeroYield[a_, dsr_/;dsr<183, e_, m_, p_, r_/;r==0, n_/;n==1, rv_:100] := 
	(rv - p)/p * (m 182)/dsr

(* zeros with more than one coupon remaining *)
ZeroYield[a_, dsr_/;dsr>=183, e_, m_, p_, r_/;r==0, n_/;n==1, rv_:100] := 
	Module[{nq, dsc, newe},
	nq = Ceiling[m (dsr/Years)];
	dsc = Mod[dsr, 183];
	newe = 183;
	m ((rv/p)^(1/(nq - 1 + dsc/newe)) -1)]

(******** continuously-compounded yields ************)
(* zero-coupon securities *)
CCYield[a_, dsr_, e_, m_, p_, r_/;r==0, n_/;n==1, rv_:100] := 
	-Years Log[p/100]/dsr

(* coupons with one coupon remainng *)
CCYield[a_, dsr_, e_, m_, p_, r_/;r>0, n_/;n==1, rv_:100] := 
	Yield[a, dsr, e, m, p, r, n, rv]
(* coupons with more than one coupon remaining *)
CCYield[a_, dsr_, e_, m_, p_, r_ /; r > 0, n_ /; n > 1, rv_:100] := 
	Yield[a, dsr, e, m, p, r, n, rv]

(*strips yield*)
StripsYield[next_, price_] := -Years Log[price/100]/next
(****************************************************)

(* duration *)
Duration[a_, dsr_, e_, m_, yld_, r_ /; r > 0, n_ /; n > 1, rv_:100] :=
	CouponDuration1[a, dsr, e, m, yld, r, n, rv]

CouponDuration1 =
Compile[{a, dsr, e, m, yld, r, n, rv},
(100*e*m*r + 100*dsr*r*yld + 100*e*n*r*yld - dsr*rv*yld^2 + 
    e*rv*yld^2 - e*n*rv*yld^2 - 100*e*m*r*(1 + yld/m)^n - 
    100*dsr*r*yld*(1 + yld/m)^n)/
  (yld*(-100*e*m*r - 100*e*r*yld + e*m*rv*yld + e*rv*yld^2 + 
      100*e*m*r*(1 + yld/m)^n + 100*e*r*yld*(1 + yld/m)^n - 
      100*a*r*yld*(1 + yld/m)^(dsr/e + n)))]

Duration[a_, dsr_, e_, m_, yld_, r_ /;r > 0, n_ /; n == 1, rv_:100] :=
	CouponDuration2[a, dsr, e, m, yld, r, n, rv]

CouponDuration2 =
Compile[{a, dsr, e, m, yld, r, n, rv},
(dsr*e^2*m*(100*r + m*rv))/
  ((e*m + dsr*yld)*(100*a*e*m*r - 100*e^2*m*r - e^2*m^2*rv + 
      100*a*dsr*r*yld))]

Duration[a_, dsr_, e_, m_, yld_, r_ /;r == 0, n_ /; n == 1, rv_:100] :=
	BillDuration[a, dsr, e, m, yld, r, n, rv]

BillDuration = 
	Compile[{a, dsr, e, m, yld, r, n, rv}, -(dsr/(360 + dsr*yld))]

(* convexity *)
Convexity[a_, dsr_, e_, m_, yld_, r_/;r>0, n_/;n>1, rv_:100] :=
	CouponConvexity1[a, dsr, e, m, yld, r, n, rv]

CouponConvexity1 =
Compile[{a, dsr, e, m, yld, r, n, rv},
	(-200*e^2*m^2*r - 200*dsr*e*m*r*yld - 200*e^2*m*r*yld - 
    200*e^2*m*n*r*yld - 100*dsr^2*r*yld^2 - 100*dsr*e*r*yld^2 - 
    200*dsr*e*n*r*yld^2 - 100*e^2*n*r*yld^2 - 100*e^2*n^2*r*yld^2 + 
    dsr^2*rv*yld^3 - dsr*e*rv*yld^3 + 2*dsr*e*n*rv*yld^3 - 
    e^2*n*rv*yld^3 + e^2*n^2*rv*yld^3 + 200*e^2*m^2*r*(1 + yld/m)^n + 
    200*dsr*e*m*r*yld*(1 + yld/m)^n + 200*e^2*m*r*yld*(1 + yld/m)^n + 
    100*dsr^2*r*yld^2*(1 + yld/m)^n + 100*dsr*e*r*yld^2*(1 + yld/m)^n)/
  (2*e*yld^2*(m + yld)*(-100*e*m*r - 100*e*r*yld + e*m*rv*yld + 
      e*rv*yld^2 + 100*e*m*r*(1 + yld/m)^n + 
      100*e*r*yld*(1 + yld/m)^n - 100*a*r*yld*(1 + yld/m)^(dsr/e + n)))]

Convexity[a_, dsr_, e_, m_, yld_, r_ /;r > 0, n_ /; n == 1, rv_:100] :=
	CouponConvexity2[a, dsr, e, m, yld, r, n, rv]

CouponConvexity2 =
Compile[{a, dsr, e, m, yld, r, n, rv},
	(dsr^2*e^2*m*(100*r + m*rv))/
	((e*m + dsr*yld)^2*(-100*a*e*m*r + 100*e^2*m*r + e^2*m^2*rv - 
		100*a*dsr*r*yld))]

Convexity[a_, dsr_, e_, m_, yld_, r_ /;r == 0, n_ /; n == 1, rv_:100] :=
	BillConvexity[a, dsr, e, m, yld, r, n, rv]

BillConvexity = 
Compile[{a, dsr, e, m, yld, r, n, rv}, dsr^2/(360 + dsr*yld)^2]

x$ /: x$[i_] > 0 = True (* x$ is a wrapper that satisfies *)
x$ /: x$[i_] > 1 = True (* the conditions for r and n     *)

Duration[arg_][name_] :=
Module[{durlist = Duration[AccruedDays, NextPayment, DaysInPeriod, 
	PaymentsPerYear, Yield@#/100&, CouponRate@#/100&, 
	Length[Payments@#]&],
	durargs = {AccruedDays, NextPayment, DaysInPeriod,
	PaymentsPerYear, Yield, CouponRate, RemainingPayments},
	pos},
	pos = Flatten[Position[durargs, arg]];
	If[Length[pos] != 0, 
		durlist[[ pos[[1]] ]] = (x$[arg] &);
		Through[durlist[name]] /. x$[a_] -> a,
	(* else *)
		Duration[name] ]]

(* price *)
(* from Standard Securities Calculation Methods *)
(* a - AccruedDays, dsr - NextPayment, e - DaysInPeriod, m - PaymentsPerYear,
	p - Price, yld - Yield, r - CouponRate, n - RemainingPayments,
	rv - redemption value *)
(* coupons with one coupon remainng *)
Price[a_, dsr_, e_, m_, yld_, r_/;r>0, n_/;n==1, rv_:100] := 
	CouponPrice1[a, dsr, e, m, yld, r, n, rv]

CouponPrice1 =
Compile[{a, dsr, e, m, yld, r, n, rv},
	(-100*a*e*m*r + 100*e^2*m*r + e^2*m^2*rv - 100*a*dsr*r*yld)/
		(e^2*m^2 + dsr*e*m*yld)]

(* coupons with more than one coupon remaining *)
Price[a_, dsr_, e_, m_, yld_, r_ /; r > 0, n_ /; n > 1, rv_:100] := 
	CouponPrice2[a, dsr, e, m, yld, r, n, rv]

CouponPrice2 =
Compile[{a, dsr, e, m, yld, r, n, rv},
	((1 + yld/m)^(-(dsr/e) - n)*(-100*e*m*r - 100*e*r*yld + e*m*rv*yld + 
	e*rv*yld^2 + 100*e*m*r*(1 + yld/m)^n + 100*e*r*yld*(1 + yld/m)^n - 
		100*a*r*yld*(1 + yld/m)^(dsr/e + n)))/(e*m*yld)]

(* bills *)
Price[a_, dsr_, e_, m_, yld_, r_/;r==0, n_/;n==1, rv_:100] := 
	BillPrice[a, dsr, e, m, yld, r, n, rv]

BillPrice = 
	Compile[{a, dsr, e, m, yld, r, n, rv}, (360 rv)/(360 + dsr yld)]

(* these routines work for one bond at a time *)
Price[couprate_, matdate:{_, _, _}, quotedate:{_, _, _}, yld_, 
	pmtsperyear_:2] :=
	N[Price[Sequence @@
		Insert[BondSetUp[couprate, matdate, quotedate, pmtsperyear], 
			yld/100, 5]]]

Duration[couprate_, matdate:{_, _, _}, quotedate:{_, _, _}, yld_, 
	pmtsperyear_:2] :=
	N[Duration[Sequence @@
		Insert[BondSetUp[couprate, matdate, quotedate, pmtsperyear], 
			yld/100, 5]]]

Convexity[couprate_, matdate:{_, _, _}, quotedate:{_, _, _}, yld_, 
	pmtsperyear_:2] :=
	N[Convexity[Sequence @@
		Insert[BondSetUp[couprate, matdate, quotedate, pmtsperyear], 
			yld/100, 5]]]

Yield[couprate_, matdate:{_, _, _}, quotedate:{_, _, _}, price_, 
	pmtsperyear_:2] :=
	N[Yield[Sequence @@ 
		Insert[BondSetUp[couprate, matdate, quotedate, pmtsperyear], 
			price, 5]]]

BondSetUp[couprate_, matdate:{_, _, _}, quotedate:{_, _, _}, 
	pmtsperyear_:2] :=
	With[{setdate = FirstBusinessDay @ 
		DaysPlus[quotedate, BusinessDaysToSettlement]},
	Module[{daysaccrued, daystopayments, npmts, nextpmt, daysinperiod},
	daystopayments = 
		PaymentDates[setdate, matdate, couprate, pmtsperyear];
	daysaccrued = -First[daystopayments];
	daystopayments = Rest[daystopayments];
	npmts = Length[daystopayments];
	nextpmt = First[daystopayments];
	daysinperiod = nextpmt + daysaccrued;
	{daysaccrued, nextpmt, daysinperiod, pmtsperyear, couprate/100, 
		npmts}]]

(***************************************************************************)

SetAttributes[StandardErrors, HoldFirst]
StandardErrors::outarg = "`1` is not a valid argument"
StandardErrors::notobj = "`1` is not a valid object"

StandardErrors[df_, gridlist_:Automatic] :=
	Which[
	MatchQ[Hold[df], Hold[Out[___]]],
		Message[StandardErrors::outarg, HoldForm[df]];
		Return[$Failed],
	Not[MatchQ[Head[df], DF]],
		Message[StandardErrors::notobj, HoldForm[df]];
		Return[$Failed],
	True,
	With[{f = ForwardCurve[df], z = ZeroCurve[df],
		maxmat = MaximumMaturity[df],
		form = FunctionalForm[df],
		b = BetaHat[df]},
	Module[{fwdrates, zerorates, grid, phimat, dphimat, 
		intphimat, fmat, zmat, fwderr, zeroerr},
		If[gridlist === Automatic, 
			grid = Table[j, {j, maxmat/60, maxmat, maxmat/60}],
			grid = gridlist];
		fwdrates =  f /@ grid; 
		zerorates = z /@ grid; 
		phimat =  SplineMatrix[Knots[df], grid, 3, 0];
		Switch[form, DLogDelta, 
			intphimat = SplineMatrix[Knots[df], grid, 3, -1],
			_, dphimat = SplineMatrix[Knots[df], grid, 3, 1]];
		fmat = Switch[form, DLogDelta, phimat, LogDelta, dphimat, 
			Delta, (dphimat.b phimat)/(phimat.b)^2 - 
					dphimat/(phimat.b)];
		zmat = Switch[form, DLogDelta, intphimat/grid, LogDelta, phimat/grid,
			Delta, -phimat/(grid phimat.b)];
		fwderr = 100 Years (Sqrt[ ResidualVariance[df] 
			#.BetaHatCovarianceMatrix[df].# ]& /@ fmat);
		zeroerr = 100 Years (Sqrt[ ResidualVariance[df] 
			#.BetaHatCovarianceMatrix[df].# ]& /@ zmat);
		SE[QuoteDate[df], N[grid], {fwdrates, fwderr}, 
			{zerorates, zeroerr}, Hold[df]]]] ]

SE /: QuoteDate[se_SE]		:= se[[1]]
SE /: StandardErrorsGrid[se_SE]	:= se[[2]]
SE /: ForwardRates[se_SE]	:= se[[3, 1]]
SE /: ForwardRateErrors[se_SE]	:= se[[3, 2]]
SE /: ZeroCouponRates[se_SE]	:= se[[4, 1]]
SE /: ZeroCouponRateErrors[se_SE] := se[[4, 2]]
SE /: WhenceDF[se_SE]		:= se[[-1]] // ReleaseHold
SE /: WhenceCP[se_SE]		:= WhenceCP[WhenceDF[se]]
SE /: WhenceMD[se_SE]		:= WhenceMD[WhenceDF[se]]
SE /: WhenceSE[se_SE]		:= se
Format[se_SE] := StringForm["-SE[`1`]-", QuoteDate[se]]

SESelectors = Sort @ {QuoteDate, StandardErrorsGrid, ForwardRates, 
	ForwardRateErrors, ZeroCouponRates, ZeroCouponRateErrors}

(* this is only useful if you can create a YC object from an SE object *)
(#[x_ /; MatchQ[Head[x], YC]] := #[WhenceSE[x]])& /@ 
	DeleteCases[SESelectors, QuoteDate]


Options[StandardErrorsPlot] = Sort @ {PlotStyle -> {PointSize[.005], 
	Thickness[.002]}, PeriodLength -> Years, PlotRates -> Both}

StandardErrorsPlot[se_SE, opts___Rule] :=
	With[{graphicsopts = FilterOptions[Graphics, opts],
		plotstyle = Sequence @@ (PlotStyle /. {opts} /. 
				Options[StandardErrorsPlot]),
		p = PeriodLength /. {opts} /. Options[StandardErrorsPlot],
		which = PlotRates /. {opts} /. Options[StandardErrorsPlot],
		g = StandardErrorsGrid[se],
		f = {ForwardRates[se], ForwardRateErrors[se]},
		z = {ZeroCouponRates[se], ZeroCouponRateErrors[se]},
		qd = DateToString[QuoteDate[se]]},
	Module[{fwdptslines, zeroptslines},
	With[{t = .002}, {fwdptslines, zeroptslines} = 
		Function[x, 
		Apply[{Point[{#1/p, #2}], 
			Line[{{#1/p, #2 + #3}, {#1/p, #2 - #3}}],
			Line[{Scaled[{t, 0}, {#1/p, #2 + #3}], 
				Scaled[{-t, 0}, {#1/p, #2 + #3}]}],
			Line[{Scaled[{t, 0}, {#1/p, #2 - #3}], 
				Scaled[{-t, 0}, {#1/p, #2 - #3}]}]}&,
			Transpose[Prepend[x, g]], {1}]] /@ {f, z}];
	Show[Graphics[{plotstyle, 
		If[which === Both || which === Forward, fwdptslines, {}],
		If[which === Both || which === Zero, zeroptslines, {}]}], 
		graphicsopts, PlotRange -> All, Axes -> False,
		GridLines -> {None, 
			{Automatic, {Dashing[{.002, .01}]}}},
		Frame -> {True, True, False, False},
		FrameLabel -> {PeriodLength[p], "percent"},
		PlotLabel -> qd]]]

Options[SESummaryPage] = Sort @ {Title -> {}, 
	SummaryTextFont -> {"Courier", 10}}

SESummaryPage[se_SE, opts___Rule] :=
	With[{	sepopts = FilterOptions[StandardErrorsPlot, opts],
		plotopts = FilterOptions[Plot, opts],
		ptopts = FilterOptions[PlaceText, opts],
		ppopts = FilterOptions[PlacePlot, opts],
		pgopts = FilterOptions[PageLayout, opts],
		title = Title /. {opts} /. Options[SESummaryPage],
		sumtxt = SummaryTextFont /. {opts} /. Options[SESummaryPage],
		desc = ToString[TableForm[Description[se], 
			TableSpacing -> {0}, 
			TableAlignments -> {Center}]]},
	PageLayout[
	PlacePlot[Block[{$DisplayFunction = Identity},
		StandardErrorsPlot[se, sepopts, plotopts,
		PlotLabel -> DateToString[QuoteDate[se]]]], 
			{4.5, 1, 2, 1}, ppopts],
	PlaceText[ToString[SESummaryStatistics[se, opts]], 
		{8.5/2, 2}, TextFont -> sumtxt], 
	PlaceText[If[title === {}, desc, title], 1, ptopts, 
		Font -> {"Courier", 12}], pgopts]]

Options[SESummaryStatistics] = {PlotRates -> Both}

SESummaryStatistics[se_SE, opts___Rule] :=
	With[{	fre = ForwardRateErrors[se], 
		zre = ZeroCouponRateErrors[se],
		headings = {"average", "median", "minimum", 
				"maximum", "range"},
		which = PlotRates /. {opts} /. 
			Options[SESummaryStatistics]},
	With[{	tabfunction = Function[x, 
			PaddedForm[
			TableForm[Through[{
				(Plus @@ #/Length[#])&,
				Median,
				Min,
				Max,
				(Max[#] - Min[#])&}[x]],
			TableHeadings -> {headings, None},
			TableSpacing -> {0}], {4, 3} ]]},
	Module[{ftab, ztab},
		ftab = If[which === Both || which === Forward,
			TableForm[{"forward rate errors", tabfunction[fre]}, 
				TableSpacing -> {0}], {}];
		ztab = If[which === Both || which === Zero,
			TableForm[{"zero rate errors", tabfunction[zre]}, 
				TableSpacing -> {0}], {}];
		TableForm[{ftab, ztab}, TableSpacing -> {2}] ]]]

(***************************************************************************)
(* mostly graphics routines that can be run after DiscountFunction or
	YieldCalc *)

PeriodLength[p_] := Switch[p, Days, "days", Weeks, "weeks",
	Months, "months", Years, "years", _, ToString[p] <> " days"]

Options[DFSummaryPage] = Sort @ {Title -> {}, 
	SummaryTextFont -> {"Courier", 10}}

DFSummaryPage[df_DF, opts___Rule] := 
	With[{	showgopts = FilterOptions[ShowGraph, opts],
		plotopts = FilterOptions[Plot, opts],
		ptopts = FilterOptions[PlaceText, opts],
		ppopts = FilterOptions[PlacePlot, opts],
		(* pgopts = FilterOptions[PageLayout, opts], *)
		title = Title /. {opts} /. Options[DFSummaryPage],
		sumtxt = SummaryTextFont /. {opts} /. Options[DFSummaryPage],
		desc = ToString[TableForm[Description[df], 
			TableSpacing -> {0}, 
			TableAlignments -> {Center}]]},
	PageLayout[
	PlacePlot[Block[{$DisplayFunction = Identity},
		ShowGraph[df, showgopts, plotopts,
		PlotLabel -> DateToString[QuoteDate[df]]]], 
			{4.5, 1, 2, 1}, ppopts],
	PlaceText[ToString[DFSummaryStatistics[df, opts]], 
		{8.5/2, 2}, TextFont -> sumtxt], 
	PlaceText[If[title === {}, desc, title], 1, ptopts, 
		Font -> {"Courier", 12}], opts]]

Options[DFSummaryStatistics] =  {PeriodLength -> Years}

DFSummaryStatistics[df_DF, opts___Rule]:= 
	With[{p = PeriodLength /. {opts} /. Options[DFSummaryStatistics]},
	With[{pl = PeriodLength[p]},
	TableForm[{
		PaddedForm[Length[KeepList[df]], {3, 0}, NumberPoint -> ""],
		StringForm["`1` `2`", 
			PaddedForm[N[Max[LastPayment[df]]]/p, {5, 2}], pl],
		PaddedForm[DropByTerm[df], {3, 0}, NumberPoint -> ""],
		FunctionalForm[df],
		RestrictionQ[df],
		GCVMinimizedQ[df],
		PaddedForm[TuningParameter[df], {3, 0}, NumberPoint -> ""],
		PaddedForm[Length[Knots[df]] - 2, {3, 0}, NumberPoint -> ""],
		ReportForm[EffectiveParameters[df], {4, 1}],
		PaddedForm[Lambda[df], {5, 2}, 
			NumberFormat -> (SequenceForm[#1, "e", #3]&)],
		ReportForm[ResidualVariance[df], {8, 5}],
		ReportForm[AverageAbsoluteError[df], {7, 4}]},
		TableSpacing -> {0}, 
		TableHeadings -> {{
			"no. of obs.........", 
			"longest payment....", 
			"no. dropped........", 
			"functional form....",
			"restriction........",
			"minimize GCV.......", 
			"tuning parameter...",
			"no. of knots.......", 
			"no. of parameters..",
			"lambda.............", 
			"residual variance..", 
			"avg. abs. error...." }}] ]]



SetAttributes[MaturityPlot, HoldFirst]
Options[MaturityPlot] = {PeriodLength -> Years}

MaturityPlot[f_[x_] /; MatchQ[Head[x], CP | DF | YC], opts___Rule] :=
With[{p = PeriodLength /. {opts} /. Options[MaturityPlot],
	listplotopts = FilterOptions[ListPlot, opts],
	qd = DateToString[QuoteDate[x]],
	varstring = ToString[HoldForm[f]]},
	ListPlot[Transpose[{LastPayment[x]/p, f[x]}]//Sort, listplotopts,
	Frame -> {True, True, False, False}, Axes -> False,
	FrameLabel -> {PeriodLength[p], varstring}, PlotRange -> All,
	PlotLabel -> qd] ]


Options[ShowGraph]  = Sort @ {Color -> False, PeriodLength -> Years, 
	PlotRates -> Both}

ShowGraph[df_DF, opts___Rule] :=
	With[{date = DateToString[QuoteDate[df]], 
		f = ForwardCurve[df], z = ZeroCurve[df], 
		k = Length[Knots[df]] - 2,
		effparms = ReportForm[EffectiveParameters[df], {3, 1}],
		form = FunctionalForm[df],
		plotopts = FilterOptions[Plot, opts],
		p = PeriodLength /. {opts} /. Options[ShowGraph],
		which = PlotRates /. {opts} /. Options[ShowGraph],
		maxmat = MaximumMaturity[df]}, (* in days *)
	Plot[Evaluate[
		{If[which === Both || which === Forward, f[m p], {}],
		If[which === Both || which === Zero, z[m p], {}]}], 
				{m, 1/p, maxmat/p}, plotopts,
		PlotRange -> All, Axes -> False,
		Frame -> {True, True, False, False},
		FrameLabel -> {PeriodLength[p], "percent"},
		GridLines -> {{Automatic, {Dashing[{.002, .01}]}}, 
			{Automatic, {Dashing[{.002, .01}]}}},
		PlotLabel -> StringForm["`1`, `2` knots, `3` parms, `4`", 
				date, k, effparms, form],
		PlotStyle -> If[Color /. {opts} /. Options[ShowGraph],
			{Red, Blue}, 
			{{Thickness[.002]}, Thickness[.004]}]]]

PlotByTerm::badterm = "`1` is not a valid term for this data set"

SetAttributes[PlotByTerm, HoldAll];

Options[PlotByTerm] = Sort @ {Color -> False, PeriodLength -> Years,
	ShowPoints -> True}

PlotByTerm[f_[x_], opts___Rule] :=
	With[{termlist = Union[Term[WhenceMD[x]]]},
	Module[{thisterm},
	Print[TableForm[{"", "Here are the available terms to maturity: ", 
		"", termlist}, TableDirections -> {Column},
		TableSpacing -> {0, 3}]]
	While[True, thisterm = Input["\nMake your choice: "];
		If[ thisterm === Exit || thisterm === EndOfFile, Return[],
		If[ MemberQ[termlist, thisterm], Break[], 
			Print["Not available! Read the list and try again.
			To quit, enter Exit"]]]];
	If[thisterm =!= Exit && thisterm =!= EndOfFile, 
		PlotByTerm[f[x], thisterm, opts]] ]]

PlotByTerm[f_[x_] /; MatchQ[Head[x], CP | DF | YC], 
	thisterm_, opts___Rule] :=
	With[{	term = Term[x],
		p = PeriodLength /. {opts} /. Options[PlotByTerm],
		color = Color /. {opts} /. Options[PlotByTerm],
		showpts = ShowPoints /. {opts} /. Options[PlotByTerm],
		listplotopts = FilterOptions[ListPlot, opts], 
		var = f[x],
		varstring = ToString[HoldForm[f]],
		quotedate = DateToString[QuoteDate[x]]},
	Module[{pos, list},
	If[FreeQ[Union[term], thisterm],
		Message[PlotByTerm::badterm, thisterm]; Return[x]];
	pos = Flatten[Position[term, thisterm]];
	list = Sort @ Transpose[{LastPayment[x]/p, var}][[pos]];
	Show[Block[{$DisplayFunction = Identity},
	ListPlot[list, listplotopts, PlotJoined -> True, PlotRange -> All,
		PlotLabel -> StringForm["Term: `1` years, Date: `2`", 
				thisterm, quotedate],
		Frame -> {True, True, False, False},
		FrameLabel -> {PeriodLength[p], varstring},
		Axes -> {True, False},
		GridLines -> {None, {Automatic, {Dashing[{.01, .01}]}}},
		PlotStyle -> {If[color, Hue[Switch[thisterm,
			.5, .8,  1, .3,  2, .4,  3, .6,  5, .7, 
			7, .2,  10, .9,  30, 1,  _, .1] ], 
				Thickness[.002]]} ]],
			If[showpts, Graphics[Point /@ list], {}] ] ]]

SetAttributes[PlotAllByTerm, HoldFirst];
PlotAllByTerm[f_[x_], opts___Rule] :=
	Module[{bigplot},
	Block[{ $DisplayFunction = Identity },
		bigplot = PlotByTerm[f[x], #, opts]& /@ 
			Union[Term[WhenceMD[x]]]];
	Show[bigplot, PlotLabel -> QuoteDate[x], PlotRange-> All] ]

SetAttributes[PlotCombined, HoldFirst];
PlotCombined[df_ /; Head[df] == DF, opts___Rule] :=
	Module[{one, two},
	Block[{$DisplayFunction = Identity},
		one = ShowGraph[df, opts];
		two = PlotAllByTerm[PriceError[df], opts]];
	Show[one, two, PlotLabel -> QuoteDate[df]] ]

Options[PlotRates3D] = Sort @ {PointsPerDay -> 20, 
	InteriorDateTicks -> Automatic, PlotRates -> Forward}

PlotRates3D::qdnotunique = "QuoteDates are not unique"
PlotRates3D::noteqmaxmat = "MaximumMaturity is not equal for all dates"
PlotRates3D::tooshort = "Too few obsevations"

PlotRates3D[x:{_DF ..}, opts___Rule] := 
	With[{lp3dopts = FilterOptions[ListPlot3D, opts],
		f = ForwardCurve /@ Reverse[Sort[x]],
		z = ZeroCurve /@ Reverse[Sort[x]],
		qd = QuoteDate /@ Reverse[Sort[x]],
		max = Union[MaximumMaturity /@ x]},
	Module[{maxmat = max[[1]], len = Length[qd], func, tickpts, 
		ratelist, pointsperday, intdateticks,
		tickfunc = {#, ToString[qd[[#]][[2]]] <> "/" <>
			StringTake[ToString[qd[[#]][[1]]], -2]}&},
	If[Length[Union[QuoteDate /@ x]] != Length[x], 
		Message[PlotRates3D::qdnotunique]; Return[x]];
	If[Length[max] > 1, Message[PlotRates3D::noteqmaxmat]; Return[x]];
	{pointsperday, intdateticks, rate} = 
		{PointsPerDay, InteriorDateTicks, PlotRates} /.
		{opts} /. Options[PlotRates3D];
	If[intdateticks === Automatic && len < 4 || intdateticks === All,
		tickpts = Array[tickfunc, len],
		tickpts = tickfunc /@ Ceiling[len {1/len, 1/3, 2/3, 1}]];
	Switch[rate, Zero, label = "Zero Rate";
		ratelist = Map[Table[#[m], 
			{m, 0.01, maxmat, maxmat/pointsperday}]&, z],
		Forward, label = "Forward Rate";
		ratelist = Map[Table[#[m],
        		{m, 0.01, maxmat, maxmat/pointsperday}]&, f]];
	ListPlot3D[ratelist, lp3dopts, PlotLabel-> label,
		AxesLabel-> {"maturity", "date", "percent"}, 
		MeshRange -> {{0, maxmat/Years}, {1, len}},
		Ticks -> {Automatic, tickpts, Automatic}] ]]


SignificanceByTerm[df_DF, sigma_] :=
	With[{term = Term[df]},
	Module[{pos, temp, termlist = Union[term]},
		pos = Position[Significance[df], _?(Abs[#] >= sigma &)] // 
			Flatten;
		temp = Transpose[{ termlist, 
				Count[term[[ pos ]], #]& /@ termlist } ];
		temp[[
		Complement[Range[Length[temp]], 
			Flatten[Position[Last /@ temp, 0]]] ]] ]]

CountByTerm[md_MD] := 
	TableForm[With[{u = Union[Term[md]]}, 
	{u, Count[Term[md], #]& /@ u}], 
		TableHeadings -> {{"Term", "Count"}, None}, 
		TableSpacing -> {0, 2}, 
		TableAlignments -> {Left, Right}]
(***************************************************************************)
ConstructPayments::badmdobj = "Something is wrong with the data in `1`."
ConstructPayments::badterm = "Warning: There are bad terms. Fix before
proceeding."
ConstructPayments::badquotedate = "Warning: the quote date is out of
bounds. The calendar runs from `1` to `2`"
ConstructPayments::badmaturitydate = "Warning: one or more maturity
dates is out of bounds. The calendar only runs until `1`"

QuoteDateWarning[quotedate_] := 
	If[OrderedQ[{quotedate, FirstDate}] || OrderedQ[{LastDate, quotedate}],
		Message[ConstructPayments::badquotedate, FirstDate, LastDate];
		Return[$Failed]]

MaturityDateWarning[maturitydate_] :=
	If[Or @@ (OrderedQ[{LastDate, #}]& /@ maturitydate),
		Message[ConstructPayments::badmaturitydate, LastDate];
		Return[$Failed]]

TermWarning[term_List, couprate_List] :=
	If[Or @@ MapThread[Negative[#1] && Positive[#2]&, {term, couprate}], 
		Message[ConstructPayments::badterm], 
		Null,
		Message[ConstructPayments::badterm];
		Return[$Failed]]

(*TermWarning[term_] :=
	If[Or @@ Negative /@ Union[term], 
		Message[ConstructPayments::badterm], 
		Null,
		Message[ConstructPayments::badterm];
		Return[$Failed]]*)

(* called by DiscountFunction to create first, lastpayment, rest, t, accrued,
	daystopayments and payments *)

Options[ConstructPayments] = Sort @ {PaymentsPerYear -> 2, FullPrice -> False}
SetAttributes[ConstructPayments, HoldFirst]
ConstructPayments::outarg = "`1` is not a valid argument"
ConstructPayments::notobj = "`1` is not a valid object"

ConstructPayments[md_, opts___Rule] :=
	Which[
	MatchQ[Hold[md], Hold[Out[___]]],
		Message[ConstructPayments::outarg, HoldForm[md]];
		Return[$Failed],
	Not[MatchQ[Head[md], MD]],
		Message[ConstructPayments::notobj, HoldForm[md]];
		Return[$Failed],
	True,
	With[{	setdate = SettlementDate[md],
		matdate = MaturityDate[md],
		quotedate = QuoteDate[md],
		term = Term[md],
		couprate = CouponRate[md],
		price = Price[md],
		id = IDNumber[md],
		nbonds = Length[CouponRate[md]]},
	Module[{daystopayments, daysaccrued, pmtsperyear, 
		rest, fraction, accrued, payments, fullprice,
		daysinperiod},
(* error checking *)
	If[Length[Union[Length /@ {matdate, term, couprate, price, id}]] =!= 1,
		Message[ConstructPayments::badmdobj, HoldForm[md]]; 
		Return[$Failed]];
	QuoteDateWarning[quotedate];
	MaturityDateWarning[maturitydate];
	TermWarning[term, couprate];
	{pmtsperyear, fullprice} = {PaymentsPerYear, FullPrice} /. 
			{opts} /. Options[ConstructPayments];
	{daysaccrued, daystopayments} = ({-First@#, Rest@#}& /@
		MapThread[ PaymentDates[setdate, #1, #2, pmtsperyear]&, 
		{matdate, couprate} ]) // Transpose;
	rest = (Length /@ daystopayments) - 1;
	payments = MapThread[Table[#1/pmtsperyear, {i, 0, #2}]&, 
			{couprate, rest}];
	payments = ReplacePart[#, Last[#] + 100, Length[#]]& /@ payments;
	If[fullprice, daysaccrued = Table[0, {nbonds}]];
	CP[QuoteDate[md], payments, daystopayments, daysaccrued,
		Table[pmtsperyear, {nbonds}], (* daysinperiod, *)
		fullprice, Hold[md]] ]] ]

(* CP objects created by ConstructPayments *)
CP /: QuoteDate[cp_CP] 		:= cp[[1]]
CP /: Payments[cp_CP] 		:= cp[[2]]
CP /: DaysToPayments[cp_CP] 	:= cp[[3]]
CP /: AccruedDays[cp_CP]	:= cp[[4]]
CP /: PaymentsPerYear[cp_CP]	:= cp[[5]]
CP /: FullPriceQ[cp_CP]		:= cp[[6]]
CP /: NextPayment[cp_CP] 	:= First /@ DaysToPayments[cp]
CP /: LastPayment[cp_CP] 	:= Last /@ DaysToPayments[cp]
CP /: RemainingPayments[cp_CP]	:= Length /@ Payments[cp]
CP /: DaysInPeriod[cp_CP]	:= NextPayment[cp] + AccruedDays[cp]
CP /: AccruedInterest[cp_CP]	:= If[FullPriceQ[cp], AccruedDays[cp],
					((CouponRate[cp]/PaymentsPerYear[cp]) *
				(AccruedDays[cp]/(AccruedDays[cp] + NextPayment[cp])))]
CP /: WhenceMD[cp_CP] 		:= cp[[-1]] // ReleaseHold
CP /: WhenceCP[cp_CP] 		:= cp
Format[cp_CP] := StringForm["-CP[`1`]-", QuoteDate[cp]]

CPSelectors = Sort @ {QuoteDate, Payments, DaysToPayments, AccruedDays, 
	PaymentsPerYear, DaysInPeriod, NextPayment, LastPayment, 
	RemainingPayments, AccruedInterest, FullPriceQ}

(#[x_ /; MatchQ[Head[x], DF | YC | SE]] := #[WhenceCP[x]])& /@ 
			DeleteCases[CPSelectors, QuoteDate]

(* PaymentDates is called by ConstructPayments[].
	the 1st element of the list returned is the number of accrued days *)
(* Note: for coupon securities, PaymentDates assumes that if the day
of the month that the security matures is >= 28, that it is the last
day of the month *)
(* For coupon secutities, PaymentDates counts backwards from the
maturity date to _before_ the settlement date to find the number of
accrued days. This will not work properly for bonds with non-standard
coupon payments *)

PaymentDates[sdate_, mdate_, couponrate_ 0 | 0., pmtsperyear_] := 
	{0, DayCount[mdate] - DayCount[sdate]}

PaymentDates[sdate:{syear_, _, _}, mdate:{myear_, mmonth_, mday_},
		couponrate_?Positive, pmtsperyear_] := 
Module[{pmtmonths, mats, pmtdates, pos},
	pmtmonths = Switch[pmtsperyear,
		2, Sort[{mmonth, Mod[mmonth + 5, 12] + 1}],
		1, {mmonth}, _, {mmonth}];
	pmtdates = Flatten[
		Table[{ syear + i - 1, #, 
		If[mday<28, mday, DaysInMonth[{syear+i-1, #}]] }& /@ pmtmonths,
		{i, 0, myear-syear+1}], 1];
	If[pmtsperyear == 2 && mmonth <= 6, pmtdates = Drop[pmtdates, -1]];
	mats = (DayCount /@ pmtdates) - DayCount[sdate];
	(* if 1st nonnegative is after pos 2, drop some *)
	Drop[ mats, Position[mats, _?NonNegative, 1, 1][[1, 1]] - 2 ] ]


(* DaysInMonth finds the last day of each month for a given year.
	It is called by PaymentDates. *)
DaysInMonth[{year_, month_}] := 
	{31, 
	If[Mod[year,4]==0 && Mod[year,100]!=0 || Mod[year,400]==0, 29, 28],
	31, 30, 31, 30, 31, 31, 30, 31, 30, 31}[[ month ]]

(* DayCount is called by PaymentDates and QuoteToPrice (for bills).
	Note:  DayCount is good from 1985 through 2035 *)
DayCount[{year_, month_, day_}] := DayCountList[[ year, month, day ]]

FirstBusinessDay[date_] := 
	Last @ Position[DayCountList, DayCount[date]]

If[ValueQ[DayCountList] // Not, 
		DayCountList = Get["daycount`"]]
(* first valid date *)
FirstDate = {Position[DayCountList, {}][[-1, 1]] + 1, 1, 1} 
(* last valid date *)
LastDate = {Length[DayCountList], 12, 31}

BusinessDaysToSettlement = 1

BusinessDayQ[date:{__Integer}] := 
	DaysBetween[{1900, 1, 1}, date] === DayCount[date]

LastBusinessDayInMonth[{yr_Integer, mo_Integer}] :=
	Module[{lastday = DaysInMonth[{yr, mo}]},
	While[!BusinessDayQ[{yr, mo, lastday}], --lastday];
	{yr, mo, lastday}]

(***************************************************************************)

(* input/output stuff *)

MassageData::noopen = "`1` could not be opened"

Options[MassageData] = {Description -> {}, QuoteToPrice -> True}

(* Note: in these MassageData routines, settlementdate is passed
"globally" to QuoteToPrice *)

MassageData[OpenMarket, filename_String:"openmkt.dat", opts___Rule] := 
	Module[{in, quotedate, maturitydate, term, couprate, price, 
		id, description, pfix, settlementdate},
	description = Description /. {opts} /. Options[MassageData];
	pfix = QuoteToPrice /. {opts} /. Options[MassageData];
	in = ReadList[filename, 
		{Word, Word, Word, Number, Number, Number}];
	If[in === $Failed, Message[MassageData::noopen, filename];
		Return[in]];
	in = Transpose[in];
	quotedate = ToExpression[ in[[1,1]] ];
	id = in[[2]];
	maturitydate = ToExpression[ in[[3]] ];
	settlementdate = FirstBusinessDay @ DaysPlus[quotedate, 
				BusinessDaysToSettlement];
	couprate = in[[4]];
	term = ( (in[[5]] /. 183 -> 182)/365 // Round ) /. 0 -> .5;
	price = If[pfix, MapThread[QuoteToPrice[##, settlementdate]&, 
		{in[[6]], maturitydate, couprate}], in[[6]]];
	MD[quotedate, maturitydate, term, couprate, 
		price, id, description] ]

MassageData[Treasury, filename_:"treasury.dat", opts___Rule] :=
	Module[{in, quotedate, maturitydate, term, couprate, price, 
		id, description},
	description = Description /. {opts} /. Options[MassageData];
	in = ReadList[filename, {Word, Word, Number, Word, Number, Number}];
	If[in === $Failed, Message[MassageData::noopen, filename];
		Return[in]]; 
	in = Transpose[in];
	quotedate = ToExpression[ in[[1, 1]] ];
	id = in[[2]];
	term = in[[3]];
	maturitydate = ToExpression[ in[[4]] ];
	couprate = in[[5]];
	price = in[[6]];
	MD[quotedate, maturitydate, term, couprate, price, 
		id, description] ]

(* a new MassageData for STRIPs *)
MassageData[Strips, filename_:"strips.dat", opts___Rule] :=
	Module[{in, len, quotedate, maturitydate, term, couprate, price, 
		id},
	in = ReadDelimited[filename, {Date, Date, Number}];
	If[in === $Failed, Message[MassageData::noopen, filename];
		Return[in]]; 
	in = Transpose[in];
	quotedate = in[[1, 1]];
	maturitydate = in[[2]];
	price = in[[3]];
	len = Length[Transpose[in]];
	id = Table["N/A", {len}];
	term = Table[-1, {len}];
	couprate = Table[0, {len}];
	MD[quotedate, maturitydate, term, couprate, price, 
		id, {"Strips"}] ]

(* ReadData for Varian's book *)
ReadData::noopen = "`1` could not be opened"

Options[ReadData] = {Description -> {}}

ReadData[filename_String, opts___Rule] :=
	Module[{in, quotedate, maturitydate, term, couprate, price, 
		id, description},
	description = Description /. {opts} /. Options[ReadData];
	in = ReadDelimited[filename, 
		{String, String, Date, Number, Number, Number}];
	If[in === $Failed, Return[$Failed]];
	quotedate = StringToDate[ in[[1,1]] ];
	{id, maturitydate, term, couprate, price} = Rest[Transpose[in]];
	MD[quotedate, maturitydate, term, couprate, 
		price, id, description]]


(* MD objects *)
MD /: QuoteDate[md_MD] 		:= md[[1]]
MD /: MaturityDate[md_MD] 	:= md[[2]]
MD /: Term[md_MD]			:= md[[3]]
MD /: CouponRate[md_MD]		:= md[[4]]
MD /: Price[md_MD]			:= md[[5]]
MD /: IDNumber[md_MD]		:= md[[6]]
MD /: Description[md_MD]	:= md[[7]]
MD /: NumberOfSecurities[md_MD] := Length[Price[md]]
MD /: SettlementDate[md_MD] := FirstBusinessDay@DaysPlus[QuoteDate[md], 
								BusinessDaysToSettlement]
MD /: WhenceMD[md_MD]		:= md

Format[md_MD] := StringForm["-MD[`1`]-", QuoteDate[md]]

MDSelectors = Sort @ {QuoteDate, MaturityDate, Term, CouponRate, 
	Price, IDNumber, Description, SettlementDate, NumberOfSecurities}

(#[x_ /; MatchQ[Head[x], CP | DF | YC | SE]] := #[WhenceMD[x]])& /@ 
	DeleteCases[MDSelectors, QuoteDate]

MDSubset[md_MD, pos_List, opts___Rule] :=
	With[{desc = Description /. {opts} /. Options[MassageData]},
	MD[QuoteDate[md], Sequence @@ Transpose[Transpose[
	Through[{MaturityDate, Term, CouponRate, Price, IDNumber}[md]]][[pos]]],
	If[desc === {}, Description[md], desc] ]]

(* QuoteToPrice is called by MassageData[OpenMarket] 
	to transform quoted prices into decimal prices *)
QuoteToPrice[quote_, maturitydate_, couponrate_?Positive, 
		settlementdate_] := 
	Floor[quote] + (quote - Floor[quote])100/32

QuoteToPrice[quote_, maturitydate_, couponrate_ 0 | 0., 
		settlementdate_] := 
	100 - (quote/360)(DayCount[maturitydate] - DayCount[settlementdate])

DataOut::notobj = "`1` is not a valid object"

DataOut[OpenMarket, yc_, filename_:"output.txt"] :=
	If[Not[MatchQ[Head[yc], YC]],
		Message[DataOut::notobj, HoldForm[yc]];
		Return[$Failed],
	WriteDelimited[filename, 
		Transpose[{
(* date *)	Table[DateToString[QuoteDate[yc]], {Length[IDNumber[yc]]}],
		IDNumber[yc],
		ReportForm[#, {4, 3}]& /@ PriceError[yc],
		ReportForm[#, {3, 1}]& /@ Abs[Significance[yc]],
		ReportForm[#, {5, 3}]& /@ Yield[yc],
		ReportForm[#, {5, 3}]& /@ PredictedYield[yc],
		ReportForm[#, {3, 3}]& /@ YieldError[yc],
		ReportForm[#, {4, 3}]& /@ N[LastPayment[yc]/Years]
		}] ] ]

(* reports *)
DurationReport::system = "Add $OperatingSystem to report function"
Options[DurationReport] = {PrintReport -> False, ShowReport -> True}

DurationReport[yc_/;Head[yc] === YC || Head[Object[yc]] === YC, opts___Rule] :=
With[{	print = PrintReport /. {opts} /. Options[DurationReport],
	show = ShowReport /. {opts} /. Options[DurationReport]},
Module[{tab1, tab2, stream},
tab1 = TableForm[Transpose[
	Through[{
		PaddedForm[#, 2]& /@ Term[#]&, 
		With[{cp = CouponRate[#]},
			MapThread[If[#2 == 0,
				StringForm["`1`    ", #1],
				StringForm["`1`-`2`", #1, InputForm[#2]]]&,
				{Floor[cp], Rationalize[cp-Floor[cp]]}]]&,
		DateToString /@ MaturityDate[#]&, 
		ReportForm[#, {5, 2}]& /@ N[LastPayment[#]/Years]&,
		ReportForm[#, {5, 2}]& /@ MacaulayDuration[#]&, 
		ReportForm[#, {5, 2}]& /@ Duration[#]&, 
		ReportForm[#, {5, 2}]& /@ Convexity[#]&
		}[yc] ]], 
	TableSpacing -> {0, 2}, 
	TableHeadings -> {Automatic, 
			{
			"Term", 
			"Coupon\nRate",
			"Maturity\nDate",
			"Years\nto Mat.", 
			"Duration", 
			"Modified\nDuration\n", 
			"Convexity"
			}},
	TableAlignments -> {Center, Right}];
tab2 = TableForm[{
		DateToString[QuoteDate[yc]], 	(* title *) 
		tab1},				(* body *)
	TableAlignments -> {Center}];
	If[print, Switch[$OperatingSystem, 
			"Unix", OutputForm[tab2] >> !lpr,
			"MSDOS", OutputForm[tab2] >> prn,
			_, Message[DurationReport::system]]];
	If[show, tab2]]]

OpenMarketReport::system = "Add $OperatingSystem to report function"

Options[OpenMarketReport] = {PrintReport -> False, ShowReport -> True}

OpenMarketReport[yc_/;Head[yc] === YC || Head[Object[yc]] === YC, opts___Rule] :=
With[{	print = PrintReport /. {opts} /. Options[OpenMarketReport],
	show = ShowReport /. {opts} /. Options[OpenMarketReport]},
Module[{tab1, tab2, stream},
tab1 = TableForm[Transpose[
	Through[{
		PaddedForm[#, 2]& /@ Term[#]&, 
		With[{cp = CouponRate[#]},
			MapThread[If[#2 == 0,
				StringForm["`1`    ", #1],
				StringForm["`1`-`2`", #1, InputForm[#2]]]&,
				{Floor[cp], Rationalize[cp-Floor[cp]]}]]&,
		DateToString /@ MaturityDate[#]&, 
		ReportForm[#, {4, 2}]& /@ N[LastPayment[#]/Years]&,
		ReportForm[#, {3, 2}]& /@ PriceError[#]&, 
		ReportForm[#, {4, 2}]& /@ Abs[Significance[#]]&,
		ReportForm[#, {4, 2}]& /@ Yield[#]&,
		ReportForm[#, {4, 2}]& /@ PredictedYield[#]&,
		ReportForm[#, {3, 2}]& /@ (YieldError[#]/100.)&
		}[yc] ]], 
	TableSpacing -> {0, 2}, 
	TableHeadings -> {Automatic, 
			{
			"Term", 
			"Coupon\nRate",
			"Maturity\nDate",
			"Years\nto Mat.", 
			"Price\nError", 
			"Std.\nErrors", 
			"Yield",
			"Pred.\nYield",
			"Yield\nError"
			}},
	TableAlignments -> {Center, Right}];
tab2 = TableForm[{
		DateToString[QuoteDate[yc]], 	(* title *) 
		tab1},				(* body *)
	TableAlignments -> {Center}];
	If[print, Switch[$OperatingSystem, 
			"Unix", OutputForm[tab2] >> !lpr,
			"MSDOS", OutputForm[tab2] >> prn,
			_, Message[OpenMarketReport::system]]];
	If[show, tab2]]]

BreushPagan[df_DF, xdata__List] :=
	With[{	kl = KeepList[df]},
	With[{	n = Length[ kl ],
		e = PriceError[df][[ kl ]],
		x = N[Prepend[#, 1]& /@ Transpose[{xdata}][[ kl ]]]},
	With[{	y = e^2/(e.e/n)},
	With[{	ydiff = x.Inverse[Transpose[x].x].Transpose[x].y - Mean[y]},
		{ydiff.ydiff, Length[{xdata}]} ]]]]


End[]
Protect[MD, CP, DF, YC, SE]
EndPackage[]
