(* $Id: LogSpec.m,v 2.1 1995/10/01 *)
(* :Title: LogSpec *)
(* :Authors:  Professor Luke Froeb, 
    with assistance from 
    Steve Sweeney
    Owen Graduate School of Management
    Vanderbilt University
    Nashville, TN  37203
    luke.froeb@macpost.vanderbilt.edu
    615 343-6009 
*)
(* :Summary:
    Functions for Log spectral analysis of time series:
    measuring and comparing smoothness and innovation variances
*)
(* :Reference: Usage messages only. *)
(* :Context: logspec`logspec` *)
(* :Package Version 2.1 *)
(* :Mathematica Version 2.2 *)
(* :Copyright: Froeb, 1995 *)
(* :History: *)
(* :Keywords: frequency, smoothness, spectrum, innovation variance *)
(* :Requirements: No special system requirements. *)
(* :Sources:
    Luke Froeb and Robert Koyak, Measuring and Comparing Smoothness 
      in Time Series, Journal of Econometrics  64
      (1994) 97-122.
    Luke Froeb, An Innovation Variance Ratio Test, manuscript.
*)

(* :Limitation: none known. *)
(* :Discussion: *)

BeginPackage["logspec`LogSpec`"]

LogSpec::usage = "logspec.m defines functions \
	used to test whether one time series is \"smoother\" or has a \
	larger innovation variance than \
	another.  Detrending, spectrum computation, and Differencing \
	functions are also defined. Available functions are: \
	Difference, DeMean, DeTrend, Periodogram, \
	Smoothness, LogInnovationVariance."

Difference::usage = "Difference[list,J(k)] returns the kth \
	difference of a time series, list. Default value of k is 1.  \
	The output vector has length N-k, where N = number \
	of observations in list."

DeMean::usage = "DeMean[list] subtracts the mean of 'list' from \
	each element of 'list.'"

DeTrend::usage = "DeTrend[list] takes a time series vector, 'list' \
	regresses it on a constant and a \
	trend, and returns the vector of residuals."

Periodogram::usage = "Periodogram[list] takes a (usually \
	detrended and demeaned) time series, 'list' (less the first \
	observation in the case of an even number of observations), \
	and returns the periodogram.  The \
	periodogram is normalized so that its mean is \
	the variance of the time series."

twoPeriods::usage = "twoPeriods is an option for the \
	InnovationVariance[p1,p2,twoPeriods->False], and for \
	Smoothness[frequency,p1,p2,twoPeriods->False].  It determines \
	standard error formulas for the difference in the  \
	two parameters.  The default value is False."

LogInnovationVariance::usage = 
	"LogInnovationVariance[Log[periodogram1], \
	(Log[periodogram2]),(twoPeriods->False)] \
	inputs 1, or 2 log periodograms and \
	returns a list of doubles, \
	{Log Innovation Variance [i], standard-error[i]}.  \
	If called with two log periodograms, the 3 doubles \
	correspond to #1, #2, (#1-#2).  When called using the \
	twoPeriods->True option, the standard errors \
	are computed by assuming the series are from different time \
	periods."

Smoothness::usage = 
	"Smoothness[frequency, Log[periodogram1], \
	(Log[periodogram2]),(twoPeriods->False)] \
	inputs 1 or 2 log periodograms and returns a \
	list of triples, {frequency[i], \
	smoothness coefficient[i], standard error[i]}.  \
	If called with two log periodograms, the 3 triples \
	correspond to #1, #2, (#1-#2).  When called using the \
	twoPeriods->True option, the standard errors \
	are computed by assuming the series are from different time \
	periods."

Options[InnovationVariance] := {twoPeriods -> False}

Options[Smoothness] := {twoPeriods -> False}

Begin["`Private`"]

FilterOptions[command_Symbol, opts___]:=
Module[{keywords = First /@ Options[command]},
    Sequence @@ Select[ {opts}, MemberQ[keywords, First[#]]& ] ]

Difference::short = "Length of `1` is <= the order of \
	differencing, `2`."

Difference::vector = "Argument must be a nonempty vector."

DeMean::vector = "Argument must be a nonempty vector."

DeTrend::vector = "Argument must be a nonempty vector."

Periodogram::lowobs = "Spectral Analysis of a time series with \
	fewer than 50 or so observations is unusual."

Periodogram::vector = "Argument must be a nonempty vector \
    (usually a detrended and demeaned time series)."

Smoothness::lowobs = "Spectral Analysis of a time \
	series with fewer than 50 or so observations is unusual."

Smoothness::vector = "Argument should be a nonempty vector."

Smoothness::badmatch = "Log Periodogram arguments should have same vector \
	length."

Difference[list_, k_Integer:1]:= Drop[list, k] - Drop[list, -k] /; 
  (VectorQ[list] || Message[Difference::vector]) && 
  (k<Length[list] || Message[Difference::short, Short[list, 1], k])
    
DeMean[list_]:=
  list-(Plus @@ list)/Length[list] /; 
  VectorQ[list] || Message[DeMean::vector]

DeTrend[list_]:=
Module[{i,fit,nobs=Length[list]},
  If[EvenQ[nobs],(Drop[list,1];--nobs)];
  With[ {fit=Fit[list,{1,i},i]}, Table[list[[i]]-fit,{i,1,nobs}] ]
] /; (VectorQ[list] && 0<Length[list]) || Message[DeTrend::vector]

Periodogram[dtlist_]:=
Module[{nobs=Length[dtlist]},
  If[If[EvenQ[nobs],(Drop[dtlist,1];--nobs),nobs]<50, 
      Message[Periodogram::lowobs]]; 
  ((nobs-1)/nobs)Abs[Take[Fourier[dtlist],{2,(nobs+1)/2}]]^2 
] /; (VectorQ[dtlist] && 0<Length[dtlist]) || 
        Message[Periodogram::vector]

Smoothness[freq_,p1_,p2_,opts___Rule]:=
      Module[{},
      If[twoPeriods/.{opts}/.Options[Smoothness],
         doSmoothness2Periods[freq,p1,p2],
         doSmoothness[freq,p1,p2]
        ]
      ]
      
doSmoothness[freq_, p1_, p2_]:=
Module[{dd,d1,d2,f=N[freq],i,pd=p1-p2,pnobs=Length[p1]},
  If[ f>N[2Pi], f=Mod[f,N[2Pi]] ];
  If[(nobs=2*pnobs+1)<50,
      Message[SmoothnessTest::lowobs]];
  d1=Array[(p1[[#+1]]-p1[[#]])&,pnobs-1];
  d2=Array[(p2[[#+1]]-p2[[#]])&,pnobs-1];
  dd=d1-d2;
  i=Round[N[f*nobs/(2Pi)-1]];
  {
   {(Plus @@ Take[p1,i])/i - 
    (Plus @@ Drop[p1,i])/(pnobs-i),
    (N[Pi^2/6](1/i+1/(pnobs-i)))^.5},
   {(Plus @@ Take[p2,i])/i - 
    (Plus @@ Drop[p2,i])/(pnobs-i),
    (N[Pi^2/6](1/i+1/(pnobs-i)))^.5},
   {(Plus @@ Take[pd,i])/i - 
    (Plus @@ Drop[pd,i])/(pnobs-i),
    Max[
      Sqrt[
        N[Pi^2/3]*((1-#1.#3/Sqrt[#1.#1*#3.#3])/i + 
          (1-#2.#4/Sqrt[#2.#2 * #4.#4])/(pnobs-i-1))
      ],
      Sqrt[#5.#5/(2i(i+1)) + 
        #6.#6/(2(pnobs-i-1)*(pnobs-i-2))
      ]
    ]
  }} /.
      { #1 -> Take[d1,i],
        #2 -> Drop[d1,i],
        #3 -> Take[d2,i],
        #4 -> Drop[d2,i],
        #5 -> Take[dd,i],
        #6 -> Drop[dd,i]
      }
] /; ((VectorQ[p1] && VectorQ[p2] && 0<Length[p1]) ||  
       Message[Smoothness::vector]) && 
     (Length[p1]==Length[p2] || 
      Message[Smoothness::badmatch])
 
doSmoothness2Periods[freq_, p1_, p2_]:=
Module[{s1,s2,i1,i2,dd,d1,d2,f=N[freq],
        pnobs1=Length[p1],pnobs2=Length[p2]},
  If[ f>N[2Pi], f=Mod[f,N[2Pi]] ];
  If[Min[(nobs1=2*pnobs1+1),(nobs2=2*pnobs2+1)]<50,
      Message[SmoothnessTest::lowobs]];
    i1=Round[N[f*nobs1/(2Pi)-1]];
    i2=Round[N[f*nobs2/(2Pi)-1]];
{
   {s1=(Plus @@ Take[p1,i1])/i1 - 
       (Plus @@ Drop[p1,i1])/(pnobs1-i1),
    (N[Pi^2/6](1/i1+1/(pnobs1-i1)))^.5},
   {s2=(Plus @@ Take[p2,i2])/i2 - 
    (Plus @@ Drop[p2,i2])/(pnobs2-i2),
    (N[Pi^2/6](1/i2+1/(pnobs2-i2)))^.5},
   {s1-s2,
    Sqrt[
         (N[Pi^2/6](1/i1+1/(pnobs1-i1))) +
         (N[Pi^2/6](1/i2+1/(pnobs2-i2)))
        ]
  }} 
] /; ((VectorQ[p1] && VectorQ[p2] && 0<Length[p1]) ||  
       Message[SmoothnessTest::vector])
              
Smoothness[f_,p_]:=
Module[{j,pi26=N[Pi^2/6],pnobs=Length[p],
        nobs=2 Length[p]+1,SE,Coeff},
        If[ f>N[2Pi], f=Mod[f,N[2Pi]] ];
        If[(nobs=2*pnobs+1)<50,
            Message[SmoothnessTest::lowobs]];
        j=Round[N[f*nobs/(2Pi)-1]];
       {f,
        Coeff=(Plus @@ Take[p,j])/(j)-(Plus @@ Drop[p,j])/(pnobs-j),
        SE=Sqrt[pi26(1/j + 1/(pnobs - j-1))]}
      ] /; (* Do only if this is true: *) (VectorQ[p] && 0<Length[p]) ||
           (* otherwise: *) Message[Smoothness::vector]

LogInnovationVariance[p1_]:=
Module[{SE1,LogIVar1,pnobs1=Length[p1]},
       {LogIVar1=(Plus @@ p1)/pnobs1 + N[EulerGamma],
        SE1=     N[Pi]/(6 pnobs1)^.5}
      ] /; (* Do only if this is true: *) (VectorQ[p1] && 0<Length[p1]) ||  
           (* otherwise: *) Message[Smoothness::vector]

LogInnovationVariance[p1_,p2_,opts___Rule]:=
      Module[{},
      If[twoPeriods/.{opts}/.Options[InnovationVariance],
         doLogInnovationVariance2Periods[p1,p2],
         doLogInnovationVariance[p1,p2]
        ]
      ]
            
doLogInnovationVariance[p1_, p2_]:=
Module[{dd,d1,d2,SE,LogIVarRatio,zStat,LogIVar1,LogIVar2,SEIVar,
        pd=p1-p2,pi23=N[Pi^2/3],pnobs=Length[p1]},
  d1=Array[(p1[[#+1]]-p1[[#]])&,pnobs-1];
  d2=Array[(p2[[#+1]]-p2[[#]])&,pnobs-1];
  dd=d1-d2;
  SE=Max[
         SE1=Sqrt[pi23*(1-d1.d2/Sqrt[d1.d1*d2.d2])/pnobs],
         SE2=Sqrt[dd.dd/(2 pnobs*(pnobs-1))]
        ];
    {
     {LogIVar1=     (Plus @@ p1)/pnobs + N[EulerGamma],
      SEIVar=      N[Pi]/(6 pnobs)^.5},
     {LogIVar2=     (Plus @@ p2)/pnobs + N[EulerGamma],
      SEIVar},
     {LogIVarRatio=(LogIVar1-LogIVar2),
      SE}
    } 
] /; ((VectorQ[p1] && VectorQ[p2] && 0<Length[p1]) ||  
         Message[Smoothness::vector]) && 
     (Length[p1]==Length[p2] || Message[Smoothness::badmatch])
     
doLogInnovationVariance2Periods[p1_, p2_]:=
Module[{SE,SE1,SE2,LogIVarRatio,LogIVar1,LogIVar2,
        pnobs1=Length[p1],pnobs2=Length[p2]},
    {
     {LogIVar1=     (Plus @@ p1)/pnobs1 + N[EulerGamma],
      SE1=          N[Pi]/(6 pnobs1)^.5},
     {LogIVar2=     (Plus @@ p2)/pnobs2 + N[EulerGamma],
      SE2=          N[Pi]/(6 pnobs2)^.5},
     {LogIVarRatio= (LogIVar1-LogIVar2),
      SE=           (SE1^2+SE2^2)^.5}
    } 
] /; ((VectorQ[p1] && VectorQ[p2] && 0<Length[p1] && 0<Length[p2]) ||  
         Message[Smoothness::vector])
         
End[]

SetAttributes[Difference, ReadProtected];
SetAttributes[DeMean, ReadProtected];
SetAttributes[DeTrend, ReadProtected];
SetAttributes[Periodogram, ReadProtected];
SetAttributes[Smoothness, ReadProtected];
SetAttributes[doSmoothness, ReadProtected];
SetAttributes[doSmoothness2Periods, ReadProtected];
SetAttributes[LogInnovationVariance, ReadProtected];
SetAttributes[doLogInnovationVariance, ReadProtected];
SetAttributes[doLogInnovationVariance2Periods, ReadProtected];

Protect[Difference, DeMean, DeTrend, Periodogram, 
Smoothness,
doSmoothness,
doSmoothness2Periods,
LogInnovationVariance,
doLogInnovationVariance,
doLogInnovationVariance2Periods];

EndPackage[]

