(* $Id: Merger.m,v 2.1 1995/08/21 *)

(* :Title: Merger Package *)

(* :Authors:  
    Luke Froeb, Owen School of Management, Vanderbilt University,
    luke.froeb@macpost.vanderbilt.edu,
    Greg Werden, Antitrust Division, U.S. Department of Justice,
    WerdenGJ@aol.com, 
    August, 1995
*)

(* :Summary:
    Functions for simulating the effects of mergers with a nested Logit 
    demand system.
*)

(* :Reference: Usage messages only. *)

(* :Context: Merger`Merger` *)

(* :Package Version 2.0 *)

(* :Mathematica Version 2.2 *)

(* :Copyright: 1995 *)

(* :History: LogitMergers.m  1992;  NestedLogitMerger.m  1993 *)

(* :Keywords: Mergers, Simulation, Bertrand, Cournot, Differentiated Products, Oligopoly, Logit, Nested Logit *)

(* :Requirements: No special system requirements. *)
 
(* :Sources:
    Gregory J. Werden and Luke M. Froeb, The Effects of Mergers in Differentiated Products \
    Industries:  Logit Demand and Merger Policy, J. Law, Economics, & Organization, 10 (1994) 407-426, \
    Luke M. Froeb, Timothy J. Tardiff, and Gregory J. Werden, The Demsetz Postulate \
    and the Effects of Mergers in Differentiated Products Industries, \
    Department of Justice working paper.*)

(* :Limitation: For some parameter values the package may not converge. *)

(* :Discussion: *)

BeginPackage["merger`Merger`"]

Merger::usage = \
	"This package defines functions \
	used to simulate mergers in differentiated \
	and homogenous product industries.  \
    Available functions:  \
    ChoiceProbabilities, \
    DemandConstants, \
    MarginalCosts, \
    PostMerger, \
    PrintResults, \
    DiffProdMerger, \
    DiffProdMergerMinCost, \
    HomoProdMerger, \
    PostMergerC."

DemandConstants::usage = "DemandConstants[(nest_List), (nestparam), \
                b, p0_List, q0_List] solves the n demand \
				equations, q(p0) = q0, for the n-1 demand constants, \
				{a(j)}, which characterize the random \
				utility model:  U(i,j) = a(j) - b*p(j) + e(i,j).  a[n] \
				is arbitrarily set equal to 10.  \
				Nestparam is the nesting parameter:  0 is a strong \
				nest; 1 is a flat logit. \
				Nest is a vector with the firms that are in a nest, \
				e.g., {1,3,4}.  \
				The last good is the 'no purchase' option or \
				'outside' good, and its price must be set equal to zero."

MarginalCosts::usage = "MarginalCosts[(nest_List), (nestparam), a_List, b, \
                p0_List, q0_List] calculates n-1 marginal costs \
				from the n-1 pre-merger first-order conditions produced by \
				differentiating each firm's profit funtion with respect to its own price.  \
				Nest is a vector with the firms that are in a nest, e.g., {1,3,4}.  \
				Nestparam is the nesting parameter:  0 is a strong nest, and 1 is a flat logit.  \
				The last good is the 'no purchase' or 'outside' good and its \
				price must be set equal to zero."

ChoiceProbabilities::usage = "ChoiceProbabilities[b, p0_List, q0_List, elasticity] \
				inputs the coefficient on price, pre-merger prices, pre-merger \
				quantities or market shares, and the aggregate elasticity of demand, \
				and ouputs a vector of choice probabilities constructed so that \
				the choice probability for the 'no purchase' option or 'outside' good \
				is consistent with the specified aggregate elasticity of demand \
				for the remaining, 'inside' goods.  \
				The length of p0 and q0 must equal the number of firms plus one.  \
				The price on the outside good can take any value, i.e. it is meaningless)."

PostMerger::usage = "PostMerger[merging_List, (nested_List), (nestParam), a_List, \
                b, e_, c0_List, p0_List] inputs demand parameters from a logit demand \
                system, pre-merger marginal costs, prices, and quanties or market shares, \
                and outputs post-merger costs, prices, and choice probabilities \
				resulting from a merger among firms in the merging list."

PostMergerC::usage = "PostMergerC[merging_List, elasticity_, p0_, s0_List] \
                computes the effects of a merger in a Cournot industry and outputs \
                the vector {{k0,p0,q0},{k1,p1,q1}}"

DiffProdMerger::usage = "DiffProdMerger[merging_List, (nested_List), (nestParam_), \
                {b,elasticity}, {FirmNames,p0,q0}] \
                This function calls all the other functions to calculate \
                the effects of a merger in a differentiated product industry.  \
                Merging is a list of the merging firms, e.g., {1,2,3}; \
                nested is a list of the products in the demand nest, e.g., {2,3,4}; \
                nestParam is a value between 0 and 1 that controls the strength of the nest--\
                1 is a flat logit, and 0 is a strong nest; \
                b is an estimated cross elasticity of demand parameter from the logit model; \
                p0 is the vector of pre-merger prices; \
                q0 is the vector of pre-merger quantities or market shares; \
                elasticity is the pre-merger aggregate elasticity for the inside goods; \
                FirmNames is a vector of firm names.  \
                Note that the last choice is the 'no purchase' option or 'outside' good \
                with a pre-merger price and quantity of zero."

DiffProdMergerMinCost::usage = "DiffProdMergerMinCost[merging_List, (nested_List), (nestParam_), \
                {b,elasticity}, {FirmNames,p0,q0}] \
                This function calls all the other functions to calculate \
                the effects of a merger in which the merged firm \
                produces at the minimum of the marginal costs of the merging \
                firms."

PrintResults::usage = "PrintResults[merging_List, (nest_List), (nestParam), \
                a_List, b_, e_, c0_List, p0_List, q0_List, \
                c1_List, p1_List, q1_List, firmNames_List] \
                inputs parameters and pre- and post-merger marginal costs, \
                prices, and quantities, and \
                outputs the full results following a merger.  \
	            The last good is the 'no purchase' or 'outside' good, \
	            and its price must be set equal to zero."

HomoProdMerger::usage = "HomoProdMerger[merging_List, elasticity_ ,p0_, s0_List, \
                FirmNames_List] inputs model primitives for simulation of a merger \
                in a linear Cournot model in which firms are distinguished \
                on the basis of their marginal costs, which are determines by their \
                capital stocks.  It computes and outputs the effects of a merger."

Begin["`Private`"]

FilterOptions[command_Symbol, opts___]:=
Module[{keywords = First /@ Options[command]},
    Sequence @@ Select[ {opts}, MemberQ[keywords, First[#]]& ] ]

ChoiceProbabilities::negative = "WARNING: 100%+ markt share \
on outside good, results are meaningless."

ChoiceProbabilities[B_,p0_List,q0_List,elasticity_]:=
     (* This function calculates the share of the           *)
     (* outside good associated with an initial elasticity. *)
     (* Note that the "no purchase" choice is included last *)
     (* in the vectors, p0 and q0.  This function inputs    *)
     (* {B,p0,q0,FirmNames}, outputs q1, and can be         *)
     (* called by Merger functions.                         *)
     (* q0 can be in either quantities or shares.           *)
     Module[{n,q00,pbar,x,q1},
	 n=Length[p0];
	 q00=q0/(Plus @@ q0);
   	 pbar=Plus @@ Drop[p0*q00/(1-q00[[n]]),-1];
     x=elasticity/(B*pbar);
     q1=q00*(1-x)/(1-q00[[n]]);
	 q1[[n]]=x;
	 If[Min[q1]<0,Message[ChoiceProbabilities::negative]];		
     q1
     ]    
         
DemandConstants[B_,p0_List,q0_List]:=
(* This function calculates the choice specific        *)
(* constants from the demand identities: q(p0)=q0.     *)
     Module[{},
     10 + B p0 + Log[q0/Last[q0]]
     ]
     
DemandConstants[nest_,nestParam_,b_,p0_List,q0_List]:=
(* This function calculates the choice specific        *)
(* constants from the demand identities: q(p0)=q0.     *)
(*                                                     *)
(* The "nest" is a list of integers denoted nested     *)
(* firms.                                              *)
(*                                                     *)
(* The nestParam ranges from zero to one.  A value     *)
(* of one means a flat logit, and a value of zero      *)
(* means that the goods in the nest are perfect        *)
(* substitutes for one another.                        *)
(*                                                     *)
(* The last good is the "no purchase" or outside       *)
(* good and its price should be set equal to zero.     *)
Module[
       {n,a,aStart,notNested,sumNested,sumNotNested,denom,
        q,startingValues,eqns,results},
n=Length[p0];
aStart=10 + b p0 + Log[q0/q0[[n]]];
notNested=Complement[Table[i,{i,n}],nest];
sumNested=(Plus @@ Part[Exp[#/nestParam],nest]) & @
             (Array[a,n]-b p0 );
sumNotNested=(Plus @@ Part[Exp[#],notNested]) & @
             (Array[a,n]-b p0 );
denom=sumNested^nestParam + sumNotNested;
q=Table[
        If[MemberQ[nest,i],
                           Exp[#/nestParam]*sumNested^(nestParam-1),
                           Exp[#]
          ] & @ (Array[a,n]-b p0)[[i]],
       {i,n}]/denom;

startingValues=Table[{a[i],aStart[[i]]},{i,1,n-1}];
eqns=Take[q-q0,{1,n-1}]/.a[n]->10;

results=FindRoot @@ 
       FlattenAt[{eqns,startingValues,MaxIterations->500},{2}];

Array[a,n]/.Flatten[{results,a[n]->10}]
]
      
MarginalCosts::negative = "WARNING: negative inferred \
marginal costs on at least one of the firms.  This means \
that premerger pricing is more competitive than was assumed \
by the model, i.e. Bertrand equilibrium.  \
Total welfare changes may be misleading.  However, if only \
the small firms have inferred negative marginal costs, \
it may be better to take them out of the choice set and \
increase the aggregate elasticity."

MarginalCosts[a_List,B_,p0_List,q0_List]:=
(* This function calculates the marginal costs from    *)
(* premerger first order conditions, assuming that     *)
(* a current Nash equilibrium in prices. Good "n" is   *)
(* the "outside" good which is produced at marginal    *)
(* cost, price=cost=1.                                 *)
     Module[{cc},
     cc=Flatten[{
                 Take[p0-1/(B(1-q0)),Length[p0]-1],0
               }];
     If[Min[cc]<0,Message[MarginalCosts::negative]];		       
     cc
     ]

MarginalCosts[nested_,nestParam_,a_List,B_,p0_List,q0_List]:=
(* This function calculates the marginal costs from    *)
(* premerger first order conditions, assuming that     *)
(* a current Nash equilibrium in prices. Good "n" is   *)
(* the "outside" good which is produced at marginal    *)
(* cost, price=cost=0.                                 *)
     Module[{n,pp,cc,notNested,sumNested,sumNotNested,denom,p,qq,q,pR,zeroList,prof,dprof,c},
       n=Length[p0];
       cc=Array[c,n];
       pp=Array[p,n];
       notNested=Complement[Table[i,{i,n}],nested];
       sumNested=   Plus @@ Part[Exp[(a-B pp)/nestParam],nested];      
       sumNotNested=Plus @@ Part[Exp[(a-B pp)          ],notNested];
       denom=sumNested^nestParam + sumNotNested;
       qq=Table[
                If[MemberQ[nested,i],
                                     Exp[(a[[i]]-B pp[[i]])/nestParam]*sumNested^(nestParam-1),
                                     Exp[ a[[i]]-B pp[[i]] ]
                  ]/denom,
               {i,n}
               ];
               
     zeroList=Table[0,{i,1,n}];
     prof:=(pp-cc)*qq;
     dprof=Table[ D[ prof[[i]],p[i] ],{i,1,n}];
     rules=Table[ p[i]->p0[[i]]  , {i,1,n} ];
     cc=cc/.Flatten[Solve[zeroList==dprof/.rules,cc]];
     cc[[n]]=0;
     If[Min[cc]<0,Message[MarginalCosts::negative]];		
     cc
     ]

PostMerger[merging_List,aa_List,b_,c0_List,p0_List]:=
(* This function calculates the postmerger costs,      *)
(* prices and quantities following a merger between    *)
(* firms in the group.  "No purchase" choice is last   *)
(* in the vectors, p0 and q0. "PostMerger" means that  *)
(* the merged firm internalizes the price effects of   *)
(* the merged firms on each other.                    *)
Module[{n,q,a,c,p,profits,mergedProfits,
        rules,eqns,startingValues,results},
n=Length[p0];
q=Exp[#]/Apply[Plus,Exp[#]]& @ (Array[a,n]-b Array[p,n]);
profits=(Array[p,n]-Array[c,n])q;
mergedProfits= Plus @@ Part[profits,merging];
rules=Flatten[{
              Table[a[i]->aa[[i]],{i,n}],
              Table[c[i]->c0[[i]],{i,n}],
              p[n]->0
             }]; 
eqns=Table[If[
                   MemberQ[merging,i],
                   D[mergedProfits,p[i]],
                   D[profits[[i]],p[i]]
                  ],{i,n-1}]/.rules;             
startingValues=Table[{p[i],p0[[i]]},{i,1,n-1}];
results=FindRoot @@ 
  FlattenAt[{eqns,startingValues,MaxIterations->500},{2}];
{
 c0,
 Array[p,n]/.Flatten[{results,rules}],     
 N[q/.Flatten[{results,rules}]]
}
]    	

PostMerger[merging_List,nest_List,nestParam_,a0_List,b_,c0_List,p0_List]:=
(* This function calculates the postmerger costs,      *)
(* prices and quantities following a merger between    *)
(* firms in the group.  "No purchase" choice is last   *)
(* in the vectors, p0 and q0. "PostMerger" means that  *)
(* the merged firm internalizes the price effects of   *)
(* the merged firms on each other.                    *)
Module[{n,a,p,notNested,sumNested,sumNotNested,denom,
        q,prof,dprof,profMerging},
     
n=Length[p0];

notNested=Complement[Table[i,{i,n}],nest];
sumNested=(Plus @@ Part[Exp[#/nestParam],nest]) & @
             (a0-b Array[p,n] );
sumNotNested=(Plus @@ Part[Exp[#],notNested]) & @
             (a0-b Array[p,n] );
denom=sumNested^nestParam + sumNotNested;

q=Table[
        If[MemberQ[nest,i],
                           Exp[#/nestParam]*sumNested^(nestParam-1),
                           Exp[#]
          ] & @ (a0-b Array[p,n])[[i]],
       {i,n}]/denom;
          

prof=(Array[p,n]-c0)*q;
    
profMerging= Plus @@ Part[prof,merging];
eqns=        Table[
                   If[
                      MemberQ[merging,i],
                      D[profMerging,p[i]],
                      D[prof[[i]],  p[i]]
                     ],{i,n-1}];
                     
startingValues=Table[{p[i],p0[[i]]},{i,1,n-1}]; 

results=FindRoot @@ 
       FlattenAt[{eqns/.p[n]->p0[[n]],startingValues,MaxIterations->500},{2}];

rules=Flatten[{results,p[n]->p0[[n]]}];
     {
     c0,
     Array[p,n]/.rules//N,
     q/.rules//N
     }
]
         	
PrintResults[merging_List,nest_List:{},nestParam_:1,
                    a_List, B_, elasticity_,
                    c0_List, p0_List, q0_List,
                    c1_List, p1_List, q1_List,
                    FirmNames_List]:=
(* This function calculates the and tabulates          *)
(* variables associated with a post merger change      *)
(* from {c0,p0,q0} to {c1,p1,q1}.                      *)
     Module[{n,prof0,prof1,dprof1,g,k,h,printNames,
             PreMerger,PostMerger,Change,PctChange,
             notMerging,notNested,sumNested,sumNotNested,denom},
     n=Length[p0];
     prof0=(p0-c0)*q0;
     prof1=(p1-c1)*q1;
	g[x_List]:=Sum[x[[i]],{i,Length[x]-1}];
	h[x_List]:=Take[x,Length[x]-1];
	revPre=q0.p0-q0[[n]]*p0[[n]];
	 notMerging=Complement[Table[i,{i,n}],merging];
	 notNested=Complement[Table[i,{i,n}],nest];
     sumNested[p_List]:=    Plus @@ Part[Exp[(a-B p)/nestParam],nest];      
     sumNotNested[p_List]:= Plus @@ Part[Exp[(a-B p)          ],notNested];
     denom[p_List]:=sumNested[p]^nestParam + sumNotNested[p];
     Labels1=   {
                 "Guidelines HHI",
                 "Quantity HHI",
                 "Industry Price",
                 "Industry Quantity",
                 "Elasticity",
                 "Average Costs",
                 "Merged Firm Price",
                 "Merged Firm Prft",
                 "Industry Profits",
                 "Consumer Surplus",
                 "Welfare"
		         };
     printNames=Table[If[MemberQ[merging,i],
                         "*"<>FirmNames[[i]],
                         " "<>FirmNames[[i]]
                         ],
                       {i,n}];
     printNames=Table[If[MemberQ[nest,i],
                         "@"<>printNames[[i]],
                         " "<>printNames[[i]]
                         ],
                       {i,n}];
     PreMerger= {preHHI=10000(Plus @@ (Take[q0/(1-q0[[n]]),{1,n-1}]^2)),
                 preHHI,
                 pPre=   h[p0].h[q0]/g[q0],
                 qPre=   1-q0[[n]],
                 ePre=   B(q0[[n]])pPre,
                 cPre=   h[c0].h[q0]/g[q0],
                 p12Pre= Part[p0,merging].Part[q0,merging]/(Plus @@ Part[q0,merging]),
                 pi12Pre= Plus @@ Part[prof0,merging],
                 piPre=   Plus @@ Take[prof0,n-1]    ,
                 " ",
                 " "
                }//N;
     PostMerger=
                {
                 postHHI2=10000((Plus @@      Part[q0/(1-q0[[n]]),merging])^2 +
                          Plus @@ (Drop[Part[q0/(1-q0[[n]]),notMerging],-1]^2)),
                 postHHI= 10000((Plus @@      Part[q1/(1-q1[[n]]),merging])^2 +
                          Plus @@ (Drop[Part[q1/(1-q1[[n]]),notMerging],-1]^2)),
                 pPost=   h[p1].h[q1]/g[q1],
                 qPost=   1-q1[[n]],
                 ePost=   B(q1[[n]])pPost,
                 cPost=   h[c1].h[q1]/g[q1] ,
                 p12Post= Part[p1,merging].Part[q1,merging]/(Plus @@ Part[q1,merging]),
                 pi12Post= Plus @@ Part[prof1,merging],
                 piPost=   Plus @@ Take[prof1,n-1]    ,
                 " ",
                 " "
                }//N;
                            
     Change=    { 
                 postHHI2-preHHI,
                 postHHI-preHHI, 
                 pPost-pPre,
                 qPost-qPre,
                 ePost-ePre,
                 cPost-cPre,
                 p12Post-p12Pre,
                 100(pi12Post-pi12Pre)/revPre,
                 deltaPi=100(piPost-piPre)/revPre,
                 deltaS=100(Log[denom[p1]]-Log[denom[p0]])/(B*revPre),
                 (deltaS+deltaPi)                                    
                 }//N;

     PctChange=             {" ", 
                             " ",
                             100 (pPost/pPre-1),
                             100 (qPost/qPre-1),
                             100 (ePost/ePre-1),
                             100 (cPost/cPre-1),
                             100 (p12Post/p12Pre-1),
                             100 (pi12Post/pi12Pre-1),
                             100 (piPost/piPre-1),
                             " ",
                             " "
                             }//N;
    
     TableForm[{
       ColumnForm[{"DIFFERENTIATED PRODUCTS MERGER SIMULATION RESULTS",
       		      "Elasticity = "<>ToString[elasticity],
                  "B = "<>ToString[B],
                  "Nest Parameter = "<>ToString[nestParam],
                  "@ signifies nest member",
                  "* signifies merging firms",
                  "Changes in profits and welfare are expressed as a % of pre-merger revenue."
                  }],

      NumberForm[TableForm[Transpose[
                                      {{"Firms",printNames},
                                       {"a",a},
                                       {"q0",q0},
                                       {"q1",q1},
                                       {"% change q",100*(q1/q0-1)}}/.Indeterminate->" "
                                     ]
                           ],
                  4],
                
      NumberForm[TableForm[Transpose[
                                      {{"Firms",Take[#,n-1]& @ printNames},
                                       {"mc0",Take[#,n-1]& @ c0},
                                       {"mc1",Take[#,n-1]& @ c1},
                                       {"p0",Take[#,n-1]& @ p0},
                                       {"p1",Take[#,n-1]& @ p1},
                                       {"% change p",Take[#,n-1]& @ (100*(p1/p0-1))}}/.Indeterminate->" "
                                     ]
                           ],
                  4],

                  
      NumberForm[TableForm[Transpose[
                                      {{"firms",Take[#,n-1]& @ printNames},
                                       {"p0-mc0",Take[#,n-1]& @ (p0-c0)},
                                       {"p1-mc1",Take[#,n-1]& @ (p1-c1)},
                                       {"elas0",Take[#,n-1]& @ (B p0 (1-q0))},
                                       {"elas1",Take[#,n-1]& @ (B p1 (1-q1))}}
                                     ]
                           ],
                  3],
                  
      NumberForm[TableForm[Transpose[
                                      {{"firms",Take[#,n-1]& @ printNames},
                                       {"prof0",Take[#,n-1]& @ (prof0    /(1))},
                                       {"prof1",Take[#,n-1]& @ (prof1    /(1))},
                                       {"change prof",Take[#,n-1]& @ ((prof1-prof0)/(1))},
                                       {"% change prof",Take[#,n-1]& @ (100*(prof1-prof0)/prof0)}}
                                     ]
                           ],
                  3],

      NumberForm[TableForm[Transpose[
                                      {{"vars.",Labels1},
                                       {"pre merger",PreMerger},
                                       {"post",PostMerger},
                                       {"change",Change},
                                       {"% change",PctChange}}
                                      ]
                            ],4]
                   }/.ComplexInfinity->" "
]]

DiffProdMerger[merging_List,demandData_List,firmData_List]:=
(* This function calls all the other functions to calculate *)
(* and print the effects of a Merger.                       *)
Module[{firmNames,p0,q0,B,e,n,a,c0,q00,c1,p1,q1},
       firmNames=firmData[[1]];
       p0=firmData[[2]];
       q0=firmData[[3]];
       B=demandData[[1]];
       e=demandData[[2]];
       n=Length[p0];
       q00=ChoiceProbabilities[B,p0,q0,e];
       a=DemandConstants[B,p0,q00];
       c0=MarginalCosts[a,B,p0,q00];
       {c1,p1,q1}=PostMerger[merging,a,B,c0,p0];
       PrintResults[merging,a,B,e,c0,p0,q00,c1,p1,q1,firmNames]
      ]
      
DiffProdMergerMinCost[merging_List,demandData_List,firmData_List]:=
(* This function calls all the other functions to calculate *)
(* and print the effects of a Merger.                       *)
Module[{firmNames,p0,q0,B,e,n,a,c0,c00,q00,c1,p1,q1},
       firmNames=firmData[[1]];
       p0=firmData[[2]];
       q0=firmData[[3]];
       B=demandData[[1]];
       e=demandData[[2]];
       n=Length[p0];
       q00=ChoiceProbabilities[B,p0,q0,e];
       a=DemandConstants[B,p0,q00];
       c0=MarginalCosts[a,B,p0,q00];
       c00=ReplacePart[c0,
            Min[Flatten[Take[c0,#]& /@ (List/@ merging)]],
                       {#}& /@ merging];
       {c1,p1,q1}=PostMerger[merging,a,B,c00,p0];
       PrintResults[merging,a,B,e,c0,p0,q00,c1,p1,q1,firmNames]
      ]

DiffProdMerger[merging_List,nest_List:{},nestParam_:1,demandData_List,firmData_List]:=
(* This function calls all the other functions to calculate *)
(* and print the effects of a Merger.                       *)
Module[{firmNames,p0,q0,B,e,n,a,c0,q00,c1,p1,q1},
       firmNames=firmData[[1]];
       p0=firmData[[2]];
       q0=firmData[[3]];
       B=demandData[[1]];
       e=demandData[[2]];
       n=Length[p0];
       q00=ChoiceProbabilities[B,p0,q0,e];
       a=DemandConstants[nest,nestParam,B,p0,q00];
       c0=MarginalCosts[nest,nestParam,a,B,p0,q00];
       {c1,p1,q1}=PostMerger[merging,nest,nestParam,a,B,c0,p0];
       PrintResults[merging,nest,nestParam,a,B,e,c0,p0,q00,c1,p1,q1,firmNames]
      ]
      
DiffProdMergerMinCost[merging_List,nest_List:{},nestParam_:1,demandData_List,firmData_List]:=
(* This function calls all the other functions to calculate *)
(* and print the effects of a Merger.                       *)
Module[{firmNames,p0,q0,B,e,n,a,c0,q00,c1,p1,q1},
       firmNames=firmData[[1]];
       p0=firmData[[2]];
       q0=firmData[[3]];
       B=demandData[[1]];
       e=demandData[[2]];
       n=Length[p0];
       q00=ChoiceProbabilities[B,p0,q0,e];
       a=DemandConstants[nest,nestParam,B,p0,q00];
       c0=MarginalCosts[nest,nestParam,a,B,p0,q00];
       c00=ReplacePart[c0,
            Min[Flatten[Take[c0,#]& /@ (List/@ merging)]],
                       {#}& /@ merging];
       {c1,p1,q1}=PostMerger[merging,nest,nestParam,a,B,c00,p0];
       PrintResults[merging,nest,nestParam,a,B,e,c0,p0,q00,c1,p1,q1,firmNames]
      ]
      
PostMergerC[merging_List,elasticity_,p0_,s0_List]:=
(* This function computes the post merger Cournot Equil*)
Module[{n,a,k0,k1,p1,q0,q1,Q0,Q1,cs0,cs1,
        cost0,cost1,mcost0,mcost1,profits0,profits1,w0,w1,
        nonMerging,sumMergedK,K0,K1,sumMergedQ,indCosts0,indCosts1,
        fProfits0,fProfits1,mfProfits0,mfProfits1,
        printNames0,printNames1,preHHIQ0,preHHIQ1,HHIQ0,HHIQ1,
        HHIK0,HHIK1,elasticity1},

       n=Length[s0];
       nonMerging=Complement[Table[i,{i,n}],merging];
       a=p0(1 + 1/elasticity);
       b=Plus @@ ((s0 elasticity)/(elasticity-s0));
       k0=s0/(b(elasticity - s0));
       sumMergedK=   Plus @@ Part[k0,merging];      
       k1=Flatten[{sumMergedK,Part[k0,nonMerging]}];
       p1=a/(1 + b Plus @@ (k1/(1 + b k1)));        
       q0=p0 k0/(1 + b k0);
       q1=p1 k1/(1 + b k1);
       {{k0,p0,q0},
        {k1,p1,q1}}
       ]      

HomoProdMerger[merging_List,elasticity_,p0_,s0_List,FirmNames_List]:=
Module[{n,a,k0,k1,p1,q0,q1,Q0,Q1,cs0,cs1,
        cost0,cost1,mcost0,mcost1,profits0,profits1,w0,w1,
        nonMerging,sumMergedK,K0,K1,sumMergedQ,indCosts0,indCosts1,
        fProfits0,fProfits1,mfProfits0,mfProfits1,
        printNames0,printNames1,preHHIQ0,preHHIQ1,HHIQ0,HHIQ1,
        HHIK0,HHIK1,elasticity1,PreMerger, PostMerger, PctChange,Labels1},
       n=Length[s0];
       nonMerging=Complement[Table[i,{i,n}],merging];
       a=p0(1 + 1/elasticity);
       b=Plus @@ ((s0 elasticity)/(elasticity-s0));
       k0=s0/(b(elasticity - s0));
       sumMergedK=   Plus @@ Part[k0,merging];      
       k1=Flatten[{sumMergedK,Part[k0,nonMerging]}];
       K0=Plus @@ k0;
       K1=Plus @@ k1;       
       p1=a/(1 + b Plus @@ (k1/(1 + b k1)));        
       q0=p0 k0/(1 + b k0);
       q1=p1 k1/(1 + b k1);
       Q0=Plus @@ q0;
       Q1=Plus @@ q1;
       sumMergedQ=   Plus @@ Part[q0,merging];      
       cs0=100(b Q0^2/2)/(p0 Q0);
       cs1=100(b Q1^2/2)/(p0 Q0);
       cost0=q0^2/(2k0);
       cost1=q1^2/(2k1);
       indCosts0= (Plus @@ cost0)/Q0;
       indCosts1= (Plus @@ cost1)/Q1;
       mcost0=q0/k0;
       mcost1=q1/k1;
       If[Min[mcost0]<0,Message[MarginalCosts::negative]];		
       
       fProfits0=(p0 q0 - cost0);
       fProfits1=(p1 q1 - cost1);
       mfProfits0=Plus @@ Part[fProfits0,merging];
       mfProfits1=        Part[fProfits1,{1}];
       profits0=100((p0 - indCosts0)Q0)/(p0 Q0);
       profits1=100((p1 - indCosts1)Q1)/(p0 Q0);
       w0=cs0+profits0;
       w1=cs1+profits1;
       printNames0=Table[If[MemberQ[merging,i],
                         "*"<>FirmNames[[i]],
                         " "<>FirmNames[[i]]
                         ],
                       {i,n}];
       printNames1=Flatten[{"Merged Firm", Part[FirmNames,nonMerging]}];
       
        Labels1=   
                {
                 "Guidelines HHI",
                 "Quantity HHI",
                 "Capacity HHI",
                 "Industry Price",
                 "Industry Quantity",
                 "elasticity",
                 "Merged Firm Profits",
                 "Industry Av. Costs",
                 "Industry Profits",
                 "Consumer Surplus",
                 "Total Welfare"
                 };
	 	    
      PreMerger=
                {
                 preHHIQ0=10000(q0/Q0).(q0/Q0),
                 HHIQ0=10000(q0/Q0).(q0/Q0),
                 HHIK0=10000(k0/K0).(k0/K0),
                 p0,
                 Q0,
                 elasticity,
                 mfProfits0,
                 indCosts0,
                 " ",
                 " ",
                 " "
                };
                 
     PostMerger=
                {
                 preHHIQ1=(10000/Q0^2) Flatten[{sumMergedQ,Part[q0,nonMerging]}].Flatten[{sumMergedQ,Part[q0,nonMerging]}],
                 HHIQ1=10000(q1/Q1).(q1/Q1),
                 HHIK1=(10000/K0^2) Flatten[{sumMergedK,Part[k0,nonMerging]}].Flatten[{sumMergedK,Part[k0,nonMerging]}],
                 p1,
                 Q1,
                 elasticity1=p1/(a-p1),
                 mfProfits1,
                 indCosts1,
                 " ",
                 " ",
                 " "
                };
                            
         Change=
                {
                 preHHIQ1-preHHIQ0,
                 HHIQ1-HHIQ0,
                 HHIK1-HHIK0,
                 p1-p0,
                 Q1-Q0,
                 elasticity1-elasticity,
                 mfProfits1-mfProfits0,
                 indCosts1-indCosts0,
                 profits1-profits0,
                 cs1-cs0,
                 w1-w0
                };


      PctChange=
                {
                 " ",
                 " ",
                 " ",
                 100(p1-p0)/p0,
                 100(Q1-Q0)/Q0,
                 100(elasticity1-elasticity)/elasticity,
                 100(mfProfits1-mfProfits0)/mfProfits0,
                 100(indCosts1-indCosts0)/indCosts0,
                 " ",
                 " ",
                 " "
                };
                
       ColumnForm[{"HOMOGENEOUS PRODUCTS MERGER SIMULATION",
                  "Elasticity = "<>ToString[elasticity],
                  "* denotes merging firm",
                  "Changes in consumer surplus and welfare are expressed as a % of premerger revenue.",
                  " ",
        NumberForm[
                  TableForm[Transpose[
                                      {
                                       {"Firms",printNames0},
                                       {"K0",k0},
                                       {"mc0",mcost0},
                                       {"q0",q0},
                                       {"cost0",cost0},
                                       {"prof0",fProfits0}
                                      }
                                     ]
                           ]
                  ,4],
                  " ",
       
        NumberForm[
                  TableForm[Transpose[
                                      {
                                       {"Firms",printNames1},
                                       {"K1",k1},
                                       {"mc1",mcost1},
                                       {"q1",q1},
                                       {"cost1",cost1},
                                       {"prof1",fProfits1}
                                      }
                                     ]
                           ]
                  ,4],
                  " ",

        NumberForm[
                  TableForm[Transpose[
                                      {
                                       {"Variables",Labels1},
                                       {"Pre",PreMerger},
                                       {"Post",PostMerger},
                                       {"Change",Change},
                                       {"%Change",PctChange}}
                                      ]
                            ],4]}]/.ComplexInfinity->"***"
     ]      
     
End[]

SetAttributes[ChoiceProbabilities, ReadProtected];
SetAttributes[DemandConstants, ReadProtected];
SetAttributes[MarginalCosts, ReadProtected];
SetAttributes[PostMerger, ReadProtected];
SetAttributes[PostMergerC, ReadProtected];
SetAttributes[PrintResults, ReadProtected];
SetAttributes[DiffProdMerger, ReadProtected];
SetAttributes[DiffProdMergerMinCost, ReadProtected];
SetAttributes[HomoProdMerger, ReadProtected];

Protect[ChoiceProbabilities, DemandConstants, MarginalCosts, 
        PostMerger, PostMergerC, PrintResults,
        DiffProdMerger, DiffProdMergerMinCost, HomoProdMerger];

EndPackage[]

