(* file version: 9/20/95	*)
(*:Version: Mathematica 2.2 *)
(*:Name: 	DEA 	*)
(*:Title: 	Tools for Data Envelopment Analysis *)
(*:Author: 	Eduardo Ley, October 1994 *)
(*:Legal: 	Copyright (c) 1995, Eduardo Ley *)
(*:Summary: This package implements some of the basic
			Data Envelopment Analysis (DEA) measures
			for the measurement of productive
			efficiency. *)
(*:Keywords: DEA, Efficiency 	*)
(*:Requirements: No special system requirements. *)
(*:Warning: None. *)
(*:Sources: Lovell C.A.K. (1994): "Linear Programming
			Approaches to the Measurement and Analysis
			of Productive Efficiency", TOP 2:2, 175-223. *)

BeginPackage["DEA`", "LinearAlgebra`MatrixManipulation`"]
			
IEfficiencyCRS::usage  = "Computes the DEA Input-Efficiency measure(s) assuming
                          Constant Returns to Scale." 
IEfficiencyNDRS::usage = "Computes the DEA Input-Efficiency measure(s) assuming
                          Non-Decreasing Returns to Scale."  
IEfficiencyVRS::usage  = "Computes the DEA Input-Efficiency measure(s) assuming
                          Variable Returns to Scale." 

OEfficiencyCRS::usage  = "Computes the DEA Output-Efficiency measure(s) assuming
                          Constant Returns to Scale." 
OEfficiencyNDRS::usage = "Computes the DEA Output-Efficiency measure(s) assuming
                          Non-Decreasing Returns to Scale." 
OEfficiencyVRS::usage  = "Computes the DEA Output-Efficiency measure(s) assuming
                          Variable Returns to Scale."  

Begin["`Private`"]

(****************************************************************
		A. Input-Based Measures 
*****************************************************************)

IEfficiencyCRS[unitNumber_Integer, Y_List, X_List]:=
Module[{s,m,n,c,M12,M11,M1,M2,M,b1,b2,b},
    s	= Part[Dimensions[Y],1];
    m	= Part[Dimensions[X],1];
    n   = Part[Dimensions[X],2];
    c 	= AppendRows[{{1}}, ZeroMatrix[1,n]][[1]];
	M12 = TakeColumns[X,{unitNumber,unitNumber}];
	M11 = ZeroMatrix[s,1];
	M1 	= AppendColumns[M11,M12];
	M2 	= AppendColumns[Y,-X];
	M   = AppendRows[M1,M2];
	b1  = Transpose[TakeColumns[Y, {unitNumber, unitNumber}]];
	b2 	= ZeroMatrix[1,m];
	b 	= Part[AppendRows[b1,b2],1];
	N[LinearProgramming[c,M,b]]
	] 

IEfficiencyNDRS[unitNumber_Integer, Y_List, X_List]:=
Module[{s,m,n,c,M12,M11,M1,M2,M,b1,b2,b},
    s	= Part[Dimensions[Y],1];
    m	= Part[Dimensions[X],1];
    n   = Part[Dimensions[X],2];
    c 	= AppendRows[{{1}}, ZeroMatrix[1,n]][[1]];
	M12 = TakeColumns[X,{unitNumber,unitNumber}];
	M11 = ZeroMatrix[s,1];
	M1 	= AppendColumns[M11,M12,{{0}}];
	M2 	= AppendColumns[Y,-X,{Table[-1,{i,1,n}]}];
	M   = AppendRows[M1,M2];
	b1  = Transpose[TakeColumns[Y, {unitNumber, unitNumber}]];
	b2 	= ZeroMatrix[1,m];
	b 	= Part[AppendRows[b1,b2,{{-1}}],1];
	N[LinearProgramming[c,M,b]]
	] 

IEfficiencyVRS[unitNumber_Integer, Y_List, X_List]:=
Module[{s,m,n,c,M12,M11,M1,M2,M,b1,b2,b},
    s	= Part[Dimensions[Y],1];
    m	= Part[Dimensions[X],1];
    n   = Part[Dimensions[X],2];
    c 	= AppendRows[{{1}}, ZeroMatrix[1,n]][[1]];
	M12 = TakeColumns[X,{unitNumber,unitNumber}];
	M11 = ZeroMatrix[s,1];
	M1 	= AppendColumns[M11,M12,{{0}},{{0}}];
	M2 	= AppendColumns[Y,-X,{Table[-1,{i,1,n}]},{Table[1,{i,1,n}]}];
	M   = AppendRows[M1,M2];
	b1  = Transpose[TakeColumns[Y, {unitNumber, unitNumber}]];
	b2 	= ZeroMatrix[1,m];
	b 	= Part[AppendRows[b1,b2,{{-1}},{{1}}],1];
	N[LinearProgramming[c,M,b]]
	] 

(*1**************************************************************) 

IEfficiencyCRS[Y_List, X_List]:=
	Table[IEfficiencyCRS[i,Y,X], {i,1,Part[Dimensions[X],2]}]

IEfficiencyNDRS[Y_List, X_List]:=
	Table[IEfficiencyNDRS[i,Y,X], {i,1,Part[Dimensions[X],2]}]

IEfficiencyVRS[Y_List, X_List]:=
	Table[IEfficiencyVRS[i,Y,X], {i,1,Part[Dimensions[X],2]}]

(*2**************************************************************) 

IEfficiencyCRS[NOutputs_Integer, NInputs_Integer, P_List]:=
Module[{X,Y},
	Y   = TakeRows[P,NOutputs];
	X   = TakeRows[P,-NInputs];
	IEfficiencyCRS[Y,X]
	]

IEfficiencyNDRS[NOutputs_Integer, NInputs_Integer, P_List]:=
Module[{X,Y},
	Y   = TakeRows[P,NOutputs];
	X   = TakeRows[P,-NInputs];
	IEfficiencyNDRS[Y,X]
	]


IEfficiencyVRS[NOutputs_Integer, NInputs_Integer, P_List]:=
Module[{X,Y},
	Y   = TakeRows[P,NOutputs];
	X   = TakeRows[P,-NInputs];
	IEfficiencyVRS[Y,X]
	]

(*3****************************************************************) 

IEfficiencyCRS[unitNumber_Integer,
			NOutputs_Integer,NInputs_Integer,P_List]:=
Module[{X,Y},
	Y   = TakeRows[P,NOutputs];
	X   = TakeRows[P,-NInputs]; 
	IEfficiencyCRS[unitNumber, Y, X]
	]


IEfficiencyNDRS[unitNumber_Integer,
			NOutputs_Integer,NInputs_Integer,P_List]:=
Module[{X,Y},
	Y   = TakeRows[P,NOutputs];
	X   = TakeRows[P,-NInputs]; 
	IEfficiencyNDRS[unitNumber, Y, X]
	]

IEfficiencyVRS[unitNumber_Integer,
			NOutputs_Integer,NInputs_Integer,P_List]:=
Module[{X,Y},
	Y   = TakeRows[P,NOutputs];
	X   = TakeRows[P,-NInputs]; 
	IEfficiencyVRS[unitNumber, Y, X]
	]


(*4****************************************************************
		B. Output-Based Measures 
*****************************************************************)

OEfficiencyCRS[unitNumber_Integer,  Y_List, X_List]:=
Module[{s,m,n,c,M12,M11,M1,M2,M,b1,b2,b},
 s	= Part[Dimensions[Y],1];
 m	= Part[Dimensions[X],1];
 n = Part[Dimensions[X],2];
 c 	= AppendRows[{{-1}}, ZeroMatrix[1,n]][[1]];
	M12 = TakeColumns[Y,{unitNumber,unitNumber}];
	M11 = ZeroMatrix[m,1];
	M1 	= AppendColumns[M11,-M12];
	M2 	= AppendColumns[-X,Y];
	M   = AppendRows[M1,M2];
	b1  = Transpose[TakeColumns[X, {unitNumber, unitNumber}]];
	b2 	= ZeroMatrix[1,s];
	b 	= Part[AppendRows[-b1,b2],1];
	N[LinearProgramming[c,M,b]]
	] 

OEfficiencyNDRS[unitNumber_Integer,  Y_List, X_List]:=
Module[{s,m,n,c,M12,M11,M1,M2,M,b1,b2,b},
 s	= Part[Dimensions[Y],1];
 m	= Part[Dimensions[X],1];
 n = Part[Dimensions[X],2];
 c 	= AppendRows[{{-1}}, ZeroMatrix[1,n]][[1]];
	M12 = TakeColumns[Y,{unitNumber,unitNumber}];
	M11 = ZeroMatrix[m,1];
	M1 	= AppendColumns[M11,-M12,{{0}}];
	M2 	= AppendColumns[-X,Y,{Table[-1,{i,1,n}]}];
	M   = AppendRows[M1,M2];
	b1  = Transpose[TakeColumns[X, {unitNumber, unitNumber}]];
	b2 	= ZeroMatrix[1,s];
	b 	= Part[AppendRows[-b1,b2,{{-1}}],1];
	N[LinearProgramming[c,M,b]]
	] 

OEfficiencyVRS[unitNumber_Integer,  Y_List, X_List]:=
Module[{s,m,n,c,M12,M11,M1,M2,M,b1,b2,b},
 s	= Part[Dimensions[Y],1];
 m	= Part[Dimensions[X],1];
 n = Part[Dimensions[X],2];
 c 	= AppendRows[{{-1}}, ZeroMatrix[1,n]][[1]];
	M12 = TakeColumns[Y,{unitNumber,unitNumber}];
	M11 = ZeroMatrix[m,1];
	M1 	= AppendColumns[M11,-M12,{{0}},{{0}}];
	M2 	= AppendColumns[-X,Y,{Table[-1,{i,1,n}]},{Table[1,{i,1,n}]}];
	M   = AppendRows[M1,M2];
	b1  = Transpose[TakeColumns[X, {unitNumber, unitNumber}]];
	b2 	= ZeroMatrix[1,s];
	b 	= Part[AppendRows[-b1,b2,{{-1}},{{1}}],1];
	N[LinearProgramming[c,M,b]]
	] 

(*1****************************************************************) 

OEfficiencyCRS[Y_List,X_List]:=
   Table[OEfficiencyCRS[i,Y,X],{i,1,Part[Dimensions[X],2]}]
   
OEfficiencyNDRS[Y_List,X_List]:=
   Table[OEfficiencyNDRS[i,Y,X],{i,1,Part[Dimensions[X],2]}]

OEfficiencyVRS[Y_List,X_List]:=
   Table[OEfficiencyVRS[i,Y,X],{i,1,Part[Dimensions[X],2]}]
   
(*2****************************************************************) 

OEfficiencyCRS[NOutputs_Integer,NInputs_Integer,P_List]:=
Module[{X,Y},
	Y   = TakeRows[P,NOutputs];
	X   = TakeRows[P,-NInputs];
	OEfficiencyCRS[Y,X]
	]

OEfficiencyNDRS[NOutputs_Integer,NInputs_Integer,P_List]:=
Module[{X,Y},
	Y   = TakeRows[P,NOutputs];
	X   = TakeRows[P,-NInputs];
	OEfficiencyNDRS[Y,X]
	]

OEfficiencyVRS[NOutputs_Integer,NInputs_Integer,P_List]:=
Module[{X,Y},
	Y   = TakeRows[P,NOutputs];
	X   = TakeRows[P,-NInputs];
	OEfficiencyVRS[Y,X]
	]

(*3****************************************************************) 

OEfficiencyCRS[unitNumber_Integer,
			NOutputs_Integer,NInputs_Integer,P_List]:=
Module[{X,Y},
	Y   = TakeRows[P,NOutputs];
	X   = TakeRows[P,-NInputs]; 
	OEfficiencyCRS[unitNumber, Y, X]
	]	

OEfficiencyNDRS[unitNumber_Integer,
			NOutputs_Integer,NInputs_Integer,P_List]:=
Module[{X,Y},
	Y   = TakeRows[P,NOutputs];
	X   = TakeRows[P,-NInputs]; 
	OEfficiencyNDRS[unitNumber, Y, X]
	]	

OEfficiencyVRS[unitNumber_Integer,
			NOutputs_Integer,NInputs_Integer,P_List]:=
Module[{X,Y},
	Y   = TakeRows[P,NOutputs];
	X   = TakeRows[P,-NInputs]; 
	OEfficiencyVRS[unitNumber, Y, X]
	]	

End[]

SetAttributes[IEfficiencyCRS, 	ReadProtected];
SetAttributes[IEfficiencyNDRS, 	ReadProtected];
SetAttributes[IEfficiencyVRS, 	ReadProtected];
SetAttributes[OEfficiencyCRS, 	ReadProtected];
SetAttributes[OEfficiencyNDRS, 	ReadProtected];
SetAttributes[OEfficiencyVRS, 	ReadProtected];

Protect[IEfficiencyCRS, IEfficiencyNDRS, IEfficiencyVRS,
        OEfficiencyCRS, OEfficiencyNDRS, OEfficiencyVRS];

EndPackage[]
