(* file version: 1/24/96 *)
(*:Version: Mathematica 2.2 *)
(*:Name: 	Outliers *)
(*:Title: 	Tools for Identifying Outliers 	*)
(*:Author: 	Eduardo Ley, October 1995 *)
(*:Legal: 	Copyright (c) 1995, Eduardo Ley *)
(*:Summary: *)
(*:Keywords: Mahalanobis, Outliers *)
(*:Requirements: No special system requirements. *)
(*:Warning: None. *)
(*:Sources: 
	Hadi, A.S. (1992): "Identifying Multiple Outliers 
		in Multivariate Data,'' Journal of the Royal 
		Statistical Society (B), 54(3): 761--771.
 	Hadi, A.S. (1994): "A Modification of a Method for 
 		the Detection of Outliers in Multivariate Samples,'' 
 		Journal of the Royal Statistical Society (B), 
 		56(2): 393--390.
*)

BeginPackage["IdentifyOutliers`",
			"Statistics`DataManipulation`",
			"Statistics`DescriptiveStatistics`",
			"Statistics`InverseStatisticalFunctions`"
			]
			
IdentifyOutliers::usage =
"IdentifyOutliers[alpha, data, mean, cov] 
implements Hadi's (1992, 1994) method for identifying
outliers in multivariate data.
	alpha is the significance level;
	data contains the data matrix (nobs by nvar);
	mean on output contains the mean of the basic set;
	cov on output contains the cov matrix of the basic set."
	
MahaDist::usage = "Computes the Mahalanobis Distances."
MahaSortIndeces::usage = "Sorts an array's indeces by the
corresponding Mahalanobis distances."	

Begin["`Private`"]

(******************************************************************) 
(* Need InverseStatisticalDistributions.m which is loaded at the top *)
(******************************************************************) 

numericalize[result_] := Module[{nresult},
	If[NumberQ[nresult = N[result, Precision[result]]],
		nresult,
		If[NumberQ[nresult = N[result]],
			nresult,
			result]]];
			
ChiSquareDistribution/: Quantile[ChiSquareDistribution[n_], q_] :=
	numericalize[2 InverseGammaRegularized[n/2, 0, q]];

CovMatrix[data_, loc_, nobs_, nvar_] := 
	Table[(1/(nobs-1))*
      Sum[(Part[data,n,i]-Part[loc,i])*
          (Part[data,n,j]-Part[loc,j]),
	{n,1,nobs}],
	{i,1,nvar},
	{j,1,nvar}];

BasicCovMatrix[data_, auxsort_, loc_, nbasic_, nvar_] := 
	Table[(1/(nbasic-1))*
      Sum[(Part[data,Part[auxsort,n],i]-Part[loc,i])*
          (Part[data,Part[auxsort,n],j]-Part[loc,j]),
	{n,1,nbasic}],
	{i,1,nvar},
	{j,1,nvar}]

BasicMean[data_, auxsort_, nbasic_, nvar_]:=
	Table[(1/nbasic)*
	Sum[Part[data, Part[auxsort,n],i], 
	{n,1,nbasic}], 
	{i,1,nvar}]

MahaDist[data_, cov_, loc_, nobs_] :=
	Table[{Sqrt[(Part[data,n]-loc).Inverse[cov].
	(Part[data,n]-loc)]}, 
	{n,1,nobs}]

MahaSortIndeces[data_, cov_, loc_, nobs_]:=
	Flatten[
	ColumnTake[
	Sort[RowJoin[MahaDist[data, cov, loc, nobs], 
	Table[{i},{i,nobs}]]],{2,2}]]

IdentifyOutliers[alpha_, datamatrix_] := 
Module[{nvar, nobs, indexmatrix, h, correction, cv, cvc, m, s},
	nvar = Length[Part[datamatrix,1]]; 
	nobs = Length[datamatrix];
	Print["*** Number of Observations: ",nobs,
			". Number of Variables: "   ,nvar];
    Print["*** Significance Level: ",alpha*100,"%"];
	indexmatrix = Table[{i},{i,nobs}];
	h = Round[(nvar+nobs+1)/2];
	If[h>nobs, Return[Print["Too Few Observations!!!"]]];
	correction = (1.0+(2/(nobs-1-3*nvar))+((nvar+1)/(nobs-nvar)))^2;
	cv = Sqrt[Quantile[ChiSquareDistribution[nvar], 1-alpha/2]];
		    (* ---------------------------------------- *)
			(* ------------ Initial Stats ------------- *)
		    (* ---------------------------------------- *)
	m = Table[Median[Column[datamatrix,i]],{i,nvar}];
	Print["Median is ",m];
	s = CovMatrix[datamatrix, m, nobs, nvar];
	auxsort = MahaSortIndeces[datamatrix, s, m, nobs];
    m = BasicMean[datamatrix, auxsort, h, nvar];
	s = BasicCovMatrix[datamatrix, auxsort, m, h, nvar];
	auxsort = MahaSortIndeces[datamatrix, s, m, nobs];
		    (* ---------------------------------------- *)
			(* ---------- Initial Iterations ---------- *)
		    (* ---------------------------------------- *)
	Print["=> Initial Iterations (Be patient!)"];
	If[nvar<h,		    
		Do[ m = BasicMean[datamatrix, auxsort, r, nvar];
			s = BasicCovMatrix[datamatrix, auxsort, m, r, nvar];
			auxsort = MahaSortIndeces[datamatrix, s, m, nobs]
			, {r, nvar+1, h}
			],
		Return[Print["Too Few Observations!!!"]]
	];
	r = h;
	Print["=> Size of Basic Set is ",r," observations."]
	Print["=> Observations outside Basic Set are:"];
	Print[Flatten[auxsort[[Table[i, {i,h+1,nobs}]]]]
	     ];		
		    (* ---------------------------------------- *)
			(* ------------- Test Forward ------------- *)
		    (* ---------------------------------------- *)
	Print["=> Testing Forward"];		    
	While[
		Part[Part[Sort[MahaDist[datamatrix, correction*s, m, nobs]], r], 1] < cv
		&& r<nobs,
		r ++;
		Print["... Basic Set increased to ",r,
		" observations.  Last obs moved in is ",auxsort[[r]] ];
		m = BasicMean[datamatrix, auxsort, r, nvar];
		s = BasicCovMatrix[datamatrix, auxsort, m, r, nvar];
		auxsort = MahaSortIndeces[datamatrix, s, m, nobs]
		];
		    (* --------------------------------------- *)
			(* ------------- TestBackward ------------ *)
		    (* --------------------------------------- *)
	Print["=> Testing Backward"];		    
	While[
		Part[Part[Sort[MahaDist[datamatrix, correction*s, m, nobs]], r], 1] > cv
		&& r>1,
		r --; 
		Print["... Basic Set decreased to ",r,
		" observations.  Obs ",auxsort[[r+1]]," moved out."]
		];
		    (* -------------------------------------- *)
			(* ------------- Final Stats ------------ *)
		    (* -------------------------------------- *)
	m = BasicMean[datamatrix, auxsort, r, nvar];
	s = BasicCovMatrix[datamatrix, auxsort, m, r, nvar];
	auxsort = MahaSortIndeces[datamatrix, s, m, nobs];
	Print["=> Size of final basic set is... ",r,". (", 
	N[(r/nobs)*100,2],"%)"];
	Print["=> OUTLIERS: "];
	Do[
		Print[
			"... Observation ",
		    Part[auxsort, rr],
		    "; distance is... ",
			Part[Part[Sort[
			MahaDist[datamatrix, correction*s, m, nobs]], rr], 1]
 			], 
		{rr,r+1,nobs}
		];
    cvc = correction*cv;
	Print["    (Threshold distance is... ", cvc,")"];
	Return[{m,s,cvc}];
]

End[]

SetAttributes[IdentifyOutliers, ReadProtected];
SetAttributes[MahaDist, ReadProtected];
SetAttributes[MahaSortIndeces, ReadProtected];
 
Protect[IdentifyOutliers, MahaDist, MahaSortIndeces];

EndPackage[]

