(*:Name:     PDF                                                 *)
(*:Authors:  Colin Rose and Murray D. Smith                      *)
(*:Version:  Mathematica 2.2  or later                           *)
(*:Legal:    Copyright (c) 1995, 
             Colin Rose and Murray D. Smith                      *)            
(*:Summary: 
             This package provides a suite of tools 
             to assist with the theoretical manipulation 
             of probability density functions.
                                                                 *)

BeginPackage["PDF`"]
	
DF::usage =
	"DF[x, f] calculates the distribution function of
	density f at x, where x may either be univariate
	or multivariate {x1, x2, x3}, as appropriate."

MrSpeedy::usage =
	"MrSpeedy[cdf, S] calculates the probability content
	of a closed region defined by set S, by making use of
	the known distribution function cdf[x1, x2, ..., xn]."     

Expect::usage =
	"Expect[u, f] calculates the mathematical expectation
	of u, where u = u(x) and x has density f(x)."	
	         
CentralM::usage =
	"CentralM[k, Z] provides the kth central moment, as
	a fn of the first k moments Z[i] about the origin."
     
CumulantM::usage = 
	"CumulantM[k, Z] provides the kth cumulant, as
	a fn of the first k moments Z[i] about the origin."
	 
J::usage =
	"J[y, x] calculates the Jacobian of transformation in
	absolute value, where y and x may either be univariate,
	or multivariate lists."
 
Transform::usage =
	"Transform[eqn, f] attempts to find the density
	of the transform u(x), where x has density f(x).
	The input eqn describes u(x)."
      
Marginal::usage =
	"Marginal[var, f] calculates the marginal joint
	pdf of the variable(s) specified in var,
	for var subset of x, and x with density f(x)."

Conditional::usage =
	"Conditional[var, f] calculates the conditional
	pdf of f, conditioned upon var."

Template::usage =
	"Template[n] provides a template for the
	n-dimensional multivariate normal pdf."

MVN::usage =
	"MVN[xvec, muvec, varcov] calculates the multivariate
	normal pdf defined on xvec, with mean vector muvec, and
	symmetric positive-definite variance covariance matrix varcov."

MVNRandom::usage = 
	"MVNRandom[n, varcov] generates n pseudorandom
	k-dimensional drawings from a multivariate normal
	distribution with zero mean vector, and symmetric
	positive-definite variance covariance matrix varcov (kxk)."

Monty::usage =
	"Monty[data, {a, b, c}, f] plots the relative frequency
	of data between the values a and b, with stepsize c.
	If f is specified, the plot is superimposed on top
	of a plot of f. Of course, this will only work if
	domain[f] has also been specified."

(* Attributes for Public Functions *)
SetAttributes[ domain,       HoldFirst                ]
SetAttributes[ DF,          {HoldRest, ReadProtected} ]
SetAttributes[ Expect,      {HoldRest, ReadProtected} ]
SetAttributes[ Transform,   {HoldRest, ReadProtected} ]
SetAttributes[ Marginal,    {HoldRest, ReadProtected} ]
SetAttributes[ Convert,     {HoldRest, ReadProtected} ]
SetAttributes[ Conditional, {HoldRest, ReadProtected} ]
SetAttributes[ Monty,       {HoldRest, ReadProtected} ]

(* This package switches off an irritating error message. 
   If you prefer to leave it On, simply remove the following line. *)

Off[On::none]

Begin["`Private`"]

SetAttributes[ TUni,        HoldRest ]
SetAttributes[ TMulti,      HoldRest ]
SetAttributes[ Extremity,   HoldRest ]

DF[x_, f_] := Module[{z, b = domain[f], i=1}, 
      If[ MatrixQ[b] == False,   
          Integrate[f, {b[[1]], b[[2]], x}],
          Nest[Function[ {z}, Integrate[z, {b[[i, 1]], b[[i, 2]], x[[i++]] } ]  
                       ], f, Length[b] ]   
        ] 
    ]

MrSpeedy[cdf_, S_] := (Apply[Outer, Prepend[S, cdf]] // Flatten) . 
                      (Nest[{-#, #}&, 1, Length[S]]  // Flatten)                                   

Expect[u_, f_] := Module[{z, b = domain[f], i=1}, 
         If[ MatrixQ[b] == False,  
             Integrate[u f, b] // PowerExpand,
             Nest[Function[{z}, Integrate[z, b[[i++]]] ], u f, Length[b] ] 
           ] 
      ]

CentralM[k_, M_] := Sum[Binomial[k, j] (-1)^(k-j) M[j]  M[1]^(k-j), {j, 0, k}]

CumulantM[k_, M_] := (-1)^(k-1)Det[ Table[
            Which[ i-j >=  0,  Binomial[i-1, Max[j-2, 0]] M[i-j+1], 
                   i-j == -1,  1, 
                   True,       0  ],
             {i, 1, k}, {j, 1, k}    ] ]

(* Transformations *)

J[y_, x_]  := Abs[Det[Map[D[Flatten[{y}],#]&, Flatten[{x}]]]] 

TUni[eqn_, f_] := Module[ {xx, xlow, xup, yy, ylow, yup, yt, inv, zz},
   {xx, xlow, xup} = domain[f];
          {yy, yt} = {eqn[[1,1]], eqn[[1,2]]};        
               inv = Solve[eqn, xx];
               If[ Length[inv] > 1, Throw[Message[Transform::m]]];
               zz = (f/. inv)  J[xx/.inv, yy];

    If[ StringMatchQ[ ToString[N[yt]], "*Log*"] == True, 
        zz[[1]],
        {ylow, yup} = Sort[{Limit[yt//N, xx -> xlow, Direction -> -1 ],
                            Limit[yt//N, xx -> xup,  Direction ->  1 ]  }];
        If[Count[{ylow<yup}, True] == 1, { zz[[1]], {yy, ylow, yup} }, zz[[1]] ]  ]  ]   
   
TMulti[eqn_, f_] := Module[ {xvec, yvec, inv, zz, b = domain[f]},
    xvec = Transpose[b][[1]];
    yvec = Map[ Part[#, 1]&, eqn];
    inv = Solve[eqn, xvec];
    If[ Length[inv] > 1, Throw[Message[Transform::m]]];
    zz = (f/. inv)  J[xvec/.inv, yvec]
    ]

Extremity[eqn_, f_] := Module[ {xvec, yvec, tttt, pop1, pop2, pop3, pop4,
                          trI1, trI2, trI3, trI4, A1, A2, AA, BB, b = domain[f]},
   
   Off[Infinity::indet]; 
   Off[Power::infy];   
   Off[List::nord]; 
   
   xvec = Transpose[b][[1]];
   yvec = Map[ Part[#, 1]&, eqn];
   tttt = Map[ Part[#, 2]&, eqn];
    
   pop1 = Prepend[Transpose[Delete[Transpose[b], 1]], List];
   pop2 = Flatten[Apply[Outer, pop1], Length[xvec]-1];
   pop3 = Map[MapThread[(#1 -> #2)&, {xvec, #}]&, pop2];
   pop4 = Flatten[Map[Permutations[#]&, pop3], 1];
   
   trI1 = Prepend[Partition[Range[2Length[xvec]], 2], List];
   trI2 = Flatten[Apply[Outer, trI1], Length[xvec]-1];
   trI3 = Flatten[Map[Permutations[#]&, trI2], 1];
   trI4 = trI3 /. {(x_ /; OddQ[x]) -> -1, (x_ /; EvenQ[x]) -> 1};
    
   Table[A1 = Table[ i = 1; Nest[ Function[{z}, Limit[z//N, pop4[[j, i]],  
                                           Direction -> trI4[[j, i++]] ] ], 
                                  tttt[[k]], 
                                  Length[xvec]  
                               ],    {j, Length[pop4]} 
                  ];
         A2 = Map[ (tttt[[k]] /. #)&, pop3];  (* value at x1&x2; not limit *)
         AA = Join[A1, A2];    (* can add other tests A3, A4 etc over here *) 
         BB = Delete[AA, Join[Position[AA, Indeterminate  ], 
                              Position[AA, ComplexInfinity] ] ]//N;
         {yvec[[k]], Min[BB], Max[BB]},     {k, Length[yvec]} 
       ] 
    ]   

Transform[eqn_, f_] := Module[ {z, w, w2, b = domain[f]},
    Catch[  If[ MatrixQ[b] == False,  
                TUni[eqn, f], 
                
                z = Map[ Part[#, 2]&, eqn];
                If[ StringMatchQ[ ToString[N[z]], "*Log*"] == True, 
                    TMulti[eqn, f][[1]],
                    
                    w = Extremity[eqn, f];
                    w2 = Map[ (# > 0)&, Transpose[w][[3]] - Transpose[w][[2]]];
                    On[Infinity::indet];
                    On[Power::infy];
                    On[List::nord];
                    If[  Count[w2, True] == Length[b], 
                        {TMulti[eqn, f][[1]], w}, 
                         TMulti[eqn, f][[1]]   ] ]] ]]

Transform::m := "Multiple Solutions. Please try manual approach."

(* Marginal and Conditional *)
Marginal[x_, f_] := Module[{i=1, b = domain[f], bb, z}, 
    bb = Delete[b, 
           Flatten[Map[Position[Transpose[b][[1]], #]&, Flatten[{x}]],1]];
    Nest[Function[{z}, Integrate[z, bb[[i++]]] ], f, Length[bb] ]   ]

Convert[x_, f_] := Complement[Transpose[domain[f]][[1]], Flatten[{x}]]

Conditional[x_, f_] := Module[{qw1, qw2, qw3, qw4}, 
   qw1 = StringReplace[ToString[Convert[x, f]], {"{"->"(", "}"->")"}];
   qw2 = ToString[HoldForm[f]] <> qw1;
   qw3 = StringDrop[StringDrop[ToString[Flatten[{x}]], -1], 1];
   qw4 = StringInsert[qw2, " | " <> qw3, -2];
   Print["Here is the conditional pdf ", qw4, "."];
   f/Marginal[x, f]   
   ] 

(* Multivariate Normal *)
Template[n_Integer] := Module[ {xvec, muvec, varcov, xbound, i, j},
   xvec = Table[ StringJoin["x",  ToString[i]] // ToExpression, {i, n}];
  muvec = Table[ StringJoin["mu", ToString[i]] // ToExpression, {i, n}];
 varcov = Table[ StringJoin["s",  ToString[i], ToString[j]] // ToExpression, {i, n}, {j, n}];
 xbound = Map[{#, -Infinity, Infinity}&, xvec];
 Print[ "     xvec = ", xvec,   ";" ];
 Print[ "    muvec = ", muvec,  ";" ];
 Print[ "   varcov = ", varcov, ";"]; 
 Print[ "        f = MVN[xvec, muvec, varcov]" ]; 
 Print[ "domain[f] = ", xbound ]; 
       
            ]

MVN[X_List, MU_List, var_] := Module[{},
  If[MatrixQ[var, NumberQ], 
      If[ var =!= Transpose[var],       Message[MVN::symm]; Return[] ];
      If[Min[N[Eigenvalues[var]]] <= 0, Message[MVN::pd];   Return[] ]
    ];
 (2 Pi)^(-Length[X]/2) Det[var]^(-1/2) Exp[(-1/2) (X-MU).Inverse[var].(X-MU)] 
                                    ]  /; MatrixQ[var]
   
MVN::symm := "Your variance-covariance matrix is not symmetric."
MVN::pd   := "Your variance-covariance matrix is not positive definite."

(* Random Number Generation *)
MVNRandom[n_Integer, var_] := Module[{k,u,d,t,A,Z},
  If[ var =!= Transpose[var],         Message[MVN::symm]; Return[]];
  If[Min[N[Eigenvalues[var]]] <= 0,   Message[MVN::pd];   Return[]];
  k = Length[var];
  {u,d,t} = SingularValues[N[var]];
  A = Transpose[u].DiagonalMatrix[Sqrt[d]];
  Z = Partition[Table[N[Sqrt[-2 Log[Random[]]] Cos[2Pi Random[]]], {n k}], k];
  Map[(A.#)&, Z]                     ]   /; MatrixQ[var, NumberQ]
  
Monty[lis_List, {a_, b_, c_}, f_] :=  
  Module[{xval, r, truefreq, a1, v, simfreq, i, modsimfreq, a2, z = domain[f][[1]]},
         
          xval = Table[r, {r, a+c/2, b-c/2, c}];
         
      truefreq = Table[f//N, Evaluate[{z, a+c/2, b-c/2, c}]];
            a1 = ListPlot[Transpose[{xval, truefreq}], PlotJoined -> True, 
                          PlotStyle -> {AbsoluteThickness[.7], Hue[1,1,1]},
                          DisplayFunction -> Identity];
                          
             v = Ceiling[(lis - a)/c];
       simfreq = Table[Count[v, i], {i, Length[xval]}];
    modsimfreq = simfreq*Apply[Plus, truefreq]/Apply[Plus, simfreq];
            a2 = ListPlot[Transpose[{xval, modsimfreq}], PlotJoined -> True,
                          PlotStyle -> AbsoluteThickness[1],
                          DisplayFunction -> Identity];

         Show[ a1, a2, Frame -> True, Axes -> False, 
               FrameLabel -> {z, ToString[HoldForm[f]], "PDF", ""}, 
               PlotRange -> All, DisplayFunction -> $DisplayFunction]
        ]

Monty[lis_List, {a_, b_, c_}] :=  Module[{xval, r, v, simfreq, i},
          xval = Table[r, {r, a+c/2, b-c/2, c}];
             v = Ceiling[(lis - a)/c];
       simfreq = Table[Count[v, i], {i, Length[xval]}];
       
       ListPlot[Transpose[{xval, simfreq}], PlotJoined -> True,
                Frame -> True, Axes -> False, PlotRange -> All]; ]

End[]
Protect[ DF,          Expect,      CentralM,    CumulantM,   
         J,           Transform,   Marginal,    Conditional, 
         Convert,     Template,    MVN,         MVNRandom,     Monty   ];
EndPackage[]
