(* :Title: Piecewise Smooth Functions *)
(* :Author: Paul A. Rubin *)
(* :Summary:
  This package provides limited support for piecewise smooth functions of
  a single real variable.  It adds "infinitesimal adjustment" operators
  used to distinguish between closed intervals and open intervals,
  provides mechanisms for recording and using piecewise smooth functions
  whose domains are described either exclusively by constants ("frozen"
  functions) or by constants and symbols ("unfrozen" functions), and extends
  kernel functions to correctly process piecewise smooth functions.
*)
(* :Context: PWS` *)
(* :Package Version: 1.0 *)
(* :Mathematica Version: 2.1 *)
(* :Copyright: Copyright 1994, Paul A. Rubin *)
(* :Keywords: piecewise smooth *)
(* :Warning:
  This package extends the definitions of D, Derivative, FindMinimum,
  Integrate, Max, Min, Plot and Sort, in manners that should not conflict
  with normal use of them. *)
(* :Requirement:
  Needs package Utilities`Message` (which should accompany this package) and
  standard package Utilities`FilterOptions`. *)
(* :Limitation:
  This package does not support piecewise smooth functions of multiple
  arguments, nor functions of a single complex argument.  The current
  version also does not support composition of two piecewise smooth functions. *)
(* :Discussion:
  The accompanying notebook provides a detailed walk-through of all features
  of this package. *)

(* Prolog *)
BeginPackage["PWS`"]

Needs["Utilities`FilterOptions`"]
Needs["message`"]

Begin["`Private`"]

sp = MessageQ[ General::spell ];
Off[ General::spell ];
sp1 = MessageQ[ General::spell1 ];
Off[ General::spell1 ];

End[];
(* Usage Messages *)
(* Infinitesimals *)
IA::usage =
"IA[ x ] denotes a number 'infinitesimally' larger than (above) x.  Its primary use is to denote an interval open at the lower end, as in {IA[ a ], b}.";
IB::usage =
"IB[ x ] denotes a number 'infinitesimally' smaller than (below) x.  Its primary use is to denote an interval open at the upper end, as in {a, IB[ b ]}.";
StripInfs::usage =
"StripInfs[f] strips any infinitesimals out of expression 'f' by changing IA[x] or IB[x] to just 'x'.";
IncreasingFunctions::usage =
"IncreasingFunctions is a list of functions of a single real variable known to be monotonically increasing.  Infinitesimals (IA, IB) commute with such functions; e.g., f[IA[x]] = IA[f[x]] when f is a member of IncreasingFunctions.  Functions can be added to or removed from the list.";
DecreasingFunctions::usage =
"DecreasingFunctions is a list of functions of a single real variable known to be monotonically decreasing.  Infinitesimals (IA, IB) are anticommutative with such functions; e.g., f[IA[x]] = IB[f[x]] when f is a member of DecreasingFunctions.  Functions can be added to or removed from the list.";
IAToLimit::usage =
"IAToLimit is a list of functions of a single real variable which are evaluated infinitesimally above an argument by taking the limit of the function value as the argument is approached from above.  Functions can be added to or removed from the list.";
IBToLimit::usage =
"IBToLimit is a list of functions of a single real variable which are evaluated infinitesimally below an argument by taking the limit of the function value as the argument is approached from below.  Functions can be added to or removed from the list.";
IAToValue::usage =
"IAToValue is a list of functions of a single real variable which are evaluated infinitesimally above an argument by ignoring the infinitesimal adjustment and evaluating the function directly at the argument.  Functions can be added to or removed from the list.";
IBToValue::usage =
"IBToValue is a list of functions of a single real variable which are evaluated infinitesimally below an argument by ignoring the infinitesimal adjustment and evaluating the function directly at the argument.  Functions can be added to or removed from the list.";
(* Piecewise Smooth Functions *)
PWS::usage =
"PWS[segs] defines a piecewise smooth function of a single real variable.  The argument 'segs' is a list of segment definitions, where each segment is in turn a list consisting of a domain and a (pure) function to be applied to that domain.  Type 'Message[PWS::notes]' for additional details, and 'Message[PWS::ex]' to see a syntax example.";
PWS::notes =
"Domains of a PWS function can be intervals (lists of two elements) or isolated points (lists of one element), and can be expressed symbolically.  They must be disjoint, but need not be contiguous.  Use infinitesimal adjustments (IA, IB) to denote open-ended intervals.";
PWS::ex =
"Example:  the function which is E^(-x) when x is positive, -E^x when x is negative, and zero when x is zero would be given by 'f = PWS[ {{{-Infinity, IB[0]}, -Exp[#]&}, {{0}, 0&}, {{IA[0], Infinity}, Exp[-#]&} ]'.  The expression 'f[a]' would yield its value at an argument 'a'.";
LeftEnds::usage =
"LeftEnds[f] returns a list of the left endpoints of the domain intervals of a piecewise smooth function f[].  Singleton points are considered to be left endpoints for their domains.";
RightEnds::usage =
"RightEnds[f] returns a list of the right endpoints of the domain intervals of a piecewise smooth function f[].  Singleton points are considered to be right endpoints for their domains.";
PWSForm::usage =
"PWSForm[f] prints a piecewise smooth function f[] (frozen or unfrozen) in a tabular format, domains in one column, formulas in another.";
(* Frozen Piecewise Smooth Functions *)
PWSf::usage =
"PWSf[segs] defines a 'frozen' piecewise smooth function of a single real variable.  It differs from a regular piecewise smooth function (PWS) only in that the domain lists must resolve to real numbers (with or without infinitesimal adjustments), and cannot contain undefined symbols.  The segment list of a PWSf function is stored in sorted order.  Although a PWSf function can be constructed directly, the preferred approach is to use PWSFreeze.
The basic arithmetic operators (addition, subtraction, multiplication and division, power) have been extended to handle two PWSf arguments or a PWSf argument and a scalar argument.";
PWSFreeze::usage =
"PWSFreeze[f] checks a functions f = PWS[segs] to verify that all segment domains resolve to real numbers (with or without infinitesimal adjustments), and that the domains are consistent (disjoint, with no singleton points involving infinitesimals).  If successful, a PWSf function is created.";
PWSOperate::usage =
"PWSOperate[f, g, op] applies the binary operator op to two frozen piecewise smooth functions f and g, producing another frozen piecewise smooth function as output.  The resulting function is defined only at points x in the intersection of the domains of f and g, where it takes the value op[f[x], g[x]].  PWSOperate[f, g, op, def] generates a function defined on the union of the domains of f and g, replacing f or g with function def[] whenever either is undefined.  PWSOperate[f, g, op, def_f, def_g] uses separated default functions in place of f and g.  If either default function is specified to be Indeterminate&, the resulting function is undefined at points outside the domain of the corresponding argument function.";
PWSAccumulate::usage =
"Given a frozen piecewise smooth function f[], PWSAccumulate[f] returns another frozen piecewise smooth function g[], such that for any x in the domain of f[], g[x] is the cumulative integral of f[] from the start of the domain to x.";
(* Extensions *)
ExtendedToPWS::usage =
"ExtendedToPWS gives a list of standard Mathematica functions (other than arithmetic operators) which have been extended to handle piecewise smooth functions.  For syntax guidance on a particular operator 'f', type '?fPWS'.";
DistributeOverPWS::usage =
"DistributeOverPWS lists unary operators which distribute across piecewise smooth functions.  If operator f is listed in DistributeOverPWS and g is a PWS or PWSf function, then f[g] will be a function of the same type (PWS or PWSf) as g, with the same domains as g, whose value at any x is f[g[x]].";
Max::usage = Max::usage <>
"\nGiven two frozen piecewise smooth functions f[] and g[], Max[f, g] is a frozen piecewise smooth function whose domain is the union of the domains of f[] and g[], taking the value Max[f[x], g[x]] when x is in the intersection of their domains.  For points in the domain of one function but not the other, Max[f, g] takes the value assigned by the function defined there.";
Min::usage = Min::usage <>
"\nGiven two frozen piecewise smooth functions f[] and g[], Min[f, g] is a frozen piecewise smooth function whose domain is the union of the domains of f[] and g[], taking the value Min[f[x], g[x]] when x is in the intersection of their domains.  For points in the domain of one function but not the other, Min[f, g] takes the value assigned by the function defined there.";
Plot::usage = Plot::usage <>
"\nPlot[f[x], {x, xlo, xhi}, options] will plot a frozen piecewise smooth (PWSf) function f of an argument x over the range xlo <= x <= xhi.  If f is not already frozen, Plot will attempt to freeze it, and will fail if unable to do so.  Options to Plot behave as usual, except that PlotStyles are applied cyclically to the domains of f, rather than to multiple functions.  For information on plotting multiple piecewise smooth functions, type '?PlotPWSList'.";
Plot::usage = Plot::usage <>
"\nPlot[{f1[x], f2[x], ...}, {x, xlo, xhi}, options] superimposes plots of a list of PWSf functions.  If an optional value for PlotStyles is supplied, it may be:  a single plot style (a single graphics directive, or list of depth one), which will be applied to every segment of every function; a list of plot styles (depth two list), which will be applied cyclically to the domains of every function; or a list of lists of plot styles (depth three list), where the level-one sublists will be assigned cyclically to the various functions.";
D::usage = D::usage <>
"\nD[f[x], x] takes the derivative of a piecewise smooth (PWS or PWSf) function f with respect to its argument x, by converting all domains to open intervals (deleting isolated points) and taking the derivative separately on each domain.";
Derivative::usage = D::usage <>
"\nDerivative[n][f] takes the n-th derivative of a piecewise smooth (PWS or PWSf) function f with respect to its argument (which is not specified in the expression), converting all domains to open intervals (deleting isolated points) and taking the derivative separately on each domain.  Partial derivatives are not supported, since PWS and PWSf functions must take a single (scalar) argument.";
Integrate::usage = Integrate::usage <>
"\nIntegrate[f[x], x] computes the indefinite integral of a piecewise smooth (PWS or PWSf) function f with argument x.  The integral is computed separately on each interval domain, dropping isolated points.  Integrate[f[x], {x, xlo, xhi}] computes the definite integral from 'xlo' to 'xhi' by freezing the function f, and fails if f cannot be frozen.";
FindMinimum::usage = FindMinimum::usage <>
"\nFindMinimum[f[x], {x}, options] searches the domains of a piecewise smooth (PWS or PWSf) function f with argument x and returns a list of local minima whose function values are within PrecisionGoal of the global minimum of f.  The result uses the standard format for FindMinimum.  Note that starting points and range limits for the argument x are neither required nor allowed.";
Distribute::usage = Distribute::usage <>
"\nDistribute[f[h], PWS] or Distribute[f[h], PWSf] causes the function f[] to be distributed over the piecewise smooth function h[].  The result is another piecewise smooth function (frozen if h[] was), having the same domains as h[], with each domain function replaced by its composition with f[].";
Join::usage = Join::usage <>
"\nJoin[f, g] combines two piecewise smooth functions f and g by taking the union of their domains, associating the appropriate function with each domain.  It is the user's responsibility to ensure that the domains of f and g do not overlap.  If both arguments are frozen (PWSf) functions, so will the result be; otherwise, an unfrozen (PWS) function is produced.";
(* Internal Definitions *)
Begin[ "`Private`" ];
(* Infinitesimals *)
(* Removing Infinitesimals *)
StripInfs[ x_ ] := x //. {IA[ u_ ] -> u, IB[ v_ ] -> v}
Protect[ StripInfs ];
(* Special case - argument is infinite *)
IA[ Infinity ] = Infinity;
IB[ Infinity ] = Infinity;
IA[ -Infinity ] = -Infinity;
IB[ -Infinity ] = -Infinity;
(* Arithmetic operators *)
IA /: Plus[ a_, IA[ x_ ] ] := IA[ a + x ]
IB /: Plus[ a_, IB[ x_ ] ] := IB[ a + x ]
IA /: Times[ a_?Positive, IA[ x_ ] ] := IA[ a x ]
IA /: Times[ a_?Negative, IA[ x_ ] ] := IB[ a x ]
IB /: Times[ a_?Positive, IB[ x_ ] ] := IB[ a x ]
IB /: Times[ a_?Negative, IB[ x_ ] ] := IA[ a x ]
IA /: Power[ IA[ x_ ], a_?Positive ] := IA[ x^a ] /; Positive[ x ]
IA /: Power[ IA[ x_ ], a_?Negative ] := IB[ x^a ] /; Positive[ x ]
IB /: Power[ IB[ x_ ], a_?Positive ] := IB[ x^a ] /; Positive[ x ]
IB /: Power[ IB[ x_ ], a_?Negative ] := IA[ x^a ] /; Positive[ x ]
IA /: Power[ a_?Positive, IA[ x_ ] ] := IA[ a^x ] /; a > 1
IA /: Power[ a_?Positive, IA[ x_ ] ] := IB[ a^x ] /; a < 1
IB /: Power[ a_?Positive, IB[ x_ ] ] := IB[ a^x ] /; a > 1
IB /: Power[ a_?Positive, IB[ x_ ] ] := IA[ a^x ] /; a < 1
IA /: IA[ IB[ _ ] ] := Indeterminate
(* Relational Operators *)
IA /: Less[ x_, IA[ y_ ] ] := (x <= y)
IB /: Less[ x_, IB[ y_ ] ] := (x < y)
IA /: Less[ IA[ x_ ], y_ ] := (x < y)
IB /: Less[ IB[ x_ ], y_ ] := (x <= y)
IA /: Less[ IA[ x_ ], IA[ x_ ] ] := False
IB /: Less[ IB[ x_ ], IB[ x_ ] ] := False
IA /: LessEqual[ x_, IA[ y_ ] ] := (x <= y)
IB /: LessEqual[ x_, IB[ y_ ] ] := (x < y)
IA /: LessEqual[ IA[ x_ ], y_ ] := (x < y)
IB /: LessEqual[ IB[ x_ ], y_ ] := (x <= y)
IA /: LessEqual[ IA[ x_ ], IA[ x_ ] ] := True
IB /: LessEqual[ IB[ x_ ], IB[ x_ ] ] := True
IA /: Greater[ x_, IA[ y_ ] ] := (x > y)
IB /: Greater[ x_, IB[ y_ ] ] := (x >= y)
IA /: Greater[ IA[ x_ ], y_ ] := (x >= y)
IB /: Greater[ IB[ x_ ], y_ ] := (x > y)
IA /: Greater[ IA[ x_ ], IA[ x_ ] ] := False
IB /: Greater[ IB[ x_ ], IB[ x_ ] ] := False
IA /: GreaterEqual[ x_, IA[ y_ ] ] := (x > y)
IB /: GreaterEqual[ x_, IB[ y_ ] ] := (x >= y)
IA /: GreaterEqual[ IA[ x_ ], y_ ] := (x >= y)
IB /: GreaterEqual[ IB[ x_ ], y_ ] := (x > y)
IA /: GreaterEqual[ IA[ x_ ], IA[ x_ ] ] := True
IB /: GreaterEqual[ IB[ x_ ], IB[ x_ ] ] := True
IA /: Equal[ IA[ _ ], _ ] := False
IA /: Equal[ _, IA[ _ ] ] := False
IB /: Equal[ IB[ _ ], _ ] := False
IB /: Equal[ _, IB[ _ ] ] := False
IA /: Unequal[ IA[ _ ], _ ] := True
IA /: Unequal[ _, IA[ _ ] ] := True
IB /: Unequal[ IB[ _ ], _ ] := True
IB /: Unequal[ _, IB[ _ ] ] := True
(* IA, IB are idempotents *)
IA /: IA[ IA[ x_ ] ] := IA[ x ]
IB /: IB[ IB[ x_ ] ] := IB[ x ]
(* Functions receiving special treatment *)
Unprotect[ Min, Max ];
Min[ {a___, IA[ x_ ], b___} ] :=
  If[ x < Min[ {a, b} ], IA[ x ], Min[ {a, b} ] ]
Min[ {a___, IB[ x_ ], b___} ] :=
  If[ x <= Min[ {a, b} ], IB[ x ], Min[ {a, b} ] ]
Min[ a__:Infinity, IA[ x_ ], b___ ] :=
  If[ x < Min[ a, b ], IA[ x ], Min[ a, b ] ]
Min[ a__:Infinity, IB[ x_ ], b___ ] :=
  If[ x <= Min[ a, b ], IB[ x ], Min[ a, b ] ]
Max[ {a___, IA[ x_ ], b___} ] :=
  If[ x >= Max[ {a, b} ], IA[ x ], Max[ {a, b} ] ]
Max[ {a___, IB[ x_ ], b___} ] :=
  If[ x > Max[ {a, b} ], IB[ x ], Max[ {a, b} ] ]
Max[ a__:-Infinity, IA[ x_ ], b___ ] :=
  If[ x >= Max[ a, b ], IA[ x ], Max[ a, b ] ]
Max[ a__:-Infinity, IB[ x_ ], b___ ] :=
  If[ x > Max[ a, b ], IB[ x ], Max[ a, b ] ]
Protect[ Min, Max ];
Unprotect[ Sort ];
Sort[ {x___, f_[ y_ ], z___} ] := Sort[ {x, f[ y ], z}, LessEqual ] /;
  (f === IA || f === IB) && And @@(NumberQ /@ {x, f[y], z})
Protect[ Sort ];
IA /: Abs[ IA[ x_ ] ] := IA[ Abs[ x ] ] /; x >= 0
IA /: Abs[ IA[ x_ ] ] := IB[ Abs[ x ] ] /; x < 0
IB /: Abs[ IB[ x_ ] ] := IB[ Abs[ x ] ] /; x >= 0
IB /: Abs[ IB[ x_ ] ] := IA[ Abs[ x ] ] /; x < 0
IA /: Chop[ IA[ x_ ] ] := IA[ x ] /; Chop[ x ] != 0
IA /: Chop[ IA[ x_ ] ] := 0 /; Chop[ x ] == 0
IB /: Chop[ IB[ x_ ] ] := IB[ x ] /; Chop[ x ] != 0
IB /: Chop[ IB[ x_ ] ] := 0 /; Chop[ x ] == 0
IA /: Floor[ IA[ x_ ] ] := Floor[ x ]
IA /: Ceiling[ IA[ x_ ] ] := If[ x == Ceiling[ x ], x + 1, Ceiling[ x ] ]
IB /: Ceiling[ IB[ x_ ] ] := Ceiling[ x ]
IB /: Floor[ IB[ x_ ] ] := If[ x == Floor[ x ], x - 1, Floor[ x ] ]
IA /: Positive[ IA[ x_ ] ] := !Negative[ x ]
IB /: Positive[ IB[ x_ ] ] := Positive[ x ]
IA /: Negative[ IA[ x_ ] ] := Negative[ x ]
IB /: Negative[ IB[ x_ ] ] := !Positive[ x ]
IA /: NonNegative[ IA[ x_ ] ] := NonNegative[ x ]
IB /: NonNegative[ IB[ x_ ] ] := Positive[ x ]
IA /: Sign[ IA[ x_ ] ] := Sign[ x ] /; x != 0
IA /: Sign[ IA[ 0 ] ] := 1
IB /: Sign[ IB[ x_ ] ] := Sign[ x ] /; x != 0
IB /: Sign[ IB[ 0 ] ] := -1
(* Lists of unary functions with known behavior w.r.t. infinitesimals *)
(* IA, IB commute with increasing functions *)
IncreasingFunctions = {ArcSin, ArcTan, Erf, Log};
IA /: f_[ IA[ x_ ] ] := IA[ f[ x ] ] /; MemberQ[ IncreasingFunctions, f ]
IB /: f_[ IB[ x_ ] ] := IB[ f[ x ] ] /; MemberQ[ IncreasingFunctions, f ]
(* IA, IB reverse when passed through decreasing functions *)
DecreasingFunctions = {ArcCos, Erfc};
IA /: f_[ IA[ x_ ] ] := IB[ f[ x ] ] /; MemberQ[ DecreasingFunctions, f ]
IB /: f_[ IB[ x_ ] ] := IA[ f[ x ] ] /; MemberQ[ DecreasingFunctions, f ]
(* Some functions may be evaluated at infinitesimal arguments using limits *)
IAToLimit = {};
IBToLimit = {};
IA /: f_[ IA[ x_ ] ] := Block[ {z}, Limit[ f[ z ], z -> x, Direction -> -1 ]
                        ] /; MemberQ[ IAToLimit, f ]
IB /: f_[ IB[ x_ ] ] := Block[ {z}, Limit[ f[ z ], z -> x, Direction -> 1 ]
                        ] /; MemberQ[ IBToLimit, f ]
(* Some functions evaluate infinitesimally near a point by ignoring the
   infinitesimal adjustment *)
IAToValue = {NumberQ, Round, ValueQ};
IBToValue = {NumberQ, Round, ValueQ};
IA /: f_[ IA[ x_ ] ] := f[ x ] /; MemberQ[ IAToValue, f ]
IB /: f_[ IB[ x_ ] ] := f[ x ] /; MemberQ[ IBToValue, f ]
Protect[ IA, IB, IncreasingFunctions, DecreasingFunctions, IAToLimit,
         IBToLimit, IAToValue, IBToValue ];
(* Piecewise Smooth Functions *)
(* Interval Selection *)
IntervalQ[ x_, {a_, b_} ] := (a <= x) && (x <= b)
IntervalQ[ x_, {a_} ] := (x == a)
(* PWS - A Piecewise Smooth Function *)
(* Evaluation *)
PWS[ a_List ][ x_?NumberQ ] :=
  Scan[
    If[ IntervalQ[ x, #[[1]] ], Return[ #[[2]][ x ] ] ]&,
    a
  ]
(* Arithmetic with one PWS operand *)
(a_) + PWSf[ b_ ] ^:=
  PWSf[ {#1[[1]],
        Function[ {x},
                  Evaluate[ Simplify[ a + #1[[2]][ x ] ] ]
        ]
       }& /@ b ] /;
    FreeQ[ a, PWS ] && FreeQ[ a, PWSf ] &&
    Head[ a ] =!= Function;
(a_) + PWSf[ b_ ] ^:=
  PWSf[ {#1[[1]],
        Function[ {x},
                  Evaluate[
                    Simplify[ a[ x ] + #1[[2]][ x ] ]
                  ]
        ]
       }& /@ b ] /;
    FreeQ[ a, PWS ] && FreeQ[ a, PWSf ] &&
    Head[ a ] == Function;
a__ + b_ + PWSf[ c_ ] ^:=
  ReleaseHold[ Hold[ a + ReleaseHold[ b + PWSf[c] ] ] ]
(a_) * PWSf[ b_ ] ^:=
  PWSf[ {#1[[1]],
        Function[ {x},
                  Evaluate[ Simplify[ a * #1[[2]][ x ] ] ]
        ]
       }& /@ b ] /;
    FreeQ[ a, PWS ] && FreeQ[ a, PWSf ] &&
    Head[ a ] =!= Function;
(a_) * PWSf[ b_ ] ^:=
  PWSf[ {#1[[1]],
        Function[ {x},
                  Evaluate[
                    Simplify[ a[ x ] * #1[[2]][ x ] ]
                  ]
        ]
       }& /@ b ] /;
    FreeQ[ a, PWS ] && FreeQ[ a, PWSf ] &&
    Head[ a ] == Function;
a__ b_ PWSf[ c_ ] ^:=
  ReleaseHold[ Hold[ a ReleaseHold[ b PWSf[c] ] ] ]
PWSf[ b_ ]^(a_) ^:=
  PWSf[ {#1[[1]],
        Function[ {x},
                  Evaluate[ Simplify[ #1[[2]][ x ]^a ] ]
        ]
       }& /@ b ] /;
    FreeQ[ a, PWS ] && FreeQ[ a, PWSf ] &&
    Head[ a ] =!= Function;
PWSf[ b_ ]^(a_) ^:=
  PWSf[ {#1[[1]],
        Function[ {x},
                  Evaluate[
                    Simplify[ #1[[2]][ x ]^a[ x ] ]
                  ]
        ]
       }& /@ b ] /;
    FreeQ[ a, PWS ] && FreeQ[ a, PWSf ] &&
    Head[ a ] == Function;
(a_)^PWSf[ b_ ] ^:=
  PWSf[ {#1[[1]],
        Function[ {x},
                  Evaluate[ Simplify[ a^#1[[2]][ x ] ] ]
        ]
       }& /@ b ] /;
    FreeQ[ a, PWS ] && FreeQ[ a, PWSf ] &&
    Head[ a ] =!= Function;
(a_)^PWSf[ b_ ] ^:=
  PWSf[ {#1[[1]],
        Function[ {x},
                  Evaluate[
                    Simplify[ a[ x ]^#1[[2]][ x ] ]
                  ]
        ]
       }& /@ b ] /;
    FreeQ[ a, PWS ] && FreeQ[ a, PWSf ] &&
    Head[ a ] == Function;
(* Printing PWS Functions *)
PWSForm[ f_[ a_ ] ] :=
  Module[
    {pw},
    pw = PageWidth /. Options[ "stdout" ];
    SetOptions[ "stdout", PageWidth -> Infinity ];
    If[
      f === PWS,
      Print[ "Unfrozen Piecewise Smooth Function:" ],
      Print[ "Frozen Piecewise Smooth Function:" ]
    ];
    Print[
      TableForm[
        {OutputForm[ #[[1]] ], "==>", OutputForm[ #[[2]] ]}& /@ a,
        TableHeadings -> {None, {"Domain", "", "Function"}}
      ]
    ];
    SetOptions[ "stdout", PageWidth -> pw ];
  ] /; f === PWS || f === PWSf
Protect[ PWSForm ];
(* Extracting Domain Endpoints *)
LeftEnds[ (PWS | PWSf)[ a_ ] ] := First /@ Transpose[ a ][[1]]
RightEnds[ (PWS | PWSf)[ a_ ] ] := Last /@ Transpose[ a ][[1]]
Protect[ LeftEnds, RightEnds ];
(* Frozen Piecewise Smooth Functions *)
(* PWSf - a piecewise smooth function with frozen (constant) domains *)
(* Evaluate same way as an unfrozen PWS *)
PWSf[ a_List ][ x_?NumberQ ] := PWS[ a ][ x ]
(* Extend arithmetic operators to frozen PWS functions; use PWSOperate
   when both arguments are frozen PWS functions *)
(a_PWSf) + (b_PWSf) ^:= PWSOperate[ a, b, Plus ];
(a_PWSf)*(b_PWSf) ^:= PWSOperate[ a, b, Times ];
(a_PWSf)^(b_PWSf) ^:= PWSOperate[ a, b, Power ];
(* Arithmetic with one PWSf operand *)
(a_) + PWSf[ b_ ] ^:=
  PWSf[ {#1[[1]],
         Function[ {x}, Evaluate[ Simplify[ a + #1[[2]][ x ] ] ] ]
        }& /@ b
  ] /; FreeQ[ a, PWS ] && FreeQ[ a, PWSf ];
(a_) * PWSf[ b_ ] ^:=
  PWSf[ {#1[[1]],
         Function[ {x}, Evaluate[ Simplify[ a * #1[[2]][ x ] ] ] ]
        }& /@ b
  ] /; FreeQ[ a, PWS ] && FreeQ[ a, PWSf ];
PWSf[ b_ ]^(a_) ^:=
  PWSf[ {#1[[1]],
         Function[ {x}, Evaluate[ Simplify[ #1[[2]][ x ]^a ] ] ]
        }& /@ b
  ] /; FreeQ[ a, PWS ] && FreeQ[ a, PWSf ];
(a_)^PWSf[ b_ ] ^:=
  PWSf[ {#1[[1]],
         Function[ {x}, Evaluate[ Simplify[ a^#1[[2]][ x ] ] ] ]
        }& /@ b
  ] /; FreeQ[ a, PWS ] && FreeQ[ a, PWSf ];
(* "Freezing" a PWS function (when domains are constant) *)
PWSFreeze[ f_PWS ] :=
  Module[
    {a = f[[1]], b, c},
    (* isolate list of domains *)
    b = N[ Transpose[ a ][[1]] ];
    (* verify all intervals are numeric *)
    If[ c = Select[ b, Not @ (And @@ (NumberQ /@
                              (N[ # ] /. Infinity -> 0)))&
            ];
        c == {},
        (* scan for singleton infinitesimals *)
        c = Cases[ b, {IA[_]} | {IB[_]} ];
        If[ c =!= {},
            Message[ PWSFreeze::sngi, # ]& /@ c;
            Return[ f ]
        ];
        (* replace singletons {x} with {x, x} and sort *)
        b = Sort[
              Replace[ #, {{u_}, g_} -> {{u, u}, g} ]& /@ a,
              LessEqual[ N[ #1[[1, 1]] ], N[ #2[[1, 1]] ] ]&
            ];
        (* test for overlap of domains *)
        c = Partition[ Transpose[ b ][[1]], 2, 1 ];
          (* c contains pairs of consecutive domains *)
        c = Select[ c, (N[ #[[1, 2]] ] >= N[ #[[2, 1]] ])& ];
          (* c lists pairs of overlapping domains *)
        If[ c =!= {},
            (* replace any singletons in c *)
            c = c /. {x_, x_} -> {x};
            Message[ PWSFreeze::over, #[[1]], #[[2]] ]& /@ c;
            Return[ f ]
        ];
        (* revert singletons to notation {x} and return *)
        b = Replace[ #, {{u_, u}, g_} -> {{u}, g} ]& /@ b;
        Return[ PWSf[ b ] ],
    (* else if any nonnumeric intervals, return original
       function unchanged *)
        Message[ PWSFreeze::nonn, # ]& /@ c;
        f
    ]
  ]
(* Freezing a frozen function has no effect *)
PWSFreeze[ g_PWSf ] := g;
(* Diagnostic messages *)
PWSFreeze::sngi =
"Defective PWS function detected, containing the following isolated point involving infinitesimals:  `1`.";
PWSFreeze::over =
"Defective PWS function detected, containing the following overlapping domains:  `1` and `2`.";
PWSFreeze::nonn =
"Unable to freeze piecewise smooth function containing non-numeric domain `1`.";
(* PWSAccumulate - cumulative integral of a PWSf function *)
PWSAccumulate[ PWSf[ aa_ ] ] :=
  Module[
    {a, a2, x, b, f, y},
    (* convert singletons to intervals *)
    a = Replace[ #, {{u_}, f_} -> {{u, u}, f} ]& /@ aa;
    (* integrate over each interval *)
    a =
      Cases[ a, {{u_, v_}, f_} :>
                {{u, v}, Function[ {x}, Evaluate[ Integrate[ f[ x ], x ] ] ]}
      ];
    (* evaluate at endpoints *)
    b = Cases[ a, {{u_, v_}, f_} :> {f[ u ], f[ v ]} ];
    (* delete infinitesimals *)
    b = Transpose[ b /. {IA[ x_ ] -> x, IB[ x_ ] -> x} ];
    (* compute running subtotals *)
    b = FoldList[ Plus, 0, # ]& /@ b;
    (* drop first entry in row 1, last in row 2, and
       subtract the rows *)
    b = Drop[ b[[2]], -1 ] - Rest[ b[[1]] ];
    (* b has cum. sum of integrals over intervals *)
    a2 = Cases[ Transpose[ a ][[2]], f_ :> f[#] ];
    a2 = Function /@ (a2 + b);
    a = Transpose[ {Transpose[ a ][[1]], a2} ];
    PWSf[ a ]
  ]
Protect[ PWSAccumulate ];
(* Applying binary operators to frozen PWS functions *)
PWSOperate[ PWSf[ aa_ ], PWSf[ bb_ ], op_,
            def1_:Indeterminate, def2_:Null ] :=
  Module[
    {a, b, x, y, z = {}, u, f, g, d1, d2, upto, next},
    upto = {IA[ u_ ] -> u, u_ -> IB[ u ]};
    next = {IB[ u_ ] -> u, u_ -> IA[ u ]};
    (* default operand behaviors *)
    d1 = def1;
    d2 = If[ def2 === Null, d1, def2 ];
    (* argument function lists *)
    a = Sort[ aa /. {{u_}, f_} :> {{u, u}, f},
              LessEqual[ N[ #1[[1, 1]] ], N[ #2[[1, 1]] ] ]&
        ];
    b = Sort[ bb /. {{u_}, f_} :> {{u, u}, f},
              LessEqual[ N[ #1[[1, 1]] ], N[ #2[[1, 1]] ]  ]&
        ];
    x = -Infinity;
    While[
      a != {} && b != {},
      (* starting point for next segment *)
      y = Min[ a[[1, 1, 1]], b[[1, 1, 1]] ];
      x = Max[ x, y ];
      (* is starting point in next a interval? *)
      If[ x < a[[1, 1, 1]],
          y = a[[1, 1, 1]] /. upto; f = d1,  (* no *)
          y = a[[1, 1, 2]]; f = a[[1, 2]]  (* yes *)
      ];
      (* is starting point in next b interval? *)
      If[ x < b[[1, 1, 1]],
          y = Min[ y, b[[1, 1, 1]] /. upto ]; g = d2,  (* no *)
          y = Min[ y, b[[1, 1, 2]] ]; g = b[[1, 2]]  (* yes *)
      ];
      (* add a segment from x to y *)
      AppendTo[ z, {{x, y}, (Evaluate[ op[ f[ # ], g[ # ] ] ])&} ];
      (* drop exhausted segments *)
      If[ !(a[[1, 1, 2]] > y), a = Rest[ a ] ];
      If[ !(b[[1, 1, 2]] > y), b = Rest[ b ] ];
      (* bump up the interval starting point *)
      x = y /. next
    ];
    (* at least one list is exhausted *)
    If[ a != {},
        a[[1, 1, 1]] = Max[ x, a[[1, 1, 1]] ];
        z =
          Join[ z,
                Cases[ a, {{u_, v_}, f_} :>
                          {{u, v}, (Evaluate[ op[ f[ # ], d2[ # ] ] ])&}
                ]
          ]
    ];
    If[ b != {},
        b[[1, 1, 1]] = Max[ x, b[[1, 1, 1]] ];
        z =
          Join[ z,
                Cases[ b, {{u_, v_}, g_} :>
                          {{u, v}, (Evaluate[ op[ d1[ # ], g[ # ] ] ])&}
                ]
          ]
    ];
    (* delete undefined segments *)
    z = Select[ z, FreeQ[ #, Indeterminate ]& ];
    Return[ PWSf[ z /. {{u_, u_}, f_} :> {{u}, f} ] ]
  ]
Protect[ PWSOperate ];
(* Extensions of Built-in Operators *)
(* List of extended functions *)
ExtendedToPWS =
  {Plot, D, Derivative, Integrate, FindMinimum, Join, Min, Max};
Protect[ ExtendedToPWS ];
(* Min, Max of two PWSf functions *)
f_[ a_PWSf, b_PWSf ] ^:=
  Module[
    {inf},
    (* set inf to +/- Infinity based on operator f *)
    inf = If[ f === Min, Infinity, -Infinity ];
    (* apply PWSOperate *)
    PWSOperate[ a, b, f, inf& ]
  ] /; f === Max || f === Min
(* Plot *)
Unprotect[ Plot ];
(* Plot frozen PWS functions by forming a list of plots, one per domain
   interval *)
Protect[ PWS`EndPoints ]; (* optional parameter name *)
Plot[ f_[ x_ ], {x_, lo_, hi_}, opts___ ] :=
  Module[
    {a = f[[1]], ps, u, v, g, s, ep, p, px, olist,
     l0 = N[ lo ], h0 = N[ hi ]},
    (* get the plot styles *)
    ps = PlotStyle /. {opts} /. Options[ Plot ];
    (* make ps a list of lists *)
    If[ VectorQ[ ps ], ps = {ps} ];
    If[ ps === Automatic, ps = {{}} ];
    (* singleton directive? *)
    If[ Head[ ps ] =!= List, ps = {{ps}} ];
    (* get the dots option *)
    ep = Flatten[ {EndPoints} /. {opts} /. EndPoints -> {} ];
    (* isolate the graphics options *)
    olist = FilterOptions[ Graphics, opts ];
    (* convert a to numbers *)
    a = ({N /@ #[[1]], #[[2]]}& /@ a) /. {IA[ u_ ] -> u, IB[ v_ ] -> v};
    (* convert singletons to intervals *)
    a = Replace[ #, {{u_}, g_} -> {{u, u}, g} ]& /@ a;
    (* restrict each interval to the plot range *)
    p =
      Replace[ #, {{u_, v_}, g_} -> {{Max[ u, l0 ], Min[ v, h0 ]}, g} ]& /@ a;
    (* drop empty intervals *)
    p = Select[ p, (#[[1, 1]] <= #[[1, 2]])& ];
    (* stretch the plot style list to match the list of plots and
       add it to a *)
    ps = Take[
           Flatten[ Table[ ps, {Ceiling[ Length[ p ]/Length[ ps ] ]} ], 1 ],
           Length[ p ]
         ];
    (* attach the styles to the plot list *)
    p = Transpose[ Append[ Transpose[ p ], ps ] ];
    (* replace each piece with a Point or Plot *)
    p =
      Replace[
        #,
        {{{u_, u_}, g_, {s__}} :>
           Graphics[ Append[ {s}, Point[ {u, g[ u ]} ] ] ],
          {{u_, u_}, g_, {}} :>
           Graphics[ {Point[ {u, g[ u ]} ]} ],
         {{u_, v_}, g_, {s__}} :>
           Plot[ g[ x ], {x, u, v}, PlotStyle -> {{s}},
                 DisplayFunction -> Identity ],
         {{u_, v_}, g_, {}} :>
           Plot[ g[ x ], {x, u, v}, PlotStyle -> Automatic,
                 DisplayFunction -> Identity ]
        }
      ]& /@ p;
    (* if EndPoints specified, append a Graphics object plotting the points *)
    If[ ep != {},
        (* list the endpoints *)
        px =
          Replace[
            #,
            {{{u_, v_}, g_} -> {{u, g[ u ]}, {v, g[ v ]}},
             {{u_}, g_} -> {{u, g[ u ]}}}
          ]& /@ a;
        px = Flatten[ px, 1 ];
        (* keep only those in range *)
        px = Sort[ Select[ px, (l0 <= #[[1]] <= h0)& ] ];
        (* plot the points *)
        px = Graphics[ Join[ ep, Point /@ px ], DisplayFunction -> Identity ];
        p = Append[ p, px ]
    ];
    (* show the plots *)
    Show[ p, olist, DisplayFunction -> $DisplayFunction ]
] /; Head[ f ] === PWSf
(* Plot unfrozen PWS functions by freezing them (fail if they cannot
   be frozen) *)
Plot[ f_[ x_ ], {x_, lo_, hi_}, opts___ ] :=
  With[
    {g = PWSFreeze[ f ]},
    If[
      Head[ g ] === PWSf,
      Plot[ g[ x ], {x, lo, hi}, opts ],
      Message[ Plot::pws ]
    ]
  ] /; Head[ f ] === PWS
(* Plot multiple PWS functions by cycling PlotStyles *)
Plot[ f:{_[ x_ ]..}, {x_, lo_, hi_}, opts___ ] :=
  Module[
    {ps, flist = f, olist, p, ep},
    (* list the styles *)
    ps = PlotStyle /. {opts} /. Options[ Plot ];
    (* check for EndPoints *)
    ep = EndPoints /. {opts} /. EndPoints -> {};
    (* list all options excluding PlotStyle *)
    olist = FilterOptions[ Graphics, opts ];
    If[
      (* check for list of style lists *)
      MatchQ[ ps, {{{___}...}...} ],
      (* extend the list to match the number of f's *)
      ps =
        Take[
          Flatten[
            Table[ ps, {Ceiling[ Length[ flist ]/Length[ ps ] ]} ],
            1
          ],
          Length[ flist ]
        ];
      (* match styles lists to functions *)
      flist = Transpose[ {flist, ps} ];
      (* plot them *)
      p = Plot[ Evaluate[ #[[1]] ], {x, lo, hi}, DisplayFunction -> Identity,
                PlotStyle -> Evaluate[ #[[2]] ], EndPoints -> ep,
                olist ]& /@ flist,
      (* else use same PlotStyles for all functions *)
      p = Plot[ #, {x, lo, hi}, DisplayFunction -> Identity,
                opts ]& /@ flist
    ];
    Show[ p, olist, DisplayFunction -> $DisplayFunction ]
] /; And @@ (MemberQ[ {PWS, PWSf}, Head[ Head[ # ] ] ]& /@ f)
(* Diagnostic messages *)
Plot::pws = "Cannot plot an unfreezable PWS function.";
Protect[ Plot ];
(* Derivative *)
(* Apply the D operator case by case to domain intervals, ignoring
   singletons; make all intervals open to avoid one-sided derivatives *)
Unprotect[ D ];
D[ h_[ a_ ][ x_], x_ ] :=
  Module[
    {a0},
    a0 = Cases[ a, {{lo_, hi_}, f_} -> {{IA[ lo ], IB[ hi ]}, f} ];
    a0 = {#[[1]], Function[ x, Evaluate[ D[ #[[2]][ x ], x ] ] ] }& /@ a0;
    h[ a0 ]
  ] /; h === PWS || h === PWSf
Protect[ D ];
(* Extend the Derivative operator to PWS functions of a single argument
   in the same manner that D was extended *)
Unprotect[ Derivative ];
Derivative[ n_ ][ h_[ a_ ] ] :=
  Module[
    {a0},
    a0 = Cases[ a, {{lo_, hi_}, f_} -> {{IA[ lo ], IB[ hi ]}, f} ];
    a0 = {#[[1]], Evaluate[ Derivative[ n ][ #[[2]] ] ] }& /@ a0;
    h[ a0 ]
  ] /; h === PWS || h === PWSf
Protect[ Derivative ];
(* Integrate *)
Unprotect[ Integrate ];
(* Pass the indefinite integral operator across all domain intervals,
   dropping singletons, in PWS or PWSf functions *)
Integrate[ h_[ a_ ][ x_ ], x_ ] :=
  Module[
    {a0},
    a0 = Cases[ a, {{_,_}, _} ];
    a0 = {#[[1]], Function[ x, Evaluate[ Integrate[ #[[2]][ x ], x ] ] ]
         }& /@ a0;
    h[ a0 ]
  ] /; h === PWS || h === PWSf
(* Compute definite integrals of PWSf functions by integrating across the
   intersection of the domain intervals (ignoring singletons) with the
   range of integration and summing the results *)
Integrate[ PWSf[ a_ ][ x_ ], {x_, lo_, hi_} ] :=
  Module[
    {a0},
    (* select domain intervals that intersect the range of integration *)
    a0 = Cases[ a, {{l_, h_}, f_} /; (l<= hi && h>=lo) ];
    (* adjust the endpoints of each interval to fall within the range of
       integration, and eliminate infinitesimals *)
    a0 = {N /@ {Max[ #[[1, 1]], lo ], Min[ #[[1, 2]], hi ]}, #[[2]]}& /@ a0;
    a0 = a0 /. {IA[ u_ ] -> u, IB[ v_ ] -> v};
    (* integrate across each interval *)
    a0 = Integrate[ #[[2]][ x ], Prepend[ #[[1]], x ] ]& /@ a0;
    (* sum the integrals *)
    Plus @@ a0
  ]
(* To compute the definite integral of an unfrozen PWS function, first
   freeze it *)
Integrate[ PWS[ a_ ][ x_ ], {x_, lo_, hi_} ] :=
  With[
    {f = PWSFreeze[ PWS[ a ] ]},
    If[
      Head[ f ] === PWSf,
      Integrate[ f[ x ], {x, lo, hi} ],
      Message[ Integrate::pws ]
    ]
  ]
(* Diagnostic messages *)
Integrate::pws =
"Cannot resolve a definite integral of an unfreezable PWS function.";
Protect[ Integrate ];
(* FindMinimum *)
(* fm1 checks the endpoints of an interval, then applies FindMinimum to
   the interior; it returns a list possibly containing the left endpoint
   (if the function is rising there), the right endpoint (if the function
   is falling there), and/or an interior minimum (if one can be found) *)
fm1[ f_, {x_, x0_, xl_, xh_}, opts___ ] :=
  Module[
    {df, res, xx0, xxl, xxh, sol = {},
     msg1 = MessageQ[ FindMinimum::regex ],
     msg2 = MessageQ[ FindMinimum::fmreg ],
     msg3 = MessageQ[ FindMinimum::fmgz ]},
    {xx0, xxl, xxh} = N[ StripInfs[ {x0, xl, xh} ] ];
    (* trap bad interval limits *)
    If[ !NumberQ[ xxl ],
        Message[ FindMinimum::linf, "lower", xl ];
        Return[ {} ]
    ];
    If[ !NumberQ[ xxh ],
        Message[ FindMinimum::linf, "upper", xh ];
        Return[ {} ]
    ];
    (* x0, xl and xh are converted to numbers (without
       infinitesimals) xx0, xxl, xxh *)
    df = D[ f, x ];  (* derivative of function f *)
    (* rising at left end => left end is local min *)
    If[ (df /. x -> xl) > 0, AppendTo[ sol, {f /. x -> xxl, {x -> xl}} ] ];
    (* falling at right end => right end is local min *)
    If[ (df /. x -> xh) < 0, AppendTo[ sol, {f /. x -> xxh, {x -> xh}} ] ];
    (* look for local min in interior *)
    Off[ FindMinimum::regex, FindMinimum::fmgz, FindMinimum::fmreg ];
    res = FindMinimum[ f, {x, xx0, xxl, xxh}, opts ];
    If[ Head[ res ] === List, AppendTo[ sol, res ] ];
    If[ msg1, On[ FindMinimum::regex ] ];
    If[ msg2, On[ FindMinimum::fmreg ] ];
    If[ msg3, On[ FindMinimum::fmgz ] ];
    sol
  ]
(* Extend FindMinimum to piecewise smooth functions with a slight variation
   in syntax - specify the argument but not starting values *)
Unprotect[ FindMinimum ];
FindMinimum[ pws_[ x_ ], {x_Symbol}, opts___ ] :=
  Module[
    {a = pws[[1]], res = {}, fmin, p},
    (* identify the precision goal (used below to
       resolve apparent ties for minimum) *)
    p = PrecisionGoal /. Options[ FindMinimum ] /. { opts };
    If[
      p === Automatic,
      p = (WorkingPrecision /. Options[ FindMinimum ] /. { opts }) - 10
    ];
    (* at singletons, evaluate the function; on intervals,
       apply function fm1, starting at the midpoint *)
    res = (# /.
             { {{u_}, f_} :> {f[u], {x -> u}},
               {{u_, v_}, f_} :> Sequence @@
                 (fm1[ f[x], {x, (N[ u ] + N[ v ])/2., u, v}, opts ]) }
          )& /@ a;
    (* dispose of any cases that failed *)
    res = Cases[ res, {_, {__}} ];
    (* find the global minimum, and pad it based on the precision allowance *)
    fmin = Min[ Sequence @@ Transpose[ res ][[1]] ] * (1 + 0.5 10^(-p) );
    (* return a list of local minima that were within
       tolerance of the global minimum *)
    Select[ res, (#[[1]] <= fmin)& ]
  ] /; (Head[ pws ] === PWS || Head[ pws ] === PWSf)
Protect[ FindMinimum ];
(* Diagnostic messages *)
FindMinimum::linf =
"FindMinimum could not process the non-numeric `1` limit `2` specified for the search interval.";
(* Join - result is PWS unless both are frozen *)
PWS /: Join[ PWS[ a_ ], (PWS | PWSf)[ b_ ] ] := PWS[ Join[ a, b ] ];
PWSf /: Join[ PWSf[ a_ ], PWS[ b_ ] ] := PWS[ Join[ a, b ] ];
PWSf /: Join[ PWSf[ a_ ], PWSf[ b_ ] ] := PWSFreeze[ PWS[ Join[ a, b ] ] ];
(* Distribute *)
(* List of functions that automatically distribute over PWS, PWSf *)
DistributeOverPWS =
  {Abs, ArcCos, ArcCosh, ArcCot, ArcCoth, ArcCsc, ArcCsch,
   ArcSec, ArcSech, ArcSin, ArcSinh, ArcTan, ArcTanh,
   Ceiling, Cos, Cosh, Cot, Coth, Csc, Csch, Erf, Erfc,
   Exp, Factorial, Floor, Gamma, Log, LogGamma,
   LogIntegral, N, Negative, NonNegative, Positive, Round,
   Sec, Sech, Sign, Sin, Sinh, Sqrt, Tan, Tanh};
Protect[ DistributeOverPWS ];
(* Automatic distribution *)
f_[ PWS[ a_ ] ] ^:=
  PWS[ Function[ {x}, {x[[1]], Evaluate[ f[ x[[2]][ # ] ] ]&} ] /@ a
  ] /; MemberQ[ DistributeOverPWS, f ]
f_[ PWSf[ a_ ] ] ^:=
  PWSf[ Function[ {x}, {x[[1]], Evaluate[ f[ x[[2]][ # ] ] ]&} ] /@ a
  ] /; MemberQ[ DistributeOverPWS, f ]
(* Manual distribution *)
PWS /: Distribute[ h_, PWS ] :=
  Module[
    {a = h[[1]][[1]], b = Head[ h[[1]] ], f = Head[ h ]},
    a = Cases[ a, {x_List, y_} :> {x, Evaluate[ f[ y[ # ] ] ]&} ];
    b[ a ]
  ] /; Head[ h[[1]] ] === PWS || Head[ h[[1]] ] === PWSf
PWSf /: Distribute[ h_, PWSf ] :=
  Module[
    {a = h[[1]][[1]], b = Head[ h[[1]] ], f = Head[ h ]},
    a = Cases[ a, {x_List, y_} :> {x, Evaluate[ f[ y[ # ] ] ]&} ];
    b[ a ]
  ] /; Head[ h[[1]] ] === PWS || Head[ h[[1]] ] === PWSf
(* Epilog *)
Protect[ PWS, PWSf ];
If[ sp, On[ General::spell ] ];
If[ sp1, On[ General::spell1 ] ];
End[];
EndPackage[];
