(* :Name: Pricing *)
(* :Title: Pricing and Cost Allocation *)
(* :Author:	William W. Sharkey *) 
(* :Summary:
	This package demonstrates standard cost allocation methods
	for both discrete and continuous cost allocation problems,
	and optimal linear and nonlinear pricing rules for 
	continuous problems. *)  
(* :Context: *)
(* :Package Version: 1.0 *)
(* :Copyright: Copyright 1995, William W. Sharkey *)
(* :History: Version 1.0 1995 *)
(* :Keywords:
	Core,Nucleolus,Shapley value,Aumann-Shapley price,
	Subsidy free price,Ramsey price,Profit maximizing
	price,Nonlinear pricing,Two-part tariffs *)
(* :Source: 
	Pricing and Cost Allocation Using Mathematica, in
	Economic and Financial Modeling with Mathematica, vol. 2,
	H. Varian, ed. *)
(* :Mathematica Version: 2.2 *)
(* :Limitation:
	Standard game theoretic solution concepts such as the
	core are defined with constraints appropriate for cost 
	sharing rather than surplus sharing. *)

BeginPackage["Pricing`"];

Game::usage = "Game[N,v] is a game object where N is the set \
of players and v is the coalitional worth function.";

CostFunction::usage = "CostFunction[x,F] defines a cost function \
where x is an arbitrary output vector and F[x] is the \
cost of producing x.";

DemandSystem::usage = "DemandSystem is the header for a demand \
system. DemandSystem[p,V,Q] defines a demand system for analysis \
of linear pricing with multiple commodities and a single customer \
type.  V[p] and Q[p] represent the indirect utility function and the \
vector of demand functions. DemandSystem[p,V,Q,q,U] defines a demand \
system for nonlinear pricing with a single commodity and multiple \
customer types.  V[p] represents a vector of indirect utility \
functions, Q[p] a vector of demand functions, and U[q] a vector of \
utility functions.";

(* Cost Allocation Methods *)
Core::usage = "Core[game] computes the set of extreme points to the \
core of game.";

CoreSelection::usage = "CoreSelection[w,game] solves the linear \
program Max (w.x) subject to the core constraints.";

CSValue::usage = "CSValue[permutation,game] computes the coalition \
structure value associated with permutation.";

Nucleolus::usage = "Nucleolus[game] computes the nucleolus of game.";

PerCapitaNucleolus::usage = "PerCapitaNucleolus[game] computes the \
per capita nucleolus of game.";

SCRB::usage = "SCRB[game] computes the separable cost, remaining \
benefit cost allocation for game.";

ShapleyValue::usage = "ShapleyValue[game] computes the Shapley value \
of game. ShapleyValue[weight,game] computes the weighted Shapley \
value.";

(* Cost Allocation Pricing *)
AumannShapleyPrice::usage = "AumannShapleyPrice[weight,output,c] \
computes the vector of weighted Aumann Shapley prices corresponding \
to output in the cost allocation problem c. If no weight is specified, \
unweighted AS prices are computed. This functin should be applied only \
to cost functions that are continuous throughout their domain.";

NucleolusPrice::usage = "NucleolusPrice[output,c] computes nucleolus \
price vector at output for a cost allocation problem c."; 

ShapleyValuePrice::usage = "ShapleyValuePrice[weight,output,c] \
computes the vector of weighted Sbapley value prices corresponding \
to output and the cost allocation problem c. If no weight is \
specified, unweighted AS prices are computed.";

SubsidyFreePrices::usage = "SubsidyFreePrices[output,c] computes \
all extreme points of the set of subsidy free prices at output in \
the cost allocation problem c.";

SubsidyFreePriceSelection::usage "SubsidyFreePriceSelection[w,output,c] \
computes a subsidy free price associated with 'w' and output in the \
cost allocation problem c.";

(* Optimal Pricing *)
KTMax::usage = "KTMax[funct,var,inequalities,equalities] takes as \
input an objective function, a list of variables, a list of \
inequaltiy constraints and a list of equality constraints. For \
every subset of inequality constraints it generates a Lagrangean \
for the system, sets its partial derivatives with respect to the \
variables equal to zero and then solves the system. It stores this \
solution if it is feasible with respect to the remaining constraints. \
Thus it solves a constrained optimization problem assuming a \
particular set of constraints is binding, and checks if that is true. \
It then works through all possible sets of binding constraints to \
identify the feasible solution that maximizes the value of the \
objective function.";

ProfitMaxPrice::usage = "ProfitMaxPrice[demands,c] computes the \
profit maximizing price associated with  demand system d \
and cost function c.";

ProfitMax2Part::usage = "ProfitMax2Part[d,c] \
computes the set of self selecting two part tariffs that maximize \
profit under demand system d and a cost function c.";

ProfitMaxNonlinear::usage = "ProfitMaxNonlinear[d,c] \
computes the set of self selecting nonlinear tariffs that maximize \
profit under demand system d and a cost function c.";

RamseyPrice::usage = "RamseyPrice[demands,c] computes the Ramsey \
price vector associated with a pre-defined demand system and cost \
function."; 

SurplusMax2Part::usage = "SurplusMax2Part[d,c] \
computes the set of self selecting two part tariffs that maximize \
consumers' surplus under demand system d and a cost function c.";

SurplusMaxNonlinear::usage = "SurplusMaxNonlinear[d,c] \
computes the set of self selecting nonlinear tariffs that maximize \
consumers' surplus under demand system d and a cost function c.";

(* Examples *)
AssignmentGame::usage = "AssignmentGame[M] generates an assignment \
game based on M.";

IntegerGame::usage = "IntegerGame[n] generates a random integer \
valued n-player game.";

RealValuedGame::usage = "RealValuedGame[n] generates a random real \
valued n-player game.";

SpanningTreeGame::usage = "SpanningTreeGame[M] generates a minimum \
cost spanning tree game.";

StableAssignments::usage = "StableAssignments[M] computes the set \
of optimal solutions to the dual of an optimal assignment problem defined by M.";

CESCostFunction::usage = "CESCostFunction[] generates a CES \
cost function.";

QuadraticCostFunction::usage = "QuadraticCostFunction[] \
generates a quadratic function.";

CESDemandSystem::usage = "CESDemandSystem[a,b,c,y] generates a CES demand \
system for the analysis of uniform pricing systems.";

CobbDouglasDemandSystem::usage = "CobbDouglasDemandSystem[a,y] generates a \

Cobb-Douglas demand system for the analysis of uniform pricing systems.";

LinearDemandSystem::usage = "LinearDemandSystem[a,b,B] generates a linear \
demand system for the analysis of uniform pricing systems.";

NonlinearDemandSystem::usage = "NonlinearDemandSystem[a,b] generates a demand system for the \
analysis of nonlinear pricing systems.";

(* Utility functions *)
BalancedCover::usage = "BalancedCover[game] generates the balanced \
cover of game.";

CoreQ::usage = "CoreQ[game] reports True if the core of game is \
nonempty.";

Cost::usage = "Cost[x,c] is the cost of producing output vector x \
in cost allocation problem c.";

Demands::usage = "Demands[p,d] is the vector of demands at price \
vector p in demand system d.";

Dual::usage = "Dual[game] generates the dual game defined by \
v#[S] = v[N] - v[N\S].";

InCoreQ::usage = "InCoreQ[x,game] reports True if x is contained \
in the core of game.";

IndirectUtility::usage = "IndirectUtility[p,d] is the indirect \
utility of price vector p in demand system d.";

MonotonicCover::usage = "MonotonicCover[game] generates the \
monotonic cover of game.";

MultilinearExtension::usage = "MultilinearExtension[output,game] \
defines Owen's multilinear extension for game at output.";

Profit::usage = "Profit[p,d,c] is the profit at price p under \
demand system d and cost allocation problem c.";

StandAloneGame::usage = "StandAloneGame[output,costfunction] defines \
a discrete game defined by output and costfunction.";

SubadditiveCover::usage = "SubadditiveCover[game] generates the \
subadditive cover of game.";

Subsets::usage = "Subsets[S] lists the subsets of set S.";

Utility::usage = "Utility[q,d] is the utility of output vector q in \
demand system d.";

Values::usage = "Values[game] generates a list of ordered pairs of \
the form {S,v[S]} for every coalition S, where v[S] is the worth of \
coalition S.";

Begin["`Private`"];

(* Cost Allocation Methods *)

Core[g_Game] := Module[{dual,constraints,points},
	dual = Dual[g];
	constraints = Drop[CoreConstraints[ZeroNormalization[dual]],
		Length[First[g]]]/.lhs_<=rhs_->lhs>=rhs;
	points = Union[#+Take[Rest[Last[dual]],Length[First[g]]] & /@
		ArgMin[x[First[g]],constraints,x /@ First[g]]];
	If[And @@ NumberQ /@ Flatten[points],points,N[points]]
	];

CoreSelection[weight_?VectorQ,g_Game] := Module[{constraints,duallp},
	If[Length[weight]!=Length[First[g]],Return[]];
	dual = Dual[g];
	constraints = Drop[CoreConstraints[ZeroNormalization[dual]],
		Length[First[g]]]/.lhs_<=rhs_->lhs>=rhs;
	lp = ConstrainedMin[(x /@ First[g]).weight,
		constraints,x /@ First[g]];
	If[NumberQ[First[lp]],Take[Rest[Last[dual]],
		Length[First[g]]]+(x /@ First[g]) /. Last[lp]]
	];

CSValue[partition:{{__}..},g_Game] := Module[
	{N,perm},
	N = Sort[First[g]];
	perm=Partition[Flatten[Fold[CSpermutations,
		Permutations[partition],Range[Length[partition]]]],Length[N]];
	Table[Plus @@ (v[Union[Take[#,Position[#,N[[i]]][[1,1]]]],g]-
		v[Union[Take[#,Position[#,N[[i]]][[1,1]]-1]],g] &/@ perm),
		{i,1,Length[N]}]/Length[perm]
	];

(* Based on an algorithm due to M. Carter *)
Nucleolus[g_Game] := Module[{constraints},
	constraints = CoreConstraints[Dual[g]]/.lhs_<=rhs_->lhs>=rhs;
	Nest[NucleolusAux,{constraints,1,First[g]},
		Length[First[g]]];
	(x /@ First[g]) /. AuxReport
	];

PerCapitaNucleolus[g_Game] := Module[{constraints},
	constraints = CoreConstraints[Dual[g]]/.lhs_<=rhs_->lhs>=rhs;
	Nest[NucleolusAux,{constraints,2,First[g]},
		Length[First[g]]];
	(x /@ First[g]) /. AuxReport
	];

SCRB[g_Game] := Module[{N,separablecost,nonseparablecost,
	alternativecost,remainingbenefit},
	N = Sort[First[g]];
	separablecost = (v[N,g] - v[Complement[N,{#}],g]) &/@ N;
	nonseparablecost = v[N,g] - Plus @@ separablecost;
	alternativecost = v[{#},g] &/@ N;
	remainingbenefit = alternativecost - separablecost;
	separablecost + nonseparablecost*remainingbenefit/
		Plus @@ remainingbenefit
	];
	
(* Due to M. Carter *)	
ShapleyValue[weight_:Null,g_Game] := Module[{N,w,value},
	N = Sort[First[g]];
	If[Length[weight]==Length[N] && Min[weight]>0,w=weight,
		w=Table[1,{Length[N]}]];
	value = w.IndicatorVector[{#},N]*
		(Potential[w,N,g]-Potential[w,DeleteCases[N,#],g]) & /@ N;
	Potential[w,#,g]=. &/@ Rest[Subsets[N]];
	Return[value]
	];

(* Cost Allocation Pricing *)
AumannShapleyPrice[weight_:Null,output_?VectorQ,c_CostFunction] := Module[{w,quant,t},
	If[Length[weight]==Length[First[c]] && Min[weight]>0,
		w=weight,w=Table[1,{Length[First[c]]}]];
	quant=Array[q,Length[First[c]]];
	MapThread[NIntegrate,
		{(w*t^(w-1)*Grad[F[quant,c],quant]/.q[i_]:>
		t^w[[i]]*output[[i]]),
		Table[{t,0,1},{Length[First[c]]}]}]
	];

NucleolusPrice[output_?VectorQ,c_CostFunction] := 
	Nucleolus[StandAloneGame[output,c]]/
		(output/.x_/;x==0->1)

ShapleyValuePrice[weight_:Null,output_?VectorQ,c_CostFunction] := 
	ShapleyValue[weight,StandAloneGame[output,c]]/
		(output/.x_/;x==0->1)

SubsidyFreePrices[output_?VectorQ,c_CostFunction] := #/
	(output/.x_/;x==0->1) &/@ Core[StandAloneGame[output,c]]

SubsidyFreePriceSelection[weight_:Null,output_?VectorQ,c_CostFunction] := Module[{cs},
	cs = CoreSelection[weight,StandAloneGame[output,c]];
	If[Length[cs]>0,cs/(output/.x_/;x==0->1)]
	];

(* Optimal Pricing *)

ProfitMaxPrice[d_DemandSystem,c_CostFunction] := Module[{p},
	If[Length[d]!=3,Return["Demand system undefined"]];
	p = d[[1]];
	p/.Last[Last[KTMax[Profit[p,d,c],p,{},{}]]]
	];

RamseyPrice[d_DemandSystem,c_CostFunction] := Module[{p},
	If[Length[d]!=3,Return["Demand system undefined"]];
	p = d[[1]];
	p/.Last[Last[KTMax[V[p,d],p,{},{Profit[p,d,c]}]]]
	];

ProfitMax2Part[d_DemandSystem,c_CostFunction] := 
	Module[{price,entry,p,e,objective,vars,ineq,eq,soln},
	If[Length[d]!=5,Return["Demand system undefined"]];
	price = Array[p,Length[d[[1]]]];
	entry = Array[e,Length[d[[1]]]];
	objective = Profit1[price,entry,d,c];
	vars = Join[price,entry];
	ineq = {};
	eq = Join[IR1[price,entry,d],DIC1[price,entry,d]];
	soln = Last[Last[KTMax[objective,vars,ineq,eq]]];
	Transpose[Join[{price/.soln},{entry/.soln}]]
	];
	
ProfitMaxNonlinear[d_DemandSystem,c_CostFunction] := 
	Module[{quant,outlay,q,o,objective,vars,ineq,eq,soln},
	If[Length[d]!=5,Return["Demand system undefined"]];
	quant = Array[q,Length[d[[4]]]];
	outlay = Array[o,Length[d[[4]]]];
	objective = Profit2[quant,outlay,d,c];
	vars = Join[quant,outlay];
	ineq = {};
	eq = Join[IR2[quant,outlay,d],DIC2[quant,outlay,d]];
	soln = Last[Last[KTMax[objective,vars,ineq,eq]]];
	Transpose[Join[{quant/.soln},{outlay/.soln}]]
	];
	
SurplusMax2Part[d_DemandSystem,c_CostFunction] := 
	Module[{price,entry,p,e,objective,vars,ineq,eq,soln},
	If[Length[d]!=5,Return["Demand system undefined"]];
	price = Array[p,Length[d[[1]]]];
	entry = Array[e,Length[d[[1]]]];
	objective = Plus @@ CS1[price,entry,d];
	vars = Join[price,entry];
	ineq = IR1[price,entry,d];
	eq = Join[DIC1[price,entry,d],{Profit1[price,entry,d,c]}];
	soln = Last[Last[KTMax[objective,vars,ineq,eq]]];
	Transpose[Join[{price/.soln},{entry/.soln}]]
	];
	
SurplusMaxNonlinear[d_DemandSystem,c_CostFunction] := 
	Module[{quant,outlay,q,o,objective,vars,ineq,eq,soln},
	If[Length[d]!=5,Return["Demand system undefined"]];
	quant = Array[q,Length[d[[4]]]];
	outlay = Array[o,Length[d[[4]]]];
	objective = Plus @@ CS2[quant,outlay,d];
	vars = Join[quant,outlay];
	ineq = IR2[quant,outlay,d];
	eq = Join[DIC2[quant,outlay,d],{Profit2[quant,outlay,d,c]}];
	soln = Last[Last[KTMax[objective,vars,ineq,eq]]];
	Transpose[Join[{quant/.soln},{outlay/.soln}]]
	];
	
(* Optimization: DMax and KTMax are due to T. Kaplan and A. Mukherji *)

DMax[funct_,vars_]:=Block[{t},
  eqns=Table[D[funct,vars[[t]] ]==0,
  {t,1,Length[vars]}];
  NSolve[eqns,vars] ];

KTMax[funct_,vars_,inequality_,equality_]:=
Module[{eq,ineq,t,xxx,yyy,t2,l1,l2,i,f,j,anslist,thelist,
	vlist,lagran,var1,var2,try,plist},
	eq = equality/.(lhs_==rhs_->lhs-rhs);
	ineq = inequality/.{lhs_>=rhs_->lhs-rhs,
		lhs_<=rhs_->rhs-lhs};
	l1=Length[ineq];
	l2=Length[eq];
	var2=Table[lam[t],{t,l1+1,l1+l2}];
	thelist=Table[Mod[Floor[j/2^i],2],
		{j,1,2^l1-1},{i,0,l1-1}];
	lagran=funct+var2.eq;   
	anslist=DMax[lagran,Join[vars,var2]];
    For[t=1,t<=Length[thelist],t++,
    	try=thelist[[t]];
    	var1=Table[lam[t2]*try[[t2]],
    		{t2,1,Length[ineq]}];
    	vlist=Select[var1,Function[x,Not[NumberQ[x]]]];
    	lagran=funct+var1.ineq+var2.eq;
    	anslist=Join[anslist,
    	DMax[lagran,Join[vars,vlist,var2]]];
    	];
    f=Function[xxx,Apply[And,Map[Function[yyy,
    	NonNegative[Chop[yyy]] && 
    	NumberQ[yyy]],ineq/.xxx]]];
    anslist=Select[anslist,f];
    Maxlist=Re[Map[Function[xxx,funct/.xxx],anslist]];
    listmax=Max[Maxlist];
    plist=Position[Maxlist,listmax];
    Clear[t];
    anslist=Table[anslist[[plist[[t]]]],{t,1,Length[plist]}];
    var1=Table[lam[t],{t,1,Length[ineq]}];
    vlist=Join[vars,var1,var2];
    f=Function[xxx,Table[vlist[[t2]]->
    	(vlist[[t2]]/.xxx/.Table[lam[t]->0,
    	{t,1,Length[ineq]}])[[1]],{t2,1,Length[vlist]}]];
    anslist=Map[f,anslist];
    Return[{listmax,anslist}/.lam->"lam"];
 	];

(* Examples *)
AssignmentGame[mat_?MatrixQ] := Module[
	{m,f,z,M,F,N,variables,constraints,v,val},
	{m,f} = Dimensions[mat];
	M = Range[m];F = Range[f]+m; 
	N = Union[M,F];
	variables=Array[z,{Length[M],Length[F]}];
	constraints[P:{__},Q:{__}] := #<=1 & /@
		Join[variables[[P,Q]].Table[1,{Length[Q]}],
		Transpose[variables[[P,Q]]].Table[1,{Length[P]}]];
	v[P:{__},Q:{__}] := First[ ConstrainedMax[
		Flatten[mat[[P,Q]]].Flatten[variables[[P,Q]]],
		constraints[P,Q],Flatten[variables[[P,Q]]] ] ];
	v[P_,{}] = 0; v[{},Q_] = 0;
	val = (v[Intersection[#,M],Intersection[#,F]-Length[M]]) &
		/@ Subsets[N];
	Game[N,val]
	];

IntegerGame[n_] := Game[Range[n],
	Prepend[Table[Random[Integer,{1,10}],{2^n-1}],0]]

MCST[M_] := Module[{N,mcst,testrow,cost,i},
	N = Range[Length[M]];
	mcst = {Last[N]};
	testrow = Last[M];
	cost = 0;
	While[Length[mcst]<Length[N],
		i = N[[First[First[Position[testrow,Min[
			testrow[[Complement[N,mcst]]]]]]]]];
		cost = cost + testrow[[i]];
		mcst = Append[mcst,i];
		testrow = Table[Min[testrow[[j]],M[[i,j]]],
			{j,1,Length[N]}];
		];
	Return[cost];
	];

RealValuedGame[n_] := Game[Range[n],
	Prepend[Table[Random[],{2^n-1}],0]]
	
SpanningTreeGame[mat_?MatrixQ] := Module[{N,Root,val},
	If[Transpose[mat]!=mat,
		Print["Cost matrix is not symmetric"];Return[]];
	N = Range[Length[mat]-1];
	Root = {Length[mat]};
	val = (MCST[mat[[Join[#,Root],Join[#,Root]]]]) & 
		/@ Subsets[N];
	Game[N,val]
	];

StableAssignments[mat_?MatrixQ] := Module[
	{m,f,M,F,c,A,b,z},
	{m,f} = Dimensions[mat];
	M = Range[m];F = Range[f];
	c = Flatten[mat];
	A1 = Table[RotateRight[Join[Table[1,{Length[F]}],
		Table[0,{(Length[M]-1)*Length[F]}]],i*Length[F]],
		{i,0,Length[M]-1}];
	A2 =Table[Flatten[Table[RotateRight[Join[{1},
		Table[0,{Length[F]-1}]],i],
		{Length[M]}],1],{i,0,Length[F]-1}];
	b = Table[1,{Length[M]+Length[F]}];
	z = Array[x,Length[b]];
	ArgMin[b.z,Table[(Transpose[Join[A1,A2]].z)[[i]]>=
		c[[i]],{i,1,Length[c]}],z]
	];

CESCostFunction[a_?VectorQ,b_?VectorQ,c_?NumberQ] := 
	Module[{q,x},q = Array[x,Length[a]];
	CostFunction[q,(Plus @@ (a*q^b))^(1/c)]/.x->"x"
	];

QuadraticCostFunction[f_?NumberQ,m_?VectorQ,M_?MatrixQ] := 
	Module[{q,x},q = Array[x,Length[m]];
	CostFunction[q,f + m.q + q.M.q]/.x->"x"
	];

CESDemandSystem[a_?VectorQ,b_?VectorQ,c_?AtomQ,m_?AtomQ] := 
	Module[{price,p,V,Q},
	price = Array[p,Length[b]];
	V[x_] := m*(a.(x^b))^(-1/c);
	Q[price] = Map[-D[V[price],#]&,price]/(V[price]/m);
	DemandSystem[price,V[price],Q[price]]/.p->"p"
	];

CobbDouglasDemandSystem[a_?VectorQ,m_?AtomQ] := 
	Module[{price,p,V,Q},
	price = Array[p,Length[a]];
	V[x_] := m/Times@@(x^(a/Plus@@a));
	Q[price] = Map[-D[V[price],#]&,price]/(V[price]/m);
	DemandSystem[price,V[price],Q[price]]/.p->"p"
	];

LinearDemandSystem[a_?NumberQ,b_?VectorQ,B_?MatrixQ] := Module[
	{price,p,V,Q},
	price = Array[p,Length[b]];
	V[x_] := a - b.x + x.B.x;
	Q[price] = Map[-D[V[price],#]&,price];
	DemandSystem[price,V[price],Q[price]]/.p->"p"
	];

NonlinearDemandSystem[a_?VectorQ,b_?VectorQ] := Module[
	{price,p,quant,q,U,Q,V,P},
	price = Array[p,Length[a]];
	quant = Array[q,Length[a]];
	U[x_] := a*x-(b*x^2)/2;
	P[quant] = Apply[D,#] & /@ Transpose[{U[quant],quant}];
	Q[price] = quant /. First[Solve[price==P[quant],quant]];
	V[price] = U[Q[price]];
	DemandSystem[price,V[price],Q[price],quant,U[quant]]/.
		{p->"p",q->"q"}
	];

(* Utility functions *)

ArgMin[objective_,constraints_,variables_] := Module[
	{eqn,ineq,c,b1,b2,A1,A2},
	eqn = Cases[constraints,lhs_==rhs_];
	ineq = Complement[constraints,eqn] /. lhs_<=rhs_->-lhs>=-rhs;
	c = -Table[D[objective,variables[[i]]],{i,1,Length[variables]}];
	b1 = eqn /. lhs_==rhs_ -> rhs;
	b2 = ineq /. lhs_ >= rhs_ -> -rhs;
	A1 = eqn /. lhs_==rhs_ :> Table[D[lhs,variables[[i]]],
		{i,1,Length[variables]}];
	A2 = ineq /. lhs_>=rhs_ :> -Table[D[lhs,variables[[i]]],
		{i,1,Length[variables]}];
	SimplexStart[c,A1,A2,b1,b2]
	];

BalancedCover[g_Game] := Game[First[g],ReplacePart[Last[g],
	First[ConstrainedMin[x[First[g]],
		CoreConstraints[g]/.{lhs_<=rhs_->lhs>=rhs,
		lhs_==rhs_->lhs>=rhs},
		(x /@ First[g])]],Length[Last[g]]]]

Coalitions[Game[N_,_]] := Subsets[N]

CoreConstraints[g_Game] := Module[{values},
	If[And @@ NumberQ /@ Last[g],values=Last[g],values=N[Last[g]]];
	Append[Table[x[Coalitions[g][[i]]]<=
		values[[i]],{i,2,Length[values]-1}],
		x[First[g]]==Last[values]]
	];

CoreQ[g_Game] := Last[Last[g]] <= First[ConstrainedMax[
	x[First[g]],Drop[CoreConstraints[g],-1],(x /@ First[g])]];

Cost[x_,c_CostFunction] := F[x,c];
	F[x_,_] := 0 /; Plus @@ x==0;
	F[x_,c_CostFunction] := c[[2]] /. Thread[Rule[First[c],x]]
	
CS1[p_,e_,d_DemandSystem] := IndirectUtility[p,d]-p*Q[p,d]-e;

CS2[q_,r_,d_DemandSystem] := Utility[q,d] - r

CSpermutations[P:{{__}..},position_] := Flatten[Table[
	ReplacePart[P[[i]],
	Permutations[P[[i,position]]][[j]],position],
	{i,1,Length[P]},
	{j,1,Length[Permutations[P[[i,position]]]]}],1];

Demands[p_,d_] := Q[p,d];
	Q[p_,d_DemandSystem] := d[[3]] /. Thread[Rule[d[[1]],p]]

DIC1[p_,e_,d_DemandSystem] := Take[CS1[p,e,d] - 
	CS1[RotateRight[p],RotateRight[e],d],-(Length[p]-1)];

DIC2[q_,r_,d_DemandSystem] := Take[CS2[q,r,d] -
	CS2[RotateRight[q],RotateRight[r],d],-(Length[q]-1)]

Dual[g_Game] := Game[First[g],
	(g[[2,-1]]-g[[2,Length[Last[g]]+1-#]]) & /@ 
	Range[Length[Last[g]]]]

(* Due to Hal Varian *)
Grad[f_,x_] := Map[D[f,#]&,x];

InCoreQ[x_,g_Game] := (Chop[Min[Last[g]-(Plus@@x[[#]]&/@
	Subsets[Range[Length[x]]])]]==0)&&(Last[Last[g]]==
	Plus@@x);

IndicatorMatrix[M:{{___}..},N:{__}] := Table[
	IndicatorVector[M[[i]],N],{i,1,Length[M]}];

IndicatorVector[S_List,N_List] := N /. 
	{i_?AtomQ:>0/;FreeQ[S,i],i_?AtomQ:>1/;MemberQ[S,i]};

IndirectUtility[p_,d_] := V[p,d];
	V[p_,d_DemandSystem] := d[[2]] /. Thread[Rule[d[[1]],p]]

IR1[p_,e_,d_DemandSystem] := Take[CS1[p,e,d],1];

IR2[q_,r_,d_DemandSystem] := Take[CS2[q,r,d],1]

(* Due to S. Skienna *)
Ksubsets[l_List,0] := { {} };
Ksubsets[l_List,1] := Partition[l,1];
Ksubsets[l_List,k_Integer?Positive] := {l} /; (k == Length[l]);
Ksubsets[l_List,k_Integer?Positive] := {}  /; (k > Length[l]);
Ksubsets[l_List,k_Integer?Positive] :=
	Join[
		Map[(Prepend[#,First[l]])&, Ksubsets[Rest[l],k-1]],
		Ksubsets[Rest[l],k]
	];

MonotonicCover[g_Game] := Module[{w},
	w[S_] := Max[v[#,g] & /@ Subsets[S]];
	Game[First[g],w[#] &/@ Subsets[First[g]]]
	];

MultilinearExtension[alpha_:Null,g_Game] := Module[{a,N,x,mle},
	If[Length[alpha]==0,a=Table[1,{Length[First[g]]}],
		If[Length[alpha]==Length[First[g]],a=alpha,
		Print["Not a valid output vector"];Return[]]];
	N = First[g];
	x = Map[q,N];
	mle = Plus @@ (v[#,g] *
		Times @@ ( (x/a)[[Select[IndicatorVector[#,N]*
		Range[Length[N]],Positive] ]] ) * Times @@ 
		((1-(x/a))[[Select[IndicatorVector[Complement[N,#],N]*
		Range[Length[N]],Positive] ]] ) & /@ Subsets[N]);
	CostFunction[x,mle]/.q->"q"
	];

NucleolusAux[{constraints_,w_,N_}] :=
    Module[{lp,newConstraints,binding,nonbinding,
    	coalitions,setequal,p,q},
	If[Count[constraints,_GreaterEqual]==0,Return[constraints]];
	lp = ConstrainedMin[p-q,
		Relax[constraints,p-q,w],
		Join[(x /@ N),{p,q}]
		];
	AuxReport = lp[[2]];
	newConstraints = Relax[constraints, lp[[1]], w];
	binding = Select[newConstraints, 
		# /. GreaterEqual :> Equal /. lp[[2]] & ];
	nonbinding = Complement[newConstraints,binding];
	binding = NullVariables[binding];
	coalitions=Cases[binding,lhs_==rhs_] /. lhs_==rhs_ :> 
		(Variables[{lhs}] /.x[i_] -> i);
	setequal=Union[Flatten[Table[
		Intersection[Complement[coalitions[[j]],coalitions[[i]]],
		NAux1[Complement[coalitions[[i]],
			coalitions[[j]] ], N]  ],
		{i,1,Length[coalitions]-1},{j,i,Length[coalitions]}],1]];
	setequal=Union[setequal,Complement[N,#]&/@setequal];
	binding = binding /. lhs_>=rhs_ :> lhs==rhs /;
		MemberQ[setequal,(Variables[lhs]/.x[i_]->i)]; 
	Join[binding,nonbinding]
	];

NAux1[S:{},N_] := N;
NAux1[S:{__},N_] := {};

NonNegativeVector[x_List] := And @@ NonNegative[x];

NonPositiveVector[x_List] := NonNegativeVector[-x];

NullVariables[ineq_] := Module[{},
	n = Count[ineq,_Equal] ; (* no of equations *)
	ns = NullSpace @ Transpose @ 
		ToMatrix[ineq /. GreaterEqual -> Equal];
	binary = Cases[{Take[#,n],Drop[#,n]} & /@ ns,
	   ({x_,y_ } /; NonPositiveVector[x] && NonNegativeVector[y] )];
	If[Length[binary] > 0,
	    (null = Flatten[First[binary]];		
	    Sort[
		#[[1]] /. (GreaterEqual :> Equal /; #[[2]] >0) &
			 /@ Thread[{#1,#2}& [ineq,null]]]),
	    ineq
		]
	];

Payoff[y_List,g_Game] := Thread[Rule[(x /@ First[g]),y]];

Profit[p_,d_DemandSystem,c_CostFunction] := p.Q[p,d] - F[Q[p,d],c];

Profit1[p_,e_,d_DemandSystem,c_CostFunction] := 
	p.Demands[p,d] + Plus @@ e - Cost[Demands[p,d],c];

Profit2[q_,r_,d_DemandSystem,c_CostFunction] := 
	Plus @@ r - Cost[q,c]

Potential[w_,{i_},g_Game] := Potential[w,{i},g] = 
	v[{i},g]/w.IndicatorVector[{i},First[g]];
Potential[w_,S_,g_Game]   := Potential[w,S,g] = 
	(v[S,g] + Plus @@ 
	Map[w.IndicatorVector[{#},First[g]]*
	Potential[w,Complement[S,{#}],g] &,S])/
	w.IndicatorVector[S,First[g]];		

Relax[ineq_, epsilon_, 1] :=  ineq /. 
	lhs_>=rhs_ :> lhs>=rhs-epsilon;
Relax[ineq_, epsilon_, 2] :=  ineq /. 
	lhs_>=rhs_ :> lhs>=rhs-(epsilon*Length[Variables[lhs]]);

SimplexLoop[d_,A_,b_,basis_,Binv_] := Module[
	{testrow,tableau,pc,pctest,pr,newBinv,newbases},
	testrow = d[[basis]].Binv.A - d;
	tableau = Binv.A;
	pc = Complement[Flatten[Position[testrow,0]],basis];
	pctest=Table[Cases[Transpose[Join[{Range[Length[b]]},
		{Binv.b},{Flatten[Transpose[tableau][[{pc[[i]]}]]]}]],
		{x1_,x2_,x3_}/; x3>0] /.
		{x1_?NumberQ,x2_?NumberQ,x3_?NumberQ}->{x2/x3,x1},
		{i,1,Length[pc]}];
	pr = Table[Last[Transpose[Cases[pctest[[i]],
		{x1_,x2_}/;x1==Min[First[Transpose[pctest[[i]]]]]]]],
		{i,1,Length[pctest]}];
	newbases = Flatten[Table[ReplacePart[basis,pc[[j]],{pr[[j,k]]}],
		{j,1,Length[pc]},{k,1,Length[pr[[j]]]}],1];
	newBinv = Flatten[Table[Transpose[ReplacePart[
		IdentityMatrix[Length[b]],
		Flatten[Table[-tableau[[i,pc[[j]]]]/tableau[[pr[[j,k]],
		pc[[j]]]],{i,1,Length[b]}]]*Join[Table[1,
		{i,1,pr[[j,k]]-1}],{-1/tableau[[pr[[j,k]],
		pc[[j]]]]},Table[1,{i,pr[[j,k]]+1,Length[b]}]],
		pr[[j,k]] ]].Binv,{j,1,Length[pc]},{k,1,Length[pr[[j]]]}],1];
	stack = Union[stack,Cases[Table[{0,newbases[[i]],
		newBinv[[i]],Sort[newbases[[i]]]},
		{i,1,Length[newBinv]}],{x1_?NumberQ,x2_?VectorQ,x3_?MatrixQ,
		x4_?VectorQ}/;FreeQ[Last[Transpose[stack]],x4]]];
	If[stack[[1,1]]==1,Return[]];
	stack[[1,1]] = 1;
	SimplexLoop[d,A,b,stack[[1,2]],stack[[1,3]]];
	];

SimplexStart[c_,A1_,A2_,b1_,b2_] := Module[
	{d,A,b,vars,lp,soln,zeroes,pos,zeroset,basis,Binv},
	d = Join[c,Table[0,{Length[b2]}]];
	A = Transpose[Join[Transpose[Join[A1,A2]],Transpose[Join[
		Table[0,{Length[b1]},{Length[b2]}],
		IdentityMatrix[Length[b2]]]]]];
	b = Join[b1,b2];
	vars = Array[z,Length[d]];
	lp = ConstrainedMax[d.vars,Table[A[[i]].vars==b[[i]],
		{i,1,Length[b]}],vars];
	If[NumberQ[First[lp]],,Return[{}]];
	soln = vars /. Last[lp];
	If[Max[soln]==0,Return[{Table[0,{Length[c]}]}]];
	zeroes = Flatten[Position[soln,0]];
	pos = Complement[Range[Length[d]],zeroes];
	zeroset = Ksubsets[zeroes,Length[b]-Length[pos]];
	basis = First[Cases[Join[pos,#] &/@ zeroset,
		x_ /; Det[Transpose[Transpose[A][[x]]]] != 0]];
	Binv = Inverse[Transpose[Transpose[A][[basis]]]];
	stack = {{1,basis,Binv,Sort[basis]}};
	SimplexLoop[d,A,b,basis,Binv];
	Union[Take[Last[Transpose[Sort[Transpose[Flatten[
		Join[{{Join[#[[2]],Complement[Range[Length[d]],
		#[[2]]]]}},{{Join[#[[3]].b,
		Table[0,{Length[d]-Length[b]}]]}}],1]
		]]]],Length[c]] & /@ stack]
	];

StandAloneGame[output_,c_CostFunction] := Module[{N,values},
	N = Range[Length[output]];
	values=(F[output*IndicatorVector[#,N],c] &/@ 
		Subsets[N]);
	Game[N,values]
	];

SubadditiveCover[g_Game] := Module[{w},
	w[{}] = 0;
	w[S:{_}] := v[S,g];
	w[S:{_,__}] := Min[v[#,g]+v[Complement[S,#],g]&/@Subsets[S]];
	Game[First[g],w[#] &/@ Subsets[First[g]]]
	];

Subsets[N_List] := Sort[Flatten /@ Distribute[{{},{#}} & 
	/@ Sort[N], List]];

ToMatrix[eqns_] := Module[{vars},	(* assumes standard form *)
	vars = Union @@
		Cases[eqns,Equal[lhs_,rhs_] :> Variables[lhs]];
	Cases[eqns,Equal[lhs_,rhs_] :>
	    Append[(Coefficient[lhs,#] & /@ vars),
			      rhs]
		]
	];

UIC1[p_,e_,d_DemandSystem] := Take[CS1[p,e,d] - 
	CS1[RotateLeft[p],RotateLeft[e],d],Length[p]-1];

UIC2[q_,r_,d_DemandSystem] := Take[CS2[q,r,d] -
	CS2[RotateLeft[q],RotateLeft[r],d],Length[q]-1]

Utility[q_,d_] := U[q,d];
	U[q_,d_DemandSystem] := If[Length[d]==5,
		d[[5]] /. Thread[Rule[d[[4]],q]]]

v[S_,g_Game] := Last[g][[First[First[Position[Subsets[First[g]],
	Sort[S]]]]]]

Values[g_Game] := Transpose[Join[{Coalitions[g]},{Last[g]}]]

x[S_List] := Plus @@ x /@ S;

ZeroNormalization[g_Game] := Game[First[g],
	Last[g]-(x[#]/.(Payoff[Take[Rest[Last[g]],
	Length[First[g]]],g]) &/@ Coalitions[g])];

End[];
EndPackage[];

