(*:Mathematica Version: 2.2 *)

(*:Package Version: 1.0 *)

(*:Name: Economics`Efficient` *)

(*:Context: Economics`Efficient` *)

(*:Title: Efficiency Analysis *)

(*:Author: Hal R. Varian *)

(*:Address: Hal Varian
            School of Information Management and Systems
            University of California
            Berkeley, CA 94720-xxxx

            hal@sims.berkeley.edu
*)
            

(*:History: Created November 1995 *)

(*:Copyright: Copyright 1995, Hal R. Varian. *)

(*:Reference: Economic and Financial Modeling with Mathematica, Volume II. *)

(*:Summary:
This package provides a number of tools for doing nonparametric efficiency analsyis.
*)

(*:Keywords: consumer demand, revealed preference, nonparametric methods *)

(*:Requirements: No special system requirements. *)

(*:Warning: Very little error checking is done. *)

(*:Sources:
 ``The Nonparametric Approach to Demand Analysis,'' {\it Econometrica}, 50,
 4, July, 1982, 945--972.
 ``Nonparametric Tests of Models of Consumer Behavior,'' {\it Review of
 Economic Studies}, 50, 1982, 99--110.
*)

(* set up the package context, included any imports *)

BeginPackage["Efficient`"]

(* usage messages for the exported functions and the context itself *)

efficOf::usage = "efficOf[i,j,w,x] produces the efficiency of observation i relative to observation j using price data w and factor input data x."

eff::usage = "eff[w,x,y] produces list of efficiencies of each observation relative to the others for price data w, factor input data x, and output data y."

costEfficiency::usage = "costEfficiency[w,x,y] produces a list of cost efficiencies associated with price data w, input data x and output data y."

rdMatrix::usage = "rdMatrix[p,x] produces a matrix whose (i,j) entry is +1 if pi*xi>=pi*xj, 0 if pi*xi<pi*xj."

tClosure::usage = "tClosure[m] produces the transitive closure of the Boolean matrix m."

obsRP::usage = "obsRP[i,m] produces the list of indices of the 
observations that are revealed preferred to observation i using the 
preference relation determined by the matrix m."

consEfficiency::usage = "consEfficiency[p,x] gives a list
whose ith entry is the minimal cost of purchasing a bundle that is
revealed preferred to bundle i as a fraction of the cost of bundle i.
If any entry is less than 1 there is a violation of GARP."

rdEMatrix::usage = "rdEMatrix[p,x,e] gives a matrix with a 1 in the (i,j) entry if observation i is directrly revealed preferred to observation j at efficiency level e."

GARPOK::usage = "GARPOK[p,x,m,e] tests GARP on the data (p,x) with
preference relation matrix m at efficiency level e."

ViolationList::usage = "NumberViolations[p,x,e] gives the number of observations that violate GARP at efficiency level e using the data (p,x)."

NumberViolations::usage = "GARPOK[p,x,m,e] tests GARP on the data (p,x) with
preference relation matrix m at efficiency level e."

NumberViolations::usage = "GARPOK[p,x,m,e] tests GARP on the data (p,x) with
preference relation matrix m at efficiency level e."

AfriatEfficiency::usage = "AfriatEfficiency[p,x,maxSteps] calculates the Afriat efficiencies associated with the data (p,x) using at most maxSteps in the iteration."

DrawABudget::usage = "DrawABudget[{p1_,p2_},{x1_,x2_}] draws the budget associated with prices (p1,p2) and choices (x1,x2)."

DrawBudgets::usage = "DrawBudgets[p_,x_] draws the budget lines associated with the data (p,x)."

PlotViolations::usage = "PlotViolations[p,x] draws the revealed preference violations in the data (p,x)."

DrawViolations::usage = "DrawViolations[p,x] draws the revealed preference violations in the data (p,x) along with the budget lines."

Begin["`Private`"]    (* begin the private context *)

(*--------------------------------------------------*)

(* efficiency of observation i relative to j *)
 
efficOf[i_,j_,w_,x_] := 
   (w[[j]].x[[i]])/(w[[j]].x[[j]])

(* list of efficiency measures *)

eff[w_,x_,y_] := 
          Table[If[y[[i]] <= y[[j]],Min[efficOf[j,i,w,x],1],1],
          {i,1,Length[y]},{j,1,Length[y]}]

(* list of minimum efficiencies *)

costEfficiency[w_,x_,y_] := Map[Min,eff[w,x,y]]


(*--------------------------------------------------*)

(* definition of the exported functions *)

(* revealed direct matrix *)

rdMatrix[p_,x_] := Module[{i,j,n},
                   n=Length[x];
                   Table[If[p[[i]].x[[i]] >= p[[i]].x[[j]],1,0],
                     {i,1,n},{j,1,n}]]

(* transitive closure matrix *)

tClosure[m_] := Sign[MatrixPower[m,Length[m]]]

(* observations revealed preferred to observation i *)

ObsRP[i_,m_] := 
              Module[{j},
              Union[Table[If[m[[j,i]]>0,j,i],{j,1,Length[m]}]]]

(* consumption efficiency *)

consEfficiency[p_,x_] :=
   Module[{m,n},
   m=tClosure[rdMatrix[p,x]];
   n=Length[x];
   Table[Min[x[[ObsRP[i,m]]].p[[i]]/p[[i]].x[[i]]],
        {i,1,n}]]

(* rd matrix at efficiency level e *)

rdEMatrix[p_,x_,e_] := Module[{i,j,n},
                    n=Length[p];
                    Table[If[(e*p[[i]].x[[i]] >= p[[i]].x[[j]])
                       || i==j,1,0],
                     {i,1,n},{j,1,n}]]

(* checks to see if GARP is satisfied *)


GARPOK[p_,x_,m_,e_] := Module[{i,j,n},
                       n=Length[p];
                        Table[
                          If[m[[i,j]]==1 && (e*p[[j]].x[[j]]
                             > p[[j]].x[[i]]),False,True],
                             {j,1,n},{i,1,n}]]

(* list of violations of GARP *)

ViolationList[p_,x_,e_] :=
            Module[{m},
            m = tClosure[rdEMatrix[p,x,e]];
            Union[Flatten[Position[GARPOK[p,x,m,e],False]]]]

(* number of violations of GARP *)

NumberViolations[p_,x_,e_] := Length[ViolationList[p,x,e]]

(* Afriat efficiency *)

AfriatEfficiency[p_,x_,maxSteps_] :=
  Module[{step,e},
  e=1/2;
    For[step=1,step<=maxSteps,step++,
      If[NumberViolations[p,x,e] > 0,
         e=e-2^(-(step+1)),
         e=e+2^(-(step+1))];
      Print[N[e]]]]

(*--------------------------------------------------*)

(* drawing budget sets *)


DrawABudget[{p1_,p2_},{x1_,x2_}] := Module[
                     {w,t},
                     w=p1*x1 + p2*x2;
                     Plot[w/p2 - p1*t/p2,{t,0,w/p1},
                     AspectRatio->1,DisplayFunction->Identity]] 
          
DrawBudgets[p_,x_] := Table[{DrawABudget[p[[i]],x[[i]]],
                            Graphics[Point[x[[i]]]]}, 
                            {i,1,Length[p]}]
                                  
PlotViolations[p_,x_] :=
     ListPlot[x[[ViolationList[p,x,1]]],
     AspectRatio->1, DisplayFunction->Identity, PlotStyle->{Hue[0],PointSize[.02]}]

DrawViolations[p_,x_] := Show[DrawBudgets[p,x],PlotViolations[p,x],
                          Prolog->PointSize[.02],
                          DisplayFunction->$DisplayFunction]

Protect[]    (* protect exported symbols *)

EndPackage[]  (* end the package context *)
